@@ -31,6 +31,7 @@
 #include "cli.h"
 #include "cli-cmd.h"
 #include "cli-mem-types.h"
+#include "protocol-common.h"
 
 #include <fnmatch.h>
 
@@ -59,6 +60,9 @@ cli_cmd_needs_connection (struct cli_cmd_word *word)
         if (!strcasecmp ("getwd", word->word))
                 return 1;
 
+        if (!strcasecmp ("exit", word->word))
+                return 0;
+
         return CLI_DEFAULT_CONN_TIMEOUT;
 }
 
@@ -281,14 +285,14 @@ seconds_from_now (unsigned secs, struct timespec *ts)
 }
 
 int
-cli_cmd_await_response ()
+cli_cmd_await_response (unsigned time)
 {
         struct  timespec        ts = {0,};
         int                     ret = 0;
 
         cli_op_ret = -1;
 
-        seconds_from_now (CLI_DEFAULT_CMD_TIMEOUT, &ts);
+        seconds_from_now (time, &ts);
         while (!cmd_done && !ret) {
                 ret = pthread_cond_timedwait (&cond, &cond_mutex,
                                         &ts);
@@ -296,8 +300,6 @@ cli_cmd_await_response ()
 
         cmd_done = 0;
 
-        cli_cmd_unlock ();
-
         if (ret)
                 return ret;
 
@@ -364,22 +366,25 @@ int
 cli_cmd_submit (void *req, call_frame_t *frame,
                 rpc_clnt_prog_t *prog,
                 int procnum, struct iobref *iobref,
-                cli_serialize_t sfunc, xlator_t *this,
-                fop_cbk_fn_t cbkfn)
+                xlator_t *this, fop_cbk_fn_t cbkfn, xdrproc_t xdrproc)
 {
-        int     ret = -1;
+        int             ret = -1;
+        unsigned        timeout = 0;
+
+        timeout = (GLUSTER_CLI_PROFILE_VOLUME == procnum) ?
+                   CLI_TOP_CMD_TIMEOUT : CLI_DEFAULT_CMD_TIMEOUT;
 
         cli_cmd_lock ();
         cmd_sent = 0;
         ret = cli_submit_request (req, frame, prog,
-                                  procnum, NULL, sfunc,
-                                  this, cbkfn);
+                                  procnum, NULL, this, cbkfn, xdrproc);
 
         if (!ret) {
                 cmd_sent = 1;
-                ret = cli_cmd_await_response ();
-        } else
-                cli_cmd_unlock ();
+                ret = cli_cmd_await_response (timeout);
+        }
+
+        cli_cmd_unlock ();
 
         gf_log ("cli", GF_LOG_DEBUG, "Returning %d", ret);
         return ret;
