@@ -74,11 +74,10 @@
 #include "call-stub.h"
 #include <fnmatch.h>
 
+#include "xdr-generic.h"
+
 extern int connected;
 /* using argp for command line parsing */
-static char gf_doc[] = "";
-
-static char argp_doc[] = "";
 
 const char *argp_program_version = ""                                 \
         PACKAGE_NAME" "PACKAGE_VERSION" built on "__DATE__" "__TIME__ \
@@ -91,10 +90,7 @@ const char *argp_program_version = ""                                 \
 
 const char *argp_program_bug_address = "<" PACKAGE_BUGREPORT ">";
 
-static struct argp_option gf_options[] = {
-        {0, 0, 0, 0, "Basic options:"},
-        {0, }
-};
+
 
 struct rpc_clnt *global_rpc;
 
@@ -103,39 +99,6 @@ rpc_clnt_prog_t *cli_rpc_prog;
 
 extern struct rpc_clnt_program cli_prog;
 
-static error_t
-parse_opts (int key, char *arg, struct argp_state *argp_state)
-{
-        struct cli_state  *state = NULL;
-        char             **argv = NULL;
-
-        state = argp_state->input;
-
-        switch (key) {
-        case ARGP_KEY_ARG:
-                if (!state->argc) {
-                        argv = calloc (state->argc + 2,
-                                       sizeof (*state->argv));
-                } else {
-                        argv = realloc (state->argv, (state->argc + 2) *
-                                        sizeof (*state->argv));
-                }
-                if (!argv)
-                        return -1;
-
-                state->argv = argv;
-
-                argv[state->argc] = strdup (arg);
-                if (!argv[state->argc])
-                        return -1;
-                state->argc++;
-                argv[state->argc] = NULL;
-
-                break;
-        }
-
-        return 0;
-}
 
 
 
@@ -188,7 +151,7 @@ glusterfs_ctx_defaults_init (glusterfs_ctx_t *ctx)
 
         ctx->page_size  = 128 * GF_UNIT_KB;
 
-        ctx->iobuf_pool = iobuf_pool_new (8 * GF_UNIT_MB, ctx->page_size);
+        ctx->iobuf_pool = iobuf_pool_new ();
         if (!ctx->iobuf_pool)
                 return -1;
 
@@ -201,22 +164,33 @@ glusterfs_ctx_defaults_init (glusterfs_ctx_t *ctx)
         if (!pool)
                 return -1;
 
-        /* frame_mem_pool size 112 * 16k */
-        pool->frame_mem_pool = mem_pool_new (call_frame_t, 16384);
-
+        /* frame_mem_pool size 112 * 64 */
+        pool->frame_mem_pool = mem_pool_new (call_frame_t, 32);
         if (!pool->frame_mem_pool)
                 return -1;
 
-        /* stack_mem_pool size 256 * 8k */
-        pool->stack_mem_pool = mem_pool_new (call_stack_t, 8192); 
+        /* stack_mem_pool size 256 * 128 */
+        pool->stack_mem_pool = mem_pool_new (call_stack_t, 16);
 
         if (!pool->stack_mem_pool)
                 return -1;
 
-        ctx->stub_mem_pool = mem_pool_new (call_stub_t, 1024);
+        ctx->stub_mem_pool = mem_pool_new (call_stub_t, 16);
         if (!ctx->stub_mem_pool)
                 return -1;
 
+        ctx->dict_pool = mem_pool_new (dict_t, 32);
+        if (!ctx->dict_pool)
+                return -1;
+
+        ctx->dict_pair_pool = mem_pool_new (data_pair_t, 512);
+        if (!ctx->dict_pair_pool)
+                return -1;
+
+        ctx->dict_data_pool = mem_pool_new (data_t, 512);
+        if (!ctx->dict_data_pool)
+                return -1;
+
         INIT_LIST_HEAD (&pool->all_frames);
         LOCK_INIT (&pool->lock);
         ctx->pool = pool;
@@ -236,27 +210,21 @@ glusterfs_ctx_defaults_init (glusterfs_ctx_t *ctx)
 
 
 static int
-logging_init (glusterfs_ctx_t *ctx)
+logging_init (struct cli_state *state)
 {
-        int         ret      = 0;
-        cmd_args_t *cmd_args = NULL;
+        char *log_file = state->log_file ? state->log_file :
+                         DEFAULT_CLI_LOG_FILE_DIRECTORY "/cli.log";
 
-        cmd_args = &ctx->cmd_args;
-
-        /* CLI should not have something to DEBUG after the release,
-           hence defaulting to INFO loglevel */
-        cmd_args->log_level = GF_LOG_INFO;
-
-        ret = gf_asprintf (&cmd_args->log_file,
-                           DEFAULT_CLI_LOG_FILE_DIRECTORY "/cli.log");
-
-        if (gf_log_init (cmd_args->log_file) == -1) {
+        if (gf_log_init (log_file) == -1) {
                 fprintf (stderr, "ERROR: failed to open logfile %s\n",
-                         cmd_args->log_file);
+                         log_file);
                 return -1;
         }
 
-        gf_log_set_loglevel (cmd_args->log_level);
+        /* CLI should not have something to DEBUG after the release,
+           hence defaulting to INFO loglevel */
+        gf_log_set_loglevel ((state->log_level == -1) ? GF_LOG_INFO :
+                             state->log_level);
 
         return 0;
 }
@@ -265,41 +233,41 @@ int
 cli_submit_request (void *req, call_frame_t *frame,
                     rpc_clnt_prog_t *prog,
                     int procnum, struct iobref *iobref,
-                    cli_serialize_t sfunc, xlator_t *this,
-                    fop_cbk_fn_t cbkfn)
+                    xlator_t *this, fop_cbk_fn_t cbkfn, xdrproc_t xdrproc)
 {
         int                     ret         = -1;
         int                     count      = 0;
-        char                    start_ping = 0;
         struct iovec            iov         = {0, };
         struct iobuf            *iobuf = NULL;
         char                    new_iobref = 0;
+        ssize_t                 xdr_size   = 0;
 
         GF_ASSERT (this);
 
-        iobuf = iobuf_get (this->ctx->iobuf_pool);
-        if (!iobuf) {
-                goto out;
-        };
-
-        if (!iobref) {
-                iobref = iobref_new ();
-                if (!iobref) {
+        if (req) {
+                xdr_size = xdr_sizeof (xdrproc, req);
+                iobuf = iobuf_get2 (this->ctx->iobuf_pool, xdr_size);
+                if (!iobuf) {
                         goto out;
+                };
+
+                if (!iobref) {
+                        iobref = iobref_new ();
+                        if (!iobref) {
+                                goto out;
+                        }
+
+                        new_iobref = 1;
                 }
 
-                new_iobref = 1;
-        }
+                iobref_add (iobref, iobuf);
 
-        iobref_add (iobref, iobuf);
-
-        iov.iov_base = iobuf->ptr;
-        iov.iov_len  = 128 * GF_UNIT_KB;
+                iov.iov_base = iobuf->ptr;
+                iov.iov_len  = iobuf_size (iobuf);
 
 
-        /* Create the xdr payload */
-        if (req && sfunc) {
-                ret = sfunc (iov, req);
+                /* Create the xdr payload */
+                ret = xdr_serialize_generic (iov, req, xdrproc);
                 if (ret == -1) {
                         goto out;
                 }
@@ -311,22 +279,13 @@ cli_submit_request (void *req, call_frame_t *frame,
         ret = rpc_clnt_submit (global_rpc, prog, procnum, cbkfn,
                                &iov, count,
                                NULL, 0, iobref, frame, NULL, 0, NULL, 0, NULL);
-
-        if (ret == 0) {
-                pthread_mutex_lock (&global_rpc->conn.lock);
-                {
-                        if (!global_rpc->conn.ping_started) {
-                                start_ping = 1;
-                        }
-                }
-                pthread_mutex_unlock (&global_rpc->conn.lock);
-        }
-
         ret = 0;
 
 out:
         if (new_iobref)
                 iobref_unref (iobref);
+        if (iobuf)
+                iobuf_unref (iobuf);
         return ret;
 }
 
@@ -376,6 +335,16 @@ cli_opt_parse (char *opt, struct cli_state *state)
 {
         char *oarg;
 
+        if (strcmp (opt, "version") == 0) {
+                puts (argp_program_version);
+                exit (0);
+        }
+
+        if (strcmp (opt, "xml") == 0) {
+                state->mode |= GLUSTER_MODE_XML;
+                return 0;
+        }
+
         oarg = strtail (opt, "mode=");
         if (oarg) {
                 if (strcmp (oarg, "script") == 0) {
@@ -393,6 +362,20 @@ cli_opt_parse (char *opt, struct cli_state *state)
                 return 0;
         }
 
+        oarg = strtail (opt, "log-file=");
+        if (oarg) {
+                state->log_file = oarg;
+                return 0;
+        }
+
+        oarg = strtail (opt, "log-level=");
+        if (oarg) {
+                state->log_level = glusterd_check_log_level(oarg);
+                if (state->log_level == -1)
+                        return -1;
+                return 0;
+        }
+
         return -1;
 }
 
@@ -403,29 +386,26 @@ parse_cmdline (int argc, char *argv[], struct cli_state *state)
         int         i = 0;
         int         j = 0;
         char        *opt = NULL;
-        struct argp argp = { 0,};
 
-        argp.options    = gf_options;
-        argp.parser     = parse_opts;
-        argp.args_doc   = argp_doc;
-        argp.doc        = gf_doc;
+        state->argc=argc-1;
+        state->argv=&argv[1];
 
-        for (i = 0; i < argc; i++) {
-                opt = strtail (argv[i], "--");
+        for (i = 0; i < state->argc; i++) {
+                opt = strtail (state->argv[i], "--");
                 if (opt) {
                         ret = cli_opt_parse (opt, state);
                         if (ret == -1) {
-                                break;
+                                cli_out ("unrecognized option --%s", opt);
+                                return ret;
                         }
-                        for (j = i; j < argc - 1; j++)
-                                argv[j] = argv[j + 1];
-                        argc--;
+                        for (j = i; j < state->argc - 1; j++)
+                                state->argv[j] = state->argv[j + 1];
+                        state->argc--;
+                        /* argv shifted, next check should be at i again */
+                        i--;
                 }
         }
 
-        ret = argp_parse (&argp, argc, argv,
-                          ARGP_IN_ORDER, NULL, state);
-
         return ret;
 }
 
@@ -451,6 +431,7 @@ cli_state_init (struct cli_state *state)
 
 
         state->remote_host = "localhost";
+        state->log_level = -1;
 
         tree = &state->tree;
         tree->state = state;
@@ -469,12 +450,36 @@ cli_usage_out (const char *usage)
         if (!usage || usage[0] == '\0')
                 return -1;
 
-        cli_out ("Usage: %s", usage);
+        cli_err ("Usage: %s", usage);
         return 0;
 }
 
 int
-cli_out (const char *fmt, ...)
+_cli_err (const char *fmt, ...)
+{
+        struct cli_state *state = NULL;
+        va_list           ap;
+        int               ret = 0;
+
+        state = global_state;
+
+        va_start (ap, fmt);
+
+#ifdef HAVE_READLINE
+        if (state->rl_enabled && !state->rl_processing)
+                return cli_rl_err(state, fmt, ap);
+#endif
+
+        ret = vfprintf (stderr, fmt, ap);
+        fprintf (stderr, "\n");
+        va_end (ap);
+
+        return ret;
+}
+
+
+int
+_cli_out (const char *fmt, ...)
 {
         struct cli_state *state = NULL;
         va_list           ap;
@@ -527,7 +532,7 @@ cli_rpc_init (struct cli_state *state)
         if (ret)
                 goto out;
 
-        rpc = rpc_clnt_new (options, this->ctx, this->name);
+        rpc = rpc_clnt_new (options, this->ctx, this->name, 16);
 
         if (!rpc)
                 goto out;
@@ -562,30 +567,78 @@ void
 cli_local_wipe (cli_local_t *local)
 {
         if (local) {
+                if (local->get_vol.volname)
+                        GF_FREE (local->get_vol.volname);
+                if (local->dict)
+                        dict_unref (local->dict);
                 GF_FREE (local);
         }
 
         return;
 }
 
-void
-cli_path_strip_trailing_slashes (char *path)
+/* If the path exists use realpath(3) to handle extra slashes and to resolve
+ * symlinks else strip the extra slashes in the path and return */
+
+int
+cli_canonicalize_path (char *path)
 {
-        int i = 0;
-        int len = 0;
+        struct stat     sb = {0};
+        int             ret = -1;
+        char            *tmppath = NULL;
+        char            *dir = NULL;
+        char            *tmpstr = NULL;
+        int             path_len = 0;
 
         if (!path)
-                return;
-
-        len = strlen (path);
-        for (i = len - 1; i > 0 ; i--) {
-                if (path[i] != '/')
-                        break;
+                return ret;
 
+        ret = stat (path, &sb);
+        if (ret == -1) {
+                /* Strip the extra slashes and return */
+                tmppath = gf_strdup (path);
+                if (tmppath == NULL) {
+                        ret = -1;
+                        gf_log ("cli", GF_LOG_ERROR, "Out of memory.");
+                        goto out;
+                }
+                bzero (path, strlen(path));
+                path[0] = '/';
+                dir = strtok_r(tmppath, "/", &tmpstr);
+                while (dir) {
+                        strncpy ((path + path_len + 1), dir, strlen(dir));
+                        path_len = strlen (path);
+                        dir = strtok_r(NULL, "/", &tmpstr);
+                        if (dir)
+                                strncpy((path + path_len), "/", 1);
+                }
+                if (path_len == 0)
+                        path[1] = '\0';
+                else
+                        path[path_len] = '\0';
+                ret = 0;
+                goto out;
+        } else {
+                tmppath = gf_strdup(path);
+                if (tmppath == NULL) {
+                        ret = -1;
+                        gf_log ("cli", GF_LOG_ERROR, "Out of memory.");
+                        goto out;
+                }
+                if (realpath (tmppath, path) == NULL) {
+                        cli_out ("Path manipulation failed: %s",
+                                 strerror(errno));
+                        gf_log ("cli", GF_LOG_ERROR, "Path manipulation "
+                                 "failed: %s", strerror(errno));
+                        ret = -1;
+                        goto out;
+                }
+                ret = 0;
         }
-
-        if (i < (len - 1))
-                path[i + 1] = '\0';
+out:
+        if (tmppath)
+                GF_FREE(tmppath);
+        return ret;
 }
 
 struct cli_state *global_state;
@@ -620,12 +673,7 @@ main (int argc, char *argv[])
         if (ret)
                 goto out;
 
-        if (geteuid ()) {
-                printf ("Only super user can run this command\n");
-                return EPERM;
-        }
-
-        ret = logging_init (ctx);
+        ret = logging_init (&state);
         if (ret)
                 goto out;
 
@@ -652,3 +700,14 @@ out:
 
         return ret;
 }
+
+void
+cli_print_line (int len)
+{
+        GF_ASSERT (len > 0);
+
+        while (len--)
+                printf ("-");
+
+        printf ("\n");
+}
