@@ -25,6 +25,7 @@
 #include <sys/types.h>
 #include <sys/resource.h>
 #include <sys/file.h>
+#include <sys/wait.h>
 #include <netdb.h>
 #include <signal.h>
 #include <libgen.h>
@@ -38,6 +39,7 @@
 #include <time.h>
 #include <semaphore.h>
 #include <errno.h>
+#include <pwd.h>
 
 #ifndef _CONFIG_H
 #define _CONFIG_H
@@ -74,13 +76,14 @@
 #include "call-stub.h"
 #include <fnmatch.h>
 #include "rpc-clnt.h"
+#include "syncop.h"
 
-#ifdef GF_DARWIN_HOST_OS
 #include "daemon.h"
-#else
-#define os_daemon(u, v) daemon (u, v)
-#endif
 
+/* process mode definitions */
+#define GF_SERVER_PROCESS   0
+#define GF_CLIENT_PROCESS   1
+#define GF_GLUSTERD_PROCESS 2
 
 /* using argp for command line parsing */
 static char gf_doc[] = "";
@@ -139,15 +142,20 @@ static struct argp_option gf_options[] = {
         {"debug", ARGP_DEBUG_KEY, 0, 0,
          "Run in debug mode.  This option sets --no-daemon, --log-level "
          "to DEBUG and --log-file to console"},
-        {"volume-name", ARGP_VOLUME_NAME_KEY, "VOLUME-NAME", 0,
-         "Volume name to be used for MOUNT-POINT [default: top most volume "
-         "in VOLFILE]"},
-        {"xlator-option", ARGP_XLATOR_OPTION_KEY,"VOLUME-NAME.OPTION=VALUE", 0,
-         "Add/override a translator option for a volume with specified value"},
+        {"volume-name", ARGP_VOLUME_NAME_KEY, "XLATOR-NAME", 0,
+         "Translator name to be used for MOUNT-POINT [default: top most volume "
+         "definition in VOLFILE]"},
+        {"xlator-option", ARGP_XLATOR_OPTION_KEY,"XLATOR-NAME.OPTION=VALUE", 0,
+         "Add/override an option for a translator in volume file with specified"
+         " value"},
         {"read-only", ARGP_READ_ONLY_KEY, 0, 0,
          "Mount the filesystem in 'read-only' mode"},
         {"acl", ARGP_ACL_KEY, 0, 0,
          "Mount the filesystem with POSIX ACL support"},
+        {"selinux", ARGP_SELINUX_KEY, 0, 0,
+         "Enable SELinux label (extened attributes) support on inodes"},
+        {"worm", ARGP_WORM_KEY, 0, 0,
+         "Mount the filesystem in 'worm' mode"},
         {"mac-compat", ARGP_MAC_COMPAT_KEY, "BOOL", OPTION_ARG_OPTIONAL,
          "Provide stubs for attributes needed for seamless operation on Macs "
 #ifdef GF_DARWIN_HOST_OS
@@ -173,10 +181,14 @@ static struct argp_option gf_options[] = {
          "[default: 1]"},
         {"client-pid", ARGP_CLIENT_PID_KEY, "PID", OPTION_HIDDEN,
          "client will authenticate itself with process id PID to server"},
+        {"user-map-root", ARGP_USER_MAP_ROOT_KEY, "USER", OPTION_HIDDEN,
+         "replace USER with root in messages"},
         {"dump-fuse", ARGP_DUMP_FUSE_KEY, "PATH", 0,
          "Dump fuse traffic to PATH"},
         {"volfile-check", ARGP_VOLFILE_CHECK_KEY, 0, 0,
          "Enable strict volume file checking"},
+        {"mem-accounting", ARGP_MEM_ACCOUNTING_KEY, 0, OPTION_HIDDEN,
+         "Enable internal memory accounting"},
         {0, 0, 0, 0, "Miscellaneous Options:"},
         {0, }
 };
@@ -188,6 +200,7 @@ int glusterfs_pidfile_cleanup (glusterfs_ctx_t *ctx);
 int glusterfs_volumes_init (glusterfs_ctx_t *ctx);
 int glusterfs_mgmt_init (glusterfs_ctx_t *ctx);
 int glusterfs_listener_init (glusterfs_ctx_t *ctx);
+int glusterfs_listener_stop (glusterfs_ctx_t *ctx);
 
 int
 create_fuse_mount (glusterfs_ctx_t *ctx)
@@ -195,6 +208,8 @@ create_fuse_mount (glusterfs_ctx_t *ctx)
         int              ret = 0;
         cmd_args_t      *cmd_args = NULL;
         xlator_t        *master = NULL;
+        char            *mount_point = NULL;
+        char            cwd[PATH_MAX] = {0,};
 
         cmd_args = &ctx->cmd_args;
 
@@ -204,6 +219,12 @@ create_fuse_mount (glusterfs_ctx_t *ctx)
                 return 0;
         }
 
+        if (ctx->process_mode != GF_CLIENT_PROCESS) {
+                gf_log("glusterfsd", GF_LOG_ERROR,
+                       "Not a client process, not performing mount operation");
+                return -1;
+        }
+
         master = GF_CALLOC (1, sizeof (*master),
                             gfd_mt_xlator_t);
         if (!master)
@@ -225,8 +246,28 @@ create_fuse_mount (glusterfs_ctx_t *ctx)
         if (!master->options)
                 goto err;
 
-        ret = dict_set_static_ptr (master->options, ZR_MOUNTPOINT_OPT,
-                                   cmd_args->mount_point);
+        /* Check if mount-point is absolute path,
+         * if not convert to absolute path by concating with CWD
+         */
+        if (cmd_args->mount_point[0] != '/') {
+                if (getcwd (cwd, PATH_MAX) != NULL) {
+                        ret = gf_asprintf (&mount_point, "%s/%s", cwd,
+                                           cmd_args->mount_point);
+                        if (ret == -1) {
+                                gf_log ("glusterfsd", GF_LOG_ERROR,
+                                        "Could not create absolute mountpoint "
+                                        "path");
+                                goto err;
+                        }
+                } else {
+                        gf_log ("glusterfsd", GF_LOG_ERROR,
+                                "Could not get current working directory");
+                        goto err;
+                }
+        } else
+                mount_point = gf_strdup (cmd_args->mount_point);
+
+        ret = dict_set_dynstr (master->options, ZR_MOUNTPOINT_OPT, mount_point);
         if (ret < 0) {
                 gf_log ("glusterfsd", GF_LOG_ERROR,
                         "failed to set mount-point to options dictionary");
@@ -267,6 +308,17 @@ create_fuse_mount (glusterfs_ctx_t *ctx)
                 }
         }
 
+        if (cmd_args->uid_map_root) {
+                ret = dict_set_int32 (master->options, "uid-map-root",
+                                      cmd_args->uid_map_root);
+                if (ret < 0) {
+                        gf_log ("glusterfsd", GF_LOG_ERROR,
+                                "failed to set dict value for key %s",
+                                "uid-map-root");
+                        goto err;
+                }
+        }
+
         if (cmd_args->volfile_check) {
                 ret = dict_set_int32 (master->options, ZR_STRICT_VOLFILE_CHECK,
                                       cmd_args->volfile_check);
@@ -298,6 +350,24 @@ create_fuse_mount (glusterfs_ctx_t *ctx)
                 }
         }
 
+        if (cmd_args->selinux) {
+                ret = dict_set_static_ptr (master->options, "selinux", "on");
+                if (ret < 0) {
+                        gf_log ("glusterfsd", GF_LOG_ERROR,
+                                "failed to set dict value for key selinux");
+                        goto err;
+                }
+        }
+
+        if (cmd_args->read_only) {
+                ret = dict_set_static_ptr (master->options, "read-only", "on");
+                if (ret < 0) {
+                        gf_log ("glusterfsd", GF_LOG_ERROR,
+                                "failed to set dict value for key read-only");
+                        goto err;
+                }
+        }
+
         switch (cmd_args->fuse_direct_io_mode) {
         case GF_OPTION_DISABLE: /* disable */
                 ret = dict_set_static_ptr (master->options, ZR_DIRECT_IO_OPT,
@@ -326,6 +396,16 @@ create_fuse_mount (glusterfs_ctx_t *ctx)
                 break;
         }
 
+        if (!cmd_args->no_daemon_mode) {
+                 ret = dict_set_static_ptr (master->options, "sync-to-mount",
+                                            "enable");
+                if (ret < 0) {
+                        gf_log ("glusterfsd", GF_LOG_ERROR,
+                                "failed to set dict value for key sync-mtab");
+                        goto err;
+                }
+        }
+
         ret = xlator_init (master);
         if (ret) {
                 gf_log ("", GF_LOG_DEBUG, "failed to initialize fuse translator");
@@ -376,7 +456,6 @@ get_volfp (glusterfs_ctx_t *ctx)
         return specfp;
 }
 
-
 static int
 gf_remember_xlator_option (struct list_head *options, char *arg)
 {
@@ -458,6 +537,7 @@ out:
 static error_t
 parse_opts (int key, char *arg, struct argp_state *state)
 {
+        glusterfs_ctx_t *ctx        = NULL;
         cmd_args_t   *cmd_args      = NULL;
         uint32_t      n             = 0;
         double        d             = 0.0;
@@ -466,6 +546,7 @@ parse_opts (int key, char *arg, struct argp_state *state)
         char          tmp_buf[2048] = {0,};
         char         *tmp_str       = NULL;
         char         *port_str      = NULL;
+        struct passwd *pw           = NULL;
 
         cmd_args = state->input;
 
@@ -494,6 +575,14 @@ parse_opts (int key, char *arg, struct argp_state *state)
                 cmd_args->acl = 1;
                 break;
 
+        case ARGP_SELINUX_KEY:
+                cmd_args->selinux = 1;
+                break;
+
+        case ARGP_WORM_KEY:
+                cmd_args->worm = 1;
+                break;
+
         case ARGP_MAC_COMPAT_KEY:
                 if (!arg)
                         arg = "on";
@@ -655,6 +744,15 @@ parse_opts (int key, char *arg, struct argp_state *state)
                               "unknown client pid %s", arg);
                 break;
 
+        case ARGP_USER_MAP_ROOT_KEY:
+                pw = getpwnam (arg);
+                if (pw)
+                        cmd_args->uid_map_root = pw->pw_uid;
+                else
+                        argp_failure (state, -1, 0,
+                                      "user %s does not exist", arg);
+                break;
+
         case ARGP_VOLFILE_CHECK_KEY:
                 cmd_args->volfile_check = 1;
                 break;
@@ -706,6 +804,12 @@ parse_opts (int key, char *arg, struct argp_state *state)
                 argp_failure (state, -1, 0,
                               "unknown brick (listen) port %s", arg);
                 break;
+
+        case ARGP_MEM_ACCOUNTING_KEY:
+                /* TODO: it should have got handled much earlier */
+                ctx = glusterfs_ctx_get ();
+                ctx->mem_accounting = 1;
+                break;
         }
 
         return 0;
@@ -732,7 +836,7 @@ cleanup_and_exit (int signum)
         ctx->cleanup_started = 1;
         glusterfs_mgmt_pmap_signout (ctx);
         if (ctx->listener) {
-                ctx->listener = NULL;
+                (void) glusterfs_listener_stop (ctx);
         }
 
         /* Call fini() of FUSE xlator first:
@@ -748,9 +852,12 @@ cleanup_and_exit (int signum)
 
         exit (0);
 #if 0
-        /* TODO: Properly do cleanup_and_exit(), with synchronisations */
-        if (ctx->mgmt)
+        /* TODO: Properly do cleanup_and_exit(), with synchronization */
+        if (ctx->mgmt) {
+                /* cleanup the saved-frames before last unref */
+                rpc_clnt_connection_cleanup (&ctx->mgmt->conn);
                 rpc_clnt_unref (ctx->mgmt);
+        }
 
         /* call fini() of each xlator */
         trav = NULL;
@@ -782,12 +889,11 @@ reincarnate (int signum)
                         "Fetching the volume file from server...");
                 ret = glusterfs_volfile_fetch (ctx);
         } else {
-                gf_log ("glusterfsd", GF_LOG_INFO,
-                        "Reloading volfile ...");
-                ret = glusterfs_volumes_init (ctx);
+                gf_log ("glusterfsd", GF_LOG_DEBUG,
+                        "Not reloading volume specification file on SIGHUP");
         }
 
-        /* Also, SIGHUP should do logroate */
+        /* Also, SIGHUP should do logrotate */
         gf_log_logrotate (1);
 
         if (ret < 0)
@@ -827,9 +933,6 @@ generate_uuid ()
         return gf_strdup (tmp_str);
 }
 
-#define GF_SERVER_PROCESS   0
-#define GF_CLIENT_PROCESS   1
-#define GF_GLUSTERD_PROCESS 2
 
 static uint8_t
 gf_get_process_mode (char *exec_name)
@@ -934,7 +1037,7 @@ glusterfs_ctx_defaults_init (glusterfs_ctx_t *ctx)
 
         ctx->page_size  = 128 * GF_UNIT_KB;
 
-        ctx->iobuf_pool = iobuf_pool_new (8 * GF_UNIT_MB, ctx->page_size);
+        ctx->iobuf_pool = iobuf_pool_new ();
         if (!ctx->iobuf_pool) {
                 gf_log ("", GF_LOG_CRITICAL,
                         "ERROR: glusterfs iobuf pool creation failed");
@@ -956,17 +1059,15 @@ glusterfs_ctx_defaults_init (glusterfs_ctx_t *ctx)
                 return -1;
         }
 
-        /* frame_mem_pool size 112 * 16k */
-        pool->frame_mem_pool = mem_pool_new (call_frame_t, 16384);
-
+        /* frame_mem_pool size 112 * 4k */
+        pool->frame_mem_pool = mem_pool_new (call_frame_t, 4096);
         if (!pool->frame_mem_pool) {
                 gf_log ("", GF_LOG_CRITICAL,
                         "ERROR: glusterfs frame pool creation failed");
                 return -1;
         }
-        /* stack_mem_pool size 256 * 8k */
-        pool->stack_mem_pool = mem_pool_new (call_stack_t, 8192);
-
+        /* stack_mem_pool size 256 * 1024 */
+        pool->stack_mem_pool = mem_pool_new (call_stack_t, 1024);
         if (!pool->stack_mem_pool) {
                 gf_log ("", GF_LOG_CRITICAL,
                         "ERROR: glusterfs stack pool creation failed");
@@ -980,6 +1081,19 @@ glusterfs_ctx_defaults_init (glusterfs_ctx_t *ctx)
                 return -1;
         }
 
+        ctx->dict_pool = mem_pool_new (dict_t, GF_MEMPOOL_COUNT_OF_DICT_T);
+        if (!ctx->dict_pool)
+                return -1;
+
+        ctx->dict_pair_pool = mem_pool_new (data_pair_t,
+                                            GF_MEMPOOL_COUNT_OF_DATA_PAIR_T);
+        if (!ctx->dict_pair_pool)
+                return -1;
+
+        ctx->dict_data_pool = mem_pool_new (data_t, GF_MEMPOOL_COUNT_OF_DATA_T);
+        if (!ctx->dict_data_pool)
+                return -1;
+
         INIT_LIST_HEAD (&pool->all_frames);
         LOCK_INIT (&pool->lock);
         ctx->pool = pool;
@@ -990,14 +1104,14 @@ glusterfs_ctx_defaults_init (glusterfs_ctx_t *ctx)
 
         /* parsing command line arguments */
         cmd_args->log_level = DEFAULT_LOG_LEVEL;
+
+        cmd_args->mac_compat = GF_OPTION_DISABLE;
 #ifdef GF_DARWIN_HOST_OS
-        cmd_args->mac_compat = GF_OPTION_DEFERRED;
         /* On Darwin machines, O_APPEND is not handled,
          * which may corrupt the data
          */
         cmd_args->fuse_direct_io_mode = GF_OPTION_DISABLE;
 #else
-        cmd_args->mac_compat = GF_OPTION_DISABLE;
         cmd_args->fuse_direct_io_mode = GF_OPTION_DEFERRED;
 #endif
         cmd_args->fuse_attribute_timeout = -1;
@@ -1012,7 +1126,6 @@ glusterfs_ctx_defaults_init (glusterfs_ctx_t *ctx)
         return 0;
 }
 
-
 static int
 logging_init (glusterfs_ctx_t *ctx)
 {
@@ -1040,6 +1153,17 @@ logging_init (glusterfs_ctx_t *ctx)
         return 0;
 }
 
+void
+gf_check_and_set_mem_acct (int argc, char *argv[], glusterfs_ctx_t *ctx)
+{
+        int i = 0;
+        for (i = 0; i < argc; i++) {
+                if (strcmp (argv[i], "--mem-accounting") == 0) {
+                        ctx->mem_accounting = 1;
+                        break;
+                }
+        }
+}
 
 int
 parse_cmdline (int argc, char *argv[], glusterfs_ctx_t *ctx)
@@ -1066,6 +1190,17 @@ parse_cmdline (int argc, char *argv[], glusterfs_ctx_t *ctx)
         }
 
         process_mode = gf_get_process_mode (argv[0]);
+        ctx->process_mode = process_mode;
+
+        /* Make sure after the parsing cli, if '--volfile-server' option is
+           given, then '--volfile-id' is mandatory */
+        if (cmd_args->volfile_server && !cmd_args->volfile_id) {
+                gf_log ("glusterfs", GF_LOG_CRITICAL,
+                        "ERROR: '--volfile-id' is mandatory if '-s' OR "
+                        "'--volfile-server' option is given");
+                ret = -1;
+                goto out;
+        }
 
         if ((cmd_args->volfile_server == NULL)
             && (cmd_args->volfile == NULL)) {
@@ -1075,6 +1210,19 @@ parse_cmdline (int argc, char *argv[], glusterfs_ctx_t *ctx)
                         cmd_args->volfile = gf_strdup (DEFAULT_GLUSTERD_VOLFILE);
                 else
                         cmd_args->volfile = gf_strdup (DEFAULT_CLIENT_VOLFILE);
+
+                /* Check if the volfile exists, if not give usage output
+                   and exit */
+                ret = stat (cmd_args->volfile, &stbuf);
+                if (ret) {
+                        gf_log ("glusterfs", GF_LOG_CRITICAL,
+                                "ERROR: parsing the volfile failed (%s)\n",
+                                strerror (errno));
+                        /* argp_usage (argp.) */
+                        fprintf (stderr, "USAGE: %s [options] [mountpoint]\n",
+                                 argv[0]);
+                        goto out;
+                }
         }
 
         if (cmd_args->run_id) {
@@ -1085,7 +1233,7 @@ parse_cmdline (int argc, char *argv[], glusterfs_ctx_t *ctx)
                 if (((ret == 0) &&
                      (S_ISREG (stbuf.st_mode) || S_ISLNK (stbuf.st_mode))) ||
                     (ret == -1)) {
-                        /* Have seperate logfile per run */
+                        /* Have separate logfile per run */
                         tm = localtime (&utime);
                         strftime (timestr, 256, "%Y%m%d.%H%M%S", tm);
                         sprintf (tmp_logfile, "%s.%s.%d",
@@ -1100,15 +1248,23 @@ parse_cmdline (int argc, char *argv[], glusterfs_ctx_t *ctx)
                                            cmd_args->log_file);
                         if (ret == -1) {
                                 fprintf (stderr, "ERROR: symlink of logfile failed\n");
-                        } else {
-                                GF_FREE (cmd_args->log_file);
-                                cmd_args->log_file = gf_strdup (tmp_logfile);
+                                goto out;
                         }
 
+                        GF_FREE (cmd_args->log_file);
+                        cmd_args->log_file = gf_strdup (tmp_logfile);
+
                         GF_FREE (tmp_logfile_dyn);
                 }
         }
 
+#ifdef GF_DARWIN_HOST_OS
+        if (cmd_args->mount_point)
+                cmd_args->mac_compat = GF_OPTION_DEFERRED;
+#endif
+
+        ret = 0;
+out:
         return ret;
 }
 
@@ -1338,6 +1494,7 @@ daemonize (glusterfs_ctx_t *ctx)
 {
         int            ret = -1;
         cmd_args_t    *cmd_args = NULL;
+        int            cstatus = 0;
 
         cmd_args = &ctx->cmd_args;
 
@@ -1351,11 +1508,24 @@ daemonize (glusterfs_ctx_t *ctx)
         if (cmd_args->debug_mode)
                 goto postfork;
 
-        ret = os_daemon (0, 0);
-        if (ret == -1) {
+        ret = os_daemon_return (0, 0);
+        switch (ret) {
+        case -1:
                 gf_log ("daemonize", GF_LOG_ERROR,
                         "Daemonization failed: %s", strerror(errno));
                 goto out;
+        case 0:
+                break;
+        default:
+                if (ctx->mnt_pid > 0) {
+                        ret = waitpid (ctx->mnt_pid, &cstatus, 0);
+                        if (!(ret == ctx->mnt_pid && cstatus == 0)) {
+                                gf_log ("daemonize", GF_LOG_ERROR,
+                                        "mount failed");
+                                exit (1);
+                        }
+                }
+                _exit (0);
         }
 
 postfork:
@@ -1474,7 +1644,10 @@ main (int argc, char *argv[])
                         "ERROR: glusterfs context not initialized");
                 return ENOMEM;
         }
-
+#ifndef DEBUG
+        /* Enable memory accounting on the fly based on argument */
+        gf_check_and_set_mem_acct (argc, argv, ctx);
+#endif
         ret = glusterfs_ctx_defaults_init (ctx);
         if (ret)
                 goto out;
@@ -1502,6 +1675,13 @@ main (int argc, char *argv[])
         if (ret)
                 goto out;
 
+	ctx->env = syncenv_new (0);
+        if (!ctx->env) {
+                gf_log ("", GF_LOG_ERROR,
+                        "Could not create new sync-environment");
+                goto out;
+        }
+
         ret = glusterfs_volumes_init (ctx);
         if (ret)
                 goto out;
