@@ -1,20 +1,11 @@
 /*
-  Copyright (c) 2007-2011 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+  This file is licensed to you under your choice of the GNU Lesser
+  General Public License, version 3 or any later version (LGPLv3 or
+  later), or the GNU General Public License, version 2 (GPLv2), in all
+  cases as published by the Free Software Foundation.
 */
 
 #ifndef _CALL_STUB_H_
@@ -34,21 +25,20 @@ typedef struct {
 	char wind;
 	call_frame_t *frame;
 	glusterfs_fop_t fop;
-       struct mem_pool *stub_mem_pool;    /* pointer to stub mempool in glusterfs ctx */
+        struct mem_pool *stub_mem_pool; /* pointer to stub mempool in ctx_t */
+        dict_t *xdata;                  /* common accross all the fops */
 
 	union {
 		/* lookup */
 		struct {
 			fop_lookup_t fn;
 			loc_t loc;
-			dict_t *xattr_req;
 		} lookup;
 		struct {
 			fop_lookup_cbk_t fn;
 			int32_t op_ret, op_errno;
 			inode_t *inode;
 			struct iatt buf;
-			dict_t *dict;
                         struct iatt postparent;
 		} lookup_cbk;
 
@@ -130,7 +120,7 @@ typedef struct {
 			loc_t loc;
 			mode_t mode;
 			dev_t rdev;
-                        dict_t *params;
+                        mode_t umask;
 		} mknod;
 		struct {
 			fop_mknod_cbk_t fn;
@@ -146,7 +136,7 @@ typedef struct {
 			fop_mkdir_t fn;
 			loc_t loc;
 			mode_t mode;
-                        dict_t *params;
+                        mode_t umask;
 		} mkdir;
 		struct {
 			fop_mkdir_cbk_t fn;
@@ -161,6 +151,7 @@ typedef struct {
 		struct {
 			fop_unlink_t fn;
 			loc_t loc;
+                        int xflag;
 		} unlink;
 		struct {
 			fop_unlink_cbk_t fn;
@@ -187,7 +178,7 @@ typedef struct {
 			fop_symlink_t fn;
 			const char *linkname;
 			loc_t loc;
-                        dict_t *params;
+                        mode_t umask;
 		} symlink;
 		struct {
 			fop_symlink_cbk_t fn;
@@ -236,7 +227,7 @@ typedef struct {
 			int32_t flags;
 			mode_t mode;
 			fd_t *fd;
-                        dict_t *params;
+                        mode_t umask;
 		} create;
 		struct {
 			fop_create_cbk_t fn;
@@ -254,7 +245,6 @@ typedef struct {
 			loc_t loc;
 			int32_t flags;
 			fd_t *fd;
-                        int32_t wbflags;
 		} open;
 		struct {
 			fop_open_cbk_t fn;
@@ -268,6 +258,7 @@ typedef struct {
 			fd_t *fd;
 			size_t size;
 			off_t off;
+                        uint32_t flags;
 		} readv;
 		struct {
 			fop_readv_cbk_t fn;
@@ -286,6 +277,7 @@ typedef struct {
 			struct iovec *vector;
 			int32_t count;
 			off_t off;
+                        uint32_t flags;
 			struct iobref *iobref;
 		} writev;
 		struct {
@@ -412,6 +404,18 @@ typedef struct {
 			int32_t op_ret, op_errno;
 		} removexattr_cbk;
 
+
+		/* fremovexattr */
+		struct {
+			fop_fremovexattr_t fn;
+			fd_t *fd;
+			const char *name;
+		} fremovexattr;
+		struct {
+			fop_fremovexattr_cbk_t fn;
+			int32_t op_ret, op_errno;
+		} fremovexattr_cbk;
+
 		/* lk */
 		struct {
 			fop_lk_t fn;
@@ -588,7 +592,7 @@ call_stub_t *
 fop_lookup_stub (call_frame_t *frame,
 		 fop_lookup_t fn,
 		 loc_t *loc,
-		 dict_t *xattr_req);
+		 dict_t *xdata);
 
 call_stub_t *
 fop_lookup_cbk_stub (call_frame_t *frame,
@@ -597,34 +601,34 @@ fop_lookup_cbk_stub (call_frame_t *frame,
 		     int32_t op_errno,
 		     inode_t *inode,
 		     struct iatt *buf,
-                     dict_t *dict,
+                     dict_t *xdata,
                      struct iatt *postparent);
 call_stub_t *
 fop_stat_stub (call_frame_t *frame,
 	       fop_stat_t fn,
-	       loc_t *loc);
+	       loc_t *loc, dict_t *xdata);
 call_stub_t *
 fop_stat_cbk_stub (call_frame_t *frame,
 		   fop_stat_cbk_t fn,
 		   int32_t op_ret,
 		   int32_t op_errno,
-		   struct iatt *buf);
+		   struct iatt *buf, dict_t *xdata);
 call_stub_t *
 fop_fstat_stub (call_frame_t *frame,
 		fop_fstat_t fn,
-		fd_t *fd);
+		fd_t *fd, dict_t *xdata);
 call_stub_t *
 fop_fstat_cbk_stub (call_frame_t *frame,
 		    fop_fstat_cbk_t fn,
 		    int32_t op_ret,
 		    int32_t op_errno,
-		    struct iatt *buf);
+		    struct iatt *buf, dict_t *xdata);
 
 call_stub_t *
 fop_truncate_stub (call_frame_t *frame,
 		   fop_truncate_t fn,
 		   loc_t *loc,
-		   off_t off);
+		   off_t off, dict_t *xdata);
 
 call_stub_t *
 fop_truncate_cbk_stub (call_frame_t *frame,
@@ -632,13 +636,13 @@ fop_truncate_cbk_stub (call_frame_t *frame,
 		       int32_t op_ret,
 		       int32_t op_errno,
 		       struct iatt *prebuf,
-                       struct iatt *postbuf);
+                       struct iatt *postbuf, dict_t *xdata);
 
 call_stub_t *
 fop_ftruncate_stub (call_frame_t *frame,
 		    fop_ftruncate_t fn,
 		    fd_t *fd,
-		    off_t off);
+		    off_t off, dict_t *xdata);
 
 call_stub_t *
 fop_ftruncate_cbk_stub (call_frame_t *frame,
@@ -646,25 +650,25 @@ fop_ftruncate_cbk_stub (call_frame_t *frame,
 			int32_t op_ret,
 			int32_t op_errno,
 			struct iatt *prebuf,
-                        struct iatt *postbuf);
+                        struct iatt *postbuf, dict_t *xdata);
 
 call_stub_t *
 fop_access_stub (call_frame_t *frame,
 		 fop_access_t fn,
 		 loc_t *loc,
-		 int32_t mask);
+		 int32_t mask, dict_t *xdata);
 
 call_stub_t *
 fop_access_cbk_stub (call_frame_t *frame,
 		     fop_access_cbk_t fn,
 		     int32_t op_ret,
-		     int32_t op_errno);
+		     int32_t op_errno, dict_t *xdata);
 
 call_stub_t *
 fop_readlink_stub (call_frame_t *frame,
 		   fop_readlink_t fn,
 		   loc_t *loc,
-		   size_t size);
+		   size_t size, dict_t *xdata);
 
 call_stub_t *
 fop_readlink_cbk_stub (call_frame_t *frame,
@@ -672,11 +676,11 @@ fop_readlink_cbk_stub (call_frame_t *frame,
 		       int32_t op_ret,
 		       int32_t op_errno,
 		       const char *path,
-                       struct iatt *buf);
+                       struct iatt *buf, dict_t *xdata);
 
 call_stub_t *
-fop_mknod_stub (call_frame_t *frame, fop_mknod_t fn,
-		loc_t *loc, mode_t mode, dev_t rdev, dict_t *params);
+fop_mknod_stub (call_frame_t *frame, fop_mknod_t fn, loc_t *loc, mode_t mode,
+                dev_t rdev, mode_t umask, dict_t *xdata);
 
 call_stub_t *
 fop_mknod_cbk_stub (call_frame_t *frame,
@@ -686,11 +690,11 @@ fop_mknod_cbk_stub (call_frame_t *frame,
 		    inode_t *inode,
                     struct iatt *buf,
                     struct iatt *preparent,
-                    struct iatt *postparent);
+                    struct iatt *postparent, dict_t *xdata);
 
 call_stub_t *
-fop_mkdir_stub (call_frame_t *frame, fop_mkdir_t fn,
-		loc_t *loc, mode_t mode, dict_t *params);
+fop_mkdir_stub (call_frame_t *frame, fop_mkdir_t fn, loc_t *loc, mode_t mode,
+                mode_t umask, dict_t *xdata);
 
 call_stub_t *
 fop_mkdir_cbk_stub (call_frame_t *frame,
@@ -700,12 +704,11 @@ fop_mkdir_cbk_stub (call_frame_t *frame,
 		    inode_t *inode,
                     struct iatt *buf,
                     struct iatt *preparent,
-                    struct iatt *postparent);
+                    struct iatt *postparent, dict_t *xdata);
 
 call_stub_t *
-fop_unlink_stub (call_frame_t *frame,
-		 fop_unlink_t fn,
-		 loc_t *loc);
+fop_unlink_stub (call_frame_t *frame, fop_unlink_t fn,
+		 loc_t *loc, int xflag, dict_t *xdata);
 
 call_stub_t *
 fop_unlink_cbk_stub (call_frame_t *frame,
@@ -713,11 +716,11 @@ fop_unlink_cbk_stub (call_frame_t *frame,
 		     int32_t op_ret,
 		     int32_t op_errno,
                      struct iatt *preparent,
-                     struct iatt *postparent);
+                     struct iatt *postparent, dict_t *xdata);
 
 call_stub_t *
 fop_rmdir_stub (call_frame_t *frame, fop_rmdir_t fn,
-		loc_t *loc, int flags);
+		loc_t *loc, int flags, dict_t *xdata);
 
 call_stub_t *
 fop_rmdir_cbk_stub (call_frame_t *frame,
@@ -725,11 +728,11 @@ fop_rmdir_cbk_stub (call_frame_t *frame,
 		    int32_t op_ret,
 		    int32_t op_errno,
                     struct iatt *preparent,
-                    struct iatt *postparent);
+                    struct iatt *postparent, dict_t *xdata);
 
 call_stub_t *
 fop_symlink_stub (call_frame_t *frame, fop_symlink_t fn,
-		  const char *linkname, loc_t *loc, dict_t *params);
+		  const char *linkname, loc_t *loc, mode_t umask, dict_t *xdata);
 
 call_stub_t *
 fop_symlink_cbk_stub (call_frame_t *frame,
@@ -739,13 +742,13 @@ fop_symlink_cbk_stub (call_frame_t *frame,
 		      inode_t *inode,
                       struct iatt *buf,
                       struct iatt *preparent,
-                      struct iatt *postparent);
+                      struct iatt *postparent, dict_t *xdata);
 
 call_stub_t *
 fop_rename_stub (call_frame_t *frame,
 		 fop_rename_t fn,
 		 loc_t *oldloc,
-		 loc_t *newloc);
+		 loc_t *newloc, dict_t *xdata);
 
 call_stub_t *
 fop_rename_cbk_stub (call_frame_t *frame,
@@ -756,13 +759,13 @@ fop_rename_cbk_stub (call_frame_t *frame,
                      struct iatt *preoldparent,
                      struct iatt *postoldparent,
                      struct iatt *prenewparent,
-                     struct iatt *postnewparent);
+                     struct iatt *postnewparent, dict_t *xdata);
 
 call_stub_t *
 fop_link_stub (call_frame_t *frame,
 	       fop_link_t fn,
 	       loc_t *oldloc,
-	       loc_t *newloc);
+	       loc_t *newloc, dict_t *xdata);
 
 call_stub_t *
 fop_link_cbk_stub (call_frame_t *frame,
@@ -772,12 +775,12 @@ fop_link_cbk_stub (call_frame_t *frame,
 		   inode_t *inode,
                    struct iatt *buf,
                    struct iatt *preparent,
-                   struct iatt *postparent);
+                   struct iatt *postparent, dict_t *xdata);
 
 call_stub_t *
 fop_create_stub (call_frame_t *frame, fop_create_t fn,
 		 loc_t *loc, int32_t flags, mode_t mode,
-                 fd_t *fd, dict_t *params);
+                 mode_t umask, fd_t *fd, dict_t *xdata);
 
 call_stub_t *
 fop_create_cbk_stub (call_frame_t *frame,
@@ -788,7 +791,7 @@ fop_create_cbk_stub (call_frame_t *frame,
 		     inode_t *inode,
 		     struct iatt *buf,
                      struct iatt *preparent,
-                     struct iatt *postparent);
+                     struct iatt *postparent, dict_t *xdata);
 
 call_stub_t *
 fop_open_stub (call_frame_t *frame,
@@ -796,21 +799,21 @@ fop_open_stub (call_frame_t *frame,
 	       loc_t *loc,
 	       int32_t flags,
 	       fd_t *fd,
-               int32_t wbflags);
+               dict_t *xdata);
 
 call_stub_t *
 fop_open_cbk_stub (call_frame_t *frame,
 		   fop_open_cbk_t fn,
 		   int32_t op_ret,
 		   int32_t op_errno,
-		   fd_t *fd);
+		   fd_t *fd, dict_t *xdata);
 
 call_stub_t *
 fop_readv_stub (call_frame_t *frame,
 		fop_readv_t fn,
 		fd_t *fd,
 		size_t size,
-		off_t off);
+		off_t off, uint32_t flags, dict_t *xdata);
 
 call_stub_t *
 fop_readv_cbk_stub (call_frame_t *frame,
@@ -820,7 +823,7 @@ fop_readv_cbk_stub (call_frame_t *frame,
 		    struct iovec *vector,
 		    int32_t count,
 		    struct iatt *stbuf,
-                    struct iobref *iobref);
+                    struct iobref *iobref, dict_t *xdata);
 
 call_stub_t *
 fop_writev_stub (call_frame_t *frame,
@@ -828,8 +831,8 @@ fop_writev_stub (call_frame_t *frame,
 		 fd_t *fd,
 		 struct iovec *vector,
 		 int32_t count,
-		 off_t off,
-                 struct iobref *iobref);
+		 off_t off, uint32_t flags,
+                 struct iobref *iobref, dict_t *xdata);
 
 call_stub_t *
 fop_writev_cbk_stub (call_frame_t *frame,
@@ -837,24 +840,24 @@ fop_writev_cbk_stub (call_frame_t *frame,
 		     int32_t op_ret,
 		     int32_t op_errno,
                      struct iatt *prebuf,
-                     struct iatt *postbuf);
+                     struct iatt *postbuf, dict_t *xdata);
 
 call_stub_t *
 fop_flush_stub (call_frame_t *frame,
 		fop_flush_t fn,
-		fd_t *fd);
+		fd_t *fd, dict_t *xdata);
 
 call_stub_t *
 fop_flush_cbk_stub (call_frame_t *frame,
 		    fop_flush_cbk_t fn,
 		    int32_t op_ret,
-		    int32_t op_errno);
+		    int32_t op_errno, dict_t *xdata);
 
 call_stub_t *
 fop_fsync_stub (call_frame_t *frame,
 		fop_fsync_t fn,
 		fd_t *fd,
-		int32_t datasync);
+		int32_t datasync, dict_t *xdata);
 
 call_stub_t *
 fop_fsync_cbk_stub (call_frame_t *frame,
@@ -862,190 +865,205 @@ fop_fsync_cbk_stub (call_frame_t *frame,
 		    int32_t op_ret,
 		    int32_t op_errno,
                     struct iatt *prebuf,
-                    struct iatt *postbuf);
+                    struct iatt *postbuf, dict_t *xdata);
 
 call_stub_t *
 fop_opendir_stub (call_frame_t *frame,
 		  fop_opendir_t fn,
-		  loc_t *loc, fd_t *fd);
+		  loc_t *loc, fd_t *fd, dict_t *xdata);
 
 call_stub_t *
 fop_opendir_cbk_stub (call_frame_t *frame,
 		      fop_opendir_cbk_t fn,
 		      int32_t op_ret,
 		      int32_t op_errno,
-		      fd_t *fd);
+		      fd_t *fd, dict_t *xdata);
 
 call_stub_t *
 fop_fsyncdir_stub (call_frame_t *frame,
 		   fop_fsyncdir_t fn,
 		   fd_t *fd,
-		   int32_t datasync);
+		   int32_t datasync, dict_t *xdata);
 
 call_stub_t *
 fop_fsyncdir_cbk_stub (call_frame_t *frame,
 		       fop_fsyncdir_cbk_t fn,
 		       int32_t op_ret,
-		       int32_t op_errno);
+		       int32_t op_errno, dict_t *xdata);
 
 call_stub_t *
 fop_statfs_stub (call_frame_t *frame,
 		 fop_statfs_t fn,
-		 loc_t *loc);
+		 loc_t *loc, dict_t *xdata);
 
 call_stub_t *
 fop_statfs_cbk_stub (call_frame_t *frame,
 		     fop_statfs_cbk_t fn,
 		     int32_t op_ret,
 		     int32_t op_errno,
-		     struct statvfs *buf);
+		     struct statvfs *buf, dict_t *xdata);
 
 call_stub_t *
 fop_setxattr_stub (call_frame_t *frame,
 		   fop_setxattr_t fn,
 		   loc_t *loc,
 		   dict_t *dict,
-		   int32_t flags);
+		   int32_t flags, dict_t *xdata);
 
 call_stub_t *
 fop_setxattr_cbk_stub (call_frame_t *frame,
 		       fop_setxattr_cbk_t fn,
 		       int32_t op_ret,
-		       int32_t op_errno);
+		       int32_t op_errno, dict_t *xdata);
 
 call_stub_t *
 fop_getxattr_stub (call_frame_t *frame,
 		   fop_getxattr_t fn,
 		   loc_t *loc,
-		   const char *name);
+		   const char *name, dict_t *xdata);
 
 call_stub_t *
 fop_getxattr_cbk_stub (call_frame_t *frame,
 		       fop_getxattr_cbk_t fn,
 		       int32_t op_ret,
 		       int32_t op_errno,
-		       dict_t *value);
+		       dict_t *value, dict_t *xdata);
 
 call_stub_t *
 fop_fsetxattr_stub (call_frame_t *frame,
                     fop_fsetxattr_t fn,
                     fd_t *fd,
                     dict_t *dict,
-                    int32_t flags);
+                    int32_t flags, dict_t *xdata);
 
 call_stub_t *
 fop_fsetxattr_cbk_stub (call_frame_t *frame,
                         fop_fsetxattr_cbk_t fn,
                         int32_t op_ret,
-                        int32_t op_errno);
+                        int32_t op_errno, dict_t *xdata);
 
 call_stub_t *
 fop_fgetxattr_stub (call_frame_t *frame,
                     fop_fgetxattr_t fn,
                     fd_t *fd,
-                    const char *name);
+                    const char *name, dict_t *xdata);
 
 call_stub_t *
 fop_fgetxattr_cbk_stub (call_frame_t *frame,
                         fop_fgetxattr_cbk_t fn,
                         int32_t op_ret,
                         int32_t op_errno,
-                        dict_t *value);
+                        dict_t *value, dict_t *xdata);
 
 call_stub_t *
 fop_removexattr_stub (call_frame_t *frame,
 		      fop_removexattr_t fn,
 		      loc_t *loc,
-		      const char *name);
+		      const char *name, dict_t *xdata);
 
 call_stub_t *
 fop_removexattr_cbk_stub (call_frame_t *frame,
 			  fop_removexattr_cbk_t fn,
 			  int32_t op_ret,
-			  int32_t op_errno);
+			  int32_t op_errno, dict_t *xdata);
+
+
+call_stub_t *
+fop_fremovexattr_stub (call_frame_t *frame,
+                       fop_fremovexattr_t fn,
+                       fd_t *fd,
+                       const char *name, dict_t *xdata);
+
+call_stub_t *
+fop_fremovexattr_cbk_stub (call_frame_t *frame,
+                           fop_fremovexattr_cbk_t fn,
+                           int32_t op_ret,
+                           int32_t op_errno, dict_t *xdata);
+
 call_stub_t *
 fop_lk_stub (call_frame_t *frame,
 	     fop_lk_t fn,
 	     fd_t *fd,
 	     int32_t cmd,
-	     struct gf_flock *lock);
+	     struct gf_flock *lock, dict_t *xdata);
 
 call_stub_t *
 fop_lk_cbk_stub (call_frame_t *frame,
 		 fop_lk_cbk_t fn,
 		 int32_t op_ret,
 		 int32_t op_errno,
-		 struct gf_flock *lock);
+		 struct gf_flock *lock, dict_t *xdata);
 
 call_stub_t *
 fop_inodelk_stub (call_frame_t *frame, fop_inodelk_t fn,
 		  const char *volume, loc_t *loc, int32_t cmd,
-                  struct gf_flock *lock);
+                  struct gf_flock *lock, dict_t *xdata);
 
 call_stub_t *
 fop_finodelk_stub (call_frame_t *frame, fop_finodelk_t fn,
 		   const char *volume, fd_t *fd, int32_t cmd,
-                   struct gf_flock *lock);
+                   struct gf_flock *lock, dict_t *xdata);
 
 call_stub_t *
 fop_entrylk_stub (call_frame_t *frame, fop_entrylk_t fn,
 		  const char *volume, loc_t *loc, const char *basename,
-		  entrylk_cmd cmd, entrylk_type type);
+		  entrylk_cmd cmd, entrylk_type type, dict_t *xdata);
 
 call_stub_t *
 fop_fentrylk_stub (call_frame_t *frame, fop_fentrylk_t fn,
 		   const char *volume, fd_t *fd, const char *basename,
-		   entrylk_cmd cmd, entrylk_type type);
+		   entrylk_cmd cmd, entrylk_type type, dict_t *xdata);
 
 call_stub_t *
 fop_inodelk_cbk_stub (call_frame_t *frame, fop_inodelk_cbk_t fn,
-		      int32_t op_ret, int32_t op_errno);
+		      int32_t op_ret, int32_t op_errno, dict_t *xdata);
 
 call_stub_t *
 fop_finodelk_cbk_stub (call_frame_t *frame, fop_inodelk_cbk_t fn,
-		       int32_t op_ret, int32_t op_errno);
+		       int32_t op_ret, int32_t op_errno, dict_t *xdata);
 
 call_stub_t *
 fop_entrylk_cbk_stub (call_frame_t *frame, fop_entrylk_cbk_t fn,
-		      int32_t op_ret, int32_t op_errno);
+		      int32_t op_ret, int32_t op_errno, dict_t *xdata);
 
 call_stub_t *
 fop_fentrylk_cbk_stub (call_frame_t *frame, fop_entrylk_cbk_t fn,
-		       int32_t op_ret, int32_t op_errno);
+		       int32_t op_ret, int32_t op_errno, dict_t *xdata);
 
 call_stub_t *
 fop_readdir_stub (call_frame_t *frame,
 		  fop_readdir_t fn,
 		  fd_t *fd,
 		  size_t size,
-		  off_t off);
+		  off_t off, dict_t *xdata);
 
 call_stub_t *
 fop_readdirp_stub (call_frame_t *frame,
-		   fop_readdir_t fn,
+		   fop_readdirp_t fn,
 		   fd_t *fd,
 		   size_t size,
-		   off_t off);
+		   off_t off,
+                   dict_t *xdata);
 
 call_stub_t *
 fop_readdirp_cbk_stub (call_frame_t *frame,
 		       fop_readdir_cbk_t fn,
 		       int32_t op_ret,
 		       int32_t op_errno,
-		       gf_dirent_t *entries);
+		       gf_dirent_t *entries, dict_t *xdata);
 
 call_stub_t *
 fop_readdir_cbk_stub (call_frame_t *frame,
 		      fop_readdir_cbk_t fn,
 		      int32_t op_ret,
 		      int32_t op_errno,
-		      gf_dirent_t *entries);
+		      gf_dirent_t *entries, dict_t *xdata);
 
 call_stub_t *
 fop_rchecksum_stub (call_frame_t *frame,
                     fop_rchecksum_t fn,
                     fd_t *fd, off_t offset,
-                    int32_t len);
+                    int32_t len, dict_t *xdata);
 
 call_stub_t *
 fop_rchecksum_cbk_stub (call_frame_t *frame,
@@ -1053,40 +1071,40 @@ fop_rchecksum_cbk_stub (call_frame_t *frame,
                         int32_t op_ret,
                         int32_t op_errno,
                         uint32_t weak_checksum,
-                        uint8_t *strong_checksum);
+                        uint8_t *strong_checksum, dict_t *xdata);
 
 call_stub_t *
 fop_xattrop_stub (call_frame_t *frame,
 		  fop_xattrop_t fn,
 		  loc_t *loc,
 		  gf_xattrop_flags_t optype,
-		  dict_t *xattr);
+		  dict_t *xattr, dict_t *xdata);
 
 call_stub_t *
 fop_xattrop_stub_cbk_stub (call_frame_t *frame,
 			   fop_xattrop_cbk_t fn,
 			   int32_t op_ret,
-			   int32_t op_errno);
+			   int32_t op_errno, dict_t *xdata);
 
 call_stub_t *
 fop_fxattrop_stub (call_frame_t *frame,
 		   fop_fxattrop_t fn,
 		   fd_t *fd,
 		   gf_xattrop_flags_t optype,
-		   dict_t *xattr);
+		   dict_t *xattr, dict_t *xdata);
 
 call_stub_t *
 fop_fxattrop_stub_cbk_stub (call_frame_t *frame,
 			    fop_xattrop_cbk_t fn,
 			    int32_t op_ret,
-			    int32_t op_errno);
+			    int32_t op_errno, dict_t *xdata);
 
 call_stub_t *
 fop_setattr_stub (call_frame_t *frame,
                   fop_setattr_t fn,
                   loc_t *loc,
                   struct iatt *stbuf,
-                  int32_t valid);
+                  int32_t valid, dict_t *xdata);
 
 call_stub_t *
 fop_setattr_cbk_stub (call_frame_t *frame,
@@ -1094,14 +1112,14 @@ fop_setattr_cbk_stub (call_frame_t *frame,
                       int32_t op_ret,
                       int32_t op_errno,
                       struct iatt *statpre,
-                      struct iatt *statpost);
+                      struct iatt *statpost, dict_t *xdata);
 
 call_stub_t *
 fop_fsetattr_stub (call_frame_t *frame,
                    fop_fsetattr_t fn,
                    fd_t *fd,
                    struct iatt *stbuf,
-                   int32_t valid);
+                   int32_t valid, dict_t *xdata);
 
 call_stub_t *
 fop_fsetattr_cbk_stub (call_frame_t *frame,
@@ -1109,7 +1127,7 @@ fop_fsetattr_cbk_stub (call_frame_t *frame,
                        int32_t op_ret,
                        int32_t op_errno,
                        struct iatt *statpre,
-                       struct iatt *statpost);
+                       struct iatt *statpost, dict_t *xdata);
 
 void call_resume (call_stub_t *stub);
 void call_stub_destroy (call_stub_t *stub);
