@@ -1,20 +1,11 @@
 /*
-   Copyright (c) 2006-2011 Gluster, Inc. <http://www.gluster.com>
-   This file is part of GlusterFS.
+  Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
+  This file is part of GlusterFS.
 
-   GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
-   by the Free Software Foundation; either version 3 of the License,
-   or (at your option) any later version.
-
-   GlusterFS is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program.  If not, see
-   <http://www.gnu.org/licenses/>.
+  This file is licensed to you under your choice of the GNU Lesser
+  General Public License, version 3 or any later version (LGPLv3 or
+  later), or the GNU General Public License, version 2 (GPLv2), in all
+  cases as published by the Free Software Foundation.
 */
 
 #ifndef _COMMON_UTILS_H
@@ -50,6 +41,7 @@ void trap (void);
 #include "uuid.h"
 
 
+
 #define min(a,b) ((a)<(b)?(a):(b))
 #define max(a,b) ((a)>(b)?(a):(b))
 #define roof(a,b) ((((a)+(b)-1)/((b)?(b):1))*(b))
@@ -68,17 +60,46 @@ void trap (void);
 #define GF_UNIT_TB_STRING    "TB"
 #define GF_UNIT_PB_STRING    "PB"
 
+#define GF_UNIT_PERCENT_STRING      "%"
+
 #define GEOREP "geo-replication"
+#define GHADOOP "glusterfs-hadoop"
 
 #define WIPE(statp) do { typeof(*statp) z = {0,}; if (statp) *statp = z; } while (0)
 
+#define IS_EXT_FS(fs_name)          \
+        (!strcmp (fs_name, "ext2") || \
+         !strcmp (fs_name, "ext3") || \
+         !strcmp (fs_name, "ext4"))
+
+/* Defining this here as it is needed by glusterd for setting
+ * nfs port in volume status.
+ */
+#define GF_NFS3_PORT    38467
+
 enum _gf_boolean
 {
 	_gf_false = 0,
 	_gf_true = 1
 };
 
+/*
+ * we could have initialized these as +ve values and treated
+ * them as negative while comparing etc.. (which would have
+ * saved us with the pain of assigning values), but since we
+ * only have a couple of clients that use this feature, it's
+ * okay.
+ */
+enum _gf_client_pid
+{
+        GF_CLIENT_PID_MAX    =  0,
+        GF_CLIENT_PID_GSYNCD = -1,
+        GF_CLIENT_PID_HADOOP = -2,
+        GF_CLIENT_PID_DEFRAG = -3,
+};
+
 typedef enum _gf_boolean gf_boolean_t;
+typedef enum _gf_client_pid gf_client_pid_t;
 typedef int (*gf_cmp) (void *, void *);
 
 void gf_global_variable_init(void);
@@ -103,7 +124,7 @@ extern char *gf_mgmt_list[GF_MGMT_MAXVALUE];
                                           "invalid argument: " #arg);   \
 			goto label;					\
 		}							\
-	} while (0);
+	} while (0)
 
 #define GF_VALIDATE_OR_GOTO(name,arg,label)   do {                      \
 		if (!arg) {                                             \
@@ -112,7 +133,7 @@ extern char *gf_mgmt_list[GF_MGMT_MAXVALUE];
                                           "invalid argument: " #arg);	\
 			goto label;                                     \
 		}                                                       \
-	} while (0);
+	} while (0)
 
 #define GF_VALIDATE_OR_GOTO_WITH_ERROR(name, arg, label, errno, error) do { \
                 if (!arg) {                                                 \
@@ -121,7 +142,15 @@ extern char *gf_mgmt_list[GF_MGMT_MAXVALUE];
                                           "invalid argument: " #arg);   \
                         goto label;                                     \
                 }                                                       \
-        }while (0);
+        }while (0)
+
+#define GF_ASSERT_AND_GOTO_WITH_ERROR(name, arg, label, errno, error) do { \
+                if (!arg) {                                             \
+                        GF_ASSERT (0);                                  \
+                        errno = error;                                  \
+                        goto label;                                     \
+                }                                                       \
+        }while (0)
 
 #define GF_VALIDATE_ABSOLUTE_PATH_OR_GOTO(name,arg,label)               \
         do {                                                            \
@@ -132,7 +161,57 @@ extern char *gf_mgmt_list[GF_MGMT_MAXVALUE];
                                           "invalid argument: " #arg);	\
                         goto label;                                     \
                 }                                                       \
-	} while (0);
+	} while (0)
+
+#define GF_REMOVE_SLASH_FROM_PATH(path, string)                         \
+        do {                                                            \
+                int i = 0;                                              \
+                for (i = 1; i < strlen (path); i++) {                   \
+                        string[i-1] = path[i];                          \
+                        if (string[i-1] == '/')                         \
+                                string[i-1] = '-';                      \
+                }                                                       \
+        } while (0)
+
+
+#define GF_IF_INTERNAL_XATTR_GOTO(pattern, dict, trav, op_errno, label) \
+        do {                                                            \
+                if (!dict) {                                            \
+                        gf_log (this->name, GF_LOG_ERROR,               \
+                                "setxattr dict is null");               \
+                        goto label;                                     \
+                }                                                       \
+                trav = dict->members_list;                              \
+                while (trav) {                                          \
+                        if (!fnmatch (pattern, trav->key, 0)) {         \
+                                op_errno = EPERM;                       \
+                                gf_log (this->name, GF_LOG_ERROR,       \
+                                        "attempt to set internal"       \
+                                        " xattr: %s: %s", trav->key,    \
+                                        strerror (op_errno));           \
+                                goto label;                             \
+                        }                                               \
+                        trav = trav->next;                              \
+                }                                                       \
+        } while (0)
+
+#define GF_IF_NATIVE_XATTR_GOTO(pattern, key, op_errno, label)          \
+        do {                                                            \
+                if (!key) {                                             \
+                        gf_log (this->name, GF_LOG_ERROR,               \
+                                "no key for removexattr");              \
+                        goto label;                                     \
+                }                                                       \
+                if (!fnmatch (pattern, key, 0)) {                       \
+                        op_errno = EPERM;                               \
+                        gf_log (this->name, GF_LOG_ERROR,               \
+                                "attempt to remove internal "           \
+                                "xattr: %s: %s", key,                   \
+                                strerror (op_errno));                   \
+                        goto label;                                     \
+                }                                                       \
+        } while (0)
+
 
 #define GF_FILE_CONTENT_REQUESTED(_xattr_req,_content_limit) \
 	(dict_get_uint64 (_xattr_req, "glusterfs.content", _content_limit) == 0)
@@ -146,9 +225,20 @@ extern char *gf_mgmt_list[GF_MGMT_MAXVALUE];
                         gf_log_callingfn ("", GF_LOG_ERROR,             \
                                           "Assertion failed: " #x);     \
                 }                                                       \
-        } while (0);
+        } while (0)
 #endif
 
+#define GF_UUID_ASSERT(u) \
+        if (uuid_is_null (u))\
+                GF_ASSERT (!"uuid null");
+
+union gf_sock_union {
+        struct sockaddr_storage storage;
+        struct sockaddr_in6 sin6;
+        struct sockaddr_in sin;
+        struct sockaddr sa;
+};
+
 #define GF_HIDDEN_PATH ".glusterfs"
 
 static inline void
@@ -302,6 +392,19 @@ memdup (const void *ptr, size_t size)
 	return newptr;
 }
 
+/*
+ * rounds up nr to power of two. If nr is already a power of two, just returns
+ * nr
+ */
+
+int32_t gf_roundup_power_of_two (uint32_t nr);
+
+/*
+ * rounds up nr to next power of two. If nr is already a power of two, next
+ * power of two is returned.
+ */
+
+int32_t gf_roundup_next_power_of_two (uint32_t nr);
 
 char *gf_trim (char *string);
 int gf_strsplit (const char *str, const char *delim,
@@ -335,6 +438,8 @@ int gf_string2uint32_base10 (const char *str, uint32_t *n);
 int gf_string2uint64_base10 (const char *str, uint64_t *n);
 
 int gf_string2bytesize (const char *str, uint64_t *n);
+int gf_string2percent_or_bytesize (const char *str, uint64_t *n,
+				   gf_boolean_t *is_percent);
 
 int gf_string2boolean (const char *str, gf_boolean_t *b);
 int gf_string2percent (const char *str, uint32_t *n);
@@ -346,23 +451,37 @@ int gf_unlockfd (int fd);
 int get_checksum_for_file (int fd, uint32_t *checksum);
 int log_base2 (unsigned long x);
 
-int gf_system (const char *command);
 int get_checksum_for_path (char *path, uint32_t *checksum);
 
 char *strtail (char *str, const char *pattern);
+void skipwhite (char **s);
+char *nwstrtail (char *str, char *pattern);
+void skip_word (char **str);
+/* returns a new string with nth word of given string. n>=1 */
+char *get_nth_word (const char *str, int n);
 
 char valid_host_name (char *address, int length);
-char valid_ipv4_address (char *address, int length);
-char valid_ipv6_address (char *address, int length);
-char valid_internet_address (char *address);
+char valid_ipv4_address (char *address, int length, gf_boolean_t wildcard_acc);
+char valid_ipv6_address (char *address, int length, gf_boolean_t wildcard_acc);
+char valid_internet_address (char *address, gf_boolean_t wildcard_acc);
+char valid_ipv4_wildcard_check (char *address);
+char valid_ipv6_wildcard_check (char *address);
+char valid_wildcard_internet_address (char *address);
 
 char *uuid_utoa (uuid_t uuid);
 char *uuid_utoa_r (uuid_t uuid, char *dst);
-void _get_md5_str (char *out_str, size_t outlen,
-                   const uint8_t *input, int n);
+char *lkowner_utoa (gf_lkowner_t *lkowner);
+char *lkowner_utoa_r (gf_lkowner_t *lkowner, char *dst, int len);
+
 void gf_array_insertionsort (void *a, int l, int r, size_t elem_size,
                              gf_cmp cmp);
 int gf_is_str_int (const char *value);
 
 char *gf_uint64_2human_readable (uint64_t);
+int validate_brick_name (char *brick);
+char *get_host_name (char *word, char **host);
+char *get_path_name (char *word, char **path);
+void gf_path_strip_trailing_slashes (char *path);
+uint64_t get_mem_size ();
+int gf_strip_whitespace (char *str, int len);
 #endif /* _COMMON_UTILS_H */
