@@ -1,20 +1,11 @@
 /*
-  Copyright (c) 2008-2011 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+  This file is licensed to you under your choice of the GNU Lesser
+  General Public License, version 3 or any later version (LGPLv3 or
+  later), or the GNU General Public License, version 2 (GPLv2), in all
+  cases as published by the Free Software Foundation.
 */
 
 #ifndef _CONFIG_H
@@ -567,7 +558,7 @@ init_compat_errno_arrays ()
         gf_error_to_errno_array[GF_ERROR_CODE_BADMACHO] = EBADMACHO;
         gf_errno_to_error_array[EBADMACHO] = GF_ERROR_CODE_BADMACHO;
 
-#if 0
+#ifdef EDOOFUS
         /*    EDOOFUS           88              / * Programming error */
         gf_error_to_errno_array[GF_ERROR_CODE_DOOFUS] = EDOOFUS;
         gf_errno_to_error_array[EDOOFUS] = GF_ERROR_CODE_DOOFUS;
@@ -858,9 +849,11 @@ init_compat_errno_arrays ()
         gf_error_to_errno_array[GF_ERROR_CODE_NOATTR] = ENOATTR;
         gf_errno_to_error_array[ENOATTR] = GF_ERROR_CODE_NOATTR;
 
+#ifdef EDOOFUS
         /*    EDOOFUS           88              / * Programming error */
         gf_error_to_errno_array[GF_ERROR_CODE_DOOFUS] = EDOOFUS;
         gf_errno_to_error_array[EDOOFUS] = GF_ERROR_CODE_DOOFUS;
+#endif
 
         /*      EBADMSG         89              / * Bad message */
         gf_error_to_errno_array[GF_ERROR_CODE_BADMSG] = EBADMSG;
