@@ -1,20 +1,11 @@
 /*
-   Copyright (c) 2006-2011 Gluster, Inc. <http://www.gluster.com>
-   This file is part of GlusterFS.
+  Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
+  This file is part of GlusterFS.
 
-   GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
-   by the Free Software Foundation; either version 3 of the License,
-   or (at your option) any later version.
-
-   GlusterFS is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program.  If not, see
-   <http://www.gnu.org/licenses/>.
+  This file is licensed to you under your choice of the GNU Lesser
+  General Public License, version 3 or any later version (LGPLv3 or
+  later), or the GNU General Public License, version 2 (GPLv2), in all
+  cases as published by the Free Software Foundation.
 */
 
 #ifndef __COMPAT_H__
@@ -58,7 +49,7 @@
 #endif /* GF_LINUX_HOST_OS */
 
 #ifdef GF_BSD_HOST_OS
-/* In case of FreeBSD */
+/* In case of FreeBSD and NetBSD */
 
 #define UNIX_PATH_MAX 104
 #include <sys/types.h>
@@ -66,16 +57,21 @@
 #include <sys/un.h>
 #include <sys/endian.h>
 #include <sys/extattr.h>
+#ifdef HAVE_SYS_XATTR_H
+#include <sys/xattr.h>
+#endif /* HAVE_SYS_XATTR_H */
 #include <limits.h>
 
 #include <libgen.h>
 
+#ifndef XATTR_CREATE
 enum {
         ATTR_CREATE = 1,
 #define XATTR_CREATE ATTR_CREATE
         ATTR_REPLACE = 2
 #define XATTR_REPLACE ATTR_REPLACE
 };
+#endif /* XATTR_CREATE */
 
 
 #ifndef sighandler_t
@@ -284,10 +280,12 @@ int solaris_rename (const char *oldpath, const char *newpath);
 
 int solaris_unlink (const char *pathname);
 
+char *mkdtemp (char *temp);
+
 #define GF_SOLARIS_XATTR_DIR ".glusterfs_xattr_inode"
 
 int solaris_xattr_resolve_path (const char *real_path, char **path);
-char *mkdtemp (char *__template);
+
 #endif /* GF_SOLARIS_HOST_OS */
 
 #ifndef HAVE_ARGP
@@ -336,6 +334,15 @@ dirent_size (struct dirent *entry)
         return size;
 }
 
+#ifdef THREAD_UNSAFE_BASENAME
+char *basename_r(const char *);
+#define basename(path) basename_r(path)
+#endif /* THREAD_UNSAFE_BASENAME */
+
+#ifdef THREAD_UNSAFE_DIRNAME
+char *dirname_r(char *path);
+#define dirname(path) dirname_r(path)
+#endif /* THREAD_UNSAFE_DIRNAME */
 
 #ifdef HAVE_STRUCT_STAT_ST_ATIM_TV_NSEC
 /* Linux, Solaris, Cygwin */
@@ -378,4 +385,9 @@ dirent_size (struct dirent *entry)
 #define IXDR_PUT_U_LONG(buf, v)       IXDR_PUT_LONG(buf, (long)(v))
 #endif
 
+#if defined(__GNUC__) && !defined(RELAX_POISONING)
+/* Use run API, see run.h */
+#pragma GCC poison system popen
+#endif
+
 #endif /* __COMPAT_H__ */
