@@ -1,20 +1,11 @@
 /*
-  Copyright (c) 2007-2011 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+  This file is licensed to you under your choice of the GNU Lesser
+  General Public License, version 3 or any later version (LGPLv3 or
+  later), or the GNU General Public License, version 2 (GPLv2), in all
+  cases as published by the Free Software Foundation.
 */
 
 #ifndef _FD_H
@@ -30,9 +21,11 @@
 #include <unistd.h>
 #include "glusterfs.h"
 #include "locking.h"
+#include "fd-lk.h"
 
 struct _inode;
 struct _dict;
+struct fd_lk_ctx;
 
 struct _fd_ctx {
         union {
@@ -50,7 +43,7 @@ struct _fd_ctx {
  * See the comment there to know why.
  */
 struct _fd {
-        pid_t             pid;
+        uint64_t             pid;
 	int32_t           flags;
         int32_t           refcount;
         struct list_head  inode_list;
@@ -59,6 +52,7 @@ struct _fd {
                                    'struct _fd_ctx' array (_ctx).*/
 	struct _fd_ctx   *_ctx;
         int               xl_count; /* Number of xl referred in this fd */
+        struct fd_lk_ctx *lk_ctx;
 };
 typedef struct _fd fd_t;
 
@@ -116,10 +110,18 @@ void
 gf_fd_fdtable_destroy (fdtable_t *fdtable);
 
 
+fd_t *
+__fd_ref (fd_t *fd);
+
+
 fd_t *
 fd_ref (fd_t *fd);
 
 
+fd_t *
+__fd_unref (fd_t *fd);
+
+
 void
 fd_unref (fd_t *fd);
 
@@ -127,10 +129,22 @@ fd_unref (fd_t *fd);
 fd_t *
 fd_create (struct _inode *inode, pid_t pid);
 
+fd_t *
+fd_create_uint64 (struct _inode *inode, uint64_t pid);
 
 fd_t *
 fd_lookup (struct _inode *inode, pid_t pid);
 
+fd_t *
+fd_lookup_uint64 (struct _inode *inode, uint64_t pid);
+
+fd_t *
+fd_anonymous (inode_t *inode);
+
+
+gf_boolean_t
+fd_is_anonymous (fd_t *fd);
+
 
 uint8_t
 fd_list_empty (struct _inode *inode);
@@ -139,6 +153,8 @@ fd_list_empty (struct _inode *inode);
 fd_t *
 fd_bind (fd_t *fd);
 
+fd_t *
+__fd_bind (fd_t *fd);
 
 int
 fd_ctx_set (fd_t *fd, xlator_t *xlator, uint64_t value);
@@ -164,9 +180,18 @@ int
 __fd_ctx_del (fd_t *fd, xlator_t *xlator, uint64_t *value);
 
 fd_t *
-_fd_ref (fd_t *fd);
+__fd_ref (fd_t *fd);
 
 void
 fd_ctx_dump (fd_t *fd, char *prefix);
 
+fdentry_t *
+gf_fd_fdtable_copy_all_fds (fdtable_t *fdtable, uint32_t *count);
+
+fdentry_t *
+__gf_fd_fdtable_copy_all_fds (fdtable_t *fdtable, uint32_t *count);
+
+void
+gf_fdptr_put (fdtable_t *fdtable, fd_t *fd);
+
 #endif /* _FD_H */
