@@ -1,20 +1,11 @@
 /*
-  Copyright (c) 2010-2011 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+  This file is licensed to you under your choice of the GNU Lesser
+  General Public License, version 3 or any later version (LGPLv3 or
+  later), or the GNU General Public License, version 2 (GPLv2), in all
+  cases as published by the Free Software Foundation.
 */
 
 #ifndef _CONFIG_H
@@ -111,6 +102,7 @@ glusterfs_ctx_init ()
         }
 
         INIT_LIST_HEAD (&glusterfs_ctx->graphs);
+        INIT_LIST_HEAD (&glusterfs_ctx->mempool_list);
         ret = pthread_mutex_init (&glusterfs_ctx->lock, NULL);
 
 out:
@@ -154,6 +146,8 @@ glusterfs_this_init ()
         global_xlator.type = "global";
         global_xlator.ctx  = glusterfs_ctx;
 
+        INIT_LIST_HEAD (&global_xlator.volume_options);
+
         return ret;
 }
 
@@ -290,6 +284,43 @@ glusterfs_uuid_buf_get ()
         return buf;
 }
 
+/* LKOWNER_BUFFER */
+
+static pthread_key_t lkowner_buf_key;
+static char global_lkowner_buf[GF_LKOWNER_BUF_SIZE];
+void
+glusterfs_lkowner_buf_destroy (void *ptr)
+{
+        if (ptr)
+                FREE (ptr);
+}
+
+int
+glusterfs_lkowner_buf_init ()
+{
+        int ret = 0;
+
+        ret = pthread_key_create (&lkowner_buf_key,
+                                  glusterfs_lkowner_buf_destroy);
+        return ret;
+}
+
+char *
+glusterfs_lkowner_buf_get ()
+{
+        char *buf;
+        int ret = 0;
+
+        buf = pthread_getspecific (lkowner_buf_key);
+        if(!buf) {
+                buf = MALLOC (GF_LKOWNER_BUF_SIZE);
+                ret = pthread_setspecific (lkowner_buf_key, (void *) buf);
+                if(ret)
+                        buf = global_lkowner_buf;
+        }
+        return buf;
+}
+
 int
 glusterfs_globals_init ()
 {
@@ -320,6 +351,13 @@ glusterfs_globals_init ()
                 goto out;
         }
 
+        ret = glusterfs_lkowner_buf_init ();
+        if(ret) {
+                gf_log ("", GF_LOG_CRITICAL,
+                        "ERROR: glusterfs lkowner buffer init failed");
+                goto out;
+        }
+
         gf_mem_acct_enable_set ();
 
         ret = synctask_init ();
@@ -333,7 +371,7 @@ out:
 }
 
 
-char eventstring[GF_EVENT_MAXVAL][64] = {
+char eventstring[GF_EVENT_MAXVAL+1][64] = {
         "Invalid event",
         "Parent Up",
         "Poll In",
@@ -342,9 +380,15 @@ char eventstring[GF_EVENT_MAXVAL][64] = {
         "Child Up",
         "Child Down",
         "Child Connecting",
+        "Child Modified",
         "Transport Cleanup",
         "Transport Connected",
         "Volfile Modified",
+        "New Volfile",
+        "Translator Info",
+        "Xlator Op",
+        "Authentication Failed",
+        "Invalid event",
 };
 
 /* Copy the string ptr contents if needed for yourself */
