@@ -1,20 +1,11 @@
 /*
-   Copyright (c) 2006-2011 Gluster, Inc. <http://www.gluster.com>
-   This file is part of GlusterFS.
+  Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
+  This file is part of GlusterFS.
 
-   GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
-   by the Free Software Foundation; either version 3 of the License,
-   or (at your option) any later version.
-
-   GlusterFS is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program.  If not, see
-   <http://www.gnu.org/licenses/>.
+  This file is licensed to you under your choice of the GNU Lesser
+  General Public License, version 3 or any later version (LGPLv3 or
+  later), or the GNU General Public License, version 2 (GPLv2), in all
+  cases as published by the Free Software Foundation.
 */
 
 #ifndef _GLUSTERFS_H
@@ -42,16 +33,21 @@
 #include <sys/poll.h>
 #include <pthread.h>
 
+
 #include "list.h"
 #include "logging.h"
 
-
 #define GF_YES 1
 #define GF_NO  0
 
 #ifndef O_LARGEFILE
 /* savannah bug #20053, patch for compiling on darwin */
-#define O_LARGEFILE 0
+#define O_LARGEFILE 0100000 /* from bits/fcntl.h */
+#endif
+
+#ifndef O_FMODE_EXEC
+/* redhat bug 843080, added from linux/fs.h */
+#define O_FMODE_EXEC 040 //0x20
 #endif
 
 #ifndef O_DIRECT
@@ -80,10 +76,23 @@
 #define ZR_STRICT_VOLFILE_CHECK "strict-volfile-check"
 #define ZR_DUMP_FUSE            "dump-fuse"
 
+#define GF_XATTR_CLRLK_CMD      "glusterfs.clrlk"
 #define GF_XATTR_PATHINFO_KEY   "trusted.glusterfs.pathinfo"
+#define GF_XATTR_NODE_UUID_KEY  "trusted.glusterfs.node-uuid"
+#define GF_XATTR_VOL_ID_KEY   "trusted.glusterfs.volume-id"
+
+#define GF_READDIR_SKIP_DIRS       "readdir-filter-directories"
+
+#define XATTR_IS_PATHINFO(x)  (strncmp (x, GF_XATTR_PATHINFO_KEY,       \
+                                        strlen (GF_XATTR_PATHINFO_KEY)) == 0)
+#define XATTR_IS_NODE_UUID(x) (strncmp (x, GF_XATTR_NODE_UUID_KEY,      \
+                                        strlen (GF_XATTR_NODE_UUID_KEY)) == 0)
+
 #define GF_XATTR_LINKINFO_KEY   "trusted.distribute.linkinfo"
 #define GFID_XATTR_KEY "trusted.gfid"
 
+#define GLUSTERFS_INTERNAL_FOP_KEY  "glusterfs-internal-fop"
+
 #define ZR_FILE_CONTENT_STR     "glusterfs.file."
 #define ZR_FILE_CONTENT_STRLEN 15
 
@@ -91,7 +100,23 @@
 #define GLUSTERFS_INODELK_COUNT "glusterfs.inodelk-count"
 #define GLUSTERFS_ENTRYLK_COUNT "glusterfs.entrylk-count"
 #define GLUSTERFS_POSIXLK_COUNT "glusterfs.posixlk-count"
+#define GLUSTERFS_PARENT_ENTRYLK "glusterfs.parent-entrylk"
 #define QUOTA_SIZE_KEY "trusted.glusterfs.quota.size"
+#define GFID_TO_PATH_KEY "glusterfs.gfid2path"
+
+/* Index xlator related */
+#define GF_XATTROP_INDEX_GFID "glusterfs.xattrop_index_gfid"
+
+#define GF_GFIDLESS_LOOKUP "gfidless-lookup"
+/* replace-brick and pump related internal xattrs */
+#define RB_PUMP_CMD_START       "glusterfs.pump.start"
+#define RB_PUMP_CMD_PAUSE       "glusterfs.pump.pause"
+#define RB_PUMP_CMD_COMMIT      "glusterfs.pump.commit"
+#define RB_PUMP_CMD_ABORT       "glusterfs.pump.abort"
+#define RB_PUMP_CMD_STATUS      "glusterfs.pump.status"
+
+#define POSIX_ACL_DEFAULT_XATTR "system.posix_acl_default"
+#define POSIX_ACL_ACCESS_XATTR "system.posix_acl_access"
 
 #define GLUSTERFS_RDMA_INLINE_THRESHOLD       (2048)
 #define GLUSTERFS_RDMA_MAX_HEADER_SIZE        (228) /* (sizeof (rdma_header_t)                 \
@@ -104,8 +129,11 @@
 #define ZR_FILE_CONTENT_REQUEST(key) (!strncmp(key, ZR_FILE_CONTENT_STR, \
 					       ZR_FILE_CONTENT_STRLEN))
 
-/* TODO: Should we use PATH-MAX? On some systems it may save space */
-#define ZR_PATH_MAX 4096
+/* GlusterFS's maximum supported Auxilary GIDs */
+/* TODO: Keeping it to 200, so that we can fit in 2KB buffer for auth data
+ * in RPC server code, if there is ever need for having more aux-gids, then
+ * we have to add aux-gid in payload of actors */
+#define GF_MAX_AUX_GROUPS   200
 
 /* NOTE: add members ONLY at the end (just before _MAXVALUE) */
 typedef enum {
@@ -154,6 +182,7 @@ typedef enum {
         GF_FOP_RELEASE,
         GF_FOP_RELEASEDIR,
         GF_FOP_GETSPEC,
+        GF_FOP_FREMOVEXATTR,
         GF_FOP_MAXVALUE,
 } glusterfs_fop_t;
 
@@ -170,15 +199,6 @@ typedef enum {
         GF_OP_TYPE_MAX,
 } gf_op_type_t;
 
-struct gf_flock {
-        short    l_type;
-        short    l_whence;
-        off_t    l_start;
-        off_t    l_len;
-        pid_t    l_pid;
-        uint64_t l_owner;
-};
-
 /* NOTE: all the miscellaneous flags used by GlusterFS should be listed here */
 typedef enum {
         GF_LK_GETLK = 0,
@@ -276,6 +296,8 @@ struct _cmd_args {
 	int              debug_mode;
         int              read_only;
         int              acl;
+        int              selinux;
+        int              worm;
         int              mac_compat;
 	struct list_head xlator_options;  /* list of xlator_option_t */
 
@@ -290,6 +312,8 @@ struct _cmd_args {
 	char            *dump_fuse;
         pid_t            client_pid;
         int              client_pid_set;
+        unsigned         uid_map_root;
+
 
 	/* key args */
 	char            *mount_point;
@@ -318,6 +342,8 @@ struct _glusterfs_graph {
 typedef struct _glusterfs_graph glusterfs_graph_t;
 
 
+typedef int32_t (*glusterfsd_mgmt_event_notify_fn_t) (int32_t event, void *data,
+                                                      ...);
 struct _glusterfs_ctx {
 	cmd_args_t          cmd_args;
 	char               *process_uuid;
@@ -342,6 +368,23 @@ struct _glusterfs_ctx {
         int                 graph_id; /* Incremented per graph, value should
                                          indicate how many times the graph has
                                          got changed */
+        pid_t               mnt_pid; /* pid of the mount agent */
+        int                 process_mode; /*mode in which process is runninng*/
+	struct syncenv      *env;         /* The env pointer to the synctasks */
+
+        struct list_head    mempool_list; /* used to keep a global list of
+                                             mempools, used to log details of
+                                             mempool in statedump */
+        char                *statedump_path;
+
+        struct mem_pool    *dict_pool;
+        struct mem_pool    *dict_pair_pool;
+        struct mem_pool    *dict_data_pool;
+
+        int                 mem_accounting; /* if value is other than 0, it
+                                               will be set */
+        glusterfsd_mgmt_event_notify_fn_t notify; /* Used for xlators to make
+                                                     call to fsd-mgmt */
 };
 typedef struct _glusterfs_ctx glusterfs_ctx_t;
 
@@ -363,12 +406,35 @@ typedef enum {
         GF_EVENT_VOLFILE_MODIFIED,
         GF_EVENT_GRAPH_NEW,
         GF_EVENT_TRANSLATOR_INFO,
+        GF_EVENT_TRANSLATOR_OP,
+        GF_EVENT_AUTH_FAILED,
+        GF_EVENT_VOLUME_DEFRAG,
+        GF_EVENT_PARENT_DOWN,
         GF_EVENT_MAXVAL,
 } glusterfs_event_t;
 
+/* gf_lkowner_t is defined in lkowner.h */
+#include "lkowner.h"
+
+struct gf_flock {
+        short        l_type;
+        short        l_whence;
+        off_t        l_start;
+        off_t        l_len;
+        pid_t        l_pid;
+        gf_lkowner_t l_owner;
+};
+
 extern char *glusterfs_strevent (glusterfs_event_t ev);
 
 #define GF_MUST_CHECK __attribute__((warn_unused_result))
+/*
+ * Some macros (e.g. ALLOC_OR_GOTO) set variables in function scope, but the
+ * calling function might not only declare the variable to keep the macro happy
+ * and not use it otherwise.  In such cases, the following can be used to
+ * suppress the "set but not used" warning that would otherwise occur.
+ */
+#define GF_UNUSED __attribute__((unused))
 
 int glusterfs_graph_prepare (glusterfs_graph_t *graph, glusterfs_ctx_t *ctx);
 int glusterfs_graph_destroy (glusterfs_graph_t *graph);
