@@ -1,20 +1,11 @@
 /*
-  Copyright (c) 2010-2011 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+  This file is licensed to you under your choice of the GNU Lesser
+  General Public License, version 3 or any later version (LGPLv3 or
+  later), or the GNU General Public License, version 2 (GPLv2), in all
+  cases as published by the Free Software Foundation.
 */
 
 #ifndef _CONFIG_H
@@ -26,7 +17,6 @@
 
 #include "common-utils.h"
 #include "xlator.h"
-#include "graph-mem-types.h"
 #include "graph-utils.h"
 
 
@@ -180,7 +170,7 @@ glusterfs_graph_print_buf (glusterfs_graph_t *graph)
         if (len == -1)
                 return NULL;
 
-        buf = GF_CALLOC (1, len + 1, gf_graph_mt_buf);
+        buf = GF_CALLOC (1, len + 1, gf_common_mt_graph_buf);
         if (!buf) {
                 return NULL;
         }
