@@ -1,20 +1,11 @@
 /*
-  Copyright (c) 2007-2011 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+  This file is licensed to you under your choice of the GNU Lesser
+  General Public License, version 3 or any later version (LGPLv3 or
+  later), or the GNU General Public License, version 2 (GPLv2), in all
+  cases as published by the Free Software Foundation.
 */
 
 #ifndef _CONFIG_H
@@ -94,7 +85,7 @@ __dentry_hash (dentry_t *dentry)
         int              hash = 0;
 
         if (!dentry) {
-                gf_log_callingfn ("", GF_LOG_WARNING, "dentry not found");
+                gf_log_callingfn (THIS->name, GF_LOG_WARNING, "dentry not found");
                 return;
         }
 
@@ -111,7 +102,7 @@ static int
 __is_dentry_hashed (dentry_t *dentry)
 {
         if (!dentry) {
-                gf_log_callingfn ("", GF_LOG_WARNING, "dentry not found");
+                gf_log_callingfn (THIS->name, GF_LOG_WARNING, "dentry not found");
                 return 0;
         }
 
@@ -123,7 +114,7 @@ static void
 __dentry_unhash (dentry_t *dentry)
 {
         if (!dentry) {
-                gf_log_callingfn ("", GF_LOG_WARNING, "dentry not found");
+                gf_log_callingfn (THIS->name, GF_LOG_WARNING, "dentry not found");
                 return;
         }
 
@@ -134,14 +125,11 @@ __dentry_unhash (dentry_t *dentry)
 static void
 __dentry_unset (dentry_t *dentry)
 {
-        struct mem_pool *tmp_pool = NULL;
-
         if (!dentry) {
-                gf_log_callingfn ("", GF_LOG_WARNING, "dentry not found");
+                gf_log_callingfn (THIS->name, GF_LOG_WARNING, "dentry not found");
                 return;
         }
 
-        tmp_pool = dentry->inode->table->dentry_pool;
         __dentry_unhash (dentry);
 
         list_del_init (&dentry->inode_list);
@@ -154,9 +142,7 @@ __dentry_unset (dentry_t *dentry)
                 dentry->parent = NULL;
         }
 
-        mem_put (tmp_pool, dentry);
-        tmp_pool = NULL;
-
+        mem_put (dentry);
 }
 
 
@@ -171,19 +157,19 @@ __foreach_ancestor_dentry (dentry_t *dentry,
         int       ret = 0;
 
         if (!dentry) {
-                gf_log_callingfn ("", GF_LOG_WARNING, "dentry not found");
+                gf_log_callingfn (THIS->name, GF_LOG_WARNING, "dentry not found");
                 return 0;
         }
 
         ret = per_dentry_fn (dentry, data);
         if (ret) {
-                gf_log ("", GF_LOG_WARNING, "per dentry fn returned %d", ret);
+                gf_log (THIS->name, GF_LOG_WARNING, "per dentry fn returned %d", ret);
                 goto out;
         }
 
         parent = dentry->parent;
         if (!parent) {
-                gf_log ("", GF_LOG_WARNING, "parent not found");
+                gf_log (THIS->name, GF_LOG_WARNING, "parent not found");
                 goto out;
         }
 
@@ -228,8 +214,8 @@ __is_dentry_cyclic (dentry_t *dentry)
 
                 gf_log (dentry->inode->table->name, GF_LOG_CRITICAL,
                         "detected cyclic loop formation during inode linkage."
-                        " inode (%"PRId64"/%s) linking under itself as %s",
-                        inode->ino, uuid_utoa (inode->gfid), name);
+                        " inode (%s) linking under itself as %s",
+                        uuid_utoa (inode->gfid), name);
         }
 
         return ret;
@@ -240,7 +226,7 @@ static void
 __inode_unhash (inode_t *inode)
 {
         if (!inode) {
-                gf_log_callingfn ("", GF_LOG_WARNING, "inode not found");
+                gf_log_callingfn (THIS->name, GF_LOG_WARNING, "inode not found");
                 return;
         }
 
@@ -252,7 +238,7 @@ static int
 __is_inode_hashed (inode_t *inode)
 {
         if (!inode) {
-                gf_log_callingfn ("", GF_LOG_WARNING, "inode not found");
+                gf_log_callingfn (THIS->name, GF_LOG_WARNING, "inode not found");
                 return 0;
         }
 
@@ -267,7 +253,7 @@ __inode_hash (inode_t *inode)
         int            hash = 0;
 
         if (!inode) {
-                gf_log_callingfn ("", GF_LOG_WARNING, "inode not found");
+                gf_log_callingfn (THIS->name, GF_LOG_WARNING, "inode not found");
                 return;
         }
 
@@ -280,18 +266,25 @@ __inode_hash (inode_t *inode)
 
 
 static dentry_t *
-__dentry_search_for_inode (inode_t *inode, ino_t par, const char *name)
+__dentry_search_for_inode (inode_t *inode, uuid_t pargfid, const char *name)
 {
         dentry_t *dentry = NULL;
         dentry_t *tmp = NULL;
 
         if (!inode || !name) {
-                gf_log_callingfn ("", GF_LOG_WARNING, "inode || name not found");
+                gf_log_callingfn (THIS->name, GF_LOG_WARNING, "inode || name not found");
                 return NULL;
         }
 
+        /* earlier, just the ino was sent, which could have been 0, now
+           we deal with gfid, and if sent gfid is null or 0, no need to
+           continue with the check */
+        if (!pargfid || uuid_is_null (pargfid))
+                return NULL;
+
         list_for_each_entry (tmp, &inode->dentry_list, inode_list) {
-                if (tmp->parent->ino == par && !strcmp (tmp->name, name)) {
+                if ((uuid_compare (tmp->parent->gfid, pargfid) == 0) &&
+                    !strcmp (tmp->name, name)) {
                         dentry = tmp;
                         break;
                 }
@@ -307,20 +300,17 @@ __inode_destroy (inode_t *inode)
         int          index = 0;
         xlator_t    *xl = NULL;
         xlator_t    *old_THIS = NULL;
-        struct mem_pool *tmp_pool = NULL;
 
         if (!inode) {
-                gf_log_callingfn ("", GF_LOG_WARNING, "inode not found");
+                gf_log_callingfn (THIS->name, GF_LOG_WARNING, "inode not found");
                 return;
         }
 
         if (!inode->_ctx) {
-                gf_log ("", GF_LOG_WARNING, "_ctx not found");
+                gf_log (THIS->name, GF_LOG_WARNING, "_ctx not found");
                 goto noctx;
         }
 
-        tmp_pool = inode->table->inode_pool;
-
         for (index = 0; index < inode->table->xl->graph->xl_count; index++) {
                 if (inode->_ctx[index].xl_key) {
                         xl = (xlator_t *)(long)inode->_ctx[index].xl_key;
@@ -336,9 +326,7 @@ __inode_destroy (inode_t *inode)
 noctx:
         LOCK_DESTROY (&inode->lock);
         //  memset (inode, 0xb, sizeof (*inode));
-        mem_put (tmp_pool, inode);
-        tmp_pool = NULL;
-
+        mem_put (inode);
 }
 
 
@@ -360,7 +348,7 @@ __inode_passivate (inode_t *inode)
         dentry_t      *t = NULL;
 
         if (!inode) {
-                gf_log_callingfn ("", GF_LOG_WARNING, "inode not found");
+                gf_log_callingfn (THIS->name, GF_LOG_WARNING, "inode not found");
                 return;
         }
 
@@ -381,7 +369,7 @@ __inode_retire (inode_t *inode)
         dentry_t      *t = NULL;
 
         if (!inode) {
-                gf_log_callingfn ("", GF_LOG_WARNING, "inode not found");
+                gf_log_callingfn (THIS->name, GF_LOG_WARNING, "inode not found");
                 return;
         }
 
@@ -402,7 +390,7 @@ __inode_unref (inode_t *inode)
         if (!inode)
                 return NULL;
 
-        if (inode->ino == 1)
+        if (__is_root_gfid(inode->gfid))
                 return inode;
 
         GF_ASSERT (inode->ref);
@@ -486,7 +474,7 @@ __dentry_create (inode_t *inode, inode_t *parent, const char *name)
         dentry_t      *newd = NULL;
 
         if (!inode || !parent || !name) {
-                gf_log_callingfn ("", GF_LOG_WARNING,
+                gf_log_callingfn (THIS->name, GF_LOG_WARNING,
                                   "inode || parent || name not found");
                 return NULL;
         }
@@ -501,7 +489,7 @@ __dentry_create (inode_t *inode, inode_t *parent, const char *name)
 
         newd->name = gf_strdup (name);
         if (newd->name == NULL) {
-                mem_put (parent->table->dentry_pool, newd);
+                mem_put (newd);
                 newd = NULL;
                 goto out;
         }
@@ -523,7 +511,7 @@ __inode_create (inode_table_t *table)
         inode_t  *newi = NULL;
 
         if (!table) {
-                gf_log_callingfn ("", GF_LOG_WARNING, "table not found");
+                gf_log_callingfn (THIS->name, GF_LOG_WARNING, "table not found");
                 return NULL;
         }
 
@@ -547,7 +535,7 @@ __inode_create (inode_table_t *table)
 
         if (newi->_ctx == NULL) {
                 LOCK_DESTROY (&newi->lock);
-                mem_put (table->inode_pool, newi);
+                mem_put (newi);
                 newi = NULL;
                 goto out;
         }
@@ -567,7 +555,7 @@ inode_new (inode_table_t *table)
         inode_t *inode = NULL;
 
         if (!table) {
-                gf_log_callingfn ("", GF_LOG_WARNING, "inode not found");
+                gf_log_callingfn (THIS->name, GF_LOG_WARNING, "inode not found");
                 return NULL;
         }
 
@@ -643,7 +631,7 @@ inode_grep (inode_table_t *table, inode_t *parent, const char *name)
         dentry_t  *dentry = NULL;
 
         if (!table || !parent || !name) {
-                gf_log_callingfn ("", GF_LOG_WARNING,
+                gf_log_callingfn (THIS->name, GF_LOG_WARNING,
                                   "table || parent || name not found");
                 return NULL;
         }
@@ -665,24 +653,94 @@ inode_grep (inode_table_t *table, inode_t *parent, const char *name)
 
 
 inode_t *
-inode_get (inode_table_t *table, ino_t ino, uint64_t gen)
+inode_resolve (inode_table_t *table, char *path)
 {
-        return NULL;
+        char    *tmp   = NULL, *bname = NULL, *str = NULL, *saveptr = NULL;
+        inode_t *inode = NULL, *parent = NULL;
+
+        if ((path == NULL) || (table == NULL)) {
+                goto out;
+        }
+
+        parent = inode_ref (table->root);
+        str = tmp = gf_strdup (path);
+
+        while (1) {
+                bname = strtok_r (str, "/", &saveptr);
+                if (bname == NULL) {
+                        break;
+                }
+
+                if (inode != NULL) {
+                        inode_unref (inode);
+                }
+
+                inode = inode_grep (table, parent, bname);
+                if (inode == NULL) {
+                        break;
+                }
+
+                if (parent != NULL) {
+                        inode_unref (parent);
+                }
+
+                parent = inode_ref (inode);
+                str = NULL;
+        }
+
+        inode_unref (parent);
+        GF_FREE (tmp);
+out:
+        return inode;
 }
 
 
 int
+inode_grep_for_gfid (inode_table_t *table, inode_t *parent, const char *name,
+                     uuid_t gfid, ia_type_t *type)
+{
+        inode_t   *inode = NULL;
+        dentry_t  *dentry = NULL;
+        int        ret = -1;
+
+        if (!table || !parent || !name) {
+                gf_log_callingfn (THIS->name, GF_LOG_WARNING,
+                                  "table || parent || name not found");
+                return ret;
+        }
+
+        pthread_mutex_lock (&table->lock);
+        {
+                dentry = __dentry_grep (table, parent, name);
+
+                if (dentry)
+                        inode = dentry->inode;
+
+                if (inode) {
+                        uuid_copy (gfid, inode->gfid);
+                        *type = inode->ia_type;
+                        ret = 0;
+                }
+        }
+        pthread_mutex_unlock (&table->lock);
+
+        return ret;
+}
+
+
+/* return 1 if gfid is of root, 0 if not */
+gf_boolean_t
 __is_root_gfid (uuid_t gfid)
 {
         uuid_t  root;
-        int     ret;
 
         memset (root, 0, 16);
         root[15] = 1;
 
-        ret = uuid_compare (gfid, root);
+        if (uuid_compare (gfid, root) == 0)
+                return _gf_true;
 
-        return ret;
+        return _gf_false;
 }
 
 
@@ -694,11 +752,11 @@ __inode_find (inode_table_t *table, uuid_t gfid)
         int        hash = 0;
 
         if (!table) {
-                gf_log_callingfn ("", GF_LOG_WARNING, "table not found");
+                gf_log_callingfn (THIS->name, GF_LOG_WARNING, "table not found");
                 goto out;
         }
 
-        if (__is_root_gfid (gfid) == 0)
+        if (__is_root_gfid (gfid))
                 return table->root;
 
         hash = hash_gfid (gfid, 65536);
@@ -721,7 +779,7 @@ inode_find (inode_table_t *table, uuid_t gfid)
         inode_t   *inode = NULL;
 
         if (!table) {
-                gf_log_callingfn ("", GF_LOG_WARNING, "table not found");
+                gf_log_callingfn (THIS->name, GF_LOG_WARNING, "table not found");
                 return NULL;
         }
 
@@ -773,7 +831,6 @@ __inode_link (inode_t *inode, inode_t *parent, const char *name,
                         return NULL;
 
                 uuid_copy (inode->gfid, iatt->ia_gfid);
-                inode->ino        = iatt->ia_ino;
                 inode->ia_type    = iatt->ia_type;
 
                 old_inode = __inode_find (table, inode->gfid);
@@ -785,12 +842,25 @@ __inode_link (inode_t *inode, inode_t *parent, const char *name,
                 }
         }
 
+        if (name) {
+                if (!strcmp(name, ".") || !strcmp(name, ".."))
+                        return link_inode;
+        }
+
         /* use only link_inode beyond this point */
         if (parent) {
                 old_dentry = __dentry_grep (table, parent, name);
 
                 if (!old_dentry || old_dentry->inode != link_inode) {
                         dentry = __dentry_create (link_inode, parent, name);
+                        if (!dentry) {
+                                gf_log_callingfn (THIS->name, GF_LOG_ERROR,
+                                                  "dentry create failed on "
+                                                  "inode %s with parent %s",
+                                                  uuid_utoa (link_inode->gfid),
+                                                  uuid_utoa (parent->gfid));
+                                return NULL;
+                        }
                         if (old_inode && __is_dentry_cyclic (dentry)) {
                                 __dentry_unset (dentry);
                                 return NULL;
@@ -814,7 +884,7 @@ inode_link (inode_t *inode, inode_t *parent, const char *name,
         inode_t       *linked_inode = NULL;
 
         if (!inode) {
-                gf_log_callingfn ("", GF_LOG_WARNING, "inode not found");
+                gf_log_callingfn (THIS->name, GF_LOG_WARNING, "inode not found");
                 return NULL;
         }
 
@@ -841,7 +911,7 @@ inode_lookup (inode_t *inode)
         inode_table_t *table = NULL;
 
         if (!inode) {
-                gf_log_callingfn ("", GF_LOG_WARNING, "inode not found");
+                gf_log_callingfn (THIS->name, GF_LOG_WARNING, "inode not found");
                 return -1;
         }
 
@@ -863,7 +933,7 @@ inode_forget (inode_t *inode, uint64_t nlookup)
         inode_table_t *table = NULL;
 
         if (!inode) {
-                gf_log_callingfn ("", GF_LOG_WARNING, "inode not found");
+                gf_log_callingfn (THIS->name, GF_LOG_WARNING, "inode not found");
                 return -1;
         }
 
@@ -889,7 +959,7 @@ __inode_unlink (inode_t *inode, inode_t *parent, const char *name)
         if (!inode || !parent || !name)
                 return;
 
-        dentry = __dentry_search_for_inode (inode, parent->ino, name);
+        dentry = __dentry_search_for_inode (inode, parent->gfid, name);
 
         /* dentry NULL for corrupted backend */
         if (dentry)
@@ -903,7 +973,7 @@ inode_unlink (inode_t *inode, inode_t *parent, const char *name)
         inode_table_t *table = NULL;
 
         if (!inode) {
-                gf_log_callingfn ("", GF_LOG_WARNING, "inode not found");
+                gf_log_callingfn (THIS->name, GF_LOG_WARNING, "inode not found");
                 return;
         }
 
@@ -925,7 +995,7 @@ inode_rename (inode_table_t *table, inode_t *srcdir, const char *srcname,
               struct iatt *iatt)
 {
         if (!inode) {
-                gf_log_callingfn ("", GF_LOG_WARNING, "inode not found");
+                gf_log_callingfn (THIS->name, GF_LOG_WARNING, "inode not found");
                 return -1;
         }
 
@@ -972,14 +1042,14 @@ __dentry_search_arbit (inode_t *inode)
 
 
 inode_t *
-inode_parent (inode_t *inode, ino_t par, const char *name)
+inode_parent (inode_t *inode, uuid_t pargfid, const char *name)
 {
         inode_t       *parent = NULL;
         inode_table_t *table = NULL;
         dentry_t      *dentry = NULL;
 
         if (!inode) {
-                gf_log_callingfn ("", GF_LOG_WARNING, "inode not found");
+                gf_log_callingfn (THIS->name, GF_LOG_WARNING, "inode not found");
                 return NULL;
         }
 
@@ -987,8 +1057,8 @@ inode_parent (inode_t *inode, ino_t par, const char *name)
 
         pthread_mutex_lock (&table->lock);
         {
-                if (par && name) {
-                        dentry = __dentry_search_for_inode (inode, par, name);
+                if (pargfid && !uuid_is_null (pargfid) && name) {
+                        dentry = __dentry_search_for_inode (inode, pargfid, name);
                 } else {
                         dentry = __dentry_search_arbit (inode);
                 }
@@ -1006,81 +1076,86 @@ inode_parent (inode_t *inode, ino_t par, const char *name)
 
 
 int
-inode_path (inode_t *inode, const char *name, char **bufp)
+__inode_path (inode_t *inode, const char *name, char **bufp)
 {
         inode_table_t *table = NULL;
-        dentry_t      *trav = NULL;
-        size_t         i = 0, size = 0;
-        int64_t        ret = 0;
-        int            len = 0;
-        char          *buf = NULL;
+        inode_t       *itrav = NULL;
+        dentry_t      *trav  = NULL;
+        size_t         i     = 0, size = 0;
+        int64_t        ret   = 0;
+        int            len   = 0;
+        char          *buf   = NULL;
 
-        if (!inode) {
-                gf_log_callingfn ("", GF_LOG_WARNING, "inode not found");
+        if (!inode || uuid_is_null (inode->gfid)) {
+                GF_ASSERT (0);
+                gf_log_callingfn (THIS->name, GF_LOG_WARNING, "invalid inode");
                 return -1;
         }
 
         table = inode->table;
 
-        pthread_mutex_lock (&table->lock);
-        {
-                for (trav = __dentry_search_arbit (inode); trav;
-                     trav = __dentry_search_arbit (trav->parent)) {
-                        i ++; /* "/" */
-                        i += strlen (trav->name);
-                        if (i > PATH_MAX) {
-                                gf_log (table->name, GF_LOG_CRITICAL,
-                                        "possible infinite loop detected, "
-                                        "forcing break. name=(%s)", name);
-                                ret = -ENOENT;
-                                goto unlock;
-                        }
-                }
-
-                if ((inode->ino != 1) &&
-                    (i == 0)) {
-                        gf_log_callingfn (table->name, GF_LOG_WARNING,
-                                "no dentry for non-root inode %"PRId64": %s",
-                                inode->ino, uuid_utoa (inode->gfid));
+        itrav = inode;
+        for (trav = __dentry_search_arbit (itrav); trav;
+             trav = __dentry_search_arbit (itrav)) {
+                itrav = trav->parent;
+                i ++; /* "/" */
+                i += strlen (trav->name);
+                if (i > PATH_MAX) {
+                        gf_log (table->name, GF_LOG_CRITICAL,
+                                "possible infinite loop detected, "
+                                "forcing break. name=(%s)", name);
                         ret = -ENOENT;
-                        goto unlock;
-                }
-
-                if (name) {
-                        i++;
-                        i += strlen (name);
-                }
-
-                ret = i;
-                size = i + 1;
-                buf = GF_CALLOC (size, sizeof (char), gf_common_mt_char);
-                if (buf) {
-
-                        buf[size - 1] = 0;
-
-                        if (name) {
-                                len = strlen (name);
-                                strncpy (buf + (i - len), name, len);
-                                buf[i-len-1] = '/';
-                                i -= (len + 1);
-                        }
-
-                        for (trav = __dentry_search_arbit (inode); trav;
-                             trav = __dentry_search_arbit (trav->parent)) {
-                                len = strlen (trav->name);
-                                strncpy (buf + (i - len), trav->name, len);
-                                buf[i-len-1] = '/';
-                                i -= (len + 1);
-                        }
-                        *bufp = buf;
-                } else {
-                        ret = -ENOMEM;
+                        goto out;
                 }
         }
-unlock:
-        pthread_mutex_unlock (&table->lock);
 
-        if (inode->ino == 1 && !name) {
+        if (!__is_root_gfid (itrav->gfid)) {
+                /* "<gfid:00000000-0000-0000-0000-000000000000>"/path */
+                i += GFID_STR_PFX_LEN;
+        }
+
+        if (name) {
+                i++;
+                i += strlen (name);
+        }
+
+        ret = i;
+        size = i + 1;
+        buf = GF_CALLOC (size, sizeof (char), gf_common_mt_char);
+        if (buf) {
+
+                buf[size - 1] = 0;
+
+                if (name) {
+                        len = strlen (name);
+                        strncpy (buf + (i - len), name, len);
+                        buf[i-len-1] = '/';
+                        i -= (len + 1);
+                }
+
+                itrav = inode;
+                for (trav = __dentry_search_arbit (itrav); trav;
+                     trav = __dentry_search_arbit (itrav)) {
+                        itrav = trav->parent;
+                        len = strlen (trav->name);
+                        strncpy (buf + (i - len), trav->name, len);
+                        buf[i-len-1] = '/';
+                        i -= (len + 1);
+                }
+
+                if (!__is_root_gfid (itrav->gfid)) {
+                        snprintf (&buf[i-GFID_STR_PFX_LEN], GFID_STR_PFX_LEN,
+                                  INODE_PATH_FMT, uuid_utoa (itrav->gfid));
+                        buf[i-1] = '>';
+                }
+
+                *bufp = buf;
+        } else {
+                ret = -ENOMEM;
+        }
+
+out:
+        if (__is_root_gfid (inode->gfid) && !name) {
                 ret = 1;
                 if (buf) {
                         GF_FREE (buf);
@@ -1094,9 +1169,33 @@ unlock:
                 }
         }
 
+        if (ret < 0)
+                *bufp = NULL;
         return ret;
 }
 
+
+int
+inode_path (inode_t *inode, const char *name, char **bufp)
+{
+        inode_table_t *table = NULL;
+        int            ret   = -1;
+
+        if (!inode)
+                return -1;
+
+        table = inode->table;
+
+        pthread_mutex_lock (&table->lock);
+        {
+                ret = __inode_path (inode, name, bufp);
+        }
+        pthread_mutex_unlock (&table->lock);
+
+        return ret;
+}
+
+
 static int
 inode_table_prune (inode_table_t *table)
 {
@@ -1165,7 +1264,7 @@ inode_table_t *
 inode_table_new (size_t lru_limit, xlator_t *xl)
 {
         inode_table_t *new = NULL;
-        int            ret = 0;
+        int            ret = -1;
         int            i = 0;
 
         new = (void *)GF_CALLOC(1, sizeof (*new), gf_common_mt_inode_table_t);
@@ -1184,41 +1283,32 @@ inode_table_new (size_t lru_limit, xlator_t *xl)
 
         new->inode_pool = mem_pool_new (inode_t, lru_limit);
 
-        if (!new->inode_pool) {
-                GF_FREE (new);
-                return NULL;
-        }
+        if (!new->inode_pool)
+                goto out;
 
         new->dentry_pool = mem_pool_new (dentry_t, lru_limit);
 
-        if (!new->dentry_pool) {
-                GF_FREE (new);
-                return NULL;
-        }
+        if (!new->dentry_pool)
+                goto out;
 
         new->inode_hash = (void *)GF_CALLOC (65536,
                                              sizeof (struct list_head),
                                              gf_common_mt_list_head);
-        if (!new->inode_hash) {
-                GF_FREE (new);
-                return NULL;
-        }
+        if (!new->inode_hash)
+                goto out;
 
         new->name_hash = (void *)GF_CALLOC (new->hashsize,
                                             sizeof (struct list_head),
                                             gf_common_mt_list_head);
-        if (!new->name_hash) {
-                GF_FREE (new->inode_hash);
-                GF_FREE (new);
-                return NULL;
-        }
+        if (!new->name_hash)
+                goto out;
 
-        new->fd_mem_pool = mem_pool_new (fd_t, 16384);
+        /* if number of fd open in one process is more than this,
+           we may hit perf issues */
+        new->fd_mem_pool = mem_pool_new (fd_t, 1024);
 
-        if (!new->fd_mem_pool) {
-                GF_FREE (new->inode_hash);
-                GF_FREE (new);
-        }
+        if (!new->fd_mem_pool)
+                goto out;
 
         for (i = 0; i < 65536; i++) {
                 INIT_LIST_HEAD (&new->inode_hash[i]);
@@ -1243,6 +1333,23 @@ inode_table_new (size_t lru_limit, xlator_t *xl)
 
         pthread_mutex_init (&new->lock, NULL);
 
+        ret = 0;
+out:
+        if (ret) {
+                if (new) {
+                        if (new->inode_hash)
+                                GF_FREE (new->inode_hash);
+                        if (new->name_hash)
+                                GF_FREE (new->name_hash);
+                        if (new->dentry_pool)
+                                mem_pool_destroy (new->dentry_pool);
+                        if (new->inode_pool)
+                                mem_pool_destroy (new->inode_pool);
+                        GF_FREE (new);
+                        new = NULL;
+                }
+        }
+
         return new;
 }
 
@@ -1308,45 +1415,47 @@ out:
 
 
 int
-__inode_ctx_put2 (inode_t *inode, xlator_t *xlator, uint64_t value1,
-                  uint64_t value2)
+__inode_ctx_set2 (inode_t *inode, xlator_t *xlator, uint64_t *value1_p,
+                  uint64_t *value2_p)
 {
         int ret = 0;
         int index = 0;
-        int put_idx = -1;
+        int set_idx = -1;
 
         if (!inode || !xlator)
                 return -1;
 
         for (index = 0; index < xlator->graph->xl_count; index++) {
                 if (!inode->_ctx[index].xl_key) {
-                        if (put_idx == -1)
-                                put_idx = index;
+                        if (set_idx == -1)
+                                set_idx = index;
                         /* dont break, to check if key already exists
                            further on */
                 }
                 if (inode->_ctx[index].xl_key == xlator) {
-                        put_idx = index;
+                        set_idx = index;
                         break;
                 }
         }
 
-        if (put_idx == -1) {
+        if (set_idx == -1) {
                 ret = -1;
                 goto out;;
         }
 
-        inode->_ctx[put_idx].xl_key = xlator;
-        inode->_ctx[put_idx].value1 = value1;
-        inode->_ctx[put_idx].value2 = value2;
+        inode->_ctx[set_idx].xl_key = xlator;
+        if (value1_p)
+                inode->_ctx[set_idx].value1 = *value1_p;
+        if (value2_p)
+                inode->_ctx[set_idx].value2 = *value2_p;
 out:
         return ret;
 }
 
 
 int
-inode_ctx_put2 (inode_t *inode, xlator_t *xlator, uint64_t value1,
-                uint64_t value2)
+inode_ctx_set2 (inode_t *inode, xlator_t *xlator, uint64_t *value1_p,
+                uint64_t *value2_p)
 {
         int ret = 0;
 
@@ -1355,7 +1464,7 @@ inode_ctx_put2 (inode_t *inode, xlator_t *xlator, uint64_t value1,
 
         LOCK (&inode->lock);
         {
-                ret = __inode_ctx_put2 (inode, xlator, value1, value2);
+                ret = __inode_ctx_set2 (inode, xlator, value1_p, value2_p);
         }
         UNLOCK (&inode->lock);
 
@@ -1451,54 +1560,14 @@ unlock:
 }
 
 
-int
-__inode_ctx_put (inode_t *inode, xlator_t *key, uint64_t value)
-{
-        return __inode_ctx_put2 (inode, key, value, 0);
-}
-
-
-int
-inode_ctx_put (inode_t *inode, xlator_t *key, uint64_t value)
-{
-        return inode_ctx_put2 (inode, key, value, 0);
-}
-
-
-int
-__inode_ctx_get (inode_t *inode, xlator_t *key, uint64_t *value)
-{
-        return __inode_ctx_get2 (inode, key, value, 0);
-}
-
-
-int
-inode_ctx_get (inode_t *inode, xlator_t *key, uint64_t *value)
-{
-        return inode_ctx_get2 (inode, key, value, 0);
-}
-
-
-int
-inode_ctx_del (inode_t *inode, xlator_t *key, uint64_t *value)
-{
-        return inode_ctx_del2 (inode, key, value, 0);
-}
-
-
 void
 inode_dump (inode_t *inode, char *prefix)
 {
-        char               key[GF_DUMP_MAX_BUF_LEN];
         int                ret       = -1;
         xlator_t          *xl        = NULL;
         int                i         = 0;
         fd_t              *fd        = NULL;
         struct _inode_ctx *inode_ctx = NULL;
-        struct  fd_wrapper {
-                fd_t *fd;
-                struct list_head next;
-        } *fd_wrapper, *tmp;
         struct list_head fd_list;
 
         if (!inode)
@@ -1507,24 +1576,15 @@ inode_dump (inode_t *inode, char *prefix)
         INIT_LIST_HEAD (&fd_list);
 
         ret = TRY_LOCK(&inode->lock);
-
         if (ret != 0) {
-                gf_log ("", GF_LOG_WARNING, "Unable to dump inode"
-                        " errno: %s", strerror (errno));
                 return;
         }
 
         {
-                gf_proc_dump_build_key(key, prefix, "gfid");
-                gf_proc_dump_write(key, "%s", uuid_utoa (inode->gfid));
-                gf_proc_dump_build_key(key, prefix, "nlookup");
-                gf_proc_dump_write(key, "%ld", inode->nlookup);
-                gf_proc_dump_build_key(key, prefix, "ref");
-                gf_proc_dump_write(key, "%u", inode->ref);
-                gf_proc_dump_build_key(key, prefix, "ino");
-                gf_proc_dump_write(key, "%ld", inode->ino);
-                gf_proc_dump_build_key(key, prefix, "ia_type");
-                gf_proc_dump_write(key, "%d", inode->ia_type);
+                gf_proc_dump_write("gfid", "%s", uuid_utoa (inode->gfid));
+                gf_proc_dump_write("nlookup", "%ld", inode->nlookup);
+                gf_proc_dump_write("ref", "%u", inode->ref);
+                gf_proc_dump_write("ia_type", "%d", inode->ia_type);
                 if (inode->_ctx) {
                         inode_ctx = GF_CALLOC (inode->table->xl->graph->xl_count,
                                                sizeof (*inode_ctx),
@@ -1538,21 +1598,12 @@ inode_dump (inode_t *inode, char *prefix)
                         }
                 }
 
-                if (list_empty (&inode->fd_list)) {
-                        goto unlock;
-                }
+		if (dump_options.xl_options.dump_fdctx != _gf_true)
+			goto unlock;
+
 
                 list_for_each_entry (fd, &inode->fd_list, inode_list) {
-                        fd_wrapper = GF_CALLOC (1, sizeof (*fd_wrapper),
-                                                gf_common_mt_char);
-                        if (fd_wrapper == NULL) {
-                                goto unlock;
-                        }
-
-                        INIT_LIST_HEAD (&fd_wrapper->next);
-                        list_add_tail (&fd_wrapper->next, &fd_list);
-
-                        fd_wrapper->fd = _fd_ref (fd);
+                        fd_ctx_dump (fd, prefix);
                 }
         }
 unlock:
@@ -1568,18 +1619,6 @@ unlock:
                 }
         }
 
-        if (!list_empty (&fd_list)
-            && (dump_options.xl_options.dump_fdctx == _gf_true)) {
-                list_for_each_entry_safe (fd_wrapper, tmp, &fd_list,
-                                          next) {
-                        list_del (&fd_wrapper->next);
-                        fd_ctx_dump (fd_wrapper->fd, prefix);
-
-                        fd_unref (fd_wrapper->fd);
-                        GF_FREE (fd_wrapper);
-                }
-        }
-
         if (inode_ctx != NULL) {
                 GF_FREE (inode_ctx);
         }
@@ -1601,8 +1640,6 @@ inode_table_dump (inode_table_t *itable, char *prefix)
         ret = pthread_mutex_trylock(&itable->lock);
 
         if (ret != 0) {
-                gf_log("", GF_LOG_WARNING, "Unable to dump inode table"
-                       " errno: %s", strerror (errno));
                 return;
         }
 
@@ -1626,3 +1663,99 @@ inode_table_dump (inode_table_t *itable, char *prefix)
 
         pthread_mutex_unlock(&itable->lock);
 }
+
+void
+inode_dump_to_dict (inode_t *inode, char *prefix, dict_t *dict)
+{
+        int             ret = -1;
+        char            key[GF_DUMP_MAX_BUF_LEN] = {0,};
+
+        ret = TRY_LOCK (&inode->lock);
+        if (ret)
+                return;
+
+        memset (key, 0, sizeof (key));
+        snprintf (key, sizeof (key), "%s.gfid", prefix);
+        ret = dict_set_dynstr (dict, key, gf_strdup (uuid_utoa (inode->gfid)));
+        if (ret)
+                goto out;
+
+        memset (key, 0, sizeof (key));
+        snprintf (key, sizeof (key), "%s.nlookup", prefix);
+        ret = dict_set_uint64 (dict, key, inode->nlookup);
+        if (ret)
+                goto out;
+
+        memset (key, 0, sizeof (key));
+        snprintf (key, sizeof (key), "%s.ref", prefix);
+        ret = dict_set_uint32 (dict, key, inode->ref);
+        if (ret)
+                goto out;
+
+        memset (key, 0, sizeof (key));
+        snprintf (key, sizeof (key), "%s.ia_type", prefix);
+        ret = dict_set_int32 (dict, key, inode->ia_type);
+
+out:
+        UNLOCK (&inode->lock);
+        return;
+}
+
+void
+inode_table_dump_to_dict (inode_table_t *itable, char *prefix, dict_t *dict)
+{
+        char            key[GF_DUMP_MAX_BUF_LEN] = {0,};
+        int             ret = 0;
+        inode_t         *inode = NULL;
+        int             count = 0;
+
+        ret = pthread_mutex_trylock (&itable->lock);
+        if (ret)
+                return;
+
+        memset (key, 0, sizeof (key));
+        snprintf (key, sizeof (key), "%s.itable.active_size", prefix);
+        ret = dict_set_uint32 (dict, key, itable->active_size);
+        if (ret)
+                goto out;
+
+        memset (key, 0, sizeof (key));
+        snprintf (key, sizeof (key), "%s.itable.lru_size", prefix);
+        ret = dict_set_uint32 (dict, key, itable->lru_size);
+        if (ret)
+                goto out;
+
+        memset (key, 0, sizeof (key));
+        snprintf (key, sizeof (key), "%s.itable.purge_size", prefix);
+        ret = dict_set_uint32 (dict, key, itable->purge_size);
+        if (ret)
+                goto out;
+
+        list_for_each_entry (inode, &itable->active, list) {
+                memset (key, 0, sizeof (key));
+                snprintf (key, sizeof (key), "%s.itable.active%d", prefix,
+                          count++);
+                inode_dump_to_dict (inode, key, dict);
+        }
+        count = 0;
+
+        list_for_each_entry (inode, &itable->lru, list) {
+                memset (key, 0, sizeof (key));
+                snprintf (key, sizeof (key), "%s.itable.lru%d", prefix,
+                          count++);
+                inode_dump_to_dict (inode, key, dict);
+        }
+        count = 0;
+
+        list_for_each_entry (inode, &itable->purge, list) {
+                memset (key, 0, sizeof (key));
+                snprintf (key, sizeof (key), "%s.itable.purge%d", prefix,
+                          count++);
+                inode_dump_to_dict (inode, key, dict);
+        }
+
+out:
+        pthread_mutex_unlock (&itable->lock);
+
+        return;
+}
