@@ -1,20 +1,11 @@
 /*
-  Copyright (c) 2006-2011 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+  This file is licensed to you under your choice of the GNU Lesser
+  General Public License, version 3 or any later version (LGPLv3 or
+  later), or the GNU General Public License, version 2 (GPLv2), in all
+  cases as published by the Free Software Foundation.
 */
 
 #ifndef _CONFIG_H
@@ -50,9 +41,10 @@ static uint8_t          logrotate = 0;
 static FILE            *logfile = NULL;
 static gf_loglevel_t    loglevel = GF_LOG_INFO;
 static int              gf_log_syslog = 1;
+static gf_loglevel_t    sys_log_level = GF_LOG_CRITICAL;
 
 char                    gf_log_xl_log_set;
-gf_loglevel_t           gf_log_loglevel; /* extern'd */
+gf_loglevel_t           gf_log_loglevel = GF_LOG_INFO; /* extern'd */
 FILE                   *gf_log_logfile;
 
 static char            *cmd_log_filename = NULL;
@@ -130,11 +122,19 @@ gf_log_globals_init (void)
 int
 gf_log_init (const char *file)
 {
+        int     fd = -1;
+
         if (!file){
                 fprintf (stderr, "ERROR: no filename specified\n");
                 return -1;
         }
 
+        if (strcmp (file, "-") == 0) {
+                gf_log_logfile = stderr;
+
+                return 0;
+        }
+
         filename = gf_strdup (file);
         if (!filename) {
                 fprintf (stderr, "ERROR: updating log-filename failed: %s\n",
@@ -142,6 +142,14 @@ gf_log_init (const char *file)
                 return -1;
         }
 
+        fd = open (file, O_CREAT | O_RDONLY, S_IRUSR | S_IWUSR);
+        if (fd < 0) {
+                fprintf (stderr, "ERROR: failed to create logfile \"%s\" (%s)\n",
+                         file, strerror (errno));
+                return -1;
+        }
+        close (fd);
+
         logfile = fopen (file, "a");
         if (!logfile){
                 fprintf (stderr, "ERROR: failed to open logfile \"%s\" (%s)\n",
@@ -186,6 +194,12 @@ gf_log_cleanup (void)
         pthread_mutex_destroy (&logfile_mutex);
 }
 
+void
+set_sys_log_level (gf_loglevel_t level)
+{
+        sys_log_level = level;
+}
+
 int
 _gf_log_nomem (const char *domain, const char *file,
                const char *function, int line, gf_loglevel_t level,
@@ -196,19 +210,18 @@ _gf_log_nomem (const char *domain, const char *file,
         xlator_t       *this            = NULL;
         struct timeval  tv              = {0,};
         int             ret             = 0;
-        gf_loglevel_t   xlator_loglevel = 0;
         char            msg[8092];
         char            timestr[256];
         char            callstr[4096];
 
         this = THIS;
 
-        xlator_loglevel = this->loglevel;
-        if (xlator_loglevel == 0)
-                xlator_loglevel = loglevel;
-
-        if (level > xlator_loglevel)
-                goto out;
+        if (gf_log_xl_log_set) {
+                if (this->loglevel && (level > this->loglevel))
+                        goto out;
+                else if (level > gf_log_loglevel)
+                        goto out;
+        }
 
         static char *level_strings[] = {"",  /* NONE */
                                         "M", /* EMERGENCY */
@@ -292,7 +305,7 @@ _gf_log_nomem (const char *domain, const char *file,
 #ifdef GF_LINUX_HOST_OS
                 /* We want only serious log in 'syslog', not our debug
                    and trace logs */
-                if (gf_log_syslog && level && (level <= GF_LOG_ERROR))
+                if (gf_log_syslog && level && (level <= sys_log_level))
                         syslog ((level-1), "%s\n", msg);
 #endif
         }
@@ -318,17 +331,16 @@ _gf_log_callingfn (const char *domain, const char *file, const char *function,
         struct timeval  tv              = {0,};
         size_t          len             = 0;
         int             ret             = 0;
-        gf_loglevel_t   xlator_loglevel = 0;
         va_list         ap;
 
         this = THIS;
 
-        xlator_loglevel = this->loglevel;
-        if (xlator_loglevel == 0)
-                xlator_loglevel = loglevel;
-
-        if (level > xlator_loglevel)
-                goto out;
+        if (gf_log_xl_log_set) {
+                if (this->loglevel && (level > this->loglevel))
+                        goto out;
+                else if (level > gf_log_loglevel)
+                        goto out;
+        }
 
         static char *level_strings[] = {"",  /* NONE */
                                         "M", /* EMERGENCY */
@@ -426,7 +438,7 @@ _gf_log_callingfn (const char *domain, const char *file, const char *function,
 #ifdef GF_LINUX_HOST_OS
                 /* We want only serious log in 'syslog', not our debug
                    and trace logs */
-                if (gf_log_syslog && level && (level <= GF_LOG_CRITICAL))
+                if (gf_log_syslog && level && (level <= sys_log_level))
                         syslog ((level-1), "%s\n", msg);
 #endif
         }
@@ -464,17 +476,17 @@ _gf_log (const char *domain, const char *file, const char *function, int line,
         char        *msg  = NULL;
         size_t       len  = 0;
         int          ret  = 0;
+        int          fd   = -1;
         xlator_t    *this = NULL;
-        gf_loglevel_t xlator_loglevel = 0;
 
         this = THIS;
 
-        xlator_loglevel = this->loglevel;
-        if (xlator_loglevel == 0)
-                xlator_loglevel = loglevel;
-
-        if (level > xlator_loglevel)
-                goto out;
+        if (gf_log_xl_log_set) {
+                if (this->loglevel && (level > this->loglevel))
+                        goto out;
+                else if (level > gf_log_loglevel)
+                        goto out;
+        }
 
         static char *level_strings[] = {"",  /* NONE */
                                         "M", /* EMERGENCY */
@@ -499,6 +511,14 @@ _gf_log (const char *domain, const char *file, const char *function, int line,
         if (logrotate) {
                 logrotate = 0;
 
+                fd = open (filename, O_CREAT | O_RDONLY, S_IRUSR | S_IWUSR);
+                if (fd < 0) {
+                        gf_log ("logrotate", GF_LOG_ERROR,
+                                "%s", strerror (errno));
+                        return -1;
+                }
+                close (fd);
+
                 new_logfile = fopen (filename, "a");
                 if (!new_logfile) {
                         gf_log ("logrotate", GF_LOG_CRITICAL,
@@ -565,7 +585,7 @@ log:
 #ifdef GF_LINUX_HOST_OS
                 /* We want only serious log in 'syslog', not our debug
                    and trace logs */
-                if (gf_log_syslog && level && (level <= GF_LOG_CRITICAL))
+                if (gf_log_syslog && level && (level <= sys_log_level))
                         syslog ((level-1), "%s\n", msg);
 #endif
         }
@@ -587,26 +607,53 @@ out:
         return (0);
 }
 
+int
+gf_log_eh (void *data)
+{
+        int    ret = -1;
+
+        ret = eh_save_history (THIS->history, data);
+
+        return ret;
+}
 
 int
 gf_cmd_log_init (const char *filename)
 {
+        int         fd   = -1;
+        xlator_t   *this = NULL;
+
+        this = THIS;
+
         if (!filename){
-                gf_log ("glusterd", GF_LOG_CRITICAL, "gf_cmd_log_init: no "
+                gf_log (this->name, GF_LOG_CRITICAL, "gf_cmd_log_init: no "
                         "filename specified\n");
                 return -1;
         }
 
         cmd_log_filename = gf_strdup (filename);
         if (!cmd_log_filename) {
-                gf_log ("glusterd", GF_LOG_CRITICAL, "gf_cmd_log_init: strdup"
-                        " error\n");
+                gf_log (this->name, GF_LOG_CRITICAL,
+                        "gf_cmd_log_init: strdup error\n");
                 return -1;
         }
+        /* close and reopen cmdlogfile for log rotate*/
+        if (cmdlogfile) {
+                fclose (cmdlogfile);
+                cmdlogfile = NULL;
+        }
+
+        fd = open (cmd_log_filename, O_CREAT | O_RDONLY, S_IRUSR | S_IWUSR);
+        if (fd < 0) {
+                gf_log (this->name, GF_LOG_CRITICAL,
+                        "%s", strerror (errno));
+                return -1;
+        }
+        close (fd);
 
         cmdlogfile = fopen (cmd_log_filename, "a");
         if (!cmdlogfile){
-                gf_log ("glusterd", GF_LOG_CRITICAL,
+                gf_log (this->name, GF_LOG_CRITICAL,
                         "gf_cmd_log_init: failed to open logfile \"%s\" "
                         "(%s)\n", cmd_log_filename, strerror (errno));
                 return -1;
