@@ -1,20 +1,11 @@
 /*
-   Copyright (c) 2006-2011 Gluster, Inc. <http://www.gluster.com>
-   This file is part of GlusterFS.
+  Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
+  This file is part of GlusterFS.
 
-   GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
-   by the Free Software Foundation; either version 3 of the License,
-   or (at your option) any later version.
-
-   GlusterFS is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program.  If not, see
-   <http://www.gnu.org/licenses/>.
+  This file is licensed to you under your choice of the GNU Lesser
+  General Public License, version 3 or any later version (LGPLv3 or
+  later), or the GNU General Public License, version 2 (GPLv2), in all
+  cases as published by the Free Software Foundation.
 */
 
 #ifndef __LOGGING_H__
@@ -33,12 +24,12 @@
 #define GF_PRI_FSBLK       "u"
 #define GF_PRI_DEV         PRId32
 #define GF_PRI_NLINK       PRIu16
-#define GF_PRI_SUSECONDS   PRId32
+#define GF_PRI_SUSECONDS   "06d"
 #else
 #define GF_PRI_FSBLK       PRIu64
 #define GF_PRI_DEV         PRIu64
 #define GF_PRI_NLINK       PRIu32
-#define GF_PRI_SUSECONDS   "ld"
+#define GF_PRI_SUSECONDS   "06ld"
 #endif
 #define GF_PRI_BLKSIZE     PRId32
 #define GF_PRI_SIZET       "zu"
@@ -108,15 +99,20 @@ extern char gf_log_xl_log_set;
 
 
 void gf_log_logrotate (int signum);
-
+int
+gf_log_eh (void *data);
 void gf_log_globals_init (void);
 int gf_log_init (const char *filename);
 void gf_log_cleanup (void);
 
-int _gf_log (const char *domain, const char *file, const char *function,
-             int32_t line, gf_loglevel_t level, const char *fmt, ...);
-int _gf_log_callingfn (const char *domain, const char *file, const char *function,
-                       int32_t line, gf_loglevel_t level, const char *fmt, ...);
+int _gf_log (const char *domain, const char *file,
+             const char *function, int32_t line, gf_loglevel_t level,
+             const char *fmt, ...)
+             __attribute__ ((__format__ (__printf__, 6, 7)));
+int _gf_log_callingfn (const char *domain, const char *file, 
+                       const char *function, int32_t line, gf_loglevel_t level,
+                       const char *fmt, ...)
+                       __attribute__ ((__format__ (__printf__, 6, 7)));
 
 int _gf_log_nomem (const char *domain, const char *file,
                    const char *function, int line, gf_loglevel_t level,
@@ -143,7 +139,10 @@ void gf_log_set_xl_loglevel (void *xl, gf_loglevel_t level);
 #define GF_ERROR(xl, format, args...)                           \
         gf_log ((xl)->name, GF_LOG_ERROR, format, ##args)
 
-int gf_cmd_log (const char *domain, const char *fmt, ...);
+int gf_cmd_log (const char *domain, const char *fmt, ...)
+                __attribute__ ((__format__ (__printf__, 2, 3)));
 
 int gf_cmd_log_init (const char *filename);
+
+void set_sys_log_level (gf_loglevel_t level);
 #endif /* __LOGGING_H__ */
