@@ -1,20 +1,11 @@
 /*
-   Copyright (c) 2008-2011 Gluster, Inc. <http://www.gluster.com>
-   This file is part of GlusterFS.
+  Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
+  This file is part of GlusterFS.
 
-   GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
-   by the Free Software Foundation; either version 3 of the License,
-   or (at your option) any later version.
-
-   GlusterFS is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program.  If not, see
-   <http://www.gnu.org/licenses/>.
+  This file is licensed to you under your choice of the GNU Lesser
+  General Public License, version 3 or any later version (LGPLv3 or
+  later), or the GNU General Public License, version 2 (GPLv2), in all
+  cases as published by the Free Software Foundation.
 */
 
 #ifndef _MEM_POOL_H_
@@ -39,6 +30,7 @@ struct mem_acct_rec {
         size_t          size;
         size_t          max_size;
         uint32_t        num_allocs;
+        uint32_t        total_allocs;
         uint32_t        max_num_allocs;
         gf_lock_t       lock;
 };
@@ -115,7 +107,7 @@ void* __gf_default_realloc (void *oldptr, size_t size)
 
 #define GF_REALLOC(ptr, size)  __gf_realloc (ptr, size)
 
-#define GF_FREE(free_ptr) __gf_free (free_ptr);
+#define GF_FREE(free_ptr) __gf_free (free_ptr)
 
 static inline
 char * gf_strdup (const char *src)
@@ -145,14 +137,21 @@ struct mem_pool {
         void             *pool;
         void             *pool_end;
         int               real_sizeof_type;
+        uint64_t          alloc_count;
+        uint64_t          pool_misses;
+        int               max_alloc;
+        int               curr_stdalloc;
+        int               max_stdalloc;
+        char             *name;
+        struct list_head  global_list;
 };
 
 struct mem_pool *
-mem_pool_new_fn (unsigned long sizeof_type, unsigned long count);
+mem_pool_new_fn (unsigned long sizeof_type, unsigned long count, char *name);
 
-#define mem_pool_new(type,count) mem_pool_new_fn (sizeof(type), count)
+#define mem_pool_new(type,count) mem_pool_new_fn (sizeof(type), count, #type)
 
-void mem_put (struct mem_pool *pool, void *ptr);
+void mem_put (void *ptr);
 void *mem_get (struct mem_pool *pool);
 void *mem_get0 (struct mem_pool *pool);
 
