@@ -1,20 +1,11 @@
 /*
-  Copyright (c) 2006-2011 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+  This file is licensed to you under your choice of the GNU Lesser
+  General Public License, version 3 or any later version (LGPLv3 or
+  later), or the GNU General Public License, version 2 (GPLv2), in all
+  cases as published by the Free Software Foundation.
 */
 
 /*
@@ -44,9 +35,10 @@ typedef struct _call_pool_t call_pool_t;
 #include "list.h"
 #include "common-utils.h"
 #include "globals.h"
+#include "lkowner.h"
 
 #define NFS_PID 1
-#define SELF_HEAL_PID -1
+#define LOW_PRIO_PROC_PID -1
 typedef int32_t (*ret_fn_t) (call_frame_t *frame,
                              call_frame_t *prev_frame,
                              xlator_t *this,
@@ -106,9 +98,9 @@ struct _call_stack_t {
         uid_t                         uid;
         gid_t                         gid;
         pid_t                         pid;
-        uint32_t                      ngrps;
-        uint32_t                      groups[GF_REQUEST_MAXGROUPS];
-        uint64_t                      lk_owner;
+        uint16_t                      ngrps;
+        uint32_t                      groups[GF_MAX_AUX_GROUPS];
+        gf_lkowner_t                  lk_owner;
 
         call_frame_t                  frames;
 
@@ -151,10 +143,10 @@ FRAME_DESTROY (call_frame_t *frame)
         }
 
         LOCK_DESTROY (&frame->lock);
-        mem_put (frame->root->pool->frame_mem_pool, frame);
+        mem_put (frame);
 
         if (local)
-                GF_FREE (local);
+                mem_put (local);
 }
 
 
@@ -181,12 +173,29 @@ STACK_DESTROY (call_stack_t *stack)
         while (stack->frames.next) {
                 FRAME_DESTROY (stack->frames.next);
         }
-        mem_put (stack->pool->stack_mem_pool, stack);
+        mem_put (stack);
 
         if (local)
-                GF_FREE (local);
+                mem_put (local);
 }
 
+static inline void
+STACK_RESET (call_stack_t *stack)
+{
+        void *local = NULL;
+
+        if (stack->frames.local) {
+                local = stack->frames.local;
+                stack->frames.local = NULL;
+        }
+
+        while (stack->frames.next) {
+                FRAME_DESTROY (stack->frames.next);
+        }
+
+        if (local)
+                mem_put (local);
+}
 
 #define cbk(x) cbk_##x
 
@@ -361,7 +370,7 @@ copy_frame (call_frame_t *frame)
         newstack->op  = oldstack->op;
         newstack->type = oldstack->type;
         memcpy (newstack->groups, oldstack->groups,
-                sizeof (uint32_t) * GF_REQUEST_MAXGROUPS);
+                sizeof (gid_t) * GF_MAX_AUX_GROUPS);
         newstack->unique = oldstack->unique;
 
         newstack->frames.this = frame->this;
@@ -414,6 +423,7 @@ create_frame (xlator_t *xl, call_pool_t *pool)
 }
 
 void gf_proc_dump_pending_frames(call_pool_t *call_pool);
-
+void gf_proc_dump_pending_frames_to_dict (call_pool_t *call_pool,
+                                          dict_t *dict);
 gf_boolean_t __is_fuse_call (call_frame_t *frame);
 #endif /* _STACK_H */
