@@ -1,20 +1,11 @@
 /*
-  Copyright (c) 2010-2011 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+  This file is licensed to you under your choice of the GNU Lesser
+  General Public License, version 3 or any later version (LGPLv3 or
+  later), or the GNU General Public License, version 2 (GPLv2), in all
+  cases as published by the Free Software Foundation.
 */
 
 #include <stdio.h>
@@ -23,7 +14,6 @@
 #include <ctype.h>
 
 #include "common-utils.h"
-#include "trie-mem-types.h"
 #include "trie.h"
 
 #define DISTANCE_EDIT 1
@@ -53,7 +43,7 @@ trie_new ()
 {
         trie_t *trie = NULL;
 
-        trie = GF_CALLOC (1, sizeof (*trie),  gf_trie_mt_trie);
+        trie = GF_CALLOC (1, sizeof (*trie),  gf_common_mt_trie_trie);
         if (!trie)
                 return NULL;
 
@@ -70,7 +60,8 @@ trie_subnode (trienode_t *node, int id)
 
         subnode = node->subnodes[id];
         if (!subnode) {
-                subnode = GF_CALLOC (1, sizeof (*subnode), gf_trie_mt_node);
+                subnode = GF_CALLOC (1, sizeof (*subnode),
+                                     gf_common_mt_trie_node);
                 if (!subnode)
                         return NULL;
 
@@ -202,7 +193,7 @@ trienode_get_word (trienode_t *node, char **bufp)
 {
         char *buf = NULL;
 
-        buf = GF_CALLOC (1, node->depth + 1, gf_trie_mt_buf);
+        buf = GF_CALLOC (1, node->depth + 1, gf_common_mt_trie_buf);
         if (!buf)
                 return -1;
         *bufp = buf;
@@ -226,7 +217,8 @@ calc_dist (trienode_t *node, void *data)
 
         word = data;
 
-        node->data = GF_CALLOC (node->trie->len, sizeof (int), gf_trie_mt_data);
+        node->data = GF_CALLOC (node->trie->len, sizeof (int),
+                                gf_common_mt_trie_data);
         if (!node->data)
                 return -1;
         row = node->data;
