@@ -1,20 +1,11 @@
 /*
-  Copyright (c) 2010-2011 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+  This file is licensed to you under your choice of the GNU Lesser
+  General Public License, version 3 or any later version (LGPLv3 or
+  later), or the GNU General Public License, version 2 (GPLv2), in all
+  cases as published by the Free Software Foundation.
 */
 
 
@@ -23,7 +14,7 @@
 #include "config.h"
 #endif
 
-#define RPC_CLNT_DEFAULT_REQUEST_COUNT 4096
+#define RPC_CLNT_DEFAULT_REQUEST_COUNT 512
 
 #include "rpc-clnt.h"
 #include "byte-order.h"
@@ -32,6 +23,7 @@
 #include "protocol-common.h"
 #include "mem-pool.h"
 #include "xdr-rpc.h"
+#include "rpc-common-xdr.h"
 
 void
 rpc_clnt_reply_deinit (struct rpc_req *req, struct mem_pool *pool);
@@ -69,6 +61,21 @@ __saved_frames_get_timedout (struct saved_frames *frames, uint32_t timeout,
 	return bailout_frame;
 }
 
+static int
+_is_lock_fop (struct saved_frame *sframe)
+{
+        int     fop     = 0;
+
+        if (SFRAME_GET_PROGNUM (sframe) == GLUSTER3_1_FOP_PROGRAM &&
+            SFRAME_GET_PROGVER (sframe) == GLUSTER3_1_FOP_VERSION)
+                fop = SFRAME_GET_PROCNUM (sframe);
+
+        return ((fop == GFS3_OP_LK) ||
+                (fop == GFS3_OP_INODELK) ||
+                (fop == GFS3_OP_FINODELK) ||
+                (fop == GFS3_OP_ENTRYLK) ||
+                (fop == GFS3_OP_FENTRYLK));
+}
 
 struct saved_frame *
 __saved_frames_put (struct saved_frames *frames, void *frame,
@@ -90,7 +97,11 @@ __saved_frames_put (struct saved_frames *frames, void *frame,
         saved_frame->rpcreq       = rpcreq;
 	gettimeofday (&saved_frame->saved_at, NULL);
 
-	list_add_tail (&saved_frame->list, &frames->sf.list);
+        if (_is_lock_fop (saved_frame))
+                list_add_tail (&saved_frame->list, &frames->lk_sf.list);
+        else
+                list_add_tail (&saved_frame->list, &frames->sf.list);
+
 	frames->count++;
 
 out:
@@ -117,7 +128,7 @@ saved_frames_delete (struct saved_frame *saved_frame,
                                        conn->rpc_clnt->reqpool);
         }
 
-        mem_put (conn->rpc_clnt->saved_frames_pool, saved_frame);
+        mem_put (saved_frame);
 out:
         return;
 }
@@ -203,7 +214,7 @@ call_bail (void *data)
                 rpc_clnt_reply_deinit (trav->rpcreq, clnt->reqpool);
                 clnt = rpc_clnt_unref (clnt);
                 list_del_init (&trav->list);
-                mem_put (conn->rpc_clnt->saved_frames_pool, trav);
+                mem_put (trav);
         }
 out:
         return;
@@ -254,6 +265,7 @@ saved_frames_new (void)
 	}
 
 	INIT_LIST_HEAD (&saved_frames->sf.list);
+	INIT_LIST_HEAD (&saved_frames->lk_sf.list);
 
 	return saved_frames;
 }
@@ -275,7 +287,15 @@ __saved_frame_copy (struct saved_frames *frames, int64_t callid,
 		if (tmp->rpcreq->xid == callid) {
 			*saved_frame = *tmp;
                         ret = 0;
-			break;
+			goto out;
+		}
+	}
+
+	list_for_each_entry (tmp, &frames->lk_sf.list, list) {
+		if (tmp->rpcreq->xid == callid) {
+			*saved_frame = *tmp;
+                        ret = 0;
+			goto out;
 		}
 	}
 
@@ -295,10 +315,20 @@ __saved_frame_get (struct saved_frames *frames, int64_t callid)
 			list_del_init (&tmp->list);
 			frames->count--;
 			saved_frame = tmp;
-			break;
+			goto out;
 		}
 	}
 
+	list_for_each_entry (tmp, &frames->lk_sf.list, list) {
+		if (tmp->rpcreq->xid == callid) {
+			list_del_init (&tmp->list);
+			frames->count--;
+			saved_frame = tmp;
+			goto out;
+		}
+	}
+
+out:
 	if (saved_frame) {
                 THIS  = saved_frame->capital_this;
         }
@@ -313,12 +343,13 @@ saved_frames_unwind (struct saved_frames *saved_frames)
         struct rpc_clnt      *clnt = NULL;
 	struct saved_frame   *trav = NULL;
 	struct saved_frame   *tmp = NULL;
-        struct mem_pool      *saved_frames_pool = NULL;
         struct tm            *frame_sent_tm = NULL;
         char                 timestr[256] = {0,};
 
         struct iovec          iov = {0,};
 
+        list_splice_init (&saved_frames->lk_sf.list, &saved_frames->sf.list);
+
 	list_for_each_entry_safe (trav, tmp, &saved_frames->sf.list, list) {
                 frame_sent_tm = localtime (&trav->saved_at.tv_sec);
                 strftime (timestr, sizeof(timestr), "%Y-%m-%d %H:%M:%S",
@@ -333,26 +364,25 @@ saved_frames_unwind (struct saved_frames *saved_frames)
                 gf_log_callingfn (trav->rpcreq->conn->trans->name,
                                   GF_LOG_ERROR,
                                   "forced unwinding frame type(%s) op(%s(%d)) "
-                                  "called at %s",
+                                  "called at %s (xid=0x%ux)",
                                   trav->rpcreq->prog->progname,
                                   ((trav->rpcreq->prog->procnames) ?
                                    trav->rpcreq->prog->procnames[trav->rpcreq->procnum]
                                    : "--"),
-                                  trav->rpcreq->procnum, timestr);
+                                  trav->rpcreq->procnum, timestr,
+                                  trav->rpcreq->xid);
 		saved_frames->count--;
 
                 clnt = rpc_clnt_ref (trav->rpcreq->conn->rpc_clnt);
                 trav->rpcreq->rpc_status = -1;
                 trav->rpcreq->cbkfn (trav->rpcreq, &iov, 1, trav->frame);
 
-                saved_frames_pool
-                        = trav->rpcreq->conn->rpc_clnt->saved_frames_pool;
                 rpc_clnt_reply_deinit (trav->rpcreq,
                                        trav->rpcreq->conn->rpc_clnt->reqpool);
 
                 clnt = rpc_clnt_unref (clnt);
 		list_del_init (&trav->list);
-                mem_put (saved_frames_pool, trav);
+                mem_put (trav);
 	}
 }
 
@@ -397,7 +427,16 @@ rpc_clnt_reconnect (void *trans_ptr)
 
                         gf_log (trans->name, GF_LOG_TRACE,
                                 "attempting reconnect");
-                        ret = rpc_transport_connect (trans, conn->config.remote_port);
+                        ret = rpc_transport_connect (trans,
+                                                     conn->config.remote_port);
+                        /* Every time there is a disconnection, processes
+                           should try to connect to 'glusterd' (ie, default
+                           port) or whichever port given as 'option remote-port'
+                           in volume file. */
+                        /* Below code makes sure the (re-)configured port lasts
+                           for just one successful attempt */
+                        if (!ret)
+                                conn->config.remote_port = 0;
 
                         conn->reconnect =
                                 gf_timer_call_after (clnt->ctx, tv,
@@ -432,7 +471,8 @@ rpc_clnt_fill_request_info (struct rpc_clnt *clnt, rpc_request_info_t *info)
         pthread_mutex_unlock (&clnt->conn.lock);
 
         if (ret == -1) {
-                gf_log (clnt->conn.trans->name, GF_LOG_CRITICAL, "cannot lookup the saved "
+                gf_log (clnt->conn.trans->name, GF_LOG_CRITICAL,
+                        "cannot lookup the saved "
                         "frame corresponding to xid (%d)", info->xid);
                 goto out;
         }
@@ -506,6 +546,12 @@ rpc_clnt_connection_cleanup (rpc_clnt_connection_t *conn)
                 }
 
                 conn->connected = 0;
+
+                if (conn->ping_timer) {
+                        gf_timer_call_cancel (clnt->ctx, conn->ping_timer);
+                        conn->ping_timer = NULL;
+                        conn->ping_started = 0;
+                }
         }
         pthread_mutex_unlock (&conn->lock);
 
@@ -596,7 +642,7 @@ rpc_clnt_reply_deinit (struct rpc_req *req, struct mem_pool *pool)
                 iobref_unref (req->rsp_iobref);
         }
 
-        mem_put (pool, req);
+        mem_put (req);
 out:
         return;
 }
@@ -630,15 +676,14 @@ rpc_clnt_reply_init (rpc_clnt_connection_t *conn, rpc_transport_pollin_t *msg,
                 goto out;
         }
 
-        gf_log (conn->trans->name, GF_LOG_TRACE, "received rpc message (RPC XID: 0x%ux"
+        gf_log (conn->trans->name, GF_LOG_TRACE,
+                "received rpc message (RPC XID: 0x%ux"
                 " Program: %s, ProgVers: %d, Proc: %d) from rpc-transport (%s)",
                 saved_frame->rpcreq->xid,
                 saved_frame->rpcreq->prog->progname,
                 saved_frame->rpcreq->prog->progver,
                 saved_frame->rpcreq->procnum, conn->trans->name);
 
-        req->rpc_status = 0;
-
 out:
         if (ret != 0) {
                 req->rpc_status = -1;
@@ -746,7 +791,7 @@ rpc_clnt_handle_reply (struct rpc_clnt *clnt, rpc_transport_pollin_t *pollin)
 out:
 
         if (saved_frame) {
-                mem_put (conn->rpc_clnt->saved_frames_pool, saved_frame);
+                mem_put (saved_frame);
         }
 
         clnt = rpc_clnt_unref (clnt);
@@ -789,17 +834,16 @@ out:
         return;
 }
 
-
 int
 rpc_clnt_notify (rpc_transport_t *trans, void *mydata,
                  rpc_transport_event_t event, void *data, ...)
 {
-        rpc_clnt_connection_t  *conn     = NULL;
-        struct rpc_clnt        *clnt     = NULL;
-        int                     ret      = -1;
-        rpc_request_info_t     *req_info = NULL;
-        rpc_transport_pollin_t *pollin   = NULL;
-        struct timeval          tv       = {0, };
+        rpc_clnt_connection_t  *conn        = NULL;
+        struct rpc_clnt        *clnt        = NULL;
+        int                     ret         = -1;
+        rpc_request_info_t     *req_info    = NULL;
+        rpc_transport_pollin_t *pollin      = NULL;
+        struct timeval          tv          = {0, };
 
         conn = mydata;
         if (conn == NULL) {
@@ -816,7 +860,8 @@ rpc_clnt_notify (rpc_transport_t *trans, void *mydata,
 
                 pthread_mutex_lock (&conn->lock);
                 {
-                        if (conn->reconnect == NULL) {
+                        if (!conn->rpc_clnt->disabled
+                            && (conn->reconnect == NULL)) {
                                 tv.tv_sec = 10;
 
                                 conn->reconnect =
@@ -828,8 +873,8 @@ rpc_clnt_notify (rpc_transport_t *trans, void *mydata,
                 pthread_mutex_unlock (&conn->lock);
 
                 if (clnt->notifyfn)
-                        ret = clnt->notifyfn (clnt, clnt->mydata, RPC_CLNT_DISCONNECT,
-                                              NULL);
+                        ret = clnt->notifyfn (clnt, clnt->mydata,
+                                              RPC_CLNT_DISCONNECT, NULL);
                 break;
         }
 
@@ -849,6 +894,12 @@ rpc_clnt_notify (rpc_transport_t *trans, void *mydata,
 
         case RPC_TRANSPORT_MSG_RECEIVED:
         {
+                pthread_mutex_lock (&conn->lock);
+                {
+                        gettimeofday (&conn->last_received, NULL);
+                }
+                pthread_mutex_unlock (&conn->lock);
+
                 pollin = data;
                 if (pollin->is_reply)
                         ret = rpc_clnt_handle_reply (clnt, pollin);
@@ -875,7 +926,8 @@ rpc_clnt_notify (rpc_transport_t *trans, void *mydata,
         case RPC_TRANSPORT_CONNECT:
         {
                 if (clnt->notifyfn)
-                        ret = clnt->notifyfn (clnt, clnt->mydata, RPC_CLNT_CONNECT, NULL);
+                        ret = clnt->notifyfn (clnt, clnt->mydata,
+                                              RPC_CLNT_CONNECT, NULL);
                 break;
         }
 
@@ -956,8 +1008,8 @@ out:
 }
 
 struct rpc_clnt *
-rpc_clnt_new (dict_t *options,
-              glusterfs_ctx_t *ctx, char *name)
+rpc_clnt_new (dict_t *options, glusterfs_ctx_t *ctx, char *name,
+              uint32_t reqpool_size)
 {
         int                    ret  = -1;
         struct rpc_clnt       *rpc  = NULL;
@@ -970,8 +1022,10 @@ rpc_clnt_new (dict_t *options,
         pthread_mutex_init (&rpc->lock, NULL);
         rpc->ctx = ctx;
 
-        rpc->reqpool = mem_pool_new (struct rpc_req,
-                                     RPC_CLNT_DEFAULT_REQUEST_COUNT);
+        if (!reqpool_size)
+                reqpool_size = RPC_CLNT_DEFAULT_REQUEST_COUNT;
+
+        rpc->reqpool = mem_pool_new (struct rpc_req, reqpool_size);
         if (rpc->reqpool == NULL) {
                 pthread_mutex_destroy (&rpc->lock);
                 GF_FREE (rpc);
@@ -980,7 +1034,7 @@ rpc_clnt_new (dict_t *options,
         }
 
         rpc->saved_frames_pool = mem_pool_new (struct saved_frame,
-                                              RPC_CLNT_DEFAULT_REQUEST_COUNT);
+                                               reqpool_size);
         if (rpc->saved_frames_pool == NULL) {
                 pthread_mutex_destroy (&rpc->lock);
                 mem_pool_destroy (rpc->reqpool);
@@ -1001,6 +1055,8 @@ rpc_clnt_new (dict_t *options,
                 goto out;
         }
 
+        rpc->auth_null = dict_get_str_boolean (options, "auth-null", 0);
+
         rpc = rpc_clnt_ref (rpc);
         INIT_LIST_HEAD (&rpc->programs);
 
@@ -1036,7 +1092,7 @@ rpc_clnt_register_notify (struct rpc_clnt *rpc, rpc_clnt_notify_t fn,
 }
 
 ssize_t
-xdr_serialize_glusterfs_auth (char *dest, struct auth_glusterfs_parms *au)
+xdr_serialize_glusterfs_auth (char *dest, struct auth_glusterfs_parms_v2 *au)
 {
         ssize_t ret = -1;
         XDR     xdr;
@@ -1044,11 +1100,11 @@ xdr_serialize_glusterfs_auth (char *dest, struct auth_glusterfs_parms *au)
         if ((!dest) || (!au))
                 return -1;
 
-        xdrmem_create (&xdr, dest, 1024,
-                       XDR_ENCODE);
+        xdrmem_create (&xdr, dest, GF_MAX_AUTH_BYTES, XDR_ENCODE);
 
-        if (!xdr_auth_glusterfs_parms (&xdr, au)) {
-                gf_log ("", GF_LOG_WARNING, "failed to encode auth glusterfs elements");
+        if (!xdr_auth_glusterfs_parms_v2 (&xdr, au)) {
+                gf_log (THIS->name, GF_LOG_WARNING,
+                        "failed to encode auth glusterfs elements");
                 ret = -1;
                 goto ret;
         }
@@ -1061,8 +1117,8 @@ ret:
 
 
 int
-rpc_clnt_fill_request (int prognum, int progver, int procnum, int payload,
-                       uint64_t xid, struct auth_glusterfs_parms *au,
+rpc_clnt_fill_request (int prognum, int progver, int procnum,
+                       uint64_t xid, struct auth_glusterfs_parms_v2 *au,
                        struct rpc_msg *request, char *auth_data)
 {
         int   ret          = -1;
@@ -1081,19 +1137,26 @@ rpc_clnt_fill_request (int prognum, int progver, int procnum, int payload,
         request->rm_call.cb_vers = progver;
         request->rm_call.cb_proc = procnum;
 
-        /* TODO: Using AUTH_GLUSTERFS for time-being. Make it modular in
-         * future so it is easy to plug-in new authentication schemes.
+        /* TODO: Using AUTH_(GLUSTERFS/NULL) in a kludgy way for time-being.
+         * Make it modular in future so it is easy to plug-in new
+         * authentication schemes.
          */
-        ret = xdr_serialize_glusterfs_auth (auth_data, au);
-        if (ret == -1) {
-                gf_log ("rpc-clnt", GF_LOG_DEBUG, "cannot encode credentials");
-                goto out;
+        if (auth_data) {
+                ret = xdr_serialize_glusterfs_auth (auth_data, au);
+                if (ret == -1) {
+                        gf_log ("rpc-clnt", GF_LOG_DEBUG,
+                                "cannot encode credentials");
+                        goto out;
+                }
+
+                request->rm_call.cb_cred.oa_flavor = AUTH_GLUSTERFS_v2;
+                request->rm_call.cb_cred.oa_base   = auth_data;
+                request->rm_call.cb_cred.oa_length = ret;
+        } else {
+                request->rm_call.cb_cred.oa_flavor = AUTH_NULL;
+                request->rm_call.cb_cred.oa_base   = NULL;
+                request->rm_call.cb_cred.oa_length = 0;
         }
-
-        request->rm_call.cb_cred.oa_flavor = AUTH_GLUSTERFS;
-        request->rm_call.cb_cred.oa_base   = auth_data;
-        request->rm_call.cb_cred.oa_length = ret;
-
         request->rm_call.cb_verf.oa_flavor = AUTH_NONE;
         request->rm_call.cb_verf.oa_base = NULL;
         request->rm_call.cb_verf.oa_length = 0;
@@ -1141,46 +1204,53 @@ out:
 
 struct iobuf *
 rpc_clnt_record_build_record (struct rpc_clnt *clnt, int prognum, int progver,
-                              int procnum, size_t payload, uint64_t xid,
-                              struct auth_glusterfs_parms *au, struct iovec *recbuf)
+                              int procnum, size_t hdrsize, uint64_t xid,
+                              struct auth_glusterfs_parms_v2 *au,
+                              struct iovec *recbuf)
 {
-        struct rpc_msg           request                            = {0, };
-        struct iobuf            *request_iob                        = NULL;
-        char                    *record                             = NULL;
-        struct iovec             recordhdr                          = {0, };
-        size_t                   pagesize                           = 0;
-        int                      ret                                = -1;
-        char                     auth_data[RPC_CLNT_MAX_AUTH_BYTES] = {0, };
+        struct rpc_msg  request                      = {0, };
+        struct iobuf   *request_iob                  = NULL;
+        char           *record                       = NULL;
+        struct iovec    recordhdr                    = {0, };
+        size_t          pagesize                     = 0;
+        int             ret                          = -1;
+        size_t          xdr_size                     = 0;
+        char            auth_data[GF_MAX_AUTH_BYTES] = {0, };
 
         if ((!clnt) || (!recbuf) || (!au)) {
                 goto out;
         }
 
-        /* First, try to get a pointer into the buffer which the RPC
-         * layer can use.
-         */
-        request_iob = iobuf_get (clnt->ctx->iobuf_pool);
-        if (!request_iob) {
-                goto out;
-        }
-
-        pagesize = ((struct iobuf_pool *)clnt->ctx->iobuf_pool)->page_size;
-
-        record = iobuf_ptr (request_iob);  /* Now we have it. */
-
         /* Fill the rpc structure and XDR it into the buffer got above. */
-        ret = rpc_clnt_fill_request (prognum, progver, procnum, payload, xid,
-                                     au, &request, auth_data);
+        if (clnt->auth_null)
+                ret = rpc_clnt_fill_request (prognum, progver, procnum,
+                                             xid, NULL, &request, NULL);
+        else
+                ret = rpc_clnt_fill_request (prognum, progver, procnum,
+                                             xid, au, &request, auth_data);
+
         if (ret == -1) {
                 gf_log (clnt->conn.trans->name, GF_LOG_WARNING,
                         "cannot build a rpc-request xid (%"PRIu64")", xid);
                 goto out;
         }
 
-        recordhdr = rpc_clnt_record_build_header (record, pagesize, &request,
-                                                  payload);
+        xdr_size = xdr_sizeof ((xdrproc_t)xdr_callmsg, &request);
 
-        //GF_FREE (request.rm_call.cb_cred.oa_base);
+        /* First, try to get a pointer into the buffer which the RPC
+         * layer can use.
+         */
+        request_iob = iobuf_get2 (clnt->ctx->iobuf_pool, (xdr_size + hdrsize));
+        if (!request_iob) {
+                goto out;
+        }
+
+        pagesize = iobuf_pagesize (request_iob);
+
+        record = iobuf_ptr (request_iob);  /* Now we have it. */
+
+        recordhdr = rpc_clnt_record_build_header (record, pagesize, &request,
+                                                  hdrsize);
 
         if (!recordhdr.iov_base) {
                 gf_log (clnt->conn.trans->name, GF_LOG_ERROR,
@@ -1201,39 +1271,45 @@ out:
 
 struct iobuf *
 rpc_clnt_record (struct rpc_clnt *clnt, call_frame_t *call_frame,
-                 rpc_clnt_prog_t *prog,int procnum, size_t payload_len,
+                 rpc_clnt_prog_t *prog, int procnum, size_t hdrlen,
                  struct iovec *rpchdr, uint64_t callid)
 {
-        struct auth_glusterfs_parms  au                    = {0, };
-        struct iobuf                *request_iob           = NULL;
+        struct auth_glusterfs_parms_v2  au          = {0, };
+        struct iobuf                   *request_iob = NULL;
+        char                            owner[4] = {0,};
 
         if (!prog || !rpchdr || !call_frame) {
                 goto out;
         }
 
-        au.pid      = call_frame->root->pid;
-        au.uid      = call_frame->root->uid;
-        au.gid      = call_frame->root->gid;
-        au.ngrps    = call_frame->root->ngrps;
-        au.lk_owner = call_frame->root->lk_owner;
-        if (!au.lk_owner)
-                au.lk_owner = au.pid;
+        au.pid                   = call_frame->root->pid;
+        au.uid                   = call_frame->root->uid;
+        au.gid                   = call_frame->root->gid;
+        au.groups.groups_len     = call_frame->root->ngrps;
+        au.lk_owner.lk_owner_len = call_frame->root->lk_owner.len;
+
+        if (au.groups.groups_len)
+                au.groups.groups_val = call_frame->root->groups;
+
+        if (call_frame->root->lk_owner.len)
+                au.lk_owner.lk_owner_val = call_frame->root->lk_owner.data;
+        else {
+                owner[0] = (char)(au.pid & 0xff);
+                owner[1] = (char)((au.pid >> 8) & 0xff);
+                owner[2] = (char)((au.pid >> 16) & 0xff);
+                owner[3] = (char)((au.pid >> 24) & 0xff);
+
+                au.lk_owner.lk_owner_val = owner;
+                au.lk_owner.lk_owner_len = 4;
+        }
 
         gf_log (clnt->conn.trans->name, GF_LOG_TRACE, "Auth Info: pid: %u, uid: %d"
-                ", gid: %d, owner: %"PRId64,
-                au.pid, au.uid, au.gid, au.lk_owner);
+                ", gid: %d, owner: %s", au.pid, au.uid, au.gid,
+                lkowner_utoa (&call_frame->root->lk_owner));
 
-        memcpy (au.groups, call_frame->root->groups, sizeof (au.groups));
-
-        //rpc_transport_get_myname (clnt->conn.trans, myname, UNIX_PATH_MAX);
-        //au.aup_machname = myname;
-
-        /* Assuming the client program would like to speak to the same versioned
-         * program on server.
-         */
         request_iob = rpc_clnt_record_build_record (clnt, prog->prognum,
                                                     prog->progver,
-                                                    procnum, payload_len,
+                                                    procnum, hdrlen,
                                                     callid, &au,
                                                     rpchdr);
         if (!request_iob) {
@@ -1335,6 +1411,12 @@ rpc_clnt_submit (struct rpc_clnt *rpc, rpc_clnt_prog_t *prog,
                 goto out;
         }
 
+        conn = &rpc->conn;
+
+        if (conn->trans == NULL) {
+                goto out;
+        }
+
         rpcreq = mem_get (rpc->reqpool);
         if (rpcreq == NULL) {
                 goto out;
@@ -1354,8 +1436,6 @@ rpc_clnt_submit (struct rpc_clnt *rpc, rpc_clnt_prog_t *prog,
 
         callid = rpc_clnt_new_callid (rpc);
 
-        conn = &rpc->conn;
-
         rpcreq->prog = prog;
         rpcreq->procnum = procnum;
         rpcreq->conn = conn;
@@ -1368,11 +1448,6 @@ rpc_clnt_submit (struct rpc_clnt *rpc, rpc_clnt_prog_t *prog,
                 proglen += iov_length (proghdr, proghdrcount);
         }
 
-        if (progpayload) {
-                proglen += iov_length (progpayload,
-                                       progpayloadcount);
-        }
-
         request_iob = rpc_clnt_record (rpc, frame, prog,
                                        procnum, proglen,
                                        &rpchdr, callid);
@@ -1402,8 +1477,12 @@ rpc_clnt_submit (struct rpc_clnt *rpc, rpc_clnt_prog_t *prog,
         pthread_mutex_lock (&conn->lock);
         {
                 if (conn->connected == 0) {
-                        rpc_transport_connect (conn->trans,
-                                               conn->config.remote_port);
+                        ret = rpc_transport_connect (conn->trans,
+                                                     conn->config.remote_port);
+                        /* Below code makes sure the (re-)configured port lasts
+                           for just one successful connect attempt */
+                        if (!ret)
+                                conn->config.remote_port = 0;
                 }
 
                 ret = rpc_transport_submit_request (rpc->conn.trans,
@@ -1418,7 +1497,6 @@ rpc_clnt_submit (struct rpc_clnt *rpc, rpc_clnt_prog_t *prog,
                 }
 
                 if ((ret >= 0) && frame) {
-                        gettimeofday (&conn->last_sent, NULL);
                         /* Save the frame in queue */
                         __save_frame (rpc, frame, rpcreq);
 
@@ -1438,7 +1516,9 @@ rpc_clnt_submit (struct rpc_clnt *rpc, rpc_clnt_prog_t *prog,
         ret = 0;
 
 out:
-        iobuf_unref (request_iob);
+        if (request_iob) {
+                iobuf_unref (request_iob);
+        }
 
         if (new_iobref && iobref) {
                 iobref_unref (iobref);
@@ -1448,7 +1528,7 @@ out:
                 if (rpcreq) {
                         rpcreq->rpc_status = -1;
                         cbkfn (rpcreq, NULL, 0, frame);
-                        mem_put (rpc->reqpool, rpcreq);
+                        mem_put (rpcreq);
                 }
         }
         return ret;
@@ -1476,12 +1556,11 @@ rpc_clnt_destroy (struct rpc_clnt *rpc)
                 return;
 
         if (rpc->conn.trans) {
-                rpc->conn.trans->mydata = NULL;
+                rpc_transport_unregister_notify (rpc->conn.trans);
+                rpc_transport_disconnect (rpc->conn.trans);
                 rpc_transport_unref (rpc->conn.trans);
-                //rpc_transport_destroy (rpc->conn.trans);
         }
 
-        rpc_clnt_connection_cleanup (&rpc->conn);
         rpc_clnt_reconnect_cleanup (&rpc->conn);
         saved_frames_destroy (rpc->conn.saved_frames);
         pthread_mutex_destroy (&rpc->lock);
@@ -1516,6 +1595,48 @@ rpc_clnt_unref (struct rpc_clnt *rpc)
 }
 
 
+void
+rpc_clnt_disable (struct rpc_clnt *rpc)
+{
+        rpc_clnt_connection_t *conn = NULL;
+
+        if (!rpc) {
+                goto out;
+        }
+
+        conn = &rpc->conn;
+
+        pthread_mutex_lock (&conn->lock);
+        {
+                rpc->disabled = 1;
+
+                if (conn->timer) {
+                        gf_timer_call_cancel (rpc->ctx, conn->timer);
+                        conn->timer = NULL;
+                }
+
+                if (conn->reconnect) {
+                        gf_timer_call_cancel (rpc->ctx, conn->reconnect);
+                        conn->reconnect = NULL;
+                }
+                conn->connected = 0;
+
+                if (conn->ping_timer) {
+                        gf_timer_call_cancel (rpc->ctx, conn->ping_timer);
+                        conn->ping_timer = NULL;
+                        conn->ping_started = 0;
+                }
+
+        }
+        pthread_mutex_unlock (&conn->lock);
+
+        rpc_transport_disconnect (rpc->conn.trans);
+
+out:
+        return;
+}
+
+
 void
 rpc_clnt_reconfig (struct rpc_clnt *rpc, struct rpc_clnt_config *config)
 {
@@ -1558,7 +1679,8 @@ rpc_clnt_reconfig (struct rpc_clnt *rpc, struct rpc_clnt_config *config)
 }
 
 int
-rpc_clnt_transport_unix_options_build (dict_t **options, char *filepath)
+rpc_clnt_transport_unix_options_build (dict_t **options, char *filepath,
+                                       int frame_timeout)
 {
         dict_t                  *dict = NULL;
         char                    *fpath = NULL;
@@ -1597,6 +1719,12 @@ rpc_clnt_transport_unix_options_build (dict_t **options, char *filepath)
         if (ret)
                 goto out;
 
+        if (frame_timeout > 0) {
+                ret = dict_set_int32 (dict, "frame-timeout", frame_timeout);
+                if (ret)
+                        goto out;
+        }
+
         *options = dict;
 out:
         if (ret) {
