@@ -1,20 +1,11 @@
 /*
-  Copyright (c) 2010-2011 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+  This file is licensed to you under your choice of the GNU Lesser
+  General Public License, version 3 or any later version (LGPLv3 or
+  later), or the GNU General Public License, version 2 (GPLv2), in all
+  cases as published by the Free Software Foundation.
 */
 
 #ifndef __RPC_TRANSPORT_H__
@@ -40,11 +31,15 @@
 #define MAX_IOVEC 16
 #endif
 
+#ifndef AI_ADDRCONFIG
+#define AI_ADDRCONFIG 0
+#endif /* AI_ADDRCONFIG */
+
 /* Given the 4-byte fragment header, returns non-zero if this fragment
- * is the last fragment for the RPC record being assemebled.
+ * is the last fragment for the RPC record being assembled.
  * RPC Record marking standard defines a 32 bit value as the fragment
  * header with the MSB signifying whether the fragment is the last
- * fragment for the record being asembled.
+ * fragment for the record being assembled.
  */
 #define RPC_LASTFRAG(fraghdr) ((uint32_t)(fraghdr & 0x80000000U))
 
@@ -73,9 +68,9 @@ typedef struct rpc_transport rpc_transport_t;
 #include "rpcsvc-common.h"
 
 struct peer_info {
-	struct sockaddr_storage sockaddr;
-	socklen_t sockaddr_len;
-	char identifier[UNIX_PATH_MAX];
+        struct sockaddr_storage sockaddr;
+        socklen_t sockaddr_len;
+        char identifier[UNIX_PATH_MAX];
 };
 typedef struct peer_info peer_info_t;
 
@@ -178,40 +173,38 @@ typedef int (*rpc_transport_notify_t) (rpc_transport_t *, void *mydata,
 
 
 struct rpc_transport {
-	struct rpc_transport_ops  *ops;
+        struct rpc_transport_ops  *ops;
         rpc_transport_t           *listener; /* listener transport to which
                                               * request for creation of this
                                               * transport came from. valid only
                                               * on server process.
                                               */
-        
-	void                      *private;
+
+        void                      *private;
         void                      *xl_private;
         void                      *xl;       /* Used for THIS */
-	void                      *mydata;
-	pthread_mutex_t            lock;
-	int32_t                    refcount;
+        void                      *mydata;
+        pthread_mutex_t            lock;
+        int32_t                    refcount;
 
         glusterfs_ctx_t           *ctx;
         dict_t                    *options;
         char                      *name;
-	void                      *dnscache;
-	data_t                    *buf;
- 	int32_t                  (*init)   (rpc_transport_t *this);
-	void                     (*fini)   (rpc_transport_t *this);
-        int32_t                  (*validate_options) (rpc_transport_t *this, 
-                                                      char **op_errstr);
+        void                      *dnscache;
+        data_t                    *buf;
+        int32_t                  (*init)   (rpc_transport_t *this);
+        void                     (*fini)   (rpc_transport_t *this);
         int                      (*reconfigure) (rpc_transport_t *this, dict_t *options);
         rpc_transport_notify_t     notify;
         void                      *notify_data;
-	peer_info_t                peerinfo;
-	peer_info_t                myinfo;
+        peer_info_t                peerinfo;
+        peer_info_t                myinfo;
 
         uint64_t                   total_bytes_read;
         uint64_t                   total_bytes_write;
 
         struct list_head           list;
-        int                        client_bind_insecure;
+        int                        bind_insecure;
 };
 
 struct rpc_transport_ops {
@@ -222,9 +215,9 @@ struct rpc_transport_ops {
                                    rpc_transport_req_t *req);
         int32_t (*submit_reply)   (rpc_transport_t *this,
                                    rpc_transport_reply_t *reply);
-	int32_t (*connect)        (rpc_transport_t *this, int port);
-	int32_t (*listen)         (rpc_transport_t *this);
-	int32_t (*disconnect)     (rpc_transport_t *this);
+        int32_t (*connect)        (rpc_transport_t *this, int port);
+        int32_t (*listen)         (rpc_transport_t *this);
+        int32_t (*disconnect)     (rpc_transport_t *this);
         int32_t (*get_peername)   (rpc_transport_t *this, char *hostname,
                                    int hostlen);
         int32_t (*get_peeraddr)   (rpc_transport_t *this, char *peeraddr,
@@ -274,6 +267,9 @@ int
 rpc_transport_register_notify (rpc_transport_t *trans, rpc_transport_notify_t,
                                void *mydata);
 
+int
+rpc_transport_unregister_notify (rpc_transport_t *trans);
+
 int32_t
 rpc_transport_get_peername (rpc_transport_t *this, char *hostname, int hostlen);
 
