@@ -1,20 +1,11 @@
 /*
-  Copyright (c) 2008-2011 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+  This file is licensed to you under your choice of the GNU Lesser
+  General Public License, version 3 or any later version (LGPLv3 or
+  later), or the GNU General Public License, version 2 (GPLv2), in all
+  cases as published by the Free Software Foundation.
 */
 
 #include <sys/types.h>
@@ -95,7 +86,7 @@ af_unix_client_bind (rpc_transport_t *this,
                 char *path = data_to_str (path_data);
                 if (!path || strlen (path) > UNIX_PATH_MAX) {
                         gf_log (this->name, GF_LOG_DEBUG,
-                                "transport.rdma.bind-path not specfied "
+                                "transport.rdma.bind-path not specified "
                                 "for unix socket, letting connect to assign "
                                 "default value");
                         goto err;
@@ -162,13 +153,11 @@ client_fill_address_family (rpc_transport_t *this, struct sockaddr *sockaddr)
                         sockaddr->sa_family = AF_INET6;
                 } else if (!strcasecmp (address_family, "inet-sdp")) {
                         sockaddr->sa_family = AF_INET_SDP;
-                } else if (!strcasecmp (address_family, "inet/inet6")
-                           || !strcasecmp (address_family, "inet6/inet")) {
-                        sockaddr->sa_family = AF_UNSPEC;
                 } else {
                         gf_log (this->name, GF_LOG_ERROR,
                                 "unknown address-family (%s) specified",
                                 address_family);
+                        sockaddr->sa_family = AF_UNSPEC;
                         return -1;
                 }
         }
@@ -530,21 +519,19 @@ gf_rdma_server_get_local_sockaddr (rpc_transport_t *this,
                         addr->sa_family = AF_INET_SDP;
                 } else if (!strcasecmp (address_family, "unix")) {
                         addr->sa_family = AF_UNIX;
-                } else if (!strcasecmp (address_family, "inet/inet6")
-                           || !strcasecmp (address_family, "inet6/inet")) {
-                        addr->sa_family = AF_UNSPEC;
                 } else {
                         gf_log (this->name, GF_LOG_ERROR,
                                 "unknown address family (%s) specified",
                                 address_family);
+                        addr->sa_family = AF_UNSPEC;
                         ret = -1;
                         goto err;
                 }
         } else {
                 gf_log (this->name, GF_LOG_DEBUG,
                         "option address-family not specified, defaulting "
-                        "to inet/inet6");
-                addr->sa_family = AF_UNSPEC;
+                        "to inet");
+                addr->sa_family = AF_INET;
         }
 
         switch (addr->sa_family)
@@ -577,40 +564,33 @@ fill_inet6_inet_identifiers (rpc_transport_t *this, struct sockaddr_storage *add
 {
         int32_t ret = 0, tmpaddr_len = 0;
         char service[NI_MAXSERV], host[NI_MAXHOST];
-        struct sockaddr_storage tmpaddr;
+        union gf_sock_union sock_union;
 
-        memset (&tmpaddr, 0, sizeof (tmpaddr));
-        tmpaddr = *addr;
+        memset (&sock_union, 0, sizeof (sock_union));
+        sock_union.storage = *addr;
         tmpaddr_len = addr_len;
 
-        if (((struct sockaddr *) &tmpaddr)->sa_family == AF_INET6) {
+        if (sock_union.sa.sa_family == AF_INET6) {
                 int32_t one_to_four, four_to_eight, twelve_to_sixteen;
                 int16_t eight_to_ten, ten_to_twelve;
 
                 one_to_four = four_to_eight = twelve_to_sixteen = 0;
                 eight_to_ten = ten_to_twelve = 0;
 
-                one_to_four = ((struct sockaddr_in6 *)
-                               &tmpaddr)->sin6_addr.s6_addr32[0];
-                four_to_eight = ((struct sockaddr_in6 *)
-                                 &tmpaddr)->sin6_addr.s6_addr32[1];
+                one_to_four = sock_union.sin6.sin6_addr.s6_addr32[0];
+                four_to_eight = sock_union.sin6.sin6_addr.s6_addr32[1];
 #ifdef GF_SOLARIS_HOST_OS
-                eight_to_ten = S6_ADDR16(((struct sockaddr_in6 *)
-                                          &tmpaddr)->sin6_addr)[4];
+                eight_to_ten = S6_ADDR16(sock_union.sin6.sin6_addr)[4];
 #else
-                eight_to_ten = ((struct sockaddr_in6 *)
-                                &tmpaddr)->sin6_addr.s6_addr16[4];
+                eight_to_ten = sock_union.sin6.sin6_addr.s6_addr16[4];
 #endif
 
 #ifdef GF_SOLARIS_HOST_OS
-                ten_to_twelve = S6_ADDR16(((struct sockaddr_in6 *)
-                                           &tmpaddr)->sin6_addr)[5];
+                ten_to_twelve = S6_ADDR16(sock_union.sin6.sin6_addr)[5];
 #else
-                ten_to_twelve = ((struct sockaddr_in6 *)
-                                 &tmpaddr)->sin6_addr.s6_addr16[5];
+                ten_to_twelve = sock_union.sin6.sin6_addr.s6_addr16[5];
 #endif
-                twelve_to_sixteen = ((struct sockaddr_in6 *)
-                                     &tmpaddr)->sin6_addr.s6_addr32[3];
+                twelve_to_sixteen = sock_union.sin6.sin6_addr.s6_addr32[3];
 
                 /* ipv4 mapped ipv6 address has
                    bits 0-80: 0
@@ -622,8 +602,8 @@ fill_inet6_inet_identifiers (rpc_transport_t *this, struct sockaddr_storage *add
                     four_to_eight == 0 &&
                     eight_to_ten == 0 &&
                     ten_to_twelve == -1) {
-                        struct sockaddr_in *in_ptr = (struct sockaddr_in *)&tmpaddr;
-                        memset (&tmpaddr, 0, sizeof (tmpaddr));
+                        struct sockaddr_in *in_ptr = &sock_union.sin;
+                        memset (&sock_union, 0, sizeof (sock_union));
 
                         in_ptr->sin_family = AF_INET;
                         in_ptr->sin_port = ((struct sockaddr_in6 *)addr)->sin6_port;
@@ -632,7 +612,7 @@ fill_inet6_inet_identifiers (rpc_transport_t *this, struct sockaddr_storage *add
                 }
         }
 
-        ret = getnameinfo ((struct sockaddr *) &tmpaddr,
+        ret = getnameinfo (&sock_union.sa,
                            tmpaddr_len,
                            host, sizeof (host),
                            service, sizeof (service),
