@@ -1,20 +1,11 @@
 /*
-  Copyright (c) 2010-2011 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+  This file is licensed to you under your choice of the GNU Lesser
+  General Public License, version 3 or any later version (LGPLv3 or
+  later), or the GNU General Public License, version 2 (GPLv2), in all
+  cases as published by the Free Software Foundation.
 */
 
 
@@ -37,13 +28,13 @@
 /* ugly #includes below */
 #include "protocol-common.h"
 #include "glusterfs3-xdr.h"
-#include "glusterfs3.h"
+#include "xdr-nfs3.h"
+#include "rpcsvc.h"
 
 #include <fcntl.h>
 #include <errno.h>
 #include <netinet/tcp.h>
 #include <rpc/xdr.h>
-
 #define GF_LOG_ERRNO(errno) ((errno == ENOTCONN) ? GF_LOG_DEBUG : GF_LOG_ERROR)
 #define SA(ptr) ((struct sockaddr *)ptr)
 
@@ -82,7 +73,7 @@
                 priv->incoming.frag.bytes_read += bytes_read;           \
                                                                         \
                 if ((ret > 0) || (priv->incoming.frag.remaining_size != 0)) { \
-                        if (priv->incoming.frag.remaining_size != 0) {  \
+                        if (priv->incoming.frag.remaining_size != 0 && ret == 0) {  \
                                 __socket_proto_reset_pending (priv);    \
                         }                                               \
                                                                         \
@@ -318,7 +309,7 @@ __socket_server_bind (rpc_transport_t *this)
                         strerror (errno));
         }
 
-        //reuse-address doesnt work for unix type sockets
+        /* reuse-address doesn't work for unix type sockets */
         if (AF_UNIX == SA (&this->myinfo.sockaddr)->sa_family) {
                 memcpy (&unix_addr, SA (&this->myinfo.sockaddr),
                         this->myinfo.sockaddr_len);
@@ -375,7 +366,7 @@ __socket_nodelay (int fd)
         ret = setsockopt (fd, IPPROTO_TCP, TCP_NODELAY,
                           &on, sizeof (on));
         if (!ret)
-                gf_log ("", GF_LOG_TRACE,
+                gf_log (THIS->name, GF_LOG_TRACE,
                         "NODELAY enabled for socket %d", fd);
 
         return ret;
@@ -398,7 +389,7 @@ __socket_keepalive (int fd, int keepalive_intvl, int keepalive_idle)
         if (keepalive_intvl == GF_USE_DEFAULT_KEEPALIVE)
                 goto done;
 
-#ifndef GF_LINUX_HOST_OS
+#if !defined(GF_LINUX_HOST_OS) && !defined(__NetBSD__)
 #ifdef GF_SOLARIS_HOST_OS
         ret = setsockopt (fd, SOL_SOCKET, SO_KEEPALIVE, &keepalive_intvl,
                           sizeof (keepalive_intvl));
@@ -429,7 +420,7 @@ __socket_keepalive (int fd, int keepalive_intvl, int keepalive_idle)
 #endif
 
 done:
-        gf_log ("", GF_LOG_TRACE, "Keep-alive enabled for socket %d, interval "
+        gf_log (THIS->name, GF_LOG_TRACE, "Keep-alive enabled for socket %d, interval "
                 "%d, idle: %d", fd, keepalive_intvl, keepalive_idle);
 
 err:
@@ -807,9 +798,14 @@ __socket_read_simple_request (rpc_transport_t *this)
 
 #define rpc_verf_addr(fragcurrent) (fragcurrent - 4)
 
+#define rpc_msgtype_addr(buf) (buf + 4)
+
+#define rpc_prognum_addr(buf) (buf + RPC_MSGTYPE_SIZE + 4)
+#define rpc_progver_addr(buf) (buf + RPC_MSGTYPE_SIZE + 8)
+#define rpc_procnum_addr(buf) (buf + RPC_MSGTYPE_SIZE + 12)
 
 inline int
-__socket_read_vectored_request (rpc_transport_t *this)
+__socket_read_vectored_request (rpc_transport_t *this, rpcsvc_vector_sizer vector_sizer)
 {
         socket_private_t *priv                   = NULL;
         int               ret                    = 0;
@@ -817,8 +813,8 @@ __socket_read_vectored_request (rpc_transport_t *this)
         char             *addr                   = NULL;
         struct iobuf     *iobuf                  = NULL;
         uint32_t          remaining_size         = 0;
-        uint32_t          gluster_write_proc_len = 0;
-        gfs3_write_req    write_req              = {{0,},};
+        ssize_t           readsize               = 0;
+        size_t            size = 0;
 
         GF_VALIDATE_OR_GOTO ("socket", this, out);
         GF_VALIDATE_OR_GOTO ("socket", this->private, out);
@@ -827,6 +823,7 @@ __socket_read_vectored_request (rpc_transport_t *this)
 
         switch (priv->incoming.frag.call_body.request.vector_state) {
         case SP_STATE_VECTORED_REQUEST_INIT:
+                priv->incoming.frag.call_body.request.vector_sizer_state = 0;
                 addr = rpc_cred_addr (iobuf_ptr (priv->incoming.iobuf));
 
                 /* also read verf flavour and verflen */
@@ -850,24 +847,13 @@ __socket_read_vectored_request (rpc_transport_t *this)
 
         case SP_STATE_READ_CREDBYTES:
                 addr = rpc_verf_addr (priv->incoming.frag.fragcurrent);
+                verflen = ntoh32 (*((uint32_t *)addr));
 
-                /* FIXME: Also handle procedures other than glusterfs-write
-                 * here
-                 */
-                /* also read proc-header */
-                gluster_write_proc_len = xdr_sizeof ((xdrproc_t) xdr_gfs3_write_req,
-                                                     &write_req);
-
-                if (gluster_write_proc_len == 0) {
-                        gf_log (this->name, GF_LOG_ERROR,
-                                "xdr_sizeof on gfs3_write_req failed");
-                        ret = -1;
-                        goto out;
+                if (verflen == 0) {
+                        priv->incoming.frag.call_body.request.vector_state
+                                = SP_STATE_READ_VERFBYTES;
+                        goto sp_state_read_verfbytes;
                 }
-
-                verflen = ntoh32 (*((uint32_t *)addr))
-                        + gluster_write_proc_len;
-
                 __socket_proto_init_pending (priv, verflen);
 
                 priv->incoming.frag.call_body.request.vector_state
@@ -884,8 +870,39 @@ __socket_read_vectored_request (rpc_transport_t *this)
                 /* fall through */
 
         case SP_STATE_READ_VERFBYTES:
+sp_state_read_verfbytes:
+                priv->incoming.frag.call_body.request.vector_sizer_state =
+                        vector_sizer (priv->incoming.frag.call_body.request.vector_sizer_state,
+                                      &readsize,
+                                      priv->incoming.frag.fragcurrent);
+                __socket_proto_init_pending (priv, readsize);
+                priv->incoming.frag.call_body.request.vector_state
+                        = SP_STATE_READING_PROGHDR;
+
+                /* fall through */
+
+        case SP_STATE_READING_PROGHDR:
+                __socket_proto_read (priv, ret);
+sp_state_reading_proghdr:
+                priv->incoming.frag.call_body.request.vector_sizer_state =
+                        vector_sizer (priv->incoming.frag.call_body.request.vector_sizer_state,
+                                      &readsize,
+                                      priv->incoming.frag.fragcurrent);
+                if (readsize == 0) {
+                        priv->incoming.frag.call_body.request.vector_state =
+                                SP_STATE_READ_PROGHDR;
+                } else {
+                        __socket_proto_init_pending (priv, readsize);
+                        __socket_proto_read (priv, ret);
+                        goto sp_state_reading_proghdr;
+                }
+
+        case SP_STATE_READ_PROGHDR:
                 if (priv->incoming.payload_vector.iov_base == NULL) {
-                        iobuf = iobuf_get (this->ctx->iobuf_pool);
+
+                        size = RPC_FRAGSIZE (priv->incoming.fraghdr) -
+                                priv->incoming.frag.bytes_read;
+                        iobuf = iobuf_get2 (this->ctx->iobuf_pool, size);
                         if (!iobuf) {
                                 ret = -1;
                                 break;
@@ -942,22 +959,15 @@ out:
         return ret;
 }
 
-
-#define rpc_msgtype_addr(buf) (buf + 4)
-
-#define rpc_prognum_addr(buf) (buf + RPC_MSGTYPE_SIZE + 4)
-
-#define rpc_procnum_addr(buf) (buf + RPC_MSGTYPE_SIZE + 12)
-
-
 inline int
 __socket_read_request (rpc_transport_t *this)
 {
         socket_private_t *priv               = NULL;
-        uint32_t          prognum            = 0, procnum = 0;
+        uint32_t          prognum            = 0, procnum = 0, progver = 0;
         uint32_t          remaining_size     = 0;
         int               ret                = -1;
         char             *buf                = NULL;
+        rpcsvc_vector_sizer     vector_sizer = NULL;
 
         GF_VALIDATE_OR_GOTO ("socket", this, out);
         GF_VALIDATE_OR_GOTO ("socket", this->private, out);
@@ -987,12 +997,21 @@ __socket_read_request (rpc_transport_t *this)
                 buf = rpc_prognum_addr (iobuf_ptr (priv->incoming.iobuf));
                 prognum = ntoh32 (*((uint32_t *)buf));
 
+                buf = rpc_progver_addr (iobuf_ptr (priv->incoming.iobuf));
+                progver = ntoh32 (*((uint32_t *)buf));
+
                 buf = rpc_procnum_addr (iobuf_ptr (priv->incoming.iobuf));
                 procnum = ntoh32 (*((uint32_t *)buf));
 
-                if ((prognum == GLUSTER3_1_FOP_PROGRAM)
-                    && (procnum == GF_FOP_WRITE)) {
-                        ret = __socket_read_vectored_request (this);
+                if (this->listener) {
+                        /* this check is needed as rpcsvc and rpc-clnt actor structures are
+                         * not same */
+                        vector_sizer = rpcsvc_get_program_vector_sizer ((rpcsvc_t *)this->mydata,
+                                                                        prognum, progver, procnum);
+                }
+
+                if (vector_sizer) {
+                        ret = __socket_read_vectored_request (this, vector_sizer);
                 } else {
                         ret = __socket_read_simple_request (this);
                 }
@@ -1024,6 +1043,7 @@ __socket_read_accepted_successful_reply (rpc_transport_t *this)
         struct iobuf     *iobuf                    = NULL;
         uint32_t          gluster_read_rsp_hdr_len = 0;
         gfs3_read_rsp     read_rsp                 = {0, };
+        size_t            size                     = 0;
 
         GF_VALIDATE_OR_GOTO ("socket", this, out);
         GF_VALIDATE_OR_GOTO ("socket", this->private, out);
@@ -1056,7 +1076,11 @@ __socket_read_accepted_successful_reply (rpc_transport_t *this)
                         = SP_STATE_READ_PROC_HEADER;
 
                 if (priv->incoming.payload_vector.iov_base == NULL) {
-                        iobuf = iobuf_get (this->ctx->iobuf_pool);
+
+                        size = (RPC_FRAGSIZE (priv->incoming.fraghdr) -
+                                priv->incoming.frag.bytes_read);
+
+                        iobuf = iobuf_get2 (this->ctx->iobuf_pool, size);
                         if (iobuf == NULL) {
                                 ret = -1;
                                 goto out;
@@ -1076,6 +1100,8 @@ __socket_read_accepted_successful_reply (rpc_transport_t *this)
 
                         priv->incoming.payload_vector.iov_base
                                 = iobuf_ptr (iobuf);
+
+                        priv->incoming.payload_vector.iov_len = size;
                 }
 
                 priv->incoming.frag.fragcurrent
@@ -1323,7 +1349,7 @@ __socket_read_reply (rpc_transport_t *this)
                 pthread_mutex_lock (&priv->lock);
 
                 if (ret == -1) {
-                        gf_log ("", GF_LOG_WARNING,
+                        gf_log (this->name, GF_LOG_WARNING,
                                 "notify for event MAP_XID failed");
                         goto out;
                 }
@@ -1456,14 +1482,6 @@ __socket_proto_state_machine (rpc_transport_t *this,
                 switch (priv->incoming.record_state) {
 
                 case SP_STATE_NADA:
-                        iobuf = iobuf_get (this->ctx->iobuf_pool);
-                        if (!iobuf) {
-                                ret = -ENOMEM;
-                                goto out;
-                        }
-
-                        priv->incoming.iobuf = iobuf;
-                        priv->incoming.iobuf_size = 0;
                         priv->incoming.total_bytes_read = 0;
                         priv->incoming.payload_vector.iov_len = 0;
 
@@ -1471,7 +1489,6 @@ __socket_proto_state_machine (rpc_transport_t *this,
                         priv->incoming.pending_vector->iov_base =
                                 &priv->incoming.fraghdr;
 
-                        priv->incoming.frag.fragcurrent = iobuf_ptr (iobuf);
                         priv->incoming.pending_vector->iov_len  =
                                 sizeof (priv->incoming.fraghdr);
 
@@ -1486,12 +1503,14 @@ __socket_proto_state_machine (rpc_transport_t *this,
                                               &priv->incoming.pending_count,
                                               NULL);
                         if (ret == -1) {
-                                gf_log (this->name,
-                                        ((priv->connected == 1) ?
-                                         GF_LOG_WARNING : GF_LOG_DEBUG),
-                                        "reading from socket failed. Error (%s)"
-                                        ", peer (%s)", strerror (errno),
-                                        this->peerinfo.identifier);
+                                if (priv->read_fail_log == 1) {
+                                        gf_log (this->name,
+                                                ((priv->connected == 1) ?
+                                                 GF_LOG_WARNING : GF_LOG_DEBUG),
+                                                "reading from socket failed. Error (%s)"
+                                                ", peer (%s)", strerror (errno),
+                                                this->peerinfo.identifier);
+                                }
                                 goto out;
                         }
 
@@ -1513,6 +1532,17 @@ __socket_proto_state_machine (rpc_transport_t *this,
                         priv->incoming.record_state = SP_STATE_READING_FRAG;
                         priv->incoming.total_bytes_read
                                 += RPC_FRAGSIZE(priv->incoming.fraghdr);
+                        iobuf = iobuf_get2 (this->ctx->iobuf_pool,
+                                            priv->incoming.total_bytes_read +
+                                            sizeof (priv->incoming.fraghdr));
+                        if (!iobuf) {
+                                ret = -ENOMEM;
+                                goto out;
+                        }
+
+                        priv->incoming.iobuf = iobuf;
+                        priv->incoming.iobuf_size = 0;
+                        priv->incoming.frag.fragcurrent = iobuf_ptr (iobuf);
                         /* fall through */
 
                 case SP_STATE_READING_FRAG:
@@ -1577,7 +1607,7 @@ __socket_proto_state_machine (rpc_transport_t *this,
                                 priv->incoming.iobuf = NULL;
 
                                 if (*pollin == NULL) {
-                                        gf_log ("", GF_LOG_WARNING,
+                                        gf_log (this->name, GF_LOG_WARNING,
                                                 "transport pollin allocation failed");
                                         ret = -1;
                                         goto out;
@@ -1900,7 +1930,7 @@ socket_server_event_handler (int fd, int idx, void *data,
                         }
                         pthread_mutex_unlock (&new_priv->lock);
                         if (ret == -1) {
-                                gf_log ("", GF_LOG_WARNING,
+                                gf_log (this->name, GF_LOG_WARNING,
                                         "failed to register the socket with event");
                                 goto unlock;
                         }
@@ -1945,10 +1975,10 @@ socket_connect (rpc_transport_t *this, int port)
         int                      ret = -1;
         int                      sock = -1;
         socket_private_t        *priv = NULL;
-        struct sockaddr_storage  sockaddr = {0, };
         socklen_t                sockaddr_len = 0;
         glusterfs_ctx_t         *ctx = NULL;
         sa_family_t              sa_family = {0, };
+        union gf_sock_union      sock_union;
 
         GF_VALIDATE_OR_GOTO ("socket", this, err);
         GF_VALIDATE_OR_GOTO ("socket", this->private, err);
@@ -1971,20 +2001,21 @@ socket_connect (rpc_transport_t *this, int port)
         if (sock != -1) {
                 gf_log_callingfn (this->name, GF_LOG_TRACE,
                                   "connect () called on transport already connected");
-                ret = 0;
+                errno = EINPROGRESS;
+                ret = -1;
                 goto err;
         }
 
-        ret = socket_client_get_remote_sockaddr (this, SA (&sockaddr),
+        ret = socket_client_get_remote_sockaddr (this, &sock_union.sa,
                                                  &sockaddr_len, &sa_family);
         if (ret == -1) {
                 /* logged inside client_get_remote_sockaddr */
                 goto err;
         }
 
-        if (port > 0)
-                ((struct sockaddr_in *) (&sockaddr))->sin_port = htons (port);
-
+        if (port > 0) {
+                sock_union.sin.sin_port = htons (port);
+        }
         pthread_mutex_lock (&priv->lock);
         {
                 if (priv->sock != -1) {
@@ -1993,7 +2024,8 @@ socket_connect (rpc_transport_t *this, int port)
                         goto unlock;
                 }
 
-                memcpy (&this->peerinfo.sockaddr, &sockaddr, sockaddr_len);
+                memcpy (&this->peerinfo.sockaddr, &sock_union.storage,
+                        sockaddr_len);
                 this->peerinfo.sockaddr_len = sockaddr_len;
 
                 priv->sock = socket (sa_family, SOCK_STREAM, 0);
@@ -2007,31 +2039,35 @@ socket_connect (rpc_transport_t *this, int port)
                 /* Cant help if setting socket options fails. We can continue
                  * working nonetheless.
                  */
-                if (setsockopt (priv->sock, SOL_SOCKET, SO_RCVBUF,
-                                &priv->windowsize,
-                                sizeof (priv->windowsize)) < 0) {
-                        gf_log (this->name, GF_LOG_ERROR,
-                                "setting receive window size failed: %d: %d: "
-                                "%s", priv->sock, priv->windowsize,
-                                strerror (errno));
-                }
+                if (priv->windowsize != 0) {
+                        if (setsockopt (priv->sock, SOL_SOCKET, SO_RCVBUF,
+                                        &priv->windowsize,
+                                        sizeof (priv->windowsize)) < 0) {
+                                gf_log (this->name, GF_LOG_ERROR,
+                                        "setting receive window "
+                                        "size failed: %d: %d: %s",
+                                        priv->sock, priv->windowsize,
+                                        strerror (errno));
+                        }
 
-                if (setsockopt (priv->sock, SOL_SOCKET, SO_SNDBUF,
-                                &priv->windowsize,
-                                sizeof (priv->windowsize)) < 0) {
-                        gf_log (this->name, GF_LOG_ERROR,
-                                "setting send window size failed: %d: %d: "
-                                "%s", priv->sock, priv->windowsize,
-                                strerror (errno));
+                        if (setsockopt (priv->sock, SOL_SOCKET, SO_SNDBUF,
+                                        &priv->windowsize,
+                                        sizeof (priv->windowsize)) < 0) {
+                                gf_log (this->name, GF_LOG_ERROR,
+                                        "setting send window size "
+                                        "failed: %d: %d: %s",
+                                        priv->sock, priv->windowsize,
+                                        strerror (errno));
+                        }
                 }
 
-
                 if (priv->nodelay) {
                         ret = __socket_nodelay (priv->sock);
+
                         if (ret == -1) {
                                 gf_log (this->name, GF_LOG_ERROR,
-                                        "setsockopt() failed for NODELAY (%s)",
-                                        strerror (errno));
+                                        "NODELAY on %d failed (%s)",
+                                        priv->sock, strerror (errno));
                         }
                 }
 
@@ -2074,7 +2110,7 @@ socket_connect (rpc_transport_t *this, int port)
                 ret = connect (priv->sock, SA (&this->peerinfo.sockaddr),
                                this->peerinfo.sockaddr_len);
 
-                if (ret == -1 && errno != EINPROGRESS) {
+                if (ret == -1 && ((errno != EINPROGRESS) && (errno != ENOENT))) {
                         gf_log (this->name, GF_LOG_ERROR,
                                 "connection attempt failed (%s)",
                                 strerror (errno));
@@ -2090,7 +2126,7 @@ socket_connect (rpc_transport_t *this, int port)
                 priv->idx = event_register (ctx->event_pool, priv->sock,
                                             socket_event_handler, this, 1, 1);
                 if (priv->idx == -1) {
-                        gf_log ("", GF_LOG_WARNING,
+                        gf_log (this->name, GF_LOG_WARNING,
                                 "failed to register the event");
                         ret = -1;
                 }
@@ -2110,7 +2146,7 @@ socket_listen (rpc_transport_t *this)
         int                      ret = -1;
         int                      sock = -1;
         struct sockaddr_storage  sockaddr;
-        socklen_t                sockaddr_len;
+        socklen_t                sockaddr_len = 0;
         peer_info_t             *myinfo = NULL;
         glusterfs_ctx_t         *ctx = NULL;
         sa_family_t              sa_family = {0, };
@@ -2130,7 +2166,7 @@ socket_listen (rpc_transport_t *this)
 
         if (sock != -1)  {
                 gf_log_callingfn (this->name, GF_LOG_DEBUG,
-                                  "alreading listening");
+                                  "already listening");
                 return ret;
         }
 
@@ -2163,22 +2199,26 @@ socket_listen (rpc_transport_t *this)
                 /* Cant help if setting socket options fails. We can continue
                  * working nonetheless.
                  */
-                if (setsockopt (priv->sock, SOL_SOCKET, SO_RCVBUF,
-                                &priv->windowsize,
-                                sizeof (priv->windowsize)) < 0) {
-                        gf_log (this->name, GF_LOG_ERROR,
-                                "setting receive window size failed: %d: %d: "
-                                "%s", priv->sock, priv->windowsize,
-                                strerror (errno));
-                }
+                if (priv->windowsize != 0) {
+                        if (setsockopt (priv->sock, SOL_SOCKET, SO_RCVBUF,
+                                        &priv->windowsize,
+                                        sizeof (priv->windowsize)) < 0) {
+                                gf_log (this->name, GF_LOG_ERROR,
+                                        "setting receive window size "
+                                        "failed: %d: %d: %s", priv->sock,
+                                        priv->windowsize,
+                                        strerror (errno));
+                        }
 
-                if (setsockopt (priv->sock, SOL_SOCKET, SO_SNDBUF,
-                                &priv->windowsize,
-                                sizeof (priv->windowsize)) < 0) {
-                        gf_log (this->name, GF_LOG_ERROR,
-                                "setting send window size failed: %d: %d: "
-                                "%s", priv->sock, priv->windowsize,
-                                strerror (errno));
+                        if (setsockopt (priv->sock, SOL_SOCKET, SO_SNDBUF,
+                                        &priv->windowsize,
+                                        sizeof (priv->windowsize)) < 0) {
+                                gf_log (this->name, GF_LOG_ERROR,
+                                        "setting send window size failed:"
+                                        " %d: %d: %s", priv->sock,
+                                        priv->windowsize,
+                                        strerror (errno));
+                        }
                 }
 
                 if (priv->nodelay) {
@@ -2408,6 +2448,7 @@ socket_getpeeraddr (rpc_transport_t *this, char *peeraddr, int addrlen,
         if (peeraddr != NULL) {
                 ret = socket_getpeername (this, peeraddr, addrlen);
         }
+        ret = 0;
 
 out:
         return ret;
@@ -2468,10 +2509,11 @@ struct rpc_transport_ops tops = {
 int
 reconfigure (rpc_transport_t *this, dict_t *options)
 {
-        socket_private_t *priv = NULL;
-        gf_boolean_t      tmp_bool = _gf_false;
-        char             *optstr = NULL;
-        int               ret = 0;
+        socket_private_t *priv          = NULL;
+        gf_boolean_t      tmp_bool      = _gf_false;
+        char             *optstr        = NULL;
+        int               ret           = 0;
+        uint64_t          windowsize    = 0;
 
         GF_VALIDATE_OR_GOTO ("socket", this, out);
         GF_VALIDATE_OR_GOTO ("socket", this->private, out);
@@ -2499,6 +2541,19 @@ reconfigure (rpc_transport_t *this, dict_t *options)
         }
         else
                 priv->keepalive = 1;
+
+        optstr = NULL;
+        if (dict_get_str (this->options, "tcp-window-size",
+                          &optstr) == 0) {
+                if (gf_string2bytesize (optstr, &windowsize) != 0) {
+                        gf_log (this->name, GF_LOG_ERROR,
+                                "invalid number format: %s", optstr);
+                        goto out;
+                }
+        }
+
+        priv->windowsize = (int)windowsize;
+
         ret = 0;
 out:
         return ret;
@@ -2534,7 +2589,6 @@ socket_init (rpc_transport_t *this)
         priv->nodelay = 1;
         priv->bio = 0;
         priv->windowsize = GF_DEFAULT_SOCKET_WINDOW_SIZE;
-
         INIT_LIST_HEAD (&priv->ioq);
 
         /* All the below section needs 'this->options' to be present */
@@ -2579,9 +2633,8 @@ socket_init (rpc_transport_t *this)
                 }
         }
 
-
         optstr = NULL;
-        if (dict_get_str (this->options, "transport.window-size",
+        if (dict_get_str (this->options, "tcp-window-size",
                           &optstr) == 0) {
                 if (gf_string2bytesize (optstr, &windowsize) != 0) {
                         gf_log (this->name, GF_LOG_ERROR,
@@ -2590,8 +2643,9 @@ socket_init (rpc_transport_t *this)
                 }
         }
 
-        optstr = NULL;
+        priv->windowsize = (int)windowsize;
 
+        optstr = NULL;
         /* Enable Keep-alive by default. */
         priv->keepalive = 1;
         priv->keepaliveintvl = 2;
@@ -2627,7 +2681,23 @@ socket_init (rpc_transport_t *this)
                 priv->backlog = backlog;
         }
 
-        priv->windowsize = (int)windowsize;
+        optstr = NULL;
+
+         /* Check if socket read failures are to be logged */
+        priv->read_fail_log = 1;
+        if (dict_get (this->options, "transport.socket.read-fail-log")) {
+                optstr = data_to_str (dict_get (this->options, "transport.socket.read-fail-log"));
+                if (gf_string2boolean (optstr, &tmp_bool) == -1) {
+                        gf_log (this->name, GF_LOG_WARNING,
+                                   "'transport.socket.read-fail-log' takes only "
+                                   "boolean options; logging socket read fails");
+                }
+                else if (tmp_bool == _gf_false) {
+                        priv->read_fail_log = 0;
+                }
+        }
+
+        optstr = NULL;
 out:
         this->private = priv;
 
@@ -2701,18 +2771,17 @@ struct volume_options options[] = {
         },
         { .key   = { "transport.address-family",
                      "address-family" },
-          .value = {"inet", "inet6", "inet/inet6", "inet6/inet",
-                    "unix", "inet-sdp" },
+          .value = {"inet", "inet6", "unix", "inet-sdp" },
           .type  = GF_OPTION_TYPE_STR
         },
 
         { .key   = {"non-blocking-io"},
           .type  = GF_OPTION_TYPE_BOOL
         },
-        { .key   = {"transport.window-size"},
+        { .key   = {"tcp-window-size"},
           .type  = GF_OPTION_TYPE_SIZET,
           .min   = GF_MIN_SOCKET_WINDOW_SIZE,
-          .max   = GF_MAX_SOCKET_WINDOW_SIZE,
+          .max   = GF_MAX_SOCKET_WINDOW_SIZE
         },
         { .key   = {"transport.socket.nodelay"},
           .type  = GF_OPTION_TYPE_BOOL
@@ -2732,5 +2801,8 @@ struct volume_options options[] = {
         { .key   = {"transport.socket.listen-backlog"},
           .type  = GF_OPTION_TYPE_INT
         },
+        { .key   = {"transport.socket.read-fail-log"},
+          .type  = GF_OPTION_TYPE_BOOL
+        },
         { .key = {NULL} }
 };
