@@ -1,20 +1,11 @@
 /*
-  Copyright (c) 2010-2011 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+  This file is licensed to you under your choice of the GNU Lesser
+  General Public License, version 3 or any later version (LGPLv3 or
+  later), or the GNU General Public License, version 2 (GPLv2), in all
+  cases as published by the Free Software Foundation.
 */
 
 #ifndef _SOCKET_H
@@ -41,19 +32,17 @@
 
 #define RPC_MAX_FRAGMENT_SIZE 0x7fffffff
 
-/* This is the size set through setsockopt for
- * both the TCP receive window size and the
- * send buffer size.
- * Till the time iobuf size becomes configurable, this size is set to include
- * two iobufs + the GlusterFS protocol headers.
+/* The default window size will be 0, indicating not to set
+ * it to any size. Default size of Linux is found to be
+ * performance friendly.
  * Linux allows us to over-ride the max values for the system.
  * Should we over-ride them? Because if we set a value larger than the default
  * setsockopt will fail. Having larger values might be beneficial for
  * IB links.
  */
-#define GF_DEFAULT_SOCKET_WINDOW_SIZE   (512 * GF_UNIT_KB)
+#define GF_DEFAULT_SOCKET_WINDOW_SIZE   (0)
 #define GF_MAX_SOCKET_WINDOW_SIZE       (1 * GF_UNIT_MB)
-#define GF_MIN_SOCKET_WINDOW_SIZE       (128 * GF_UNIT_KB)
+#define GF_MIN_SOCKET_WINDOW_SIZE       (0)
 #define GF_USE_DEFAULT_KEEPALIVE        (-1)
 
 typedef enum {
@@ -81,6 +70,8 @@ typedef enum {
         SP_STATE_READ_CREDBYTES,        /* read credential data. */
         SP_STATE_READING_VERFBYTES,
         SP_STATE_READ_VERFBYTES,        /* read verifier data */
+        SP_STATE_READING_PROGHDR,
+        SP_STATE_READ_PROGHDR,
         SP_STATE_READING_PROG,
 } sp_rpcfrag_vectored_request_state_t;
 
@@ -112,6 +103,7 @@ struct ioq {
 typedef struct {
         sp_rpcfrag_request_header_state_t header_state;
         sp_rpcfrag_vectored_request_state_t vector_state;
+        int vector_sizer_state;
 } sp_rpcfrag_request_state_t;
 
 typedef enum {
@@ -193,6 +185,7 @@ typedef struct {
         int                    keepaliveidle;
         int                    keepaliveintvl;
         uint32_t               backlog;
+        gf_boolean_t           read_fail_log;
 } socket_private_t;
 
 
