@@ -1,20 +1,11 @@
 /*
-  Copyright (c) 2007-2011 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+  This file is licensed to you under your choice of the GNU Lesser
+  General Public License, version 3 or any later version (LGPLv3 or
+  later), or the GNU General Public License, version 2 (GPLv2), in all
+  cases as published by the Free Software Foundation.
 */
 
 
@@ -49,10 +40,9 @@
 #include "afr-self-heal.h"
 #include "afr-self-heal-common.h"
 
-
 int
 afr_examine_dir_sh_unwind (call_frame_t *frame, xlator_t *this, int32_t op_ret,
-                           int32_t op_errno)
+                           int32_t op_errno, int32_t sh_failed)
 {
         afr_local_t *local  = NULL;
 
@@ -61,7 +51,7 @@ afr_examine_dir_sh_unwind (call_frame_t *frame, xlator_t *this, int32_t op_ret,
         afr_set_opendir_done (this, local->fd->inode);
 
         AFR_STACK_UNWIND (opendir, frame, local->op_ret,
-                          local->op_errno, local->fd);
+                          local->op_errno, local->fd, NULL);
 
         return 0;
 }
@@ -100,22 +90,24 @@ __checksums_differ (uint32_t *checksum, int child_count,
 int32_t
 afr_examine_dir_readdir_cbk (call_frame_t *frame, void *cookie,
                              xlator_t *this, int32_t op_ret, int32_t op_errno,
-                             gf_dirent_t *entries)
+                             gf_dirent_t *entries, dict_t *xdata)
 {
         afr_private_t *   priv        = NULL;
         afr_local_t *     local       = NULL;
         afr_self_heal_t * sh          = NULL;
         gf_dirent_t *     entry       = NULL;
         gf_dirent_t *     tmp         = NULL;
+        char              *reason     = NULL;
         int               child_index = 0;
         uint32_t          entry_cksum = 0;
         int               call_count  = 0;
         off_t             last_offset = 0;
-        char              sh_type_str[256] = {0,};
+        inode_t           *inode      = NULL;
 
         priv  = this->private;
         local = frame->local;
         sh    = &local->self_heal;
+        inode = local->fd->inode;
 
         child_index = (long) cookie;
 
@@ -136,7 +128,7 @@ afr_examine_dir_readdir_cbk (call_frame_t *frame, void *cookie,
         }
 
         list_for_each_entry_safe (entry, tmp, &entries->list, list) {
-                entry_cksum = gf_rsync_weak_checksum (entry->d_name,
+                entry_cksum = gf_rsync_weak_checksum ((unsigned char *)entry->d_name,
                                                       strlen (entry->d_name));
                 local->cont.opendir.checksum[child_index] ^= entry_cksum;
         }
@@ -151,7 +143,7 @@ afr_examine_dir_readdir_cbk (call_frame_t *frame, void *cookie,
                            (void *) (long) child_index,
                            priv->children[child_index],
                            priv->children[child_index]->fops->readdir,
-                           local->fd, 131072, last_offset);
+                           local->fd, 131072, last_offset, NULL);
 
         return 0;
 
@@ -163,27 +155,18 @@ out:
                                         priv->child_count,
                                         local->child_up)) {
 
-                        sh->need_entry_self_heal  = _gf_true;
+                        sh->do_entry_self_heal  = _gf_true;
                         sh->forced_merge          = _gf_true;
-                        sh->type                  = local->fd->inode->ia_type;
-                        sh->background            = _gf_false;
-                        sh->unwind                = afr_examine_dir_sh_unwind;
 
-                        afr_self_heal_type_str_get(&local->self_heal,
-                                                   sh_type_str,
-                                                   sizeof(sh_type_str));
-                        gf_log (this->name, GF_LOG_INFO,
-                                "%s self-heal triggered. path: %s, "
-                                "reason: checksums of directory differ,"
-                                " forced merge option set",
-                                sh_type_str, local->loc.path);
-
-                        afr_self_heal (frame, this, local->fd->inode);
+                        reason = "checksums of directory differ";
+                        afr_launch_self_heal (frame, this, inode, _gf_false,
+                                              inode->ia_type, reason, NULL,
+                                              afr_examine_dir_sh_unwind);
                 } else {
-                        afr_set_opendir_done (this, local->fd->inode);
+                        afr_set_opendir_done (this, inode);
 
                         AFR_STACK_UNWIND (opendir, frame, local->op_ret,
-                                          local->op_errno, local->fd);
+                                          local->op_errno, local->fd, NULL);
                 }
         }
 
@@ -206,7 +189,7 @@ afr_examine_dir (call_frame_t *frame, xlator_t *this)
                                                   sizeof (*local->cont.opendir.checksum),
                                                   gf_afr_mt_int32_t);
 
-        call_count = afr_up_children_count (priv->child_count, local->child_up);
+        call_count = afr_up_children_count (local->child_up, priv->child_count);
 
         local->call_count = call_count;
 
@@ -216,7 +199,7 @@ afr_examine_dir (call_frame_t *frame, xlator_t *this)
                                            (void *) (long) i,
                                            priv->children[i],
                                            priv->children[i]->fops->readdir,
-                                           local->fd, 131072, 0);
+                                           local->fd, 131072, 0, NULL);
 
                         if (!--call_count)
                                 break;
@@ -230,27 +213,37 @@ afr_examine_dir (call_frame_t *frame, xlator_t *this)
 int32_t
 afr_opendir_cbk (call_frame_t *frame, void *cookie,
                  xlator_t *this, int32_t op_ret, int32_t op_errno,
-                 fd_t *fd)
+                 fd_t *fd, dict_t *xdata)
 {
         afr_private_t *priv              = NULL;
         afr_local_t   *local             = NULL;
         int32_t        up_children_count = 0;
         int            ret               = -1;
         int            call_count        = -1;
+        int32_t        child_index       = 0;
 
         priv  = this->private;
         local = frame->local;
+        child_index = (long) cookie;
 
-        up_children_count = afr_up_children_count (priv->child_count,
-                                                   local->child_up);
+        up_children_count = afr_up_children_count (local->child_up,
+                                                   priv->child_count);
 
         LOCK (&frame->lock);
         {
-                if (op_ret >= 0)
+                if (op_ret >= 0) {
                         local->op_ret = op_ret;
+                        ret = afr_child_fd_ctx_set (this, fd, child_index, 0);
+                        if (ret) {
+                                local->op_ret = -1;
+                                local->op_errno = -ret;
+                                goto unlock;
+                        }
+                }
 
                 local->op_errno = op_errno;
         }
+unlock:
         UNLOCK (&frame->lock);
 
         call_count = afr_frame_return (frame);
@@ -259,15 +252,6 @@ afr_opendir_cbk (call_frame_t *frame, void *cookie,
                 if (local->op_ret != 0)
                         goto out;
 
-                ret = afr_fd_ctx_set (this, local->fd);
-                if (ret) {
-                        local->op_ret = -1;
-                        local->op_errno = -1;
-                        gf_log (this->name, GF_LOG_ERROR,
-                                "failed to set fd ctx for fd %p",
-                                local->fd);
-                        goto out;
-                }
                 if (!afr_is_opendir_done (this, local->fd->inode) &&
                     up_children_count > 1) {
 
@@ -278,7 +262,7 @@ afr_opendir_cbk (call_frame_t *frame, void *cookie,
                          * to regular entry self-heal because the readdir
                          * call is sent only to the first subvolume, and
                          * thus files that exist only there will never be healed
-                         * otherwise (assuming changelog shows no anamolies).
+                         * otherwise (assuming changelog shows no anomalies).
                          */
 
                         gf_log (this->name, GF_LOG_TRACE,
@@ -297,7 +281,7 @@ afr_opendir_cbk (call_frame_t *frame, void *cookie,
 
 out:
         AFR_STACK_UNWIND (opendir, frame, local->op_ret,
-                          local->op_errno, local->fd);
+                          local->op_errno, local->fd, NULL);
 
         return 0;
 }
@@ -313,7 +297,6 @@ afr_opendir (call_frame_t *frame, xlator_t *this,
         int             i           = 0;
         int             ret         = -1;
         int             call_count  = -1;
-        int32_t         op_ret      = -1;
         int32_t         op_errno    = 0;
 
         VALIDATE_OR_GOTO (frame, out);
@@ -324,37 +307,36 @@ afr_opendir (call_frame_t *frame, xlator_t *this,
 
         child_count = priv->child_count;
 
-        ALLOC_OR_GOTO (local, afr_local_t, out);
-        ret = AFR_LOCAL_INIT (local, priv);
-        if (ret < 0) {
-                op_errno = -ret;
+        AFR_LOCAL_ALLOC_OR_GOTO (frame->local, out);
+        local = frame->local;
+
+        ret = afr_local_init (local, priv, &op_errno);
+        if (ret < 0)
                 goto out;
-        }
 
         loc_copy (&local->loc, loc);
 
-        frame->local = local;
         local->fd    = fd_ref (fd);
 
         call_count = local->call_count;
 
         for (i = 0; i < child_count; i++) {
                 if (local->child_up[i]) {
-                        STACK_WIND (frame, afr_opendir_cbk,
-                                    priv->children[i],
-                                    priv->children[i]->fops->opendir,
-                                    loc, fd);
+                        STACK_WIND_COOKIE (frame, afr_opendir_cbk,
+                                           (void*) (long) i,
+                                           priv->children[i],
+                                           priv->children[i]->fops->opendir,
+                                           loc, fd, NULL);
 
                         if (!--call_count)
                                 break;
                 }
         }
 
-        op_ret = 0;
+        ret = 0;
 out:
-        if (op_ret == -1) {
-                AFR_STACK_UNWIND (opendir, frame, op_ret, op_errno, fd);
-        }
+        if (ret < 0)
+                AFR_STACK_UNWIND (opendir, frame, -1, op_errno, fd, NULL);
 
         return 0;
 }
@@ -484,26 +466,18 @@ afr_forget_entries (fd_t *fd)
 int32_t
 afr_readdir_cbk (call_frame_t *frame, void *cookie,
                  xlator_t *this, int32_t op_ret, int32_t op_errno,
-                 gf_dirent_t *entries)
+                 gf_dirent_t *entries, dict_t *xdata)
 {
-        afr_private_t * priv        = NULL;
         afr_local_t *   local       = NULL;
         gf_dirent_t *   entry       = NULL;
         gf_dirent_t *   tmp         = NULL;
-        int             child_index = -1;
 
-        priv     = this->private;
         local = frame->local;
-        child_index = (long) cookie;
 
         if (op_ret == -1)
                 goto out;
 
         list_for_each_entry_safe (entry, tmp, &entries->list, list) {
-                entry->d_ino = afr_itransform (entry->d_ino,
-                                               priv->child_count,
-                                               child_index);
-
                 if ((local->fd->inode == local->fd->inode->table->root)
                     && !strcmp (entry->d_name, GF_REPLICATE_TRASH_DIR)) {
                         list_del_init (&entry->list);
@@ -512,7 +486,7 @@ afr_readdir_cbk (call_frame_t *frame, void *cookie,
         }
 
 out:
-        AFR_STACK_UNWIND (readdir, frame, op_ret, op_errno, entries);
+        AFR_STACK_UNWIND (readdir, frame, op_ret, op_errno, entries, NULL);
 
         return 0;
 }
@@ -520,31 +494,41 @@ out:
 
 int32_t
 afr_readdirp_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-                  int32_t op_ret, int32_t op_errno, gf_dirent_t *entries)
+                  int32_t op_ret, int32_t op_errno, gf_dirent_t *entries,
+                  dict_t *xdata)
 {
-        afr_private_t *  priv        = NULL;
-        afr_local_t *    local       = NULL;
-        xlator_t **      children    = NULL;
-        int              call_child  = 0;
-        int              first_call_child  = 0;
-        int              ret         = 0;
-        gf_dirent_t *    entry       = NULL;
-        gf_dirent_t *    tmp         = NULL;
-        uint64_t         ctx         = 0;
-        afr_fd_ctx_t    *fd_ctx      = NULL;
-        off_t            offset      = 0;
+        afr_private_t *  priv            = NULL;
+        afr_local_t *    local           = NULL;
+        xlator_t **      children        = NULL;
+        int32_t          next_call_child = -1;
+        int              ret             = 0;
+        gf_dirent_t *    entry           = NULL;
+        gf_dirent_t *    tmp             = NULL;
+        int32_t          *last_index     = NULL;
+        int32_t          read_child      = -1;
+        int32_t         *fresh_children   = NULL;
+        uint64_t         ctx             = 0;
+        afr_fd_ctx_t    *fd_ctx          = NULL;
+        off_t            offset          = 0;
+        int32_t         call_child       = -1;
 
         priv     = this->private;
         children = priv->children;
 
         local = frame->local;
 
-        first_call_child = (long) cookie;
-        if (local->cont.readdir.last_tried == -1) {
-                call_child = (long) cookie;
-        } else {
-                call_child = local->cont.readdir.last_tried;
-        }
+        read_child = (long) cookie;
+        last_index = &local->cont.readdir.last_index;
+        fresh_children = local->fresh_children;
+
+        /* the value of the last_index changes if afr_next_call_child is
+         * called. So to find the call_child of this callback use last_index
+         * before the next_call_child call.
+         */
+        if (*last_index == -1)
+                call_child = read_child;
+        else
+                call_child = fresh_children[*last_index];
 
         if (priv->strict_readdir) {
                 ret = fd_ctx_get (local->fd, this, &ctx);
@@ -558,28 +542,27 @@ afr_readdirp_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
                 fd_ctx = (afr_fd_ctx_t *)(long) ctx;
 
-                if (child_went_down (op_ret, op_errno)) {
-                        if ((call_child + 1) % priv->child_count
-                                       == first_call_child) {
-                                gf_log (this->name, GF_LOG_INFO,
-                                        "all options tried going out");
+                if (op_ret == -1) {
+                        next_call_child = afr_next_call_child (fresh_children,
+                                                               local->child_up,
+                                                               priv->child_count,
+                                                               last_index,
+                                                               read_child);
+                        if (next_call_child < 0)
                                 goto out;
-                        }
-
-                        call_child = (call_child + 1) % priv->child_count;
-                        local->cont.readdir.last_tried = call_child;
-
                         gf_log (this->name, GF_LOG_TRACE,
                                 "starting readdir afresh on child %d, offset %"PRId64,
-                                call_child, (uint64_t) 0);
+                                next_call_child, (uint64_t) 0);
 
                         fd_ctx->failed_over = _gf_true;
 
                         STACK_WIND_COOKIE (frame, afr_readdirp_cbk,
-                                           (void *) (long) first_call_child,
-                                           children[call_child],
-                                           children[call_child]->fops->readdirp, local->fd,
-                                           local->cont.readdir.size, 0);
+                                           (void *) (long) read_child,
+                                           children[next_call_child],
+                                           children[next_call_child]->fops->readdirp,
+                                           local->fd,
+                                           local->cont.readdir.size, 0,
+                                           local->cont.readdir.dict);
                         return 0;
                 }
         }
@@ -609,7 +592,7 @@ afr_readdirp_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                         if (list_empty (&entries->list)) {
                                 /* All the entries we got were duplicate. We
                                    shouldn't send an empty list now, because
-                                   that'll make the application stop reading. So
+                                   that will make the application stop reading. So
                                    try to get more entries */
 
                                 gf_log (this->name, GF_LOG_TRACE,
@@ -618,10 +601,11 @@ afr_readdirp_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                                         offset, children[call_child]->name);
 
                                 STACK_WIND_COOKIE (frame, afr_readdirp_cbk,
-                                                   (void *) (long) first_call_child,
+                                                   (void *) (long) read_child,
                                                    children[call_child],
                                                    children[call_child]->fops->readdirp,
-                                                   local->fd, local->cont.readdir.size, offset);
+                                                   local->fd, local->cont.readdir.size, offset,
+                                                   local->cont.readdir.dict);
                                 return 0;
                         }
                 } else {
@@ -630,15 +614,14 @@ afr_readdirp_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         }
 
 out:
-        AFR_STACK_UNWIND (readdirp, frame, op_ret, op_errno, entries);
+        AFR_STACK_UNWIND (readdirp, frame, op_ret, op_errno, entries, NULL);
 
         return 0;
 }
 
-
 int32_t
 afr_do_readdir (call_frame_t *frame, xlator_t *this,
-                fd_t *fd, size_t size, off_t offset, int whichop)
+                fd_t *fd, size_t size, off_t offset, int whichop, dict_t *dict)
 {
         afr_private_t *  priv       = NULL;
         xlator_t **      children   = NULL;
@@ -647,9 +630,8 @@ afr_do_readdir (call_frame_t *frame, xlator_t *this,
         uint64_t         ctx        = 0;
         afr_fd_ctx_t    *fd_ctx     = NULL;
         int              ret        = -1;
-        int32_t          op_ret     = -1;
         int32_t          op_errno   = 0;
-        int32_t          read_child = -1;
+        uint64_t         read_child = 0;
 
         VALIDATE_OR_GOTO (frame, out);
         VALIDATE_OR_GOTO (this, out);
@@ -658,33 +640,33 @@ afr_do_readdir (call_frame_t *frame, xlator_t *this,
         priv     = this->private;
         children = priv->children;
 
-        ALLOC_OR_GOTO (local, afr_local_t, out);
-        ret = AFR_LOCAL_INIT (local, priv);
+        AFR_LOCAL_ALLOC_OR_GOTO (frame->local, out);
+        local = frame->local;
+
+        ret = afr_local_init (local, priv, &op_errno);
+        if (ret < 0)
+                goto out;
+
+        local->fresh_children = afr_children_create (priv->child_count);
+        if (!local->fresh_children) {
+                op_errno = ENOMEM;
+                goto out;
+        }
+
+        read_child = afr_inode_get_read_ctx (this, fd->inode,
+                                             local->fresh_children);
+        ret = afr_get_call_child (this, local->child_up, read_child,
+                                     local->fresh_children,
+                                     &call_child,
+                                     &local->cont.readdir.last_index);
         if (ret < 0) {
                 op_errno = -ret;
                 goto out;
         }
 
-        frame->local = local;
-
-        read_child = afr_read_child (this, fd->inode);
-
-        if ((read_child >= 0) && (priv->child_up[read_child])) {
-                call_child = read_child;
-                local->cont.readdir.last_tried = -1;
-        } else {
-                call_child = afr_first_up_child (priv);
-                if (call_child == -1) {
-                        op_errno = ENOTCONN;
-                        gf_log (this->name, GF_LOG_INFO,
-                                "no child is up");
-                        goto out;
-                }
-                local->cont.readdir.last_tried = call_child;
-        }
-
         local->fd                  = fd_ref (fd);
         local->cont.readdir.size   = size;
+        local->cont.readdir.dict   = (dict)? dict_ref (dict) : NULL;
 
         if (priv->strict_readdir) {
                 ret = fd_ctx_get (fd, this, &ctx);
@@ -715,37 +697,36 @@ afr_do_readdir (call_frame_t *frame, xlator_t *this,
                                    (void *) (long) call_child,
                                    children[call_child],
                                    children[call_child]->fops->readdir, fd,
-                                   size, offset);
+                                   size, offset, dict);
         else
                 STACK_WIND_COOKIE (frame, afr_readdirp_cbk,
                                    (void *) (long) call_child,
                                    children[call_child],
                                    children[call_child]->fops->readdirp, fd,
-                                   size, offset);
+                                   size, offset, dict);
 
-        op_ret = 0;
+        ret = 0;
 out:
-        if (op_ret == -1) {
-                AFR_STACK_UNWIND (readdir, frame, op_ret, op_errno, NULL);
-        }
+        if (ret < 0)
+                AFR_STACK_UNWIND (readdir, frame, -1, op_errno, NULL, NULL);
         return 0;
 }
 
 
 int32_t
 afr_readdir (call_frame_t *frame, xlator_t *this, fd_t *fd, size_t size,
-             off_t offset)
+             off_t offset, dict_t *xdata)
 {
-        afr_do_readdir (frame, this, fd, size, offset, GF_FOP_READDIR);
+        afr_do_readdir (frame, this, fd, size, offset, GF_FOP_READDIR, xdata);
         return 0;
 }
 
 
 int32_t
 afr_readdirp (call_frame_t *frame, xlator_t *this, fd_t *fd, size_t size,
-              off_t offset)
+              off_t offset, dict_t *dict)
 {
-        afr_do_readdir (frame, this, fd, size, offset, GF_FOP_READDIRP);
+        afr_do_readdir (frame, this, fd, size, offset, GF_FOP_READDIRP, dict);
         return 0;
 }
 
