@@ -1,20 +1,11 @@
 /*
-   Copyright (c) 2008-2011 Gluster, Inc. <http://www.gluster.com>
-   This file is part of GlusterFS.
+  Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
+  This file is part of GlusterFS.
 
-   GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
-   by the Free Software Foundation; either version 3 of the License,
-   or (at your option) any later version.
-
-   GlusterFS is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program.  If not, see
-   <http://www.gnu.org/licenses/>.
+  This file is licensed to you under your choice of the GNU Lesser
+  General Public License, version 3 or any later version (LGPLv3 or
+  later), or the GNU General Public License, version 2 (GPLv2), in all
+  cases as published by the Free Software Foundation.
 */
 
 #ifndef __AFR_SELF_HEAL_H__
@@ -45,16 +36,12 @@ afr_self_heal_data (call_frame_t *frame, xlator_t *this);
 int
 afr_self_heal_metadata (call_frame_t *frame, xlator_t *this);
 
-void
-afr_self_heal_find_sources (xlator_t *this, afr_local_t *local, dict_t **xattr,
-                            afr_transaction_type transaction_type);
+int
+afr_self_heal_get_source (xlator_t *this, afr_local_t *local, dict_t **xattr);
 
 int
 afr_self_heal (call_frame_t *frame, xlator_t *this, inode_t *inode);
 
-gf_boolean_t
-afr_is_fresh_read_child (int32_t *sources, int32_t child_count,
-                         int32_t read_child);
 int
 afr_lookup_select_read_child_by_txn_type (xlator_t *this, afr_local_t *local,
                                           dict_t **xattr,
