@@ -1,20 +1,11 @@
 /*
-  Copyright (c) 2008-2011 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+  This file is licensed to you under your choice of the GNU Lesser
+  General Public License, version 3 or any later version (LGPLv3 or
+  later), or the GNU General Public License, version 2 (GPLv2), in all
+  cases as published by the Free Software Foundation.
 */
 
 
@@ -29,8 +20,10 @@
 #include "call-stub.h"
 #include "compat-errno.h"
 #include "afr-mem-types.h"
+#include "afr-self-heal-algorithm.h"
 
 #include "libxlator.h"
+#include "timer.h"
 
 #define AFR_XATTR_PREFIX "trusted.afr"
 #define AFR_PATHINFO_HEADER "REPLICATE:"
@@ -42,14 +35,69 @@ typedef int (*afr_expunge_done_cbk_t) (call_frame_t *frame, xlator_t *this,
                                        int32_t op_errno);
 
 typedef int (*afr_impunge_done_cbk_t) (call_frame_t *frame, xlator_t *this,
-                                       int child, int32_t op_error,
-                                       int32_t op_errno);
+                                       int32_t op_error, int32_t op_errno);
 typedef int (*afr_post_remove_call_t) (call_frame_t *frame, xlator_t *this);
 
 typedef int (*afr_lock_cbk_t) (call_frame_t *frame, xlator_t *this);
 typedef void (*afr_lookup_done_cbk_t) (call_frame_t *frame, xlator_t *this,
                                       int32_t op_ret, int32_t op_errno);
 
+typedef enum {
+        AFR_POS_UNKNOWN,
+        AFR_POS_LOCAL,
+        AFR_POS_REMOTE
+} afr_child_pos_t;
+
+typedef enum {
+        SPLIT_BRAIN = 1,
+        ALL_FOOLS = 2
+} afr_subvol_status_t;
+
+typedef enum {
+        AFR_INODE_SET_READ_CTX = 1,
+        AFR_INODE_RM_STALE_CHILDREN,
+        AFR_INODE_SET_OPENDIR_DONE,
+        AFR_INODE_SET_SPLIT_BRAIN,
+        AFR_INODE_GET_READ_CTX,
+        AFR_INODE_GET_OPENDIR_DONE,
+        AFR_INODE_GET_SPLIT_BRAIN,
+} afr_inode_op_t;
+
+typedef struct afr_inode_params_ {
+        afr_inode_op_t op;
+        union {
+                gf_boolean_t value;
+                struct {
+                        int32_t read_child;
+                        int32_t *children;
+                } read_ctx;
+        } u;
+} afr_inode_params_t;
+
+typedef struct afr_inode_ctx_ {
+        uint64_t masks;
+        int32_t  *fresh_children;//increasing order of latency
+} afr_inode_ctx_t;
+
+typedef enum {
+        NONE,
+        INDEX,
+        FULL,
+} afr_crawl_type_t;
+
+typedef struct afr_self_heald_ {
+        gf_boolean_t     enabled;
+        gf_boolean_t     iamshd;
+        afr_crawl_type_t *pending;
+        gf_boolean_t     *inprogress;
+        afr_child_pos_t  *pos;
+        gf_timer_t       **timer;
+        eh_t             *healed;
+        eh_t             *heal_failed;
+        eh_t             *split_brain;
+        char             *node_uuid;
+} afr_self_heald_t;
+
 typedef struct _afr_private {
         gf_lock_t lock;               /* to guard access to child_count, etc */
         unsigned int child_count;     /* total number of children   */
@@ -59,7 +107,6 @@ typedef struct _afr_private {
 
         xlator_t **children;
 
-        gf_lock_t root_inode_lk;
         int first_lookup;
         inode_t *root_inode;
 
@@ -67,7 +114,7 @@ typedef struct _afr_private {
 
         char **pending_key;
 
-        gf_boolean_t data_self_heal;              /* on/off */
+        char         *data_self_heal;              /* on/off/open */
         char *       data_self_heal_algorithm;    /* name of algorithm */
         unsigned int data_self_heal_window_size;  /* max number of pipelined
                                                      read/writes */
@@ -85,10 +132,6 @@ typedef struct _afr_private {
         int favorite_child;  /* subvolume to be preferred in resolving
                                          split-brain cases */
 
-        unsigned int data_lock_server_count;
-        unsigned int metadata_lock_server_count;
-        unsigned int entry_lock_server_count;
-
         gf_boolean_t inodelk_trace;
         gf_boolean_t entrylk_trace;
 
@@ -104,39 +147,34 @@ typedef struct _afr_private {
 
         pthread_mutex_t  mutex;
         struct list_head saved_fds;   /* list of fds on which locks have succeeded */
-        gf_boolean_t     optimistic_change_log;
+        gf_boolean_t      optimistic_change_log;
+        gf_boolean_t      eager_lock;
+        unsigned int      quorum_count;
 
         char                   vol_uuid[UUID_SIZE + 1];
         int32_t                *last_event;
+        afr_self_heald_t       shd;
 } afr_private_t;
 
 typedef struct {
         /* External interface: These are variables (some optional) that
            are set by whoever has triggered self-heal */
 
-        inode_t      *inode;
-        gf_boolean_t need_data_self_heal;
-        gf_boolean_t need_metadata_self_heal;
-        gf_boolean_t need_entry_self_heal;
-        gf_boolean_t need_gfid_self_heal;
-        gf_boolean_t need_missing_entry_self_heal;
+        gf_boolean_t do_data_self_heal;
+        gf_boolean_t do_metadata_self_heal;
+        gf_boolean_t do_entry_self_heal;
+        gf_boolean_t do_gfid_self_heal;
+        gf_boolean_t do_missing_entry_self_heal;
 
         gf_boolean_t forced_merge;        /* Is this a self-heal triggered to
                                              forcibly merge the directories? */
 
-        gf_boolean_t healing_fd_opened;   /* true if caller has already
-                                             opened fd */
-
-        gf_boolean_t data_lock_held;      /* true if caller has already
-                                             acquired 0-0 lock */
-
-        fd_t *healing_fd;                 /* set if callers has opened fd */
-
         gf_boolean_t background;          /* do self-heal in background
                                              if possible */
-
         ia_type_t type;                   /* st_mode of the entry we're doing
                                              self-heal on */
+        inode_t   *inode;                 /* inode on which the self-heal is
+                                             performed on */
         uuid_t  sh_gfid_req;                 /* gfid self-heal needs to be done
                                              with this gfid if it is not null */
 
@@ -144,7 +182,7 @@ typedef struct {
            background, this function will be called as soon as possible. */
 
         int (*unwind) (call_frame_t *frame, xlator_t *this, int32_t op_ret,
-                       int32_t op_errno);
+                       int32_t op_errno, int32_t sh_failed);
 
         /* End of external interface members */
 
@@ -157,14 +195,13 @@ typedef struct {
 
         afr_expunge_done_cbk_t expunge_done;
         afr_impunge_done_cbk_t impunge_done;
-        int32_t impunge_ret_child;
 
         /* array of xattr's, one for each child */
         dict_t **xattr;
 
         /* array containing if the lookups succeeded in the order of response
          */
-        int32_t *child_success;
+        int32_t *success_children;
         int     success_count;
         /* array containing the fresh children found in the self-heal process */
         int32_t *fresh_children;
@@ -187,46 +224,49 @@ typedef struct {
         int source;
         int active_source;
         int active_sinks;
-        int *success;
+        unsigned char *success;
         unsigned char *locked_nodes;
         int lock_count;
 
-        mode_t impunging_entry_mode;
         const char *linkname;
         gf_boolean_t entries_skipped;
 
         int   op_failed;
 
+        gf_boolean_t sync_done;
+        gf_boolean_t data_lock_held;
+        gf_boolean_t eof_reached;
+        fd_t  *healing_fd;
         int   file_has_holes;
         blksize_t block_size;
         off_t file_size;
         off_t offset;
+        unsigned char *write_needed;
+        uint8_t *checksum;
         afr_post_remove_call_t post_remove_call;
 
         loc_t parent_loc;
 
         call_frame_t *orig_frame;
+        call_frame_t *old_loop_frame;
         gf_boolean_t unwound;
 
-        /* private data for the particular self-heal algorithm */
-        void *private;
-        unsigned char *write_needed;
-
-        int (*flush_self_heal_cbk) (call_frame_t *frame, xlator_t *this);
+        afr_sh_algo_private_t *private;
 
+        struct afr_sh_algorithm  *algo;
+        afr_lock_cbk_t data_lock_success_handler;
+        afr_lock_cbk_t data_lock_failure_handler;
         int (*completion_cbk) (call_frame_t *frame, xlator_t *this);
+        int (*sh_data_algo_start) (call_frame_t *frame, xlator_t *this);
         int (*algo_completion_cbk) (call_frame_t *frame, xlator_t *this);
         int (*algo_abort_cbk) (call_frame_t *frame, xlator_t *this);
         void (*gfid_sh_success_cbk) (call_frame_t *sh_frame, xlator_t *this);
+        gf_boolean_t mdata_spb;
+        gf_boolean_t data_spb;
 
         call_frame_t *sh_frame;
 } afr_self_heal_t;
 
-typedef enum {
-        SPLIT_BRAIN = 1,
-        ALL_FOOLS = 2
-} afr_subvol_status_t;
-
 typedef enum {
         AFR_DATA_TRANSACTION,          /* truncate, write, ... */
         AFR_METADATA_TRANSACTION,      /* chmod, chown, ... */
@@ -312,12 +352,11 @@ typedef struct {
 
         uint64_t lock_number;
         int32_t lk_call_count;
+        int32_t lk_expected_count;
 
         int32_t lock_op_ret;
         int32_t lock_op_errno;
-
-        int (*lock_cbk) (call_frame_t *, xlator_t *);
-
+        afr_lock_cbk_t lock_cbk;
 } afr_internal_lock_t;
 
 typedef struct _afr_locked_fd {
@@ -331,10 +370,7 @@ typedef struct _afr_local {
         unsigned int call_count;
         unsigned int success_count;
         unsigned int enoent_count;
-
-
         unsigned int govinda_gOvinda;
-
         unsigned int read_child_index;
         unsigned char read_child_returned;
         unsigned int first_up_child;
@@ -350,10 +386,12 @@ typedef struct _afr_local {
         loc_t newloc;
 
         fd_t *fd;
+        unsigned char *fd_open_on;
 
         glusterfs_fop_t fop;
 
         unsigned char *child_up;
+        int32_t       *fresh_children; //in the order of response
 
         int32_t *child_errno;
 
@@ -371,7 +409,8 @@ typedef struct _afr_local {
         dict_t  *dict;
         int      optimistic_change_log;
 
-        int (*openfd_flush_cbk) (call_frame_t *frame, xlator_t *this);
+        gf_boolean_t    fop_paused;
+        int (*fop_call_continue) (call_frame_t *frame, xlator_t *this);
 
         /*
           This struct contains the arguments for the "continuation"
@@ -386,25 +425,24 @@ typedef struct _afr_local {
                 } statfs;
 
                 struct {
+                        uint32_t parent_entrylk;
                         uuid_t  gfid_req;
                         inode_t *inode;
                         struct iatt buf;
                         struct iatt postparent;
-                        ino_t ino;
-                        uint64_t gen;
-                        ino_t parent_ino;
                         dict_t **xattrs;
                         dict_t *xattr;
                         struct iatt *postparents;
                         struct iatt *bufs;
                         int32_t read_child;
-                        int32_t *child_success;//in the order of response
                         int32_t *sources;
+                        int32_t *success_children;
+                        gf_boolean_t fresh_lookup;
+                        gf_boolean_t possible_spb;
                 } lookup;
 
                 struct {
                         int32_t flags;
-                        int32_t wbflags;
                 } open;
 
                 struct {
@@ -418,36 +456,33 @@ typedef struct _afr_local {
 
                 struct {
                         int32_t mask;
-                        int last_tried;  /* index of the child we tried previously */
+                        int last_index;  /* index of the child we tried previously */
                 } access;
 
                 struct {
-                        int last_tried;
-                        ino_t ino;
+                        int last_index;
                 } stat;
 
                 struct {
-                        int last_tried;
-                        ino_t ino;
+                        int last_index;
                 } fstat;
 
                 struct {
                         size_t size;
-                        int last_tried;
-                        ino_t ino;
+                        int last_index;
                 } readlink;
 
                 struct {
                         char *name;
-                        int last_tried;
-                        long pathinfo_len;
+                        int last_index;
+                        long xattr_len;
                 } getxattr;
 
                 struct {
-                        ino_t ino;
                         size_t size;
                         off_t offset;
-                        int last_tried;
+                        int last_index;
+                        uint32_t flags;
                 } readv;
 
                 /* dir read */
@@ -465,26 +500,13 @@ typedef struct _afr_local {
                         int32_t op_errno;
                         size_t size;
                         off_t offset;
-
+                        dict_t *dict;
                         gf_boolean_t failed;
-                        int last_tried;
+                        int last_index;
                 } readdir;
-
-                struct {
-                        int32_t op_ret;
-                        int32_t op_errno;
-
-                        size_t size;
-                        off_t offset;
-                        int32_t flag;
-
-                        int last_tried;
-                } getdents;
-
                 /* inode write */
 
                 struct {
-                        ino_t ino;
                         struct iatt prebuf;
                         struct iatt postbuf;
 
@@ -494,30 +516,27 @@ typedef struct _afr_local {
                         struct iobref *iobref;
                         int32_t count;
                         off_t offset;
+                        uint32_t flags;
                 } writev;
 
                 struct {
-                        ino_t ino;
                         struct iatt prebuf;
                         struct iatt postbuf;
                 } fsync;
 
                 struct {
-                        ino_t ino;
                         off_t offset;
                         struct iatt prebuf;
                         struct iatt postbuf;
                 } truncate;
 
                 struct {
-                        ino_t ino;
                         off_t offset;
                         struct iatt prebuf;
                         struct iatt postbuf;
                 } ftruncate;
 
                 struct {
-                        ino_t ino;
                         struct iatt in_buf;
                         int32_t valid;
                         struct iatt preop_buf;
@@ -525,7 +544,6 @@ typedef struct _afr_local {
                 } setattr;
 
                 struct {
-                        ino_t ino;
                         struct iatt in_buf;
                         int32_t valid;
                         struct iatt preop_buf;
@@ -537,16 +555,26 @@ typedef struct _afr_local {
                         int32_t flags;
                 } setxattr;
 
+                struct {
+                        dict_t *dict;
+                        int32_t flags;
+                } fsetxattr;
+
                 struct {
                         char *name;
                 } removexattr;
 
+                struct {
+                        dict_t *xattr;
+                } xattrop;
+
+                struct {
+                        dict_t *xattr;
+                } fxattrop;
+
                 /* dir write */
 
                 struct {
-                        ino_t ino;
-                        uint64_t gen;
-                        ino_t parent_ino;
                         fd_t *fd;
                         dict_t *params;
                         int32_t flags;
@@ -559,9 +587,6 @@ typedef struct _afr_local {
                 } create;
 
                 struct {
-                        ino_t ino;
-                        uint64_t gen;
-                        ino_t parent_ino;
                         dev_t dev;
                         mode_t mode;
                         dict_t *params;
@@ -573,9 +598,6 @@ typedef struct _afr_local {
                 } mknod;
 
                 struct {
-                        ino_t ino;
-                        uint64_t gen;
-                        ino_t parent_ino;
                         int32_t mode;
                         dict_t *params;
                         inode_t *inode;
@@ -586,7 +608,6 @@ typedef struct _afr_local {
                 } mkdir;
 
                 struct {
-                        ino_t parent_ino;
                         int32_t op_ret;
                         int32_t op_errno;
                         struct iatt preparent;
@@ -595,7 +616,6 @@ typedef struct _afr_local {
 
                 struct {
                         int   flags;
-                        ino_t parent_ino;
                         int32_t op_ret;
                         int32_t op_errno;
                         struct iatt preparent;
@@ -603,9 +623,6 @@ typedef struct _afr_local {
                 } rmdir;
 
                 struct {
-                        ino_t oldparent_ino;
-                        ino_t newparent_ino;
-                        ino_t ino;
                         struct iatt buf;
                         struct iatt read_child_buf;
                         struct iatt preoldparent;
@@ -615,9 +632,6 @@ typedef struct _afr_local {
                 } rename;
 
                 struct {
-                        ino_t ino;
-                        uint64_t gen;
-                        ino_t parent_ino;
                         inode_t *inode;
                         struct iatt buf;
                         struct iatt read_child_buf;
@@ -626,9 +640,6 @@ typedef struct _afr_local {
                 } link;
 
                 struct {
-                        ino_t ino;
-                        uint64_t gen;
-                        ino_t parent_ino;
                         inode_t *inode;
                         dict_t *params;
                         struct iatt buf;
@@ -648,6 +659,8 @@ typedef struct _afr_local {
         struct {
                 off_t start, len;
 
+                int *eager_lock;
+
                 char *basename;
                 char *new_basename;
 
@@ -662,6 +675,7 @@ typedef struct _afr_local {
 
                 int last_tried;
                 int32_t *child_errno;
+                unsigned char   *pre_op;
 
                 call_frame_t *main_frame;
 
@@ -679,16 +693,35 @@ typedef struct _afr_local {
         afr_self_heal_t self_heal;
 
         struct marker_str     marker;
+
+        /* extra data for fops */
+        dict_t         *xdata_req;
+        dict_t         *xdata_rsp;
+
+        mode_t          umask;
+        int             xflag;
 } afr_local_t;
 
+typedef enum {
+        AFR_FD_NOT_OPENED,
+        AFR_FD_OPENED,
+        AFR_FD_OPENING
+} afr_fd_open_status_t;
+
+typedef struct {
+        struct list_head call_list;
+        call_frame_t    *frame;
+} afr_fd_paused_call_t;
 
 typedef struct {
         unsigned int *pre_op_done;
-        unsigned int *opened_on;     /* which subvolumes the fd is open on */
+        afr_fd_open_status_t *opened_on; /* which subvolumes the fd is open on */
         unsigned int *pre_op_piggyback;
 
+        unsigned int *lock_piggyback;
+        unsigned int *lock_acquired;
+
         int flags;
-        int32_t wbflags;
         uint64_t up_count;   /* number of CHILD_UPs this fd has seen */
         uint64_t down_count; /* number of CHILD_DOWNs this fd has seen */
 
@@ -699,19 +732,19 @@ typedef struct {
         struct list_head entries; /* needed for readdir failover */
 
         unsigned char *locked_on; /* which subvolumes locks have been successful */
+	struct list_head  paused_calls; /* queued calls while fix_open happens  */
 } afr_fd_ctx_t;
 
 
 /* try alloc and if it fails, goto label */
-#define ALLOC_OR_GOTO(var, type, label) do {                    \
-                var = GF_CALLOC (sizeof (type), 1,              \
-                                 gf_afr_mt_##type);             \
-                if (!var) {                                     \
-                        gf_log (this->name, GF_LOG_ERROR,       \
-                                "out of memory :(");            \
-                        op_errno = ENOMEM;                      \
-                        goto label;                             \
-                }                                               \
+#define AFR_LOCAL_ALLOC_OR_GOTO(var, label) do {                    \
+                var = mem_get0 (THIS->local_pool);                  \
+                if (!var) {                                         \
+                        gf_log (this->name, GF_LOG_ERROR,           \
+                                "out of memory :(");                \
+                        op_errno = ENOMEM;                          \
+                        goto label;                                 \
+                }                                                   \
         } while (0);
 
 
@@ -732,8 +765,7 @@ int
 pump_command_reply (call_frame_t *frame, xlator_t *this);
 
 int32_t
-afr_notify (xlator_t *this, int32_t event,
-            void *data, ...);
+afr_notify (xlator_t *this, int32_t event, void *data, void *data2);
 
 int
 afr_attempt_lock_recovery (xlator_t *this, int32_t child_index);
@@ -746,7 +778,7 @@ afr_mark_locked_nodes (xlator_t *this, fd_t *fd,
                        unsigned char *locked_nodes);
 
 void
-afr_set_lk_owner (call_frame_t *frame, xlator_t *this);
+afr_set_lk_owner (call_frame_t *frame, xlator_t *this, void *lk_owner);
 
 int
 afr_set_lock_number (call_frame_t *frame, xlator_t *this);
@@ -770,23 +802,37 @@ afr_blocking_lock (call_frame_t *frame, xlator_t *this);
 int
 afr_internal_lock_finish (call_frame_t *frame, xlator_t *this);
 
+void
+afr_lk_transfer_datalock (call_frame_t *dst, call_frame_t *src,
+                          unsigned int child_count);
 
 int pump_start (call_frame_t *frame, xlator_t *this);
 
 int
-afr_fd_ctx_set (xlator_t *this, fd_t *fd);
-
-uint64_t
-afr_read_child (xlator_t *this, inode_t *inode);
-
-void
-afr_set_read_child (xlator_t *this, inode_t *inode, int32_t read_child);
-
-void
-afr_build_parent_loc (loc_t *parent, loc_t *child);
+__afr_fd_ctx_set (xlator_t *this, fd_t *fd);
 
 int
-afr_up_children_count (int child_count, unsigned char *child_up);
+afr_fd_ctx_set (xlator_t *this, fd_t *fd);
+
+int32_t
+afr_inode_get_read_ctx (xlator_t *this, inode_t *inode, int32_t *fresh_children);
+
+void
+afr_inode_set_read_ctx (xlator_t *this, inode_t *inode, int32_t read_child,
+                        int32_t *fresh_children);
+
+int
+afr_build_parent_loc (loc_t *parent, loc_t *child, int32_t *op_errno);
+
+unsigned int
+afr_up_children_count (unsigned char *child_up, unsigned int child_count);
+
+unsigned int
+afr_locked_children_count (unsigned char *children, unsigned int child_count);
+
+unsigned int
+afr_pre_op_done_children_count (unsigned char *pre_op,
+                                unsigned int child_count);
 
 gf_boolean_t
 afr_is_fresh_lookup (loc_t *loc, xlator_t *this);
@@ -797,19 +843,13 @@ afr_update_loc_gfids (loc_t *loc, struct iatt *buf, struct iatt *postparent);
 int
 afr_locked_nodes_count (unsigned char *locked_nodes, int child_count);
 
-ino64_t
-afr_itransform (ino64_t ino, int child_count, int child_index);
-
-int
-afr_deitransform (ino64_t ino, int child_count);
-
 void
 afr_local_cleanup (afr_local_t *local, xlator_t *this);
 
 int
 afr_frame_return (call_frame_t *frame);
 
-uint64_t
+gf_boolean_t
 afr_is_split_brain (xlator_t *this, inode_t *inode);
 
 void
@@ -817,12 +857,12 @@ afr_set_split_brain (xlator_t *this, inode_t *inode, gf_boolean_t set);
 
 int
 afr_open (call_frame_t *frame, xlator_t *this, loc_t *loc, int32_t flags,
-          fd_t *fd, int32_t wbflags);
+          fd_t *fd, dict_t *xdata);
 
 void
 afr_set_opendir_done (xlator_t *this, inode_t *inode);
 
-uint64_t
+gf_boolean_t
 afr_is_opendir_done (xlator_t *this, inode_t *inode);
 
 void
@@ -832,7 +872,7 @@ int
 afr_cleanup_fd_ctx (xlator_t *this, fd_t *fd);
 
 int
-afr_openfd_flush (call_frame_t *frame, xlator_t *this, fd_t *fd);
+afr_launch_openfd_self_heal (call_frame_t *frame, xlator_t *this, fd_t *fd);
 
 #define AFR_STACK_UNWIND(fop, frame, params ...)                \
         do {                                                    \
@@ -844,22 +884,27 @@ afr_openfd_flush (call_frame_t *frame, xlator_t *this, fd_t *fd);
                         frame->local = NULL;                    \
                 }                                               \
                 STACK_UNWIND_STRICT (fop, frame, params);       \
-                afr_local_cleanup (__local, __this);            \
-                GF_FREE (__local);                              \
-        } while (0);
+                if (__local) {                                  \
+                        afr_local_cleanup (__local, __this);    \
+                        mem_put (__local);                      \
+                }                                               \
+        } while (0)
 
-#define AFR_STACK_DESTROY(frame)                        \
-        do {                                            \
-                afr_local_t *__local = NULL;            \
-                xlator_t    *__this = NULL;             \
-                __local = frame->local;                 \
-                __this = frame->this;                   \
-                frame->local = NULL;                    \
-                STACK_DESTROY (frame->root);            \
-                afr_local_cleanup (__local, __this);    \
-                GF_FREE (__local);                      \
+#define AFR_STACK_DESTROY(frame)                                \
+        do {                                                    \
+                afr_local_t *__local = NULL;                    \
+                xlator_t    *__this = NULL;                     \
+                __local = frame->local;                         \
+                __this = frame->this;                           \
+                frame->local = NULL;                            \
+                STACK_DESTROY (frame->root);                    \
+                if (__local) {                                  \
+                        afr_local_cleanup (__local, __this);    \
+                        mem_put (__local);                      \
+                }                                               \
         } while (0);
 
+#define AFR_NUM_CHANGE_LOGS            3 /*data + metadata + entry*/
 /* allocate and return a string that is the basename of argument */
 static inline char *
 AFR_BASENAME (const char *str)
@@ -872,151 +917,63 @@ AFR_BASENAME (const char *str)
         return __basename_str;
 }
 
-/* initialize local_t */
-static inline int
-AFR_LOCAL_INIT (afr_local_t *local, afr_private_t *priv)
-{
-        int  child_up_count = 0;
-
-        local->child_up = GF_CALLOC (sizeof (*local->child_up),
-                                     priv->child_count,
-                                     gf_afr_mt_char);
-        if (!local->child_up) {
-                return -ENOMEM;
-        }
-
-        memcpy (local->child_up, priv->child_up,
-                sizeof (*local->child_up) * priv->child_count);
-
-        child_up_count = afr_up_children_count (priv->child_count, local->child_up);
-
-        if (priv->optimistic_change_log && child_up_count == priv->child_count)
-                local->optimistic_change_log = 1;
-
-        local->call_count = afr_up_children_count (priv->child_count, local->child_up);
-        if (local->call_count == 0) {
-                gf_log (THIS->name, GF_LOG_INFO, "no subvolumes up");
-                return -ENOTCONN;
-        }
-
-        local->transaction.erase_pending = 1;
-
-        local->op_ret = -1;
-        local->op_errno = EUCLEAN;
-
-        local->internal_lock.lock_op_ret   = -1;
-        local->internal_lock.lock_op_errno = EUCLEAN;
-
-
-        return 0;
-}
-
-
-/**
- * first_up_child - return the index of the first child that is up
- */
-
-static inline int
-afr_first_up_child (afr_private_t *priv)
-{
-        xlator_t ** children = NULL;
-        int         ret      = -1;
-        int         i        = 0;
-
-        LOCK (&priv->lock);
-        {
-                children = priv->children;
-                for (i = 0; i < priv->child_count; i++) {
-                        if (priv->child_up[i]) {
-                                ret = i;
-                                break;
-                        }
-                }
-        }
-        UNLOCK (&priv->lock);
-
-        return ret;
-}
-
-
-static inline int
-afr_transaction_local_init (afr_local_t *local, afr_private_t *priv)
-{
-        int i;
-
-        local->first_up_child = afr_first_up_child (priv);
-
-        local->child_errno = GF_CALLOC (sizeof (*local->child_errno),
-                                        priv->child_count,
-                                        gf_afr_mt_int32_t);
-        if (!local->child_errno) {
-                return -ENOMEM;
-        }
-
-        local->pending = GF_CALLOC (sizeof (*local->pending),
-                                    priv->child_count,
-                                    gf_afr_mt_int32_t);
-
-        if (!local->pending) {
-                return -ENOMEM;
-        }
-
-        for (i = 0; i < priv->child_count; i++) {
-                local->pending[i] = GF_CALLOC (sizeof (*local->pending[i]),
-                                               3, /* data + metadata + entry */
-                                               gf_afr_mt_int32_t);
-                if (!local->pending[i])
-                        return -ENOMEM;
-        }
-
-        local->internal_lock.inode_locked_nodes =
-                GF_CALLOC (sizeof (*local->internal_lock.inode_locked_nodes),
-                           priv->child_count,
-                           gf_afr_mt_char);
-
-        local->internal_lock.entry_locked_nodes =
-                GF_CALLOC (sizeof (*local->internal_lock.entry_locked_nodes),
-                           priv->child_count,
-                           gf_afr_mt_char);
-
-        local->internal_lock.locked_nodes =
-                GF_CALLOC (sizeof (*local->internal_lock.locked_nodes),
-                           priv->child_count,
-                           gf_afr_mt_char);
-
-        local->internal_lock.lower_locked_nodes
-                = GF_CALLOC (sizeof (*local->internal_lock.lower_locked_nodes),
-                             priv->child_count,
-                             gf_afr_mt_char);
-
-        local->transaction.child_errno = GF_CALLOC (sizeof (*local->transaction.child_errno),
-                                                    priv->child_count,
-                                                    gf_afr_mt_int32_t);
-
-        local->internal_lock.transaction_lk_type = AFR_TRANSACTION_LK;
-
-        return 0;
-}
+int
+afr_transaction_local_init (afr_local_t *local, xlator_t *this);
 
 int32_t
 afr_marker_getxattr (call_frame_t *frame, xlator_t *this,
                      loc_t *loc, const char *name,afr_local_t *local, afr_private_t *priv );
 
+int32_t *
+afr_children_create (int32_t child_count);
+
+int
+afr_local_init (afr_local_t *local, afr_private_t *priv, int32_t *op_errno);
+
+int
+afr_internal_lock_init (afr_internal_lock_t *lk, size_t child_count,
+                        transaction_lk_type_t lk_type);
+
+int
+afr_first_up_child (unsigned char *child_up, size_t child_count);
+
+int
+afr_select_read_child_from_policy (int32_t *fresh_children, int32_t child_count,
+                                   int32_t prev_read_child,
+                                   int32_t config_read_child, int32_t *sources);
+
+void
+afr_set_read_ctx_from_policy (xlator_t *this, inode_t *inode,
+                              int32_t *fresh_children, int32_t prev_read_child,
+                              int32_t config_read_child);
+
+int32_t
+afr_get_call_child (xlator_t *this, unsigned char *child_up, int32_t read_child,
+                    int32_t *fresh_children,
+                    int32_t *call_child, int32_t *last_index);
+
+int32_t
+afr_next_call_child (int32_t *fresh_children, unsigned char *child_up,
+                     size_t child_count, int32_t *last_index,
+                     int32_t read_child);
 void
 afr_get_fresh_children (int32_t *success_children, int32_t *sources,
-                        int32_t *fresh_children, unsigned int child_count);
+                        int32_t *children, unsigned int child_count);
 void
-afr_fresh_children_add_child (int32_t *fresh_children, int32_t child,
+afr_children_add_child (int32_t *children, int32_t child,
                               int32_t child_count);
 void
-afr_reset_children (int32_t *fresh_children, int32_t child_count);
+afr_children_rm_child (int32_t *children, int32_t child,
+                             int32_t child_count);
+void
+afr_reset_children (int32_t *children, int32_t child_count);
 gf_boolean_t
 afr_error_more_important (int32_t old_errno, int32_t new_errno);
 int
 afr_errno_count (int32_t *children, int *child_errno,
                  unsigned int child_count, int32_t op_errno);
 int
-afr_get_children_count (int32_t *fresh_children, unsigned int child_count);
+afr_get_children_count (int32_t *children, unsigned int child_count);
 gf_boolean_t
 afr_is_child_present (int32_t *success_children, int32_t child_count,
                       int32_t child);
@@ -1043,6 +1000,65 @@ afr_transaction_type_get (ia_type_t ia_type);
 int32_t
 afr_resultant_errno_get (int32_t *children,
                          int *child_errno, unsigned int child_count);
-int32_t*
-afr_children_create (unsigned int child_count);
+void
+afr_inode_rm_stale_children (xlator_t *this, inode_t *inode,
+                             int32_t *stale_children);
+void
+afr_launch_self_heal (call_frame_t *frame, xlator_t *this, inode_t *inode,
+                      gf_boolean_t background, ia_type_t ia_type, char *reason,
+                      void (*gfid_sh_success_cbk) (call_frame_t *sh_frame,
+                                                   xlator_t *this),
+                      int (*unwind) (call_frame_t *frame, xlator_t *this,
+                                     int32_t op_ret, int32_t op_errno,
+                                     int32_t sh_failed));
+int
+afr_fix_open (call_frame_t *frame, xlator_t *this, afr_fd_ctx_t *fd_ctx,
+              int need_open_count, int *need_open);
+int
+afr_open_fd_fix (call_frame_t *frame, xlator_t *this, gf_boolean_t pause_fop);
+int
+afr_set_elem_count_get (unsigned char *elems, int child_count);
+
+afr_fd_ctx_t *
+afr_fd_ctx_get (fd_t *fd, xlator_t *this);
+
+gf_boolean_t
+afr_open_only_data_self_heal (char *data_self_heal);
+
+gf_boolean_t
+afr_data_self_heal_enabled (char *data_self_heal);
+
+void
+afr_set_low_priority (call_frame_t *frame);
+int
+afr_child_fd_ctx_set (xlator_t *this, fd_t *fd, int32_t child,
+                      int flags);
+
+gf_boolean_t
+afr_have_quorum (char *logname, afr_private_t *priv);
+
+void
+afr_matrix_cleanup (int32_t **pending, unsigned int m);
+
+int32_t**
+afr_matrix_create (unsigned int m, unsigned int n);
+/*
+ * Special value indicating we should use the "auto" quorum method instead of
+ * a fixed value (including zero to turn off quorum enforcement).
+ */
+#define AFR_QUORUM_AUTO INT_MAX
+
+/*
+ * Having this as a macro will make debugging a bit weirder, but does reduce
+ * the probability of functions handling this check inconsistently.
+ */
+#define QUORUM_CHECK(_func,_label) do {                                  \
+        if (priv->quorum_count && !afr_have_quorum(this->name,priv)) { \
+                gf_log(this->name,GF_LOG_WARNING,                        \
+                       "failing "#_func" due to lack of quorum");        \
+                op_errno = EROFS;                                        \
+                goto _label;                                             \
+        }                                                                \
+} while (0);
+
 #endif /* __AFR_H__ */
