@@ -25,6 +25,8 @@
 #include "config.h"
 #endif
 
+#include "error-gen-mem-types.h"
+
 #define GF_FAILURE_DEFAULT 10
 
 typedef struct {
@@ -32,6 +34,7 @@ typedef struct {
         int op_count;
         int failure_iter_no;
         char *error_no;
+        gf_boolean_t random_failure;
         gf_lock_t lock;
 } eg_t;
 
