@@ -8,18 +8,25 @@ import logging
 import signal
 import optparse
 import fcntl
+import fnmatch
 from optparse import OptionParser, SUPPRESS_HELP
 from logging import Logger
 from errno import EEXIST, ENOENT
 
+from ipaddr import IPAddress, IPNetwork
+
 from gconf import gconf
 from syncdutils import FreeObject, norm, grabpidfile, finalize, log_raise_exception
-from syncdutils import select
+from syncdutils import GsyncdError, select
 from configinterface import GConffile
 import resource
 from monitor import monitor
 
 class GLogger(Logger):
+    """Logger customizations for gsyncd.
+
+    It implements a log format similar to that of glusterfs.
+    """
 
     def makeRecord(self, name, level, *a):
         rv = Logger.makeRecord(self, name, level, *a)
@@ -51,8 +58,8 @@ class GLogger(Logger):
         logging.getLogger().handlers = []
         logging.basicConfig(**lprm)
 
-
 def startup(**kw):
+    """set up logging, pidfile grabbing, daemonization"""
     if getattr(gconf, 'pid_file', None) and kw.get('go_daemon') != 'postconn':
         if not grabpidfile():
             sys.stderr.write("pidfile is taken, exiting.\n")
@@ -72,7 +79,7 @@ def startup(**kw):
             os.dup2(dn, f.fileno())
         if getattr(gconf, 'pid_file', None):
             if not grabpidfile(gconf.pid_file + '.tmp'):
-                raise RuntimeError("cannot grap temporary pidfile")
+                raise GsyncdError("cannot grab temporary pidfile")
             os.rename(gconf.pid_file + '.tmp', gconf.pid_file)
         # wait for parent to terminate
         # so we can start up with
@@ -91,12 +98,15 @@ def startup(**kw):
             lkw['stream'] = sys.stdout
         else:
             lkw['filename'] = kw['log_file']
+
     GLogger.setup(label=kw.get('label'), **lkw)
 
     lkw.update({'saved_label': kw.get('label')})
     gconf.log_metadata = lkw
+    gconf.log_exit = True
 
 def main():
+    """main routine, signal/exception handling boilerplates"""
     signal.signal(signal.SIGTERM, lambda *a: finalize(*a, **{'exval': 1}))
     GLogger.setup()
     excont = FreeObject(exval = 0)
@@ -109,6 +119,17 @@ def main():
         finalize(exval = excont.exval)
 
 def main_i():
+    """internal main routine
+
+    parse command line, decide what action will be taken;
+    we can either:
+    - query/manipulate configuration
+    - format gsyncd urls using gsyncd's url parsing engine
+    - start service in following modes, in given stages:
+      - monitor: startup(), monitor()
+      - master: startup(), connect_remote(), connect(), service_loop()
+      - slave: startup(), connect(), service_loop()
+    """
     rconf = {'go_daemon': 'should'}
 
     def store_abs(opt, optstr, val, parser):
@@ -123,23 +144,27 @@ def main_i():
         return lambda o, oo, vx, p: store_local(o, oo, FreeObject(op=op, **dmake(vx)), p)
 
     op = OptionParser(usage="%prog [options...] <master> <slave>", version="%prog 0.0.1")
-    op.add_option('--gluster-command',     metavar='CMD',   default='glusterfs')
+    op.add_option('--gluster-command-dir', metavar='DIR',   default='')
     op.add_option('--gluster-log-file',    metavar='LOGF',  default=os.devnull, type=str, action='callback', callback=store_abs)
     op.add_option('--gluster-log-level',   metavar='LVL')
+    op.add_option('--gluster-params',      metavar='PRMS',  default='')
+    op.add_option('--gluster-cli-options', metavar='OPTS',  default='--log-file=-')
+    op.add_option('--mountbroker',         metavar='LABEL')
     op.add_option('-p', '--pid-file',      metavar='PIDF',  type=str, action='callback', callback=store_abs)
     op.add_option('-l', '--log-file',      metavar='LOGF',  type=str, action='callback', callback=store_abs)
     op.add_option('--state-file',          metavar='STATF', type=str, action='callback', callback=store_abs)
+    op.add_option('--ignore-deletes',      default=False, action='store_true')
     op.add_option('-L', '--log-level',     metavar='LVL')
     op.add_option('-r', '--remote-gsyncd', metavar='CMD',   default=os.path.abspath(sys.argv[0]))
     op.add_option('--volume-id',           metavar='UUID')
     op.add_option('--session-owner',       metavar='ID')
     op.add_option('-s', '--ssh-command',   metavar='CMD',   default='ssh')
     op.add_option('--rsync-command',       metavar='CMD',   default='rsync')
-    op.add_option('--rsync-extra',         metavar='ARGS',  default='-sS', help=SUPPRESS_HELP)
     op.add_option('--timeout',             metavar='SEC',   type=int, default=120)
     op.add_option('--connection-timeout',  metavar='SEC',   type=int, default=60, help=SUPPRESS_HELP)
     op.add_option('--sync-jobs',           metavar='N',     type=int, default=3)
     op.add_option('--turns',               metavar='N',     type=int, default=0, help=SUPPRESS_HELP)
+    op.add_option('--allow-network',       metavar='IPS',   default='')
 
     op.add_option('-c', '--config-file',   metavar='CONF',  type=str, action='callback', callback=store_local)
     # duh. need to specify dest or value will be mapped to None :S
@@ -155,7 +180,7 @@ def main_i():
         op.add_option('--config-' + a,      metavar='OPT',  type=str, dest='config', action='callback',
                       callback=store_local_obj(a, lambda vx: {'opt': vx}))
     op.add_option('--config-get-all', dest='config', action='callback', callback=store_local_obj('get', lambda vx: {'opt': None}))
-    for m in ('', '-rx'):
+    for m in ('', '-rx', '-glob'):
         # call this code 'Pythonic' eh?
         # have to define a one-shot local function to be able to inject (a value depending on the)
         # iteration variable into the inner lambda
@@ -164,12 +189,15 @@ def main_i():
                           callback=store_local_obj('set', lambda vx: {'opt': vx[0], 'val': vx[1], 'rx': rx}))
             op.add_option('--config-del' + m,   metavar='OPT',  type=str, dest='config', action='callback',
                           callback=store_local_obj('del', lambda vx: {'opt': vx, 'rx': rx}))
-        conf_mod_opt_regex_variant(not not m)
+        conf_mod_opt_regex_variant(m and m[1:] or False)
 
-    op.add_option('--canonicalize-url',        dest='do_canon', action='callback', callback=store_local_curry('raw'))
-    op.add_option('--canonicalize-escape-url', dest='do_canon', action='callback', callback=store_local_curry('escaped'))
+    op.add_option('--normalize-url',           dest='url_print', action='callback', callback=store_local_curry('normal'))
+    op.add_option('--canonicalize-url',        dest='url_print', action='callback', callback=store_local_curry('canon'))
+    op.add_option('--canonicalize-escape-url', dest='url_print', action='callback', callback=store_local_curry('canon_esc'))
 
-    tunables = [ norm(o.get_opt_string()[2:]) for o in op.option_list if o.callback in (store_abs, None) and o.get_opt_string() not in ('--version', '--help') ]
+    tunables = [ norm(o.get_opt_string()[2:]) for o in op.option_list if o.callback in (store_abs, 'store_true', None) and o.get_opt_string() not in ('--version', '--help') ]
+    remote_tunables = [ 'listen', 'go_daemon', 'timeout', 'session_owner', 'config_file' ]
+    rq_remote_tunables = { 'listen': True }
 
     # precedence for sources of values: 1) commandline, 2) cfg file, 3) defaults
     # -- for this to work out we need to tell apart defaults from explicitly set
@@ -181,21 +209,41 @@ def main_i():
     if not (len(args) == 2 or \
             (len(args) == 1 and rconf.get('listen')) or \
             (len(args) <= 2 and confdata) or \
-            rconf.get('do_canon')):
+            rconf.get('url_print')):
         sys.stderr.write("error: incorrect number of arguments\n\n")
         sys.stderr.write(op.get_usage() + "\n")
         sys.exit(1)
 
-    if getattr(confdata, 'rx', None):
+    restricted = os.getenv('_GSYNCD_RESTRICTED_')
+
+    if restricted:
+        allopts = {}
+        allopts.update(opts.__dict__)
+        allopts.update(rconf)
+        bannedtuns = set(allopts.keys()) - set(remote_tunables)
+        if bannedtuns:
+            raise GsyncdError('following tunables cannot be set with restricted SSH invocaton: ' + \
+                              ', '.join(bannedtuns))
+        for k, v in rq_remote_tunables.items():
+            if not k in allopts or allopts[k] != v:
+                raise GsyncdError('tunable %s is not set to value %s required for restricted SSH invocaton' % \
+                                  (k, v))
+
+    confrx = getattr(confdata, 'rx', None)
+    if confrx:
         # peers are regexen, don't try to parse them
+        if confrx == 'glob':
+            args = [ '\A' + fnmatch.translate(a) for a in args ]
         canon_peers = args
         namedict = {}
     else:
         rscs = [resource.parse_url(u) for u in args]
-        dc = rconf.get('do_canon')
+        dc = rconf.get('url_print')
         if dc:
             for r in rscs:
-                print(r.get_url(canonical=True, escaped=(dc=='escaped')))
+                print(r.get_url(**{'normal': {},
+                                   'canon': {'canonical': True},
+                                   'canon_esc': {'canonical': True, 'escaped': True}}[dc]))
             return
         local = remote = None
         if rscs:
@@ -203,7 +251,7 @@ def main_i():
             if len(rscs) > 1:
                 remote = rscs[1]
             if not local.can_connect_to(remote):
-                raise RuntimeError("%s cannot work with %s" % (local.path, remote and remote.path))
+                raise GsyncdError("%s cannot work with %s" % (local.path, remote and remote.path))
         pa = ([], [], [])
         urlprms = ({}, {'canonical': True}, {'canonical': True, 'escaped': True})
         for x in rscs:
@@ -237,7 +285,7 @@ def main_i():
             else:
                 sys.exit(1)
         elif not opt_ok:
-            raise RuntimeError("not a valid option: " + confdata.opt)
+            raise GsyncdError("not a valid option: " + confdata.opt)
         if confdata.op == 'get':
             gcnf.get(confdata.opt)
         elif confdata.op == 'set':
@@ -251,6 +299,22 @@ def main_i():
     gconf.__dict__.update(opts.__dict__)
     gconf.configinterface = gcnf
 
+    if restricted and gconf.allow_network:
+        ssh_conn = os.getenv('SSH_CONNECTION')
+        if not ssh_conn:
+            #legacy env var
+            ssh_conn = os.getenv('SSH_CLIENT')
+        if ssh_conn:
+            allowed_networks = [ IPNetwork(a) for a in gconf.allow_network.split(',') ]
+            client_ip = IPAddress(ssh_conn.split()[0])
+            allowed = False
+            for nw in allowed_networks:
+                if client_ip in nw:
+                    allowed = True
+                    break
+            if not allowed:
+                raise GsyncdError("client IP address is not allowed")
+
     ffd = rconf.get('feedback_fd')
     if ffd:
         fcntl.fcntl(ffd, fcntl.F_SETFD, fcntl.FD_CLOEXEC)
@@ -263,7 +327,7 @@ def main_i():
         # I have _never_ _ever_ seen such an utterly braindead
         # error condition
         if lvl2 == "Level " + lvl1:
-            raise RuntimeError('cannot recognize log level "%s"' % lvl0)
+            raise GsyncdError('cannot recognize log level "%s"' % lvl0)
         gconf.log_level = lvl2
 
     go_daemon = rconf['go_daemon']
@@ -288,6 +352,7 @@ def main_i():
         return monitor()
 
     logging.info("syncing: %s" % " -> ".join(peers))
+    resource.Popen.init_errhandler()
     if remote:
         go_daemon = remote.connect_remote(go_daemon=go_daemon)
         if go_daemon:
@@ -299,8 +364,6 @@ def main_i():
         os.close(ffd)
     local.service_loop(*[r for r in [remote] if r])
 
-    logging.info("exiting.")
-
 
 if __name__ == "__main__":
     main()
