@@ -1,15 +1,21 @@
 import os
 import sys
+import pwd
 import time
 import fcntl
 import shutil
 import logging
 from threading import Lock, Thread as baseThread
-from errno import EACCES, EAGAIN, EINTR
+from errno import EACCES, EAGAIN, EPIPE, ENOTCONN, ECONNABORTED, EINTR, errorcode
 from signal import SIGTERM, SIGKILL
 from time import sleep
 import select as oselect
 from os import waitpid as owaitpid
+try:
+    from cPickle import PickleError
+except ImportError:
+    # py 3
+    from pickle import PickleError
 
 from gconf import gconf
 
@@ -20,9 +26,12 @@ except ImportError:
     import urllib
 
 def escape(s):
+    """the chosen flavor of string escaping, used all over
+       to turn whatever data to creatable representation"""
     return urllib.quote_plus(s)
 
 def unescape(s):
+    """inverse of .escape"""
     return urllib.unquote_plus(s)
 
 def norm(s):
@@ -60,6 +69,10 @@ def update_file(path, updater, merger = lambda f: True):
                 fx.close()
 
 def grabfile(fname, content=None):
+    """open @fname + contest for its fcntl lock
+
+    @content: if given, set the file content to it
+    """
     # damn those messy open() mode codes
     fd = os.open(fname, os.O_CREAT|os.O_RDWR)
     f = os.fdopen(fd, 'r+b', 0)
@@ -83,6 +96,7 @@ def grabfile(fname, content=None):
     return f
 
 def grabpidfile(fname=None, setpid=True):
+    """.grabfile customization for pid files"""
     if not fname:
         fname = gconf.pid_file
     content = None
@@ -93,6 +107,10 @@ def grabpidfile(fname=None, setpid=True):
 final_lock = Lock()
 
 def finalize(*a, **kw):
+    """all those messy final steps we go trough upon termination
+
+    Do away with pidfile, ssh control dir and logging.
+    """
     final_lock.acquire()
     if getattr(gconf, 'pid_file', None):
         rm_pidf = gconf.pid_file_owned
@@ -120,18 +138,51 @@ def finalize(*a, **kw):
                     raise
     if gconf.ssh_ctl_dir and not gconf.cpid:
         shutil.rmtree(gconf.ssh_ctl_dir)
+    if gconf.log_exit:
+        logging.info("exiting.")
     sys.stdout.flush()
     sys.stderr.flush()
     os._exit(kw.get('exval', 0))
 
 def log_raise_exception(excont):
+    """top-level exception handler
+
+    Try to some fancy things to cover up we face with an error.
+    Translate some weird sounding but well understood exceptions
+    into human-friendly lingo
+    """
+    is_filelog = False
+    for h in logging.getLogger().handlers:
+        fno = getattr(getattr(h, 'stream', None), 'fileno', None)
+        if fno and not os.isatty(fno()):
+            is_filelog = True
+
     exc = sys.exc_info()[1]
     if isinstance(exc, SystemExit):
         excont.exval = exc.code or 0
         raise
     else:
-        logging.exception("FAIL: ")
-        sys.stderr.write("failed with %s.\n" % type(exc).__name__)
+        logtag = None
+        if isinstance(exc, GsyncdError):
+            if is_filelog:
+                logging.error(exc.message)
+            sys.stderr.write('failure: ' + exc.message + "\n")
+        elif isinstance(exc, PickleError) or isinstance(exc, EOFError) or \
+             ((isinstance(exc, OSError) or isinstance(exc, IOError)) and \
+              exc.errno == EPIPE):
+            logging.error('connection to peer is broken')
+            if hasattr(gconf, 'transport'):
+                gconf.transport.wait()
+                gconf.transport.terminate_geterr()
+        elif isinstance(exc, OSError) and exc.errno in (ENOTCONN, ECONNABORTED):
+            logging.error('glusterfs session went down [%s]', errorcode[exc.errno])
+        else:
+            logtag = "FAIL"
+        if not logtag and logging.getLogger().isEnabledFor(logging.DEBUG):
+            logtag = "FULL EXCEPTION TRACE"
+        if logtag:
+            logging.exception(logtag + ": ")
+            sys.stderr.write("failed with %s.\n" % type(exc).__name__)
         excont.exval = 1
         sys.exit(excont.exval)
 
@@ -140,11 +191,16 @@ class FreeObject(object):
     """wildcard class for which any attribute can be set"""
 
     def __init__(self, **kw):
-        for k,v in kw.iteritems():
+        for k,v in kw.items():
             setattr(self, k, v)
 
 class Thread(baseThread):
+    """thread class flavor for gsyncd
 
+    - always a daemon thread
+    - force exit for whole program if thread
+      function coughs up an exception
+    """
     def __init__(self, *a, **kw):
         tf = kw.get('target')
         if tf:
@@ -164,6 +220,39 @@ class Thread(baseThread):
 class GsyncdError(Exception):
     pass
 
+def getusername(uid = None):
+    if uid == None:
+        uid = os.geteuid()
+    return pwd.getpwuid(uid).pw_name
+
+def privileged():
+    return os.geteuid() == 0
+
+def boolify(s):
+    """
+    Generic string to boolean converter
+
+    return
+    - Quick return if string 's' is of type bool
+    - True if it's in true_list
+    - False if it's in false_list
+    - Warn if it's not present in either and return False
+    """
+    true_list  = ['true', 'yes', '1', 'on']
+    false_list = ['false', 'no', '0', 'off']
+
+    if isinstance(s, bool):
+        return s
+
+    rv = False
+    lstr = s.lower()
+    if lstr in true_list:
+        rv = True
+    elif not lstr in false_list:
+        logging.warn("Unknown string (%s) in string to boolean conversion defaulting to False\n" % (s))
+
+    return rv
+
 def eintr_wrap(func, exc, *a):
     """
     wrapper around syscalls resilient to interrupt caused
@@ -172,12 +261,13 @@ def eintr_wrap(func, exc, *a):
     while True:
         try:
             return func(*a)
-        except exc, ex:
-            if not ex[0] == EINTR:
-                raise GsyncdError(ex[1])
+        except exc:
+            ex = sys.exc_info()[1]
+            if not ex.args[0] == EINTR:
+                raise
 
 def select(*a):
     return eintr_wrap(oselect.select, oselect.error, *a)
 
-def waitpid(*a):
+def waitpid (*a):
     return eintr_wrap(owaitpid, OSError, *a)
