@@ -23,8 +23,7 @@
 #include "mem-types.h"
 
 enum gf_quota_mem_types_ {
-        gf_quota_mt_quota_local_t = gf_common_mt_end + 1,
-        gf_quota_mt_quota_priv_t,
+        gf_quota_mt_quota_priv_t = gf_common_mt_end + 1,
         gf_quota_mt_quota_inode_ctx_t,
         gf_quota_mt_loc_t,
         gf_quota_mt_char,
