@@ -60,12 +60,6 @@
                 UNLOCK (lock);                  \
         } while (0)
 
-#define QUOTA_LOCAL_ALLOC_OR_GOTO(local, type, label)           \
-        do {                                                    \
-                QUOTA_ALLOC_OR_GOTO (local, type, label);       \
-                LOCK_INIT (&local->lock);                       \
-        } while (0)
-
 #define QUOTA_ALLOC_OR_GOTO(var, type, label)           \
         do {                                            \
                 var = GF_CALLOC (sizeof (type), 1,      \
@@ -89,7 +83,7 @@
                 }                                                       \
                 STACK_UNWIND_STRICT (fop, frame, params);               \
                 quota_local_cleanup (_this, _local);                    \
-                GF_FREE (_local);                                       \
+                mem_put (_local);                                       \
         } while (0)
 
 #define QUOTA_FREE_CONTRIBUTION_NODE(_contribution)     \
@@ -125,7 +119,7 @@
 
 struct quota_dentry {
         char            *name;
-        ino_t            par;
+        uuid_t           par;
         struct list_head next;
 };
 typedef struct quota_dentry quota_dentry_t;
@@ -160,16 +154,18 @@ struct quota_local {
 typedef struct quota_local quota_local_t;
 
 struct quota_priv {
-        int64_t  timeout;
-        struct list_head limit_head;
+        int64_t           timeout;
+        struct list_head  limit_head;
+        gf_lock_t         lock;
 };
 typedef struct quota_priv quota_priv_t;
 
 struct limits {
         struct list_head  limit_list;
         char             *path;
-        int64_t          value;
+        int64_t           value;
+        uuid_t            gfid;
 };
-typedef struct limits limits_t;
+typedef struct limits     limits_t;
 
 uint64_t cn = 1;
