@@ -58,6 +58,7 @@ typedef enum glusterd_op_sm_state_ {
         GD_OP_STATE_BRICK_OP_FAILED,
         GD_OP_STATE_BRICK_COMMITTED,
         GD_OP_STATE_BRICK_COMMIT_FAILED,
+        GD_OP_STATE_ACK_DRAIN,
         GD_OP_STATE_MAX,
 } glusterd_op_sm_state_t;
 
@@ -75,6 +76,7 @@ typedef enum glusterd_op_sm_event_type_ {
         GD_OP_EVENT_UNLOCK,
         GD_OP_EVENT_START_UNLOCK,
         GD_OP_EVENT_ALL_ACK,
+        GD_OP_EVENT_LOCAL_UNLOCK_NO_RESP,
         GD_OP_EVENT_MAX
 } glusterd_op_sm_event_type_t;
 
@@ -104,29 +106,18 @@ struct glusterd_op_info_ {
         int32_t                         pending_count;
         int32_t                         brick_pending_count;
         int32_t                         op_count;
-        glusterd_op_t                   op[GD_OP_MAX];
-        glusterd_op_t                   pending_op[GD_OP_MAX];
-        glusterd_op_t                   commit_op[GD_OP_MAX];
+        glusterd_op_t                   op;
         struct list_head                op_peers;
-        void                            *op_ctx[GD_OP_MAX];
+        void                            *op_ctx;
         rpcsvc_request_t                *req;
         int32_t                         op_ret;
         int32_t                         op_errno;
-        pthread_mutex_t                 lock;
-        int32_t                         cli_op;
-        gf_boolean_t                    ctx_free[GD_OP_MAX];
         char                            *op_errstr;
         struct  list_head               pending_bricks;
 };
 
 typedef struct glusterd_op_info_ glusterd_op_info_t;
 
-struct glusterd_op_delete_volume_ctx_ {
-        char                    volume_name[GD_VOLUME_NAME_MAX];
-};
-
-typedef struct glusterd_op_delete_volume_ctx_ glusterd_op_delete_volume_ctx_t;
-
 struct glusterd_op_log_filename_ctx_ {
         char                    volume_name[GD_VOLUME_NAME_MAX];
         char                    brick[GD_VOLUME_NAME_MAX];
@@ -155,7 +146,7 @@ typedef struct glusterd_op_brick_rsp_ctx_ {
         char *op_errstr;
         dict_t *rsp_dict;
         glusterd_req_ctx_t *commit_ctx;
-        glusterd_brickinfo_t *brickinfo;
+        glusterd_pending_node_t *pending_node;
 } glusterd_op_brick_rsp_ctx_t;
 
 typedef struct glusterd_pr_brick_rsp_conv_t {
@@ -163,12 +154,18 @@ typedef struct glusterd_pr_brick_rsp_conv_t {
         dict_t *dict;
 } glusterd_pr_brick_rsp_conv_t;
 
-typedef struct glusterd_gsync_slaves {
-        char *slave;
-        char *host_uuid;
-        int   ret_status;
-        char rmt_hostname[256];
-} glusterd_gsync_slaves_t;
+typedef struct glusterd_heal_rsp_conv_ {
+        dict_t *dict;
+        glusterd_volinfo_t *volinfo;
+        xlator_t *this;
+} glusterd_heal_rsp_conv_t;
+
+typedef struct glusterd_status_rsp_conv_ {
+        int count;
+        int brick_index_max;
+        int other_count;
+        dict_t *dict;
+} glusterd_status_rsp_conv_t;
 
 typedef struct glusterd_gsync_status_temp {
         dict_t *rsp_dict;
@@ -188,19 +185,13 @@ int
 glusterd_op_sm ();
 
 int32_t
-glusterd_op_set_ctx (glusterd_op_t op, void *ctx);
+glusterd_op_set_ctx (void *ctx);
 
 int32_t
 glusterd_op_set_op (glusterd_op_t op);
 
-int32_t
-glusterd_op_clear_pending_op (glusterd_op_t op);
-
-int32_t
-glusterd_op_clear_commit_op (glusterd_op_t op);
-
 int
-glusterd_op_build_payload (glusterd_op_t op, dict_t **req);
+glusterd_op_build_payload (dict_t **req, char **op_errstr);
 
 int32_t
 glusterd_op_stage_validate (glusterd_op_t op, dict_t *req, char **op_errstr,
@@ -211,14 +202,11 @@ glusterd_op_commit_perform (glusterd_op_t op, dict_t *req, char **op_errstr,
                             dict_t* dict);
 
 void *
-glusterd_op_get_ctx (glusterd_op_t op);
+glusterd_op_get_ctx ();
 
 int32_t
 glusterd_op_set_req (rpcsvc_request_t *req);
 
-int32_t
-glusterd_op_set_cli_op (glusterd_op_t op);
-
 int32_t
 glusterd_op_send_cli_response (glusterd_op_t op, int32_t op_ret,
                                int32_t op_errno, rpcsvc_request_t *req,
@@ -227,28 +215,10 @@ int32_t
 glusterd_op_get_op ();
 
 int32_t
-glusterd_op_clear_pending_op (glusterd_op_t op);
+glusterd_op_clear_op ();
 
 int32_t
-glusterd_op_clear_commit_op (glusterd_op_t op);
-
-int32_t
-glusterd_op_clear_op (glusterd_op_t op);
-
-int32_t
-glusterd_op_free_ctx (glusterd_op_t op, void *ctx, gf_boolean_t ctx_free);
-
-int32_t
-glusterd_opinfo_unlock();
-
-int32_t
-glusterd_op_set_ctx_free (glusterd_op_t op, gf_boolean_t ctx_free);
-
-int32_t
-glusterd_op_clear_ctx_free (glusterd_op_t op);
-
-gf_boolean_t
-glusterd_op_get_ctx_free (glusterd_op_t op);
+glusterd_op_free_ctx (glusterd_op_t op, void *ctx);
 
 int
 glusterd_check_option_exists(char *optstring, char **completion);
@@ -259,7 +229,7 @@ set_xlator_option (dict_t *dict, char *key, char *value);
 void
 glusterd_do_replace_brick (void *data);
 int
-glusterd_options_reset (glusterd_volinfo_t *volinfo, int32_t is_force);
+glusterd_options_reset (glusterd_volinfo_t *volinfo, char *key, int32_t is_force);
 
 char*
 glusterd_op_sm_state_name_get (int state);
@@ -271,14 +241,17 @@ glusterd_op_bricks_select (glusterd_op_t op, dict_t *dict, char **op_errstr);
 int
 glusterd_brick_op_build_payload (glusterd_op_t op, glusterd_brickinfo_t *brickinfo,
                                  gd1_mgmt_brick_op_req **req, dict_t *dict);
+int
+glusterd_node_op_build_payload (glusterd_op_t op, gd1_mgmt_brick_op_req **req,
+                               dict_t *dict);
 int32_t
-glusterd_handle_brick_rsp (glusterd_brickinfo_t *brickinfo,
-                           glusterd_op_t op, dict_t *rsp_dict, dict_t *ctx_dict,
-                           char **op_errstr);
+glusterd_handle_brick_rsp (void *pending_entry, glusterd_op_t op,
+                           dict_t *rsp_dict, dict_t *ctx_dict, char **op_errstr,
+                           gd_node_type type);
 int32_t
 glusterd_op_init_ctx (glusterd_op_t op);
 int32_t
-glusterd_op_fini_ctx (glusterd_op_t op);
+glusterd_op_fini_ctx ();
 int32_t
 glusterd_volume_stats_read_perf (char *brick_path, int32_t blk_size,
                 int32_t blk_count, double *throughput, double *time);
@@ -304,4 +277,8 @@ glusterd_gsync_get_param_file (char *prmfile, const char *ext, char *master,
                                 char *slave, char *gl_workdir);
 int
 glusterd_check_gsync_running (glusterd_volinfo_t *volinfo, gf_boolean_t *flag);
+
+int
+glusterd_defrag_volume_node_rsp (dict_t *req_dict, dict_t *rsp_dict,
+                                 dict_t *op_ctx);
 #endif
