@@ -30,7 +30,8 @@
 #include "glusterd.h"
 #include "glusterd-utils.h"
 
-#include "portmap.h"
+#include "portmap-xdr.h"
+#include "xdr-generic.h"
 #include "protocol-common.h"
 #include "rpcsvc.h"
 
@@ -290,27 +291,6 @@ out:
         return 0;
 }
 
-
-typedef ssize_t (*gfs_serialize_t) (struct iovec outmsg, void *data);
-
-
-static int
-xdr_to_glusterfs_req (rpcsvc_request_t *req, void *arg, gfs_serialize_t sfunc)
-{
-        int                     ret = -1;
-
-        if (!req)
-                return -1;
-
-        ret = sfunc (req->msg[0], arg);
-
-        if (ret > 0)
-                ret = 0;
-
-        return ret;
-}
-
-
 int
 gluster_pmap_portbybrick (rpcsvc_request_t *req)
 {
@@ -319,7 +299,8 @@ gluster_pmap_portbybrick (rpcsvc_request_t *req)
         char                     *brick = NULL;
         int                       port = 0;
 
-        if (xdr_to_glusterfs_req (req, &args, xdr_to_pmap_port_by_brick_req)) {
+        if (!xdr_to_generic (req->msg[0], &args,
+                             (xdrproc_t)xdr_pmap_port_by_brick_req)) {
                 req->rpc_err = GARBAGE_ARGS;
                 goto fail;
         }
@@ -335,7 +316,7 @@ gluster_pmap_portbybrick (rpcsvc_request_t *req)
 
 fail:
         glusterd_submit_reply (req, &rsp, NULL, 0, NULL,
-                               (gd_serialize_t)xdr_from_pmap_port_by_brick_rsp);
+                               (xdrproc_t)xdr_pmap_port_by_brick_rsp);
         if (args.brick)
                 free (args.brick);//malloced by xdr
 
@@ -349,7 +330,8 @@ gluster_pmap_brickbyport (rpcsvc_request_t *req)
         pmap_brick_by_port_req    args = {0,};
         pmap_brick_by_port_rsp    rsp  = {0,};
 
-        if (xdr_to_glusterfs_req (req, &args, xdr_to_pmap_brick_by_port_req)) {
+        if (!xdr_to_generic (req->msg[0], &args,
+                             (xdrproc_t)xdr_pmap_brick_by_port_req)) {
                 req->rpc_err = GARBAGE_ARGS;
                 goto fail;
         }
@@ -362,7 +344,7 @@ gluster_pmap_brickbyport (rpcsvc_request_t *req)
 fail:
 
         glusterd_submit_reply (req, &rsp, NULL, 0, NULL,
-                               (gd_serialize_t)xdr_from_pmap_brick_by_port_rsp);
+                               (xdrproc_t)xdr_pmap_brick_by_port_rsp);
 
         return 0;
 }
@@ -383,7 +365,8 @@ gluster_pmap_signup (rpcsvc_request_t *req)
         pmap_signup_rsp    rsp  = {0,};
 
 
-        if (xdr_to_glusterfs_req (req, &args, xdr_to_pmap_signup_req)) {
+        if (!xdr_to_generic (req->msg[0], &args,
+                             (xdrproc_t)xdr_pmap_signup_req)) {
                 req->rpc_err = GARBAGE_ARGS;
                 goto fail;
         }
@@ -393,7 +376,7 @@ gluster_pmap_signup (rpcsvc_request_t *req)
 
 fail:
         glusterd_submit_reply (req, &rsp, NULL, 0, NULL,
-                               (gd_serialize_t)xdr_from_pmap_signup_rsp);
+                               (xdrproc_t)xdr_pmap_signup_rsp);
         if (args.brick)
                 free (args.brick);//malloced by xdr
 
@@ -408,7 +391,8 @@ gluster_pmap_signin (rpcsvc_request_t *req)
         glusterd_brickinfo_t *brickinfo = NULL;
         int                ret = -1;
 
-        if (xdr_to_glusterfs_req (req, &args, xdr_to_pmap_signin_req)) {
+        if (!xdr_to_generic (req->msg[0], &args,
+                             (xdrproc_t)xdr_pmap_signin_req)) {
                 req->rpc_err = GARBAGE_ARGS;
                 goto fail;
         }
@@ -420,7 +404,7 @@ gluster_pmap_signin (rpcsvc_request_t *req)
                                       &brickinfo);
 fail:
         glusterd_submit_reply (req, &rsp, NULL, 0, NULL,
-                               (gd_serialize_t)xdr_from_pmap_signin_rsp);
+                               (xdrproc_t)xdr_pmap_signin_rsp);
         if (args.brick)
                 free (args.brick);//malloced by xdr
 
@@ -440,7 +424,8 @@ gluster_pmap_signout (rpcsvc_request_t *req)
         int                 ret = -1;
         glusterd_brickinfo_t *brickinfo = NULL;
 
-        if (xdr_to_glusterfs_req (req, &args, xdr_to_pmap_signout_req)) {
+        if (!xdr_to_generic (req->msg[0], &args,
+                             (xdrproc_t)xdr_pmap_signout_req)) {
                 //failed to decode msg;
                 req->rpc_err = GARBAGE_ARGS;
                 goto fail;
@@ -453,7 +438,7 @@ gluster_pmap_signout (rpcsvc_request_t *req)
                                       &brickinfo);
 fail:
         glusterd_submit_reply (req, &rsp, NULL, 0, NULL,
-                               (gd_serialize_t)xdr_from_pmap_signout_rsp);
+                               (xdrproc_t)xdr_pmap_signout_rsp);
         if (args.brick)
                 free (args.brick);//malloced by xdr
 
@@ -464,17 +449,17 @@ fail:
 }
 
 rpcsvc_actor_t gluster_pmap_actors[] = {
-        [GF_PMAP_NULL] = {"NULL", GF_HNDSK_NULL, NULL, NULL, NULL },
+        [GF_PMAP_NULL] = {"NULL", GF_PMAP_NULL, NULL, NULL, NULL, 0},
         [GF_PMAP_PORTBYBRICK] = {"PORTBYBRICK", GF_PMAP_PORTBYBRICK,
-                                 gluster_pmap_portbybrick, NULL, NULL },
+                                 gluster_pmap_portbybrick, NULL, NULL, 0},
         [GF_PMAP_BRICKBYPORT] = {"BRICKBYPORT", GF_PMAP_BRICKBYPORT,
-                                 gluster_pmap_brickbyport, NULL, NULL },
+                                 gluster_pmap_brickbyport, NULL, NULL, 0},
         [GF_PMAP_SIGNIN] = {"SIGNIN", GF_PMAP_SIGNIN,
-                              gluster_pmap_signin, NULL, NULL },
+                              gluster_pmap_signin, NULL, NULL, 0},
         [GF_PMAP_SIGNOUT] = {"SIGNOUT", GF_PMAP_SIGNOUT,
-                              gluster_pmap_signout, NULL, NULL },
+                              gluster_pmap_signout, NULL, NULL, 0},
         [GF_PMAP_SIGNUP] = {"SIGNUP", GF_PMAP_SIGNUP,
-                              gluster_pmap_signup, NULL, NULL },
+                              gluster_pmap_signup, NULL, NULL, 0},
 };
 
 
