@@ -44,8 +44,6 @@
 #include "glusterd-utils.h"
 #include "glusterd-store.h"
 
-#include "glusterd1.h"
-#include "cli1.h"
 #include "rpc-clnt.h"
 #include "common-utils.h"
 
@@ -98,7 +96,7 @@ glusterd_store_mkstemp (glusterd_store_handle_t *shandle)
         GF_ASSERT (shandle->path);
 
         snprintf (tmppath, sizeof (tmppath), "%s.tmp", shandle->path);
-        fd = open (tmppath, O_RDWR | O_CREAT | O_TRUNC, 0644);
+        fd = open (tmppath, O_RDWR | O_CREAT | O_TRUNC, 0600);
         if (fd <= 0) {
                 gf_log ("glusterd", GF_LOG_ERROR, "Failed to open %s, "
                         "error: %s", tmppath, strerror (errno));
@@ -227,6 +225,41 @@ glusterd_store_brickinfopath_set (glusterd_volinfo_t *volinfo,
         snprintf (brickpath, len, "%s/%s", brickdirpath, brickfname);
 }
 
+gf_boolean_t
+glusterd_store_is_valid_brickpath (char *volname, char *brick)
+{
+        char                    brickpath[PATH_MAX] = {0};
+        glusterd_brickinfo_t    *brickinfo = NULL;
+        glusterd_volinfo_t      *volinfo = NULL;
+        int32_t                 ret = 0;
+
+        ret = glusterd_brickinfo_from_brick (brick, &brickinfo);
+        if (ret) {
+                gf_log ("", GF_LOG_WARNING, "brick path validation failed");
+                ret = 0;
+                goto out;
+        }
+        ret = glusterd_volinfo_new (&volinfo);
+        if (ret) {
+                gf_log ("", GF_LOG_WARNING, "brick path validation failed");
+                ret = 0;
+                goto out;
+        }
+        strncpy (volinfo->volname, volname, sizeof (volinfo->volname));
+        glusterd_store_brickinfopath_set (volinfo, brickinfo, brickpath,
+                                                sizeof (brickpath));
+
+        ret = (strlen (brickpath) < _POSIX_PATH_MAX);
+
+out:
+        if (brickinfo)
+                glusterd_brickinfo_delete (brickinfo);
+        if (volinfo)
+                glusterd_volinfo_delete (volinfo);
+
+        return ret;
+}
+
 int32_t
 glusterd_store_volinfo_brick_fname_write (int vol_fd,
                                          glusterd_brickinfo_t *brickinfo,
@@ -288,6 +321,10 @@ glusterd_store_brickinfo_write (int fd, glusterd_brickinfo_t *brickinfo)
         ret = glusterd_store_save_value (fd, GLUSTERD_STORE_KEY_BRICK_RDMA_PORT,
                                          value);
 
+        snprintf (value, sizeof(value), "%d", brickinfo->decommissioned);
+        ret = glusterd_store_save_value (fd, GLUSTERD_STORE_KEY_BRICK_DECOMMISSIONED,
+                                         value);
+
 out:
         gf_log ("", GF_LOG_DEBUG, "Returning %d", ret);
         return ret;
@@ -529,6 +566,8 @@ void _storeopts (dict_t *this, char *key, data_t *value, void *data)
 int32_t
 glusterd_volume_exclude_options_write (int fd, glusterd_volinfo_t *volinfo)
 {
+        char      *str   = NULL;
+
         GF_ASSERT (fd > 0);
         GF_ASSERT (volinfo);
 
@@ -556,6 +595,18 @@ glusterd_volume_exclude_options_write (int fd, glusterd_volinfo_t *volinfo)
         if (ret)
                 goto out;
 
+        snprintf (buf, sizeof (buf), "%d", volinfo->stripe_count);
+        ret = glusterd_store_save_value (fd, GLUSTERD_STORE_KEY_VOL_STRIPE_CNT,
+                                         buf);
+        if (ret)
+                goto out;
+
+        snprintf (buf, sizeof (buf), "%d", volinfo->replica_count);
+        ret = glusterd_store_save_value (fd, GLUSTERD_STORE_KEY_VOL_REPLICA_CNT,
+                                         buf);
+        if (ret)
+                goto out;
+
         snprintf (buf, sizeof (buf), "%d", volinfo->version);
         ret = glusterd_store_save_value (fd, GLUSTERD_STORE_KEY_VOL_VERSION,
                                          buf);
@@ -573,6 +624,24 @@ glusterd_volume_exclude_options_write (int fd, glusterd_volinfo_t *volinfo)
         if (ret)
                 goto out;
 
+        str = glusterd_auth_get_username (volinfo);
+        if (str) {
+                ret = glusterd_store_save_value (fd,
+                                                 GLUSTERD_STORE_KEY_USERNAME,
+                                                 str);
+                if (ret)
+                goto out;
+        }
+
+        str = glusterd_auth_get_password (volinfo);
+        if (str) {
+                ret = glusterd_store_save_value (fd,
+                                                 GLUSTERD_STORE_KEY_PASSWORD,
+                                                 str);
+                if (ret)
+                        goto out;
+        }
+
 out:
         if (ret)
                 gf_log ("", GF_LOG_ERROR, "Unable to write volume values"
@@ -640,7 +709,7 @@ glusterd_store_rbstatepath_set (glusterd_volinfo_t *volinfo, char *rbstatepath,
         char    voldirpath[PATH_MAX] = {0,};
         GF_ASSERT (volinfo);
         GF_ASSERT (rbstatepath);
-        GF_ASSERT (len >= PATH_MAX);
+        GF_ASSERT (len <= PATH_MAX);
 
         glusterd_store_voldirpath_set (volinfo, voldirpath,
                                        sizeof (voldirpath));
@@ -655,13 +724,28 @@ glusterd_store_volfpath_set (glusterd_volinfo_t *volinfo, char *volfpath,
         char    voldirpath[PATH_MAX] = {0,};
         GF_ASSERT (volinfo);
         GF_ASSERT (volfpath);
-        GF_ASSERT (len >= PATH_MAX);
+        GF_ASSERT (len <= PATH_MAX);
 
         glusterd_store_voldirpath_set (volinfo, voldirpath,
                                        sizeof (voldirpath));
         snprintf (volfpath, len, "%s/%s", voldirpath, GLUSTERD_VOLUME_INFO_FILE);
 }
 
+static void
+glusterd_store_node_state_path_set (glusterd_volinfo_t *volinfo,
+                                    char *node_statepath, size_t len)
+{
+        char    voldirpath[PATH_MAX] = {0,};
+        GF_ASSERT (volinfo);
+        GF_ASSERT (node_statepath);
+        GF_ASSERT (len <= PATH_MAX);
+
+        glusterd_store_voldirpath_set (volinfo, voldirpath,
+                                       sizeof (voldirpath));
+        snprintf (node_statepath, len, "%s/%s", voldirpath,
+                  GLUSTERD_NODE_STATE_FILE);
+}
+
 int32_t
 glusterd_store_create_rbstate_shandle_on_absence (glusterd_volinfo_t *volinfo)
 {
@@ -690,6 +774,23 @@ glusterd_store_create_vol_shandle_on_absence (glusterd_volinfo_t *volinfo)
         return ret;
 }
 
+int32_t
+glusterd_store_create_nodestate_sh_on_absence (glusterd_volinfo_t *volinfo)
+{
+        char            node_state_path[PATH_MAX] = {0};
+        int32_t         ret                   = 0;
+
+        GF_ASSERT (volinfo);
+
+        glusterd_store_node_state_path_set (volinfo, node_state_path,
+                                            sizeof (node_state_path));
+        ret =
+          glusterd_store_handle_create_on_absence (&volinfo->node_state_shandle,
+                                                   node_state_path);
+
+        return ret;
+}
+
 int32_t
 glusterd_store_brickinfos (glusterd_volinfo_t *volinfo, int vol_fd)
 {
@@ -778,6 +879,56 @@ out:
         return ret;
 }
 
+int32_t
+glusterd_store_node_state_write (int fd, glusterd_volinfo_t *volinfo)
+{
+        int     ret             = -1;
+        char    buf[PATH_MAX]   = {0, };
+
+        GF_ASSERT (fd > 0);
+        GF_ASSERT (volinfo);
+
+        if (volinfo->defrag_cmd == GF_DEFRAG_CMD_STATUS) {
+                ret = 0;
+                goto out;
+        }
+
+        snprintf (buf, sizeof (buf), "%d", volinfo->defrag_cmd);
+        ret = glusterd_store_save_value (fd, GLUSTERD_STORE_KEY_VOL_DEFRAG,
+                                         buf);
+out:
+        gf_log (THIS->name, GF_LOG_DEBUG, "Returning %d", ret);
+        return ret;
+}
+
+int32_t
+glusterd_store_perform_node_state_store (glusterd_volinfo_t *volinfo)
+{
+        int                         fd = -1;
+        int32_t                     ret = -1;
+        GF_ASSERT (volinfo);
+
+        fd = glusterd_store_mkstemp (volinfo->node_state_shandle);
+        if (fd <= 0) {
+                ret = -1;
+                goto out;
+        }
+
+        ret = glusterd_store_node_state_write (fd, volinfo);
+        if (ret)
+                goto out;
+
+        ret = glusterd_store_rename_tmppath (volinfo->node_state_shandle);
+
+out:
+        if (ret && (fd > 0))
+                glusterd_store_unlink_tmppath (volinfo->node_state_shandle);
+        if (fd > 0)
+                close (fd);
+        gf_log ("", GF_LOG_DEBUG, "Returning %d", ret);
+        return ret;
+}
+
 int32_t
 glusterd_store_perform_volume_store (glusterd_volinfo_t *volinfo)
 {
@@ -844,6 +995,10 @@ glusterd_store_volinfo (glusterd_volinfo_t *volinfo, glusterd_volinfo_ver_ac_t a
         if (ret)
                 goto out;
 
+        ret = glusterd_store_create_nodestate_sh_on_absence (volinfo);
+        if (ret)
+                goto out;
+
         ret = glusterd_store_perform_volume_store (volinfo);
         if (ret)
                 goto out;
@@ -852,6 +1007,10 @@ glusterd_store_volinfo (glusterd_volinfo_t *volinfo, glusterd_volinfo_ver_ac_t a
         if (ret)
                 goto out;
 
+        ret = glusterd_store_perform_node_state_store (volinfo);
+        if (ret)
+                goto out;
+
         //checksum should be computed at the end
         ret = glusterd_volume_compute_cksum (volinfo);
         if (ret)
@@ -957,7 +1116,7 @@ glusterd_store_read_and_tokenize (FILE *file, char *str,
         GF_ASSERT (store_errno);
 
         ret = fscanf (file, "%s", str);
-        if (ret <= 0) {
+        if (ret <= 0 || feof (file)) {
                 ret = -1;
                 *store_errno = GD_STORE_EOF;
                 goto out;
@@ -999,12 +1158,17 @@ glusterd_store_retrieve_value (glusterd_store_handle_t *handle,
 
         handle->fd = open (handle->path, O_RDWR);
 
+        if (handle->fd == -1) {
+                gf_log ("", GF_LOG_ERROR, "Unable to open file %s errno: %s",
+                        handle->path, strerror (errno));
+                goto out;
+        }
         if (!handle->read)
                 handle->read = fdopen (handle->fd, "r");
 
         if (!handle->read) {
-                gf_log ("", GF_LOG_ERROR, "Unable to open file %s errno: %d",
-                        handle->path, errno);
+                gf_log ("", GF_LOG_ERROR, "Unable to open file %s errno: %s",
+                        handle->path, strerror (errno));
                 goto out;
         }
 
@@ -1061,29 +1225,38 @@ int32_t
 glusterd_store_save_value (int fd, char *key, char *value)
 {
         int32_t         ret = -1;
+        FILE           *fp  = NULL;
 
         GF_ASSERT (fd > 0);
         GF_ASSERT (key);
         GF_ASSERT (value);
 
-        ret = write (fd, key, strlen (key));
-        if (ret > 0)
-                ret = write (fd, "=", strlen ("="));
-        if (ret > 0)
-                ret = write (fd, value, strlen (value));
-        if (ret > 0)
-                ret = write (fd, "\n", strlen ("\n"));
+        fp = fdopen (fd, "a+");
+        if (fp == NULL) {
+                gf_log ("", GF_LOG_WARNING, "fdopen failed.");
+                ret = -1;
+                goto out;
+        }
 
-        if (ret <= 0) {
-                gf_log ("", GF_LOG_CRITICAL, "Unable to store key: %s,"
+        ret = fprintf (fp, "%s=%s\n", key, value);
+        if (ret < 0) {
+                gf_log ("", GF_LOG_WARNING, "Unable to store key: %s,"
                         "value: %s, error: %s", key, value,
                         strerror (errno));
                 ret = -1;
                 goto out;
         }
 
-        ret = 0;
+        ret = fflush (fp);
+        if (feof (fp)) {
+                gf_log ("", GF_LOG_WARNING,
+                        "fflush failed, error: %s",
+                        strerror (errno));
+                ret = -1;
+                goto out;
+        }
 
+        ret = 0;
 out:
 
         gf_log ("", GF_LOG_DEBUG, "returning: %d", ret);
@@ -1107,7 +1280,7 @@ glusterd_store_handle_new (char *path, glusterd_store_handle_t **handle)
         if (!spath)
                 goto out;
 
-        fd = open (path, O_RDWR | O_CREAT | O_APPEND, 0644);
+        fd = open (path, O_RDWR | O_CREAT | O_APPEND, 0600);
         if (fd <= 0) {
                 gf_log ("glusterd", GF_LOG_ERROR, "Failed to open file: %s, "
                         "error: %s", path, strerror (errno));
@@ -1136,16 +1309,13 @@ out:
 }
 
 int
-glusterd_store_handle_retrieve (char *path, glusterd_store_handle_t **handle,
-                                int *op_errno)
+glusterd_store_handle_retrieve (char *path, glusterd_store_handle_t **handle)
 {
         int32_t                 ret = -1;
         struct stat statbuf = {0};
 
         ret = stat (path, &statbuf);
         if (ret) {
-                if (op_errno)
-                        *op_errno = errno;
                 gf_log ("glusterd", GF_LOG_ERROR, "Unable to retrieve store "
                         "handle for %s, error: %s", path, strerror (errno));
                 goto out;
@@ -1185,8 +1355,10 @@ glusterd_store_uuid ()
         char            path[PATH_MAX] = {0,};
         int32_t         ret = -1;
         glusterd_store_handle_t *handle = NULL;
+        xlator_t       *this    = NULL;
 
-        priv = THIS->private;
+        this = THIS;
+        priv = this->private;
 
         snprintf (path, PATH_MAX, "%s/%s", priv->workdir,
                   GLUSTERD_INFO_FILE);
@@ -1195,8 +1367,8 @@ glusterd_store_uuid ()
                 ret = glusterd_store_handle_new (path, &handle);
 
                 if (ret) {
-                        gf_log ("", GF_LOG_ERROR, "Unable to get store"
-                                " handle!");
+                        gf_log (this->name, GF_LOG_ERROR,
+                                "Unable to get store handle!");
                         goto out;
                 }
 
@@ -1205,6 +1377,14 @@ glusterd_store_uuid ()
                 handle = priv->handle;
         }
 
+        /* make glusterd's uuid available for users */
+        ret = chmod (handle->path, 0644);
+        if (ret) {
+                gf_log (this->name, GF_LOG_ERROR, "chmod error for %s: %s",
+                        GLUSTERD_INFO_FILE, strerror (errno));
+                goto out;
+        }
+
         handle->fd = open (handle->path, O_RDWR | O_CREAT | O_TRUNC, 0644);
         if (handle->fd <= 0) {
                 ret = -1;
@@ -1214,8 +1394,8 @@ glusterd_store_uuid ()
                                          uuid_utoa (priv->uuid));
 
         if (ret) {
-                gf_log ("", GF_LOG_CRITICAL, "Storing uuid failed"
-                        "ret = %d", ret);
+                gf_log (this->name, GF_LOG_CRITICAL,
+                        "Storing uuid failed ret = %d", ret);
                 goto out;
         }
 
@@ -1225,7 +1405,7 @@ out:
                 close (handle->fd);
                 handle->fd = 0;
         }
-        gf_log ("", GF_LOG_DEBUG, "Returning %d", ret);
+        gf_log (this->name, GF_LOG_DEBUG, "Returning %d", ret);
         return ret;
 }
 
@@ -1244,7 +1424,7 @@ glusterd_retrieve_uuid ()
         if (!priv->handle) {
                 snprintf (path, PATH_MAX, "%s/%s", priv->workdir,
                           GLUSTERD_INFO_FILE);
-                ret = glusterd_store_handle_retrieve (path, &handle, NULL);
+                ret = glusterd_store_handle_retrieve (path, &handle);
 
                 if (ret) {
                         gf_log ("", GF_LOG_ERROR, "Unable to get store "
@@ -1550,7 +1730,7 @@ glusterd_store_retrieve_bricks (glusterd_volinfo_t *volinfo)
 
                 tmpvalue = NULL;
 
-                ret = glusterd_store_handle_retrieve (path, &brickinfo->shandle, NULL);
+                ret = glusterd_store_handle_retrieve (path, &brickinfo->shandle);
 
                 if (ret)
                         goto out;
@@ -1592,6 +1772,9 @@ glusterd_store_retrieve_bricks (glusterd_volinfo_t *volinfo)
                                 pmap = pmap_registry_get (THIS);
                                 if (pmap->last_alloc <= brickinfo->rdma_port)
                                         pmap->last_alloc = brickinfo->rdma_port + 1;
+                        } else if (!strncmp (key, GLUSTERD_STORE_KEY_BRICK_DECOMMISSIONED,
+                                             strlen (GLUSTERD_STORE_KEY_BRICK_DECOMMISSIONED))) {
+                                gf_string2int (value, &brickinfo->decommissioned);
                         } else {
                                 gf_log ("", GF_LOG_ERROR, "Unknown key: %s",
                                         key);
@@ -1628,7 +1811,7 @@ out:
 
 
 int32_t
-glusterd_store_retrieve_rbstate (char   *volname, int *rb_errno)
+glusterd_store_retrieve_rbstate (char   *volname)
 {
         int32_t                   ret                   = -1;
         glusterd_volinfo_t        *volinfo              = NULL;
@@ -1653,7 +1836,7 @@ glusterd_store_retrieve_rbstate (char   *volname, int *rb_errno)
         snprintf (path, sizeof (path), "%s/%s", volpath,
                   GLUSTERD_VOLUME_RBSTATE_FILE);
 
-        ret = glusterd_store_handle_retrieve (path, &volinfo->rb_shandle, rb_errno);
+        ret = glusterd_store_handle_retrieve (path, &volinfo->rb_shandle);
 
         if (ret)
                 goto out;
@@ -1712,6 +1895,68 @@ out:
         return ret;
 }
 
+int32_t
+glusterd_store_retrieve_node_state (char   *volname)
+{
+        int32_t                   ret                   = -1;
+        glusterd_volinfo_t        *volinfo              = NULL;
+        glusterd_store_iter_t     *iter                 = NULL;
+        char                      *key                  = NULL;
+        char                      *value                = NULL;
+        char                      volpath[PATH_MAX]     = {0,};
+        glusterd_conf_t           *priv                 = NULL;
+        char                      path[PATH_MAX]        = {0,};
+        glusterd_store_op_errno_t op_errno              = GD_STORE_SUCCESS;
+
+        priv = THIS->private;
+
+        ret = glusterd_volinfo_find (volname, &volinfo);
+        if (ret) {
+                gf_log (THIS->name, GF_LOG_ERROR, "Couldn't get"
+                        "volinfo for %s.", volname);
+                goto out;
+        }
+
+        GLUSTERD_GET_VOLUME_DIR(volpath, volinfo, priv);
+        snprintf (path, sizeof (path), "%s/%s", volpath,
+                  GLUSTERD_NODE_STATE_FILE);
+
+        ret = glusterd_store_handle_retrieve (path,
+                                              &volinfo->node_state_shandle);
+
+        if (ret)
+                goto out;
+
+        ret = glusterd_store_iter_new (volinfo->node_state_shandle, &iter);
+
+        if (ret)
+                goto out;
+
+        ret = glusterd_store_iter_get_next (iter, &key, &value, &op_errno);
+        if (ret)
+                goto out;
+        if (!strncmp (key, GLUSTERD_STORE_KEY_VOL_DEFRAG,
+                       strlen (GLUSTERD_STORE_KEY_VOL_DEFRAG))) {
+                 volinfo->defrag_cmd = atoi (value);
+        }
+
+        GF_FREE (key);
+        GF_FREE (value);
+
+        if (op_errno != GD_STORE_EOF)
+                goto out;
+
+        ret = glusterd_store_iter_destroy (iter);
+
+        if (ret)
+                goto out;
+
+out:
+        gf_log ("", GF_LOG_DEBUG, "Returning with %d", ret);
+
+        return ret;
+}
+
 int32_t
 glusterd_store_retrieve_volume (char    *volname)
 {
@@ -1739,7 +1984,7 @@ glusterd_store_retrieve_volume (char    *volname)
         snprintf (path, sizeof (path), "%s/%s", volpath,
                   GLUSTERD_VOLUME_INFO_FILE);
 
-        ret = glusterd_store_handle_retrieve (path, &volinfo->shandle, NULL);
+        ret = glusterd_store_handle_retrieve (path, &volinfo->shandle);
 
         if (ret)
                 goto out;
@@ -1772,6 +2017,12 @@ glusterd_store_retrieve_volume (char    *volname)
                 } else if (!strncmp (key, GLUSTERD_STORE_KEY_VOL_SUB_COUNT,
                                      strlen (GLUSTERD_STORE_KEY_VOL_SUB_COUNT))) {
                         volinfo->sub_count = atoi (value);
+                } else if (!strncmp (key, GLUSTERD_STORE_KEY_VOL_STRIPE_CNT,
+                                     strlen (GLUSTERD_STORE_KEY_VOL_STRIPE_CNT))) {
+                        volinfo->stripe_count = atoi (value);
+                } else if (!strncmp (key, GLUSTERD_STORE_KEY_VOL_REPLICA_CNT,
+                                     strlen (GLUSTERD_STORE_KEY_VOL_REPLICA_CNT))) {
+                        volinfo->replica_count = atoi (value);
                 } else if (!strncmp (key, GLUSTERD_STORE_KEY_VOL_TRANSPORT,
                                      strlen (GLUSTERD_STORE_KEY_VOL_TRANSPORT))) {
                         volinfo->transport_type = atoi (value);
@@ -1786,6 +2037,16 @@ glusterd_store_retrieve_volume (char    *volname)
                                 gf_log ("", GF_LOG_WARNING,
                                         "failed to parse uuid");
 
+                } else if (!strncmp (key, GLUSTERD_STORE_KEY_USERNAME,
+                                     strlen (GLUSTERD_STORE_KEY_USERNAME))) {
+
+                        glusterd_auth_set_username (volinfo, value);
+
+                } else if (!strncmp (key, GLUSTERD_STORE_KEY_PASSWORD,
+                                     strlen (GLUSTERD_STORE_KEY_PASSWORD))) {
+
+                        glusterd_auth_set_password (volinfo, value);
+
                 } else if (strstr (key, "slave")) {
                         ret = dict_set_dynstr (volinfo->gsync_slaves, key,
                                                 gf_strdup (value));
@@ -1796,8 +2057,7 @@ glusterd_store_retrieve_volume (char    *volname)
                         }
                         gf_log ("", GF_LOG_DEBUG, "Parsed as "GEOREP" "
                                 " slave:key=%s,value:%s", key, value);
-                }
-                else {
+                } else {
                         exists = glusterd_check_option_exists (key, NULL);
                         if (exists == -1) {
                                 ret = -1;
@@ -1828,6 +2088,42 @@ glusterd_store_retrieve_volume (char    *volname)
                 ret = glusterd_store_iter_get_next (iter, &key, &value,
                                                     &op_errno);
         }
+
+        /* backward compatibility */
+        {
+
+                switch (volinfo->type) {
+
+                        case GF_CLUSTER_TYPE_NONE:
+                                volinfo->stripe_count  = 1;
+                                volinfo->replica_count = 1;
+                        break;
+
+                        case GF_CLUSTER_TYPE_STRIPE:
+                                volinfo->stripe_count  = volinfo->sub_count;
+                                volinfo->replica_count = 1;
+                        break;
+
+                        case GF_CLUSTER_TYPE_REPLICATE:
+                                volinfo->stripe_count  = 1;
+                                volinfo->replica_count = volinfo->sub_count;
+                        break;
+
+                        case GF_CLUSTER_TYPE_STRIPE_REPLICATE:
+                                /* Introduced in 3.3 */
+                                GF_ASSERT (volinfo->stripe_count > 0);
+                                GF_ASSERT (volinfo->replica_count > 0);
+                        break;
+
+                        default:
+                                GF_ASSERT (0);
+                        break;
+                }
+
+                volinfo->dist_leaf_count = (volinfo->stripe_count *
+                                            volinfo->replica_count);
+        }
+
         if (op_errno != GD_STORE_EOF)
                 goto out;
 
@@ -1862,7 +2158,6 @@ glusterd_store_retrieve_volumes (xlator_t  *this)
         DIR                   *dir              = NULL;
         struct dirent         *entry            = NULL;
         glusterd_volinfo_t    *volinfo          = NULL;
-        int                    rb_errno         = 0;
 
         GF_ASSERT (this);
         priv = this->private;
@@ -1890,20 +2185,26 @@ glusterd_store_retrieve_volumes (xlator_t  *this)
                         goto out;
                 }
 
-                ret = glusterd_store_retrieve_rbstate (entry->d_name,
-                                                       &rb_errno);
+                ret = glusterd_store_retrieve_rbstate (entry->d_name);
                 if (ret) {
                         /* Backward compatibility */
-                        if (rb_errno == ENOENT) {
-                                gf_log ("", GF_LOG_INFO, "Creating a new rbstate "
-                                        "for volume: %s.", entry->d_name);
-                                ret = glusterd_volinfo_find (entry->d_name, &volinfo);
-                                ret = glusterd_store_create_rbstate_shandle_on_absence (volinfo);
-                                ret = glusterd_store_perform_rbstate_store (volinfo);
-                        } else {
-                                gf_log ("", GF_LOG_CRITICAL, "Unable to retrieve rbstate.");
-                                goto out;
-                        }
+                        gf_log ("", GF_LOG_INFO, "Creating a new rbstate "
+                                "for volume: %s.", entry->d_name);
+                        ret = glusterd_volinfo_find (entry->d_name, &volinfo);
+                        ret = glusterd_store_create_rbstate_shandle_on_absence (volinfo);
+                        ret = glusterd_store_perform_rbstate_store (volinfo);
+                }
+
+                ret = glusterd_store_retrieve_node_state (entry->d_name);
+                if (ret) {
+                        /* Backward compatibility */
+                        gf_log ("", GF_LOG_INFO, "Creating a new node_state "
+                                "for volume: %s.", entry->d_name);
+                        ret = glusterd_volinfo_find (entry->d_name, &volinfo);
+                        ret =
+                        glusterd_store_create_nodestate_sh_on_absence (volinfo);
+                        ret = glusterd_store_perform_node_state_store (volinfo);
+
                 }
                 glusterd_for_each_entry (entry, dir);
         }
@@ -2199,7 +2500,7 @@ glusterd_store_retrieve_peers (xlator_t *this)
 
         while (entry) {
                 snprintf (filepath, PATH_MAX, "%s/%s", path, entry->d_name);
-                ret = glusterd_store_handle_retrieve (filepath, &shandle, NULL);
+                ret = glusterd_store_handle_retrieve (filepath, &shandle);
                 if (ret)
                         goto out;
 
@@ -2246,7 +2547,7 @@ glusterd_store_retrieve_peers (xlator_t *this)
 
                 args.mode = GD_MODE_SWITCH_ON;
                 ret = glusterd_friend_add (hostname, 0, state, &uuid,
-                                           NULL, &peerinfo, 1, &args);
+                                           &peerinfo, 1, &args);
 
                 GF_FREE (hostname);
                 if (ret)
