@@ -30,6 +30,7 @@
 
 #include "glusterfs.h"
 #include "xlator.h"
+#include "run.h"
 #include "logging.h"
 #include "call-stub.h"
 #include "fd.h"
@@ -50,6 +51,8 @@ typedef enum glusterd_store_ver_ac_{
 #define GLUSTERD_STORE_KEY_VOL_STATUS     "status"
 #define GLUSTERD_STORE_KEY_VOL_PORT       "port"
 #define GLUSTERD_STORE_KEY_VOL_SUB_COUNT  "sub_count"
+#define GLUSTERD_STORE_KEY_VOL_STRIPE_CNT  "stripe_count"
+#define GLUSTERD_STORE_KEY_VOL_REPLICA_CNT "replica_count"
 #define GLUSTERD_STORE_KEY_VOL_BRICK      "brick"
 #define GLUSTERD_STORE_KEY_VOL_VERSION    "version"
 #define GLUSTERD_STORE_KEY_VOL_TRANSPORT  "transport-type"
@@ -57,11 +60,15 @@ typedef enum glusterd_store_ver_ac_{
 #define GLUSTERD_STORE_KEY_RB_STATUS      "rb_status"
 #define GLUSTERD_STORE_KEY_RB_SRC_BRICK   "rb_src"
 #define GLUSTERD_STORE_KEY_RB_DST_BRICK   "rb_dst"
+#define GLUSTERD_STORE_KEY_VOL_DEFRAG     "rebalance_status"
+#define GLUSTERD_STORE_KEY_USERNAME       "username"
+#define GLUSTERD_STORE_KEY_PASSWORD       "password"
 
 #define GLUSTERD_STORE_KEY_BRICK_HOSTNAME "hostname"
 #define GLUSTERD_STORE_KEY_BRICK_PATH     "path"
 #define GLUSTERD_STORE_KEY_BRICK_PORT     "listen-port"
 #define GLUSTERD_STORE_KEY_BRICK_RDMA_PORT "rdma.listen-port"
+#define GLUSTERD_STORE_KEY_BRICK_DECOMMISSIONED "decommissioned"
 
 #define GLUSTERD_STORE_KEY_PEER_UUID      "uuid"
 #define GLUSTERD_STORE_KEY_PEER_HOSTNAME  "hostname"
@@ -79,6 +86,7 @@ typedef enum glusterd_store_ver_ac_{
                 }\
         } while (0); \
 
+
 typedef enum {
         GD_STORE_SUCCESS,
         GD_STORE_KEY_NULL,
@@ -130,4 +138,9 @@ glusterd_restore ();
 void
 glusterd_perform_volinfo_version_action (glusterd_volinfo_t *volinfo,
                                          glusterd_volinfo_ver_ac_t ac);
+gf_boolean_t
+glusterd_store_is_valid_brickpath (char *volname, char *brick);
+
+int32_t
+glusterd_store_perform_node_state_store (glusterd_volinfo_t *volinfo);
 #endif
