@@ -2,7 +2,7 @@
   Copyright (c) 2006-2011 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
-  GlusterFS is GF_FREE software; you can redistribute it and/or modify
+  GlusterFS is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
@@ -23,6 +23,7 @@
 #include "config.h"
 #endif
 #include <time.h>
+#include <grp.h>
 #include <sys/uio.h>
 #include <sys/resource.h>
 
@@ -43,17 +44,23 @@
 #include "glusterd-sm.h"
 #include "glusterd-op-sm.h"
 #include "glusterd-store.h"
+#include "glusterd-hooks.h"
 #include "glusterd-utils.h"
 #include "common-utils.h"
+#include "run.h"
+
+#include "syncop.h"
+
+#include "glusterd-mountbroker.h"
 
 static uuid_t glusterd_uuid;
-extern struct rpcsvc_program glusterd1_mop_prog;
-extern struct rpcsvc_program gd_svc_mgmt_prog;
-extern struct rpcsvc_program gd_svc_cli_prog;
 extern struct rpcsvc_program gluster_handshake_prog;
 extern struct rpcsvc_program gluster_pmap_prog;
 extern glusterd_op_info_t opinfo;
-extern struct rpc_clnt_program glusterd_glusterfs_3_1_mgmt_prog;
+extern struct rpcsvc_program gd_svc_mgmt_prog;
+extern struct rpcsvc_program gd_svc_peer_prog;
+extern struct rpcsvc_program gd_svc_cli_prog;
+extern struct rpc_clnt_program gd_brick_prog;
 
 rpcsvc_cbk_program_t glusterd_cbk_prog = {
         .progname  = "Gluster Callback",
@@ -67,7 +74,7 @@ glusterd_opinfo_init ()
 {
         int32_t ret = -1;
 
-        ret = pthread_mutex_init (&opinfo.lock, NULL);
+        opinfo.op = GD_OP_NONE;
 
         return ret;
 }
@@ -221,12 +228,6 @@ glusterd_rpcsvc_options_build (dict_t *options)
         int             ret = 0;
         uint32_t        backlog = 0;
 
-        if (!dict_get (options, "rpc-auth-allow-insecure")) {
-                ret = dict_set_str (options, "rpc-auth-allow-insecure", "on");
-                if (ret)
-                        goto out;
-        }
-
         ret = dict_get_uint32 (options, "transport.socket.listen-backlog",
                                &backlog);
 
@@ -245,69 +246,91 @@ out:
         return ret;
 }
 
-/* defined in usterd-utils.c -- no
- * glusterd header where it would be
- * appropriate to put to, and too
- * accidental routine to place in
- * libglusterfs.
- *
- * (Indeed, XXX: we'd rather need a general
- * "mkdir -p" like routine in
- * libglusterfs)
- */
-extern int mkdir_if_missing (char *path);
-
 #if SYNCDAEMON_COMPILE
 static int
-glusterd_check_gsync_present ()
+glusterd_check_gsync_present (int *valid_state)
 {
-        FILE               *in = NULL;
         char                buff[PATH_MAX] = {0, };
-        char                cmd[PATH_MAX + 256] = {0, };
+        runner_t            runner = {0,};
         char               *ptr = NULL;
         int                 ret = 0;
 
-        if (strlen (GSYNCD_PREFIX)+1 > PATH_MAX-strlen("/gsyncd")) {
-                ret = -1;
+        runinit (&runner);
+        runner_add_args (&runner, GSYNCD_PREFIX"/gsyncd", "--version", NULL);
+        runner_redir (&runner, STDOUT_FILENO, RUN_PIPE);
+        ret = runner_start (&runner);
+        if (ret == -1) {
+                if (errno == ENOENT) {
+                        gf_log ("glusterd", GF_LOG_INFO, GEOREP
+                                 " module not installed in the system");
+                        *valid_state = 0;
+                }
+                else {
+                        gf_log ("glusterd", GF_LOG_ERROR, GEOREP
+                                  " module not working as desired");
+                        *valid_state = -1;
+                }
                 goto out;
         }
 
-        snprintf (cmd, sizeof(cmd), GSYNCD_PREFIX"/gsyncd --version");
-
-        if (!(in = popen(cmd, "r"))) {
-                gf_log ("", GF_LOG_INFO, "geo-replication module not installed"
-                        " in the system");
-                ret = -1;
-                goto out;
-        }
-
-        ptr = fgets(buff, sizeof(buff), in);
+        ptr = fgets(buff, sizeof(buff), runner_chio (&runner, STDOUT_FILENO));
         if (ptr) {
                 if (!strstr (buff, "gsyncd")) {
                         ret = -1;
+                        gf_log ("glusterd", GF_LOG_ERROR, GEOREP" module not "
+                                 "working as desired");
+                        *valid_state = -1;
                         goto out;
                 }
         } else {
                 ret = -1;
+                gf_log ("glusterd", GF_LOG_ERROR, GEOREP" module not "
+                         "working as desired");
+                *valid_state = -1;
                 goto out;
         }
+
         ret = 0;
  out:
-        if ((in)&& (-1 == pclose (in))) {
-                        ret = -1;
-                        gf_log ("", GF_LOG_INFO, "geo-replication module not"
-                                " installed in the system");
-                }
 
-        gf_log ("", GF_LOG_DEBUG, "Returning %d", ret);
+        runner_end (&runner);
+
+        gf_log ("glusterd", GF_LOG_DEBUG, "Returning %d", ret);
         return ret;
 
 }
 
-int
+static int
+group_write_allow (char *path, gid_t gid)
+{
+        struct stat st = {0,};
+        int ret        = 0;
+
+        ret = stat (path, &st);
+        if (ret == -1)
+                goto out;
+        GF_ASSERT (S_ISDIR (st.st_mode));
+
+        ret = chown (path, -1, gid);
+        if (ret == -1)
+                goto out;
+
+        ret = chmod (path, (st.st_mode & ~S_IFMT) | S_IWGRP|S_IXGRP|S_ISVTX);
+
+ out:
+        if (ret == -1)
+                gf_log ("", GF_LOG_CRITICAL,
+                        "failed to set up write access to %s for group %d (%s)",
+                        path, gid, strerror (errno));
+        return ret;
+}
+
+static int
 glusterd_crt_georep_folders (char *georepdir, glusterd_conf_t *conf)
 {
-        int ret = 0;
+        char *greplg_s   = NULL;
+        struct group *gr = NULL;
+        int ret          = 0;
 
         GF_ASSERT (georepdir);
         GF_ASSERT (conf);
@@ -321,7 +344,7 @@ glusterd_crt_georep_folders (char *georepdir, glusterd_conf_t *conf)
         }
 
         snprintf (georepdir, PATH_MAX, "%s/"GEOREP, conf->workdir);
-        ret = mkdir_if_missing (georepdir);
+        ret = mkdir_if_missing (georepdir, NULL);
         if (-1 == ret) {
                 gf_log ("glusterd", GF_LOG_CRITICAL,
                         "Unable to create "GEOREP" directory %s",
@@ -336,7 +359,7 @@ glusterd_crt_georep_folders (char *georepdir, glusterd_conf_t *conf)
                         georepdir);
                 goto out;
         }
-        ret = mkdir_if_missing (DEFAULT_LOG_FILE_DIRECTORY"/"GEOREP);
+        ret = mkdir_if_missing (DEFAULT_LOG_FILE_DIRECTORY"/"GEOREP, NULL);
         if (-1 == ret) {
                 gf_log ("glusterd", GF_LOG_CRITICAL,
                         "Unable to create "GEOREP" log directory");
@@ -350,135 +373,183 @@ glusterd_crt_georep_folders (char *georepdir, glusterd_conf_t *conf)
                         georepdir);
                 goto out;
         }
-        ret = mkdir_if_missing (DEFAULT_LOG_FILE_DIRECTORY"/"GEOREP"-slaves");
+        ret = mkdir_if_missing (DEFAULT_LOG_FILE_DIRECTORY"/"GEOREP"-slaves",
+                                NULL);
         if (-1 == ret) {
                 gf_log ("glusterd", GF_LOG_CRITICAL,
                         "Unable to create "GEOREP" slave log directory");
                 goto out;
         }
-        ret = 0;
+
+        ret = dict_get_str (THIS->options, GEOREP"-log-group", &greplg_s);
+        if (ret)
+                ret = 0;
+        else {
+                gr = getgrnam (greplg_s);
+                if (!gr) {
+                        gf_log ("glusterd", GF_LOG_CRITICAL,
+                                "group "GEOREP"-log-group %s does not exist", greplg_s);
+                        ret = -1;
+                        goto out;
+                }
+
+                ret = group_write_allow (DEFAULT_LOG_FILE_DIRECTORY"/"GEOREP,
+                                         gr->gr_gid);
+                if (ret == 0)
+                        ret = group_write_allow (DEFAULT_LOG_FILE_DIRECTORY"/"
+                                                 GEOREP"-slaves", gr->gr_gid);
+        }
+
  out:
         gf_log("", GF_LOG_DEBUG, "Returning %d", ret);
         return ret;
-
 }
 #endif
 
+static void
+runinit_gsyncd_setrx (runner_t *runner, glusterd_conf_t *conf)
+{
+        runinit (runner);
+        runner_add_args (runner, GSYNCD_PREFIX"/gsyncd", "-c", NULL);
+        runner_argprintf (runner, "%s/"GSYNC_CONF,conf->workdir);
+        runner_add_arg (runner, "--config-set-rx");
+}
+
 static int
 configure_syncdaemon (glusterd_conf_t *conf)
-#define libc_system system
-#define RUN_GSYNCD_CMD(prf) do {                                                \
-        ret = prf##_system (cmd);                                               \
-        if (ret) {                                                              \
-                gf_log ("", GF_LOG_ERROR, "command failed: \"%s\"", cmd);       \
-                goto out;                                                       \
-        }                                                                       \
+#define RUN_GSYNCD_CMD do {                                                          \
+        ret = runner_run_reuse (&runner);                                            \
+        if (ret == -1) {                                                             \
+                runner_log (&runner, "glusterd", GF_LOG_ERROR, "command failed");    \
+                runner_end (&runner);                                                \
+                goto out;                                                            \
+        }                                                                            \
+        runner_end (&runner);                                                        \
 } while (0)
 {
         int ret = 0;
 #if SYNCDAEMON_COMPILE
+        runner_t runner = {0,};
         char georepdir[PATH_MAX] = {0,};
-        char cmd[2*PATH_MAX + 1024] = {0,};
-        char volid[64] = {0,};
-        int blen = 0;
+        int valid_state = 0;
 
         ret = setenv ("_GLUSTERD_CALLED_", "1", 1);
         if (ret < 0) {
                 ret = 0;
                 goto out;
         }
-
-        ret = glusterd_check_gsync_present ();
+        valid_state = -1;
+        ret = glusterd_check_gsync_present (&valid_state);
         if (-1 == ret) {
-                ret = 0;
+                ret = valid_state;
                 goto out;
         }
 
-
         glusterd_crt_georep_folders (georepdir, conf);
         if (ret) {
                 ret = 0;
                 goto out;
         }
 
-        blen = snprintf (cmd, sizeof(cmd), GSYNCD_PREFIX"/gsyncd -c %s/"
-                         GSYNC_CONF " --config-set-rx ", conf->workdir);
-
-        /* Calling out to gsyncd to configure it:
-         * - use system(3) for options with multi-word values as system
-         *   groks quotes;
-         * - use gf_system() for options with template expanders so
-         *   that they are not messed up by shell
-         */
-
         /************
          * master pre-configuration
          ************/
 
         /* remote-gsyncd */
-        strcpy (cmd + blen, "remote-gsyncd " GSYNCD_PREFIX"/gsyncd . .");
-        RUN_GSYNCD_CMD(libc);
+        runinit_gsyncd_setrx (&runner, conf);
+        runner_add_args (&runner, "remote-gsyncd", GSYNCD_PREFIX"/gsyncd", ".", ".", NULL);
+        RUN_GSYNCD_CMD;
 
-        strcpy (cmd + blen,
-                "remote-gsyncd /usr/local/libexec/glusterfs/gsyncd . ^ssh:");
-        RUN_GSYNCD_CMD(libc);
+        runinit_gsyncd_setrx (&runner, conf);
+        runner_add_args (&runner, "remote-gsyncd",
+                         "/usr/local/libexec/glusterfs/gsyncd", ".", "^ssh:", NULL);
+        RUN_GSYNCD_CMD;
 
-        /* gluster-command */
-        /* XXX $sbindir should be used (throughout the codebase) */
-        strcpy (cmd + blen,
-                "gluster-command '"GFS_PREFIX"/sbin/glusterfs "
-                "--xlator-option *-dht.assert-no-child-down=true' . .");
-        RUN_GSYNCD_CMD(libc);
+        /* gluster-command-dir */
+        runinit_gsyncd_setrx (&runner, conf);
+        runner_add_args (&runner, "gluster-command-dir", SBIN_DIR"/",
+                         ".", ".", NULL);
+        RUN_GSYNCD_CMD;
+
+        /* gluster-params */
+        runinit_gsyncd_setrx (&runner, conf);
+        runner_add_args (&runner, "gluster-params",
+                         "xlator-option=*-dht.assert-no-child-down=true",
+                         ".", ".", NULL);
+        RUN_GSYNCD_CMD;
 
         /* ssh-command */
-        sprintf (cmd + blen,
-                 "ssh-command "
-                 "'ssh -oPasswordAuthentication=no -oStrictHostKeyChecking=no "
-                 "-i %s/secret.pem' . .", georepdir);
-        RUN_GSYNCD_CMD(libc);
-
-        /* session-owner */
-        uuid_unparse (conf->uuid, volid);
-        sprintf (cmd + blen, "session-owner %s . .", volid);
-        RUN_GSYNCD_CMD(libc);
+        runinit_gsyncd_setrx (&runner, conf);
+        runner_add_arg (&runner, "ssh-command");
+        runner_argprintf (&runner,
+                          "ssh -oPasswordAuthentication=no "
+                           "-oStrictHostKeyChecking=no "
+                           "-i %s/secret.pem", georepdir);
+        runner_add_args (&runner, ".", ".", NULL);
+        RUN_GSYNCD_CMD;
 
         /* pid-file */
-        sprintf (cmd + blen, "pid-file %s/${mastervol}/${eSlave}.pid . .", georepdir);
-        RUN_GSYNCD_CMD(gf);
+        runinit_gsyncd_setrx (&runner, conf);
+        runner_add_arg (&runner, "pid-file");
+        runner_argprintf (&runner, "%s/${mastervol}/${eSlave}.pid", georepdir);
+        runner_add_args (&runner, ".", ".", NULL);
+        RUN_GSYNCD_CMD;
 
         /* state-file */
-        sprintf (cmd + blen, "state-file %s/${mastervol}/${eSlave}.status . .", georepdir);
-        RUN_GSYNCD_CMD(gf);
+        runinit_gsyncd_setrx (&runner, conf);
+        runner_add_arg (&runner, "state-file");
+        runner_argprintf (&runner, "%s/${mastervol}/${eSlave}.status", georepdir);
+        runner_add_args (&runner, ".", ".", NULL);
+        RUN_GSYNCD_CMD;
 
         /* log-file */
-        strcpy (cmd + blen,
-                "log-file "DEFAULT_LOG_FILE_DIRECTORY"/"GEOREP"/${mastervol}/${eSlave}.log . .");
-        RUN_GSYNCD_CMD(gf);
+        runinit_gsyncd_setrx (&runner, conf);
+        runner_add_args (&runner,
+                         "log-file",
+                         DEFAULT_LOG_FILE_DIRECTORY"/"GEOREP"/${mastervol}/${eSlave}.log",
+                         ".", ".", NULL);
+        RUN_GSYNCD_CMD;
 
         /* gluster-log-file */
-        strcpy (cmd + blen, "gluster-log-file "
-                DEFAULT_LOG_FILE_DIRECTORY"/"GEOREP"/${mastervol}/${eSlave}.gluster.log . .");
-        RUN_GSYNCD_CMD(gf);
+        runinit_gsyncd_setrx (&runner, conf);
+        runner_add_args (&runner,
+                         "gluster-log-file",
+                         DEFAULT_LOG_FILE_DIRECTORY"/"GEOREP"/${mastervol}/${eSlave}.gluster.log",
+                         ".", ".", NULL);
+        RUN_GSYNCD_CMD;
 
         /************
          * slave pre-configuration
          ************/
 
-        /* gluster-command */
-        strcpy (cmd + blen,
-                "gluster-command '"GFS_PREFIX"/sbin/glusterfs "
-                "--xlator-option *-dht.assert-no-child-down=true' .");
-        RUN_GSYNCD_CMD(libc);
+        /* gluster-command-dir */
+        runinit_gsyncd_setrx (&runner, conf);
+        runner_add_args (&runner, "gluster-command-dir", SBIN_DIR"/",
+                         ".", NULL);
+        RUN_GSYNCD_CMD;
+
+        /* gluster-params */
+        runinit_gsyncd_setrx (&runner, conf);
+        runner_add_args (&runner, "gluster-params",
+                         "xlator-option=*-dht.assert-no-child-down=true",
+                         ".", NULL);
+        RUN_GSYNCD_CMD;
 
         /* log-file */
-        strcpy (cmd + blen,
-                "log-file "DEFAULT_LOG_FILE_DIRECTORY"/"GEOREP"-slaves/${session_owner}:${eSlave}.log .");
-        RUN_GSYNCD_CMD(gf);
+        runinit_gsyncd_setrx (&runner, conf);
+        runner_add_args (&runner,
+                         "log-file",
+                         DEFAULT_LOG_FILE_DIRECTORY"/"GEOREP"-slaves/${session_owner}:${eSlave}.log",
+                         ".", NULL);
+        RUN_GSYNCD_CMD;
 
         /* gluster-log-file */
-        strcpy (cmd + blen, "gluster-log-file "
-                DEFAULT_LOG_FILE_DIRECTORY"/"GEOREP"-slaves/${session_owner}:${eSlave}.gluster.log .");
-        RUN_GSYNCD_CMD(gf);
+        runinit_gsyncd_setrx (&runner, conf);
+        runner_add_args (&runner,
+                         "gluster-log-file",
+                         DEFAULT_LOG_FILE_DIRECTORY"/"GEOREP"-slaves/${session_owner}:${eSlave}.gluster.log",
+                         ".", NULL);
+        RUN_GSYNCD_CMD;
 
  out:
 #else
@@ -486,9 +557,189 @@ configure_syncdaemon (glusterd_conf_t *conf)
 #endif
         return ret ? -1 : 0;
 }
-#undef libc_system
 #undef RUN_GSYNCD_CMD
 
+static int
+check_prepare_mountbroker_root (char *mountbroker_root)
+{
+        int dfd0        = -1;
+        int dfd         = -1;
+        int dfd2        = -1;
+        struct stat st  = {0,};
+        struct stat st2 = {0,};
+        int ret         = 0;
+
+        ret = open (mountbroker_root, O_RDONLY);
+        if (ret != -1) {
+                dfd = ret;
+                ret = fstat (dfd, &st);
+        }
+        if (ret == -1 || !S_ISDIR (st.st_mode)) {
+                gf_log ("", GF_LOG_ERROR,
+                        "cannot access mountbroker-root directory %s",
+                        mountbroker_root);
+                ret = -1;
+                goto out;
+        }
+        if (st.st_uid != 0 ||
+            (st.st_mode & (S_IWGRP|S_IWOTH))) {
+                gf_log ("", GF_LOG_ERROR,
+                        "permissions on mountbroker-root directory %s are "
+                        "too liberal", mountbroker_root);
+                ret = -1;
+                goto out;
+        }
+        if (!(st.st_mode & (S_IXGRP|S_IXOTH))) {
+                gf_log ("", GF_LOG_WARNING,
+                        "permissions on mountbroker-root directory %s are "
+                        "probably too strict", mountbroker_root);
+        }
+
+        dfd0 = dup (dfd);
+
+        for (;;) {
+                ret = openat (dfd, "..", O_RDONLY);
+                if (ret != -1) {
+                        dfd2 = ret;
+                        ret = fstat (dfd2, &st2);
+                }
+                if (ret == -1) {
+                        gf_log ("", GF_LOG_ERROR,
+                                "error while checking mountbroker-root ancestors "
+                                "%d (%s)", errno, strerror (errno));
+                        goto out;
+                }
+
+                if (st2.st_ino == st.st_ino)
+                        break; /* arrived to root */
+
+                if (st2.st_uid != 0 ||
+                    ((st2.st_mode & (S_IWGRP|S_IWOTH)) &&
+                     !(st2.st_mode & S_ISVTX))) {
+                        gf_log ("", GF_LOG_ERROR,
+                                "permissions on ancestors of mountbroker-root "
+                                "directory are too liberal");
+                        ret = -1;
+                        goto out;
+                }
+                if (!(st.st_mode & (S_IXGRP|S_IXOTH))) {
+                        gf_log ("", GF_LOG_WARNING,
+                                "permissions on ancestors of mountbroker-root "
+                                "directory are probably too strict");
+                }
+
+                close (dfd);
+                dfd = dfd2;
+                st = st2;
+        }
+
+        ret = mkdirat (dfd0, MB_HIVE, 0711);
+        if (ret == -1 && errno == EEXIST)
+                ret = 0;
+        if (ret != -1)
+                ret = fstatat (dfd0, MB_HIVE, &st, AT_SYMLINK_NOFOLLOW);
+        if (ret == -1 || st.st_mode != (S_IFDIR|0711)) {
+                gf_log ("", GF_LOG_ERROR,
+                        "failed to set up mountbroker-root directory %s",
+                        mountbroker_root);
+                ret = -1;
+                goto out;
+        }
+
+        ret = 0;
+
+ out:
+        if (dfd0 != -1)
+                close (dfd0);
+        if (dfd != -1)
+                close (dfd);
+        if (dfd2 != -1)
+                close (dfd2);
+
+        return ret;
+}
+
+static void
+_install_mount_spec (dict_t *opts, char *key, data_t *value, void *data)
+{
+        glusterd_conf_t *priv           = THIS->private;
+        char            *label          = NULL;
+        gf_boolean_t     georep         = _gf_false;
+        gf_boolean_t     ghadoop        = _gf_false;
+        char            *pdesc          = value->data;
+        char            *volname        = NULL;
+        int             *ret            = data;
+        int              rv             = 0;
+        gf_mount_spec_t *mspec          = NULL;
+        char            *user           = NULL;
+        char            *volfile_server = NULL;
+
+        if (*ret == -1)
+                return;
+
+        label = strtail (key, "mountbroker.");
+
+        /* check for presence of geo-rep/hadoop label */
+        if (!label) {
+                label = strtail (key, "mountbroker-"GEOREP".");
+                if (label)
+                        georep = _gf_true;
+                else {
+                        label = strtail (key, "mountbroker-"GHADOOP".");
+                        if (label)
+                                ghadoop = _gf_true;
+                }
+        }
+
+        if (!label)
+                return;
+
+        mspec = GF_CALLOC (1, sizeof (*mspec), gf_gld_mt_mount_spec);
+        if (!mspec)
+                goto err;
+        mspec->label = label;
+
+        if (georep || ghadoop) {
+                volname = gf_strdup (pdesc);
+                if (!volname)
+                        goto err;
+                user = strchr (volname, ':');
+                if (user) {
+                        *user = '\0';
+                        user++;
+                } else
+                        user = label;
+
+                if (georep)
+                        rv = make_georep_mountspec (mspec, volname, user);
+
+                if (ghadoop) {
+                        volfile_server = strchr (user, ':');
+                        if (volfile_server)
+                                *volfile_server++ = '\0';
+                        else
+                                volfile_server = "localhost";
+
+                        rv = make_ghadoop_mountspec (mspec, volname, user, volfile_server);
+                }
+
+                GF_FREE (volname);
+                if (rv != 0)
+                        goto err;
+        } else if (parse_mount_pattern_desc (mspec, pdesc) != 0)
+                goto err;
+
+        list_add_tail (&mspec->speclist, &priv->mount_specs);
+
+        return;
+ err:
+
+        gf_log ("", GF_LOG_ERROR,
+                "adding %smount spec failed: label: %s desc: %s",
+                georep ? GEOREP" " : "", label, pdesc);
+
+        *ret = -1;
+}
 
 /*
  * init - called during glusterd initialization
@@ -504,50 +755,56 @@ init (xlator_t *this)
         glusterd_conf_t   *conf              = NULL;
         data_t            *dir_data          = NULL;
         struct stat        buf               = {0,};
-        char               voldir [PATH_MAX] = {0,};
-        char               dirname [PATH_MAX];
+        char               storedir [PATH_MAX] = {0,};
+        char               workdir [PATH_MAX] = {0,};
+        char               hooks_dir [PATH_MAX] = {0,};
         char               cmd_log_filename [PATH_MAX] = {0,};
         int                first_time        = 0;
+        char              *mountbroker_root  = NULL;
 
+#ifdef DEBUG
+        char              *valgrind_str      = NULL;
+#endif
         dir_data = dict_get (this->options, "working-directory");
 
         if (!dir_data) {
                 //Use default working dir
-                strncpy (dirname, GLUSTERD_DEFAULT_WORKDIR, PATH_MAX);
+                strncpy (workdir, GLUSTERD_DEFAULT_WORKDIR, PATH_MAX);
         } else {
-                strncpy (dirname, dir_data->data, PATH_MAX);
+                strncpy (workdir, dir_data->data, PATH_MAX);
         }
 
-        ret = stat (dirname, &buf);
+        ret = stat (workdir, &buf);
         if ((ret != 0) && (ENOENT != errno)) {
                 gf_log (this->name, GF_LOG_ERROR,
                         "stat fails on %s, exiting. (errno = %d)",
-			dirname, errno);
+			workdir, errno);
                 exit (1);
         }
 
         if ((!ret) && (!S_ISDIR(buf.st_mode))) {
                 gf_log (this->name, GF_LOG_CRITICAL,
                         "Provided working area %s is not a directory,"
-                        "exiting", dirname);
+                        "exiting", workdir);
                 exit (1);
         }
 
 
         if ((-1 == ret) && (ENOENT == errno)) {
-                ret = mkdir (dirname, 0777);
+                ret = mkdir (workdir, 0777);
 
                 if (-1 == ret) {
                         gf_log (this->name, GF_LOG_CRITICAL,
                                 "Unable to create directory %s"
-                                " ,errno = %d", dirname, errno);
+                                " ,errno = %d", workdir, errno);
                         exit (1);
                 }
+
                 first_time = 1;
         }
 
         gf_log (this->name, GF_LOG_INFO, "Using %s as working directory",
-                dirname);
+                workdir);
 
         snprintf (cmd_log_filename, PATH_MAX,"%s/.cmd_log_history",
                   DEFAULT_LOG_FILE_DIRECTORY);
@@ -559,50 +816,68 @@ init (xlator_t *this)
                 exit (1);
         }
 
-        snprintf (voldir, PATH_MAX, "%s/vols", dirname);
+        snprintf (storedir, PATH_MAX, "%s/vols", workdir);
 
-        ret = mkdir (voldir, 0777);
+        ret = mkdir (storedir, 0777);
 
         if ((-1 == ret) && (errno != EEXIST)) {
                 gf_log (this->name, GF_LOG_CRITICAL,
                         "Unable to create volume directory %s"
-                        " ,errno = %d", voldir, errno);
+                        " ,errno = %d", storedir, errno);
                 exit (1);
         }
 
-        snprintf (voldir, PATH_MAX, "%s/peers", dirname);
+        snprintf (storedir, PATH_MAX, "%s/peers", workdir);
 
-        ret = mkdir (voldir, 0777);
+        ret = mkdir (storedir, 0777);
 
         if ((-1 == ret) && (errno != EEXIST)) {
                 gf_log (this->name, GF_LOG_CRITICAL,
                         "Unable to create peers directory %s"
-                        " ,errno = %d", voldir, errno);
+                        " ,errno = %d", storedir, errno);
                 exit (1);
         }
 
-        snprintf (voldir, PATH_MAX, "%s/bricks", DEFAULT_LOG_FILE_DIRECTORY);
-        ret = mkdir (voldir, 0777);
+        snprintf (storedir, PATH_MAX, "%s/bricks", DEFAULT_LOG_FILE_DIRECTORY);
+        ret = mkdir (storedir, 0777);
         if ((-1 == ret) && (errno != EEXIST)) {
                 gf_log (this->name, GF_LOG_CRITICAL,
                         "Unable to create logs directory %s"
-                        " ,errno = %d", voldir, errno);
+                        " ,errno = %d", storedir, errno);
                 exit (1);
         }
 
-        snprintf (voldir, PATH_MAX, "%s/nfs", dirname);
-        ret = mkdir (voldir, 0777);
+        snprintf (storedir, PATH_MAX, "%s/nfs", workdir);
+        ret = mkdir (storedir, 0777);
         if ((-1 == ret) && (errno != EEXIST)) {
                 gf_log (this->name, GF_LOG_CRITICAL,
                         "Unable to create nfs directory %s"
-                        " ,errno = %d", voldir, errno);
+                        " ,errno = %d", storedir, errno);
+                exit (1);
+        }
+
+        snprintf (storedir, PATH_MAX, "%s/glustershd", workdir);
+        ret = mkdir (storedir, 0777);
+        if ((-1 == ret) && (errno != EEXIST)) {
+                gf_log (this->name, GF_LOG_CRITICAL,
+                        "Unable to create glustershd directory %s"
+                        " ,errno = %d", storedir, errno);
+                exit (1);
+        }
+
+        snprintf (storedir, PATH_MAX, "%s/groups", workdir);
+        ret = mkdir (storedir, 0777);
+        if ((-1 == ret) && (errno != EEXIST)) {
+                gf_log (this->name, GF_LOG_CRITICAL,
+                        "Unable to create glustershd directory %s"
+                        " ,errno = %d", storedir, errno);
                 exit (1);
         }
 
         ret = glusterd_rpcsvc_options_build (this->options);
         if (ret)
                 goto out;
-        rpc = rpcsvc_init (this->ctx, this->options);
+        rpc = rpcsvc_init (this, this->ctx, this->options, 64);
         if (rpc == NULL) {
                 gf_log (this->name, GF_LOG_ERROR,
                         "failed to init rpc");
@@ -628,27 +903,27 @@ init (xlator_t *this)
                 goto out;
         }
 
-        ret = glusterd_program_register (this, rpc, &glusterd1_mop_prog);
+        ret = glusterd_program_register (this, rpc, &gd_svc_peer_prog);
         if (ret) {
                 goto out;
         }
 
         ret = glusterd_program_register (this, rpc, &gd_svc_cli_prog);
         if (ret) {
-                rpcsvc_program_unregister (rpc, &glusterd1_mop_prog);
+                rpcsvc_program_unregister (rpc, &gd_svc_peer_prog);
                 goto out;
         }
 
         ret = glusterd_program_register (this, rpc, &gd_svc_mgmt_prog);
         if (ret) {
-                rpcsvc_program_unregister (rpc, &glusterd1_mop_prog);
+                rpcsvc_program_unregister (rpc, &gd_svc_peer_prog);
                 rpcsvc_program_unregister (rpc, &gd_svc_cli_prog);
                 goto out;
         }
 
         ret = glusterd_program_register (this, rpc, &gluster_pmap_prog);
         if (ret) {
-                rpcsvc_program_unregister (rpc, &glusterd1_mop_prog);
+                rpcsvc_program_unregister (rpc, &gd_svc_peer_prog);
                 rpcsvc_program_unregister (rpc, &gd_svc_cli_prog);
                 rpcsvc_program_unregister (rpc, &gd_svc_mgmt_prog);
                 goto out;
@@ -656,7 +931,7 @@ init (xlator_t *this)
 
         ret = glusterd_program_register (this, rpc, &gluster_handshake_prog);
         if (ret) {
-                rpcsvc_program_unregister (rpc, &glusterd1_mop_prog);
+                rpcsvc_program_unregister (rpc, &gd_svc_peer_prog);
                 rpcsvc_program_unregister (rpc, &gluster_pmap_prog);
                 rpcsvc_program_unregister (rpc, &gd_svc_cli_prog);
                 rpcsvc_program_unregister (rpc, &gd_svc_mgmt_prog);
@@ -666,14 +941,25 @@ init (xlator_t *this)
         conf = GF_CALLOC (1, sizeof (glusterd_conf_t),
                           gf_gld_mt_glusterd_conf_t);
         GF_VALIDATE_OR_GOTO(this->name, conf, out);
+        conf->shd = GF_CALLOC (1, sizeof (nodesrv_t),
+                               gf_gld_mt_nodesrv_t);
+        GF_VALIDATE_OR_GOTO(this->name, conf->shd, out);
+        conf->nfs = GF_CALLOC (1, sizeof (nodesrv_t),
+                               gf_gld_mt_nodesrv_t);
+        GF_VALIDATE_OR_GOTO(this->name, conf->nfs, out);
+
         INIT_LIST_HEAD (&conf->peers);
         INIT_LIST_HEAD (&conf->volumes);
         pthread_mutex_init (&conf->mutex, NULL);
         conf->rpc = rpc;
-        conf->gfs_mgmt = &glusterd_glusterfs_3_1_mgmt_prog;
-        strncpy (conf->workdir, dirname, PATH_MAX);
+        conf->gfs_mgmt = &gd_brick_prog;
+        strncpy (conf->workdir, workdir, PATH_MAX);
 
         INIT_LIST_HEAD (&conf->xprt_list);
+
+        glusterd_friend_sm_init ();
+        glusterd_op_sm_init ();
+        glusterd_opinfo_init ();
         ret = glusterd_sm_tr_log_init (&conf->op_sm_log,
                                        glusterd_op_sm_state_name_get,
                                        glusterd_op_sm_event_name_get,
@@ -681,13 +967,52 @@ init (xlator_t *this)
         if (ret)
                 goto out;
 
+        /* Set option to run bricks on valgrind if enabled in glusterd.vol */
+#ifdef DEBUG
+        conf->valgrind = _gf_false;
+        ret = dict_get_str (this->options, "run-with-valgrind", &valgrind_str);
+        if (ret < 0) {
+                gf_log (this->name, GF_LOG_DEBUG,
+                        "cannot get run-with-valgrind value");
+        }
+        if (valgrind_str) {
+                if (gf_string2boolean (valgrind_str, &(conf->valgrind))) {
+                        gf_log (this->name, GF_LOG_WARNING,
+                                "run-with-valgrind value not a boolean string");
+                }
+        }
+#endif
+
         this->private = conf;
-        //this->ctx->top = this;
+        (void) glusterd_nodesvc_set_running ("glustershd", _gf_false);
 
         ret = glusterd_uuid_init (first_time);
         if (ret < 0)
                 goto out;
 
+        GLUSTERD_GET_HOOKS_DIR (hooks_dir, GLUSTERD_HOOK_VER, conf);
+        if (stat (hooks_dir, &buf)) {
+                ret = glusterd_hooks_create_hooks_directory (conf->workdir);
+                if (-1 == ret) {
+                        gf_log (this->name, GF_LOG_CRITICAL,
+                                "Unable to create hooks directory ");
+                        exit (1);
+                }
+        }
+
+        INIT_LIST_HEAD (&conf->mount_specs);
+        dict_foreach (this->options, _install_mount_spec, &ret);
+        if (ret)
+                goto out;
+        ret = dict_get_str (this->options, "mountbroker-root",
+                            &mountbroker_root);
+        if (ret)
+                ret = 0;
+        else
+                ret = check_prepare_mountbroker_root (mountbroker_root);
+        if (ret)
+                goto out;
+
         ret = configure_syncdaemon (conf);
         if (ret)
                 goto out;
@@ -696,10 +1021,6 @@ init (xlator_t *this)
         if (ret < 0)
                 goto out;
 
-        glusterd_friend_sm_init ();
-        glusterd_op_sm_init ();
-        glusterd_opinfo_init ();
-
         ret = glusterd_handle_upgrade_downgrade (this->options, conf);
         if (ret)
                 goto out;
@@ -708,6 +1029,12 @@ init (xlator_t *this)
         ret = glusterd_restart_gsyncds (conf);
         if (ret)
                 goto out;
+
+        ret = glusterd_hooks_spawn_worker (this);
+        if (ret)
+                goto out;
+
+        glusterd_restart_rebalance (conf);
         ret = 0;
 out:
         if (ret < 0) {
@@ -715,6 +1042,7 @@ out:
                         GF_FREE (this->private);
                         this->private = NULL;
                 }
+
         }
 
         return ret;
@@ -814,6 +1142,28 @@ struct volume_options options[] = {
         { .key  = {"downgrade"},
           .type = GF_OPTION_TYPE_BOOL,
         },
-
+        { .key = {"bind-insecure"},
+          .type = GF_OPTION_TYPE_BOOL,
+        },
+        { .key  = {"mountbroker-root"},
+          .type = GF_OPTION_TYPE_PATH,
+        },
+        { .key  = {"mountbroker.*"},
+          .type = GF_OPTION_TYPE_ANY,
+        },
+        { .key  = {"mountbroker-"GEOREP".*"},
+          .type = GF_OPTION_TYPE_ANY,
+        },
+        { .key  = {"mountbroker-"GHADOOP".*"},
+          .type = GF_OPTION_TYPE_ANY,
+        },
+        { .key = {GEOREP"-log-group"},
+          .type = GF_OPTION_TYPE_ANY,
+        },
+#ifdef DEBUG
+        { .key = {"run-with-valgrind"},
+          .type = GF_OPTION_TYPE_BOOL,
+        },
+#endif
         { .key   = {NULL} },
 };
