@@ -54,8 +54,9 @@
 
 #include "list.h"
 #include "dict.h"
+#include "syncop.h"
 
-#ifdef GF_LINUX_HOST_OS
+#if defined(GF_LINUX_HOST_OS) || defined(__NetBSD__)
 #define FUSE_OP_HIGH (FUSE_POLL + 1)
 #endif
 #ifdef GF_DARWIN_HOST_OS
@@ -65,8 +66,6 @@
 
 #define MAX_FUSE_PROC_DELAY 1
 
-#define DISABLE_SELINUX 1
-
 typedef struct fuse_in_header fuse_in_header_t;
 typedef void (fuse_handler_t) (xlator_t *this, fuse_in_header_t *finh,
                                void *msg);
@@ -106,10 +105,35 @@ struct fuse_private {
 
         pid_t                client_pid;
         gf_boolean_t         client_pid_set;
+        unsigned             uid_map_root;
         gf_boolean_t         acl;
+        gf_boolean_t         selinux;
+        gf_boolean_t         read_only;
+        fdtable_t           *fdtable;
+
+        /* For fuse-reverse-validation */
+        int                  revchan_in;
+        int                  revchan_out;
+        gf_boolean_t         reverse_fuse_thread_started;
+
+        /* For communicating with separate mount thread. */
+        int                  status_pipe[2];
 };
 typedef struct fuse_private fuse_private_t;
 
+struct fuse_graph_switch_args {
+        xlator_t        *this;
+        xlator_t        *old_subvol;
+        xlator_t        *new_subvol;
+};
+typedef struct fuse_graph_switch_args fuse_graph_switch_args_t;
+
+#define INVAL_BUF_SIZE (sizeof (struct fuse_out_header) +               \
+                        max (sizeof (struct fuse_notify_inval_inode_out), \
+                             sizeof (struct fuse_notify_inval_entry_out) + \
+                             NAME_MAX + 1))
+
+
 #define _FH_TO_FD(fh) ((fd_t *)(uintptr_t)(fh))
 
 #define FH_TO_FD(fh) ((_FH_TO_FD (fh))?(fd_ref (_FH_TO_FD (fh))):((fd_t *) 0))
@@ -118,6 +142,7 @@ typedef struct fuse_private fuse_private_t;
         do {                                                            \
                 call_frame_t *frame = NULL;                             \
                 xlator_t *xl = NULL;                                    \
+                int32_t   op_ret = 0, op_errno = 0;                     \
                                                                         \
                 frame = get_call_frame_for_req (state);                 \
                 if (!frame) {                                           \
@@ -127,7 +152,7 @@ typedef struct fuse_private fuse_private_t;
                           * better than trying to go on with a NULL     \
                           * frame ...                                   \
                           */                                            \
-                        gf_log ("glusterfs-fuse",                       \
+                        gf_log_callingfn ("glusterfs-fuse",             \
                                 GF_LOG_ERROR,                           \
                                 "FUSE message"                          \
                                 " unique %"PRIu64" opcode %d:"          \
@@ -135,23 +160,44 @@ typedef struct fuse_private fuse_private_t;
                                 state->finh->unique,                    \
                                 state->finh->opcode);                   \
                         free_fuse_state (state);                        \
-                        return;                                         \
+                        /* ideally, need to 'return', but let the */    \
+                        /* calling function take care of it */          \
+                        break;                                          \
                 }                                                       \
                                                                         \
                 frame->root->state = state;                             \
                 frame->root->op    = op_num;                            \
                 frame->op          = op_num;                            \
                                                                         \
-                xl = fuse_state_subvol (state);                         \
+                xl = state->active_subvol;				\
                 if (!xl) {                                              \
-                        gf_log ("glusterfs-fuse", GF_LOG_ERROR,         \
-                                "xl is NULL");                          \
-                        send_fuse_err (state->this, state->finh, ENOENT); \
+                        gf_log_callingfn ("glusterfs-fuse", GF_LOG_ERROR, \
+                                          "xl is NULL");                  \
+                        op_errno = ENOENT;                              \
+                        op_ret = -1;                                    \
+                } else if (state->resolve.op_ret < 0) {                 \
+                        op_errno = state->resolve.op_errno;               \
+                        op_ret = -1;                                      \
+/*                        gf_log_callingfn ("glusterfs-fuse", GF_LOG_WARNING, \
+                                          "resolve failed (%s)",             \
+                                          strerror (op_errno));             */ \
+                } else if (state->resolve2.op_ret < 0) {                     \
+                        op_errno = state->resolve2.op_errno;                 \
+                        op_ret = -1;                                         \
+                        /* gf_log_callingfn ("glusterfs-fuse", GF_LOG_WARNING, \
+                                          "resolve of second entity "        \
+                                          "failed (%s)",                     \
+                                          strerror (op_errno));             */ \
+                }                                                            \
+                                                                             \
+                if (op_ret < 0) {                                            \
+                        send_fuse_err (state->this, state->finh, op_errno);  \
                         free_fuse_state (state);                        \
                         STACK_DESTROY (frame->root);                    \
                 } else {                                                \
                         STACK_WIND (frame, ret, xl, xl->fops->fop, args); \
                 }                                                       \
+                                                                        \
         } while (0)
 
 
@@ -201,6 +247,17 @@ typedef struct fuse_private fuse_private_t;
 
 
 
+static inline xlator_t *
+fuse_active_subvol (xlator_t *fuse)
+{
+        fuse_private_t *priv = NULL;
+
+        priv = fuse->private;
+
+        return priv->active_subvol;
+}
+
+
 typedef enum {
         RESOLVE_MUST = 1,
         RESOLVE_NOT,
@@ -209,35 +266,27 @@ typedef enum {
         RESOLVE_EXACT
 } fuse_resolve_type_t;
 
-struct fuse_resolve_comp {
-        char      *basename;
-        ino_t      ino;
-        uint64_t   gen;
-        inode_t   *inode;
-};
 
 typedef struct {
         fuse_resolve_type_t    type;
-        ino_t                  ino;
-        uint64_t               gen;
-        ino_t                  par;
         fd_t                  *fd;
         char                  *path;
         char                  *bname;
         u_char                 gfid[16];
+	inode_t               *hint;
         u_char                 pargfid[16];
+	inode_t               *parhint;
 	char                  *resolved;
         int                    op_ret;
         int                    op_errno;
-        loc_t                  deep_loc;
-        struct fuse_resolve_comp *components;
-        int                    comp_count;
+        loc_t                  resolve_loc;
 } fuse_resolve_t;
 
 
 typedef struct {
         void             *pool;
         xlator_t         *this;
+	xlator_t         *active_subvol;
         inode_table_t    *itable;
         loc_t             loc;
         loc_t             loc2;
@@ -247,7 +296,8 @@ typedef struct {
         size_t            size;
         unsigned long     nlookup;
         fd_t             *fd;
-        dict_t           *dict;
+        dict_t           *xattr;
+        dict_t           *xdata;
         char             *name;
         char              is_revalidate;
         gf_boolean_t      truncate_needed;
@@ -268,16 +318,19 @@ typedef struct {
         int            mask;
         dev_t          rdev;
         mode_t         mode;
+        mode_t         umask;
         struct iatt    attr;
         struct gf_flock   lk_lock;
         struct iovec   vector;
 
         uuid_t         gfid;
-        xlator_t       *old_xl;
+        uint32_t       io_flags;
+        int32_t        fd_no;
 } fuse_state_t;
 
 typedef struct fuse_fd_ctx {
         uint32_t  open_flags;
+        char      migration_failed;
         fd_t     *fd;
 } fuse_fd_ctx_t;
 
@@ -291,12 +344,19 @@ fuse_state_t *get_fuse_state (xlator_t *this, fuse_in_header_t *finh);
 void free_fuse_state (fuse_state_t *state);
 void gf_fuse_stat2attr (struct iatt *st, struct fuse_attr *fa);
 uint64_t inode_to_fuse_nodeid (inode_t *inode);
-xlator_t *fuse_state_subvol (fuse_state_t *state);
 xlator_t *fuse_active_subvol (xlator_t *fuse);
 inode_t *fuse_ino_to_inode (uint64_t ino, xlator_t *fuse);
-int fuse_resolve_and_resume (fuse_state_t *state, fuse_resume_fn_t fn);
 int send_fuse_err (xlator_t *this, fuse_in_header_t *finh, int error);
 int fuse_gfid_set (fuse_state_t *state);
+int fuse_flip_xattr_ns (struct fuse_private *priv, char *okey, char **nkey);
 fuse_fd_ctx_t * __fuse_fd_ctx_check_n_create (fd_t *fd, xlator_t *this);
 fuse_fd_ctx_t * fuse_fd_ctx_check_n_create (fd_t *fd, xlator_t *this);
+
+int fuse_resolve_and_resume (fuse_state_t *state, fuse_resume_fn_t fn);
+int fuse_resolve_inode_init (fuse_state_t *state, fuse_resolve_t *resolve,
+			     ino_t ino);
+int fuse_resolve_entry_init (fuse_state_t *state, fuse_resolve_t *resolve,
+			     ino_t par, char *name);
+int fuse_resolve_fd_init (fuse_state_t *state, fuse_resolve_t *resolve,
+			  fd_t *fd);
 #endif /* _GF_FUSE_BRIDGE_H_ */
