@@ -18,42 +18,19 @@
 */
 
 #include "fuse-bridge.h"
+#if defined(GF_SOLARIS_HOST_OS)
+#include <sys/procfs.h>
+#else
+#include <sys/sysctl.h>
+#endif
 
-xlator_t *
-fuse_state_subvol (fuse_state_t *state)
-{
-        xlator_t *subvol = NULL;
-
-        if (!state)
-                return NULL;
-
-        if (state->loc.inode)
-                subvol = state->loc.inode->table->xl;
-
-        if (state->fd)
-                subvol = state->fd->inode->table->xl;
-
-        return subvol;
-}
-
-
-xlator_t *
-fuse_active_subvol (xlator_t *fuse)
-{
-        fuse_private_t *priv = NULL;
-
-        priv = fuse->private;
-
-        return priv->active_subvol;
-}
-
-
+#ifndef GF_REQUEST_MAXGROUPS
+#define GF_REQUEST_MAXGROUPS    16
+#endif /* GF_REQUEST_MAXGROUPS */
 
 static void
 fuse_resolve_wipe (fuse_resolve_t *resolve)
 {
-        struct fuse_resolve_comp *comp = NULL;
-
         if (resolve->path)
                 GF_FREE ((void *)resolve->path);
 
@@ -63,35 +40,46 @@ fuse_resolve_wipe (fuse_resolve_t *resolve)
         if (resolve->resolved)
                 GF_FREE ((void *)resolve->resolved);
 
-        loc_wipe (&resolve->deep_loc);
+        if (resolve->fd)
+                fd_unref (resolve->fd);
 
-        comp = resolve->components;
+        loc_wipe (&resolve->resolve_loc);
 
-        if (comp) {
-                int                  i = 0;
+	if (resolve->hint) {
+		inode_unref (resolve->hint);
+		resolve->hint = 0;
+	}
 
-                for (i = 0; comp[i].basename; i++) {
-                        if (comp[i].inode) {
-                                inode_unref (comp[i].inode);
-                                comp[i].inode = NULL;
-                        }
-                }
-
-                GF_FREE ((void *)resolve->components);
-        }
+	if (resolve->parhint) {
+		inode_unref (resolve->parhint);
+		resolve->parhint = 0;
+	}
 }
 
+
 void
 free_fuse_state (fuse_state_t *state)
 {
+        xlator_t       *this     = NULL;
+        fuse_private_t *priv     = NULL;
+        uint64_t        winds    = 0;
+        char            switched = 0;
+
+        this = state->this;
+
+        priv = this->private;
+
         loc_wipe (&state->loc);
 
         loc_wipe (&state->loc2);
 
-        if (state->dict) {
-                dict_unref (state->dict);
-                state->dict = (void *)0xaaaaeeee;
+        if (state->xdata) {
+                dict_unref (state->xdata);
+                state->xdata = (void *)0xaaaaeeee;
         }
+        if (state->xattr)
+                dict_unref (state->xattr);
+
         if (state->name) {
                 GF_FREE (state->name);
                 state->name = NULL;
@@ -108,6 +96,18 @@ free_fuse_state (fuse_state_t *state)
         fuse_resolve_wipe (&state->resolve);
         fuse_resolve_wipe (&state->resolve2);
 
+        pthread_mutex_lock (&priv->sync_mutex);
+        {
+                winds = --state->active_subvol->winds;
+                switched = state->active_subvol->switched;
+        }
+        pthread_mutex_unlock (&priv->sync_mutex);
+
+        if ((winds == 0) && (switched)) {
+                xlator_notify (state->active_subvol, GF_EVENT_PARENT_DOWN,
+                               state->active_subvol, NULL);
+        }
+
 #ifdef DEBUG
         memset (state, 0x90, sizeof (*state));
 #endif
@@ -119,12 +119,28 @@ free_fuse_state (fuse_state_t *state)
 fuse_state_t *
 get_fuse_state (xlator_t *this, fuse_in_header_t *finh)
 {
-        fuse_state_t *state = NULL;
+        fuse_state_t   *state         = NULL;
+	xlator_t       *active_subvol = NULL;
+        fuse_private_t *priv          = NULL;
 
         state = (void *)GF_CALLOC (1, sizeof (*state),
                                    gf_fuse_mt_fuse_state_t);
         if (!state)
                 return NULL;
+
+	state->this = THIS;
+        priv = this->private;
+
+        pthread_mutex_lock (&priv->sync_mutex);
+        {
+                active_subvol = fuse_active_subvol (state->this);
+                active_subvol->winds++;
+        }
+        pthread_mutex_unlock (&priv->sync_mutex);
+
+	state->active_subvol = active_subvol;
+	state->itable = active_subvol->itable;
+
         state->pool = this->ctx->pool;
         state->finh = finh;
         state->this = this;
@@ -138,25 +154,26 @@ get_fuse_state (xlator_t *this, fuse_in_header_t *finh)
 void
 frame_fill_groups (call_frame_t *frame)
 {
-        char         filename[128];
-        char         line[128];
+#if defined(GF_LINUX_HOST_OS)
+        char         filename[32];
+        char         line[4096];
         char        *ptr = NULL;
-        int          ret = 0;
         FILE        *fp = NULL;
         int          idx = 0;
         long int     id = 0;
         char        *saveptr = NULL;
         char        *endptr = NULL;
+        int          ret = 0;
 
-        ret = snprintf (filename, 128, "/proc/%d/status", frame->root->pid);
-        if (ret == 128)
+        ret = snprintf (filename, sizeof filename, "/proc/%d/status", frame->root->pid);
+        if (ret >= sizeof filename)
                 goto out;
 
         fp = fopen (filename, "r");
         if (!fp)
                 goto out;
 
-        while ((ptr = fgets (line, 128, fp))) {
+        while ((ptr = fgets (line, sizeof line, fp))) {
                 if (strncmp (ptr, "Groups:", 7) != 0)
                         continue;
 
@@ -172,7 +189,7 @@ frame_fill_groups (call_frame_t *frame)
                         if (!endptr || *endptr)
                                 break;
                         frame->root->groups[idx++] = id;
-                        if (idx == GF_REQUEST_MAXGROUPS)
+                        if (idx == GF_MAX_AUX_GROUPS)
                                 break;
                 }
 
@@ -182,9 +199,54 @@ frame_fill_groups (call_frame_t *frame)
 out:
         if (fp)
                 fclose (fp);
-        return;
-}
+#elif defined(GF_SOLARIS_HOST_OS)
+        char         filename[32];
+        char         scratch[128];
+        prcred_t    *prcred = (prcred_t *) scratch;
+        FILE        *fp = NULL;
+        int          ret = 0;
 
+        ret = snprintf (filename, sizeof filename,
+                        "/proc/%d/cred", frame->root->pid);
+
+        if (ret < sizeof filename) {
+                fp = fopen (filename, "r");
+                if (fp != NULL) {
+                        if (fgets (scratch, sizeof scratch, fp) != NULL) {
+                                frame->root->ngrps = MIN(prcred->pr_ngroups,
+                                                         GF_REQUEST_MAXGROUPS);
+                        }
+                        fclose (fp);
+                 }
+         }
+#elif defined(CTL_KERN) /* DARWIN and *BSD */
+        /* 
+           N.B. CTL_KERN is an enum on Linux. (Meaning, if it's not
+           obvious, that it's not subject to preprocessor directives 
+           like '#if defined'.)
+           Unlike Linux, on Mac OS and the BSDs it is a #define. We
+           could test to see that KERN_PROC is defined, but, barring any 
+           evidence to the contrary, I think that's overkill.
+           We might also test that GF_DARWIN_HOST_OS is defined, why
+           limit this to just Mac OS. It's equally valid for the BSDs
+           and we do have people building on NetBSD and FreeBSD.
+        */
+        int name[] = { CTL_KERN, KERN_PROC, KERN_PROC_PID, frame->root->pid };
+        size_t namelen = sizeof name / sizeof name[0];
+        struct kinfo_proc kp;
+        size_t kplen = sizeof(kp);
+        int i, ngroups;
+
+        if (sysctl(name, namelen, &kp, &kplen, NULL, 0) != 0)
+                return;
+        ngroups = MIN(kp.kp_eproc.e_ucred.cr_ngroups, GF_REQUEST_MAXGROUPS);
+        for (i = 0; i < ngroups; i++)
+                frame->root->groups[i] = kp.kp_eproc.e_ucred.cr_groups[i];
+        frame->root->ngrps = ngroups;
+#else
+        frame->root->ngrps = 0;
+#endif /* GF_LINUX_HOST_OS */
+}
 
 call_frame_t *
 get_call_frame_for_req (fuse_state_t *state)
@@ -208,8 +270,9 @@ get_call_frame_for_req (fuse_state_t *state)
                 frame->root->uid      = finh->uid;
                 frame->root->gid      = finh->gid;
                 frame->root->pid      = finh->pid;
-                frame->root->lk_owner = state->lk_owner;
                 frame->root->unique   = finh->unique;
+                set_lk_owner_from_uint64 (&frame->root->lk_owner,
+                                          state->lk_owner);
         }
 
         frame_fill_groups (frame);
@@ -244,7 +307,7 @@ fuse_ino_to_inode (uint64_t ino, xlator_t *fuse)
 uint64_t
 inode_to_fuse_nodeid (inode_t *inode)
 {
-        if (!inode || inode->ino == 1)
+        if (!inode || __is_root_gfid (inode->gfid))
                 return 1;
 
         return (unsigned long) inode;
@@ -259,6 +322,7 @@ fuse_loc_fill (loc_t *loc, fuse_state_t *state, ino_t ino,
         inode_t  *parent = NULL;
         int32_t   ret = -1;
         char     *path = NULL;
+        uuid_t    null_gfid = {0,};
 
         /* resistance against multiple invocation of loc_fill not to get
            reference leaks via inode_search() */
@@ -268,6 +332,8 @@ fuse_loc_fill (loc_t *loc, fuse_state_t *state, ino_t ino,
                 if (!parent) {
                         parent = fuse_ino_to_inode (par, state->this);
                         loc->parent = parent;
+                        if (parent)
+                                uuid_copy (loc->pargfid, parent->gfid);
                 }
 
                 inode = loc->inode;
@@ -279,8 +345,8 @@ fuse_loc_fill (loc_t *loc, fuse_state_t *state, ino_t ino,
                 ret = inode_path (parent, name, &path);
                 if (ret <= 0) {
                         gf_log ("glusterfs-fuse", GF_LOG_DEBUG,
-                                "inode_path failed for %"PRId64"/%s",
-                                (parent)?parent->ino:0, name);
+                                "inode_path failed for %s/%s",
+                                (parent)?uuid_utoa (parent->gfid):"0", name);
                         goto fail;
                 }
                 loc->path = path;
@@ -289,27 +355,29 @@ fuse_loc_fill (loc_t *loc, fuse_state_t *state, ino_t ino,
                 if (!inode) {
                         inode = fuse_ino_to_inode (ino, state->this);
                         loc->inode = inode;
+                        if (inode)
+                                uuid_copy (loc->gfid, inode->gfid);
                 }
 
                 parent = loc->parent;
                 if (!parent) {
-                        parent = inode_parent (inode, par, name);
+                        parent = inode_parent (inode, null_gfid, NULL);
                         loc->parent = parent;
+                        if (parent)
+                                uuid_copy (loc->pargfid, parent->gfid);
+
                 }
 
                 ret = inode_path (inode, NULL, &path);
                 if (ret <= 0) {
                         gf_log ("glusterfs-fuse", GF_LOG_DEBUG,
-                                "inode_path failed for %"PRId64,
-                                (inode)?inode->ino:0);
+                                "inode_path failed for %s",
+                                (inode) ? uuid_utoa (inode->gfid) : "0");
                         goto fail;
                 }
                 loc->path = path;
         }
 
-        if (inode)
-                loc->ino = inode->ino;
-
         if (loc->path) {
                 loc->name = strrchr (loc->path, '/');
                 if (loc->name)
@@ -327,6 +395,10 @@ fuse_loc_fill (loc_t *loc, fuse_state_t *state, ino_t ino,
         }
         ret = 0;
 fail:
+        /* this should not happen as inode_path returns -1 when buf is NULL
+           for sure */
+        if (path && !loc->path)
+                GF_FREE (path);
         return ret;
 }
 
@@ -360,5 +432,79 @@ gf_fuse_stat2attr (struct iatt *st, struct fuse_attr *fa)
 #endif
 }
 
+static int
+fuse_do_flip_xattr_ns (char *okey, const char *nns, char **nkey)
+{
+        int   ret = 0;
+        char *key = NULL;
 
+        okey = strchr (okey, '.');
+        GF_ASSERT (okey);
 
+        key = GF_CALLOC (1, strlen (nns) + strlen(okey) + 1,
+                         gf_common_mt_char);
+        if (!key) {
+                ret = -1;
+                goto out;
+        }
+
+        strcpy (key, nns);
+        strcat (key, okey);
+
+        *nkey = key;
+
+ out:
+        return ret;
+}
+
+static int
+fuse_xattr_alloc_default (char *okey, char **nkey)
+{
+        int ret = 0;
+
+        *nkey = gf_strdup (okey);
+        if (!*nkey)
+                ret = -1;
+        return ret;
+}
+
+#define PRIV_XA_NS   "trusted"
+#define UNPRIV_XA_NS "system"
+
+int
+fuse_flip_xattr_ns (fuse_private_t *priv, char *okey, char **nkey)
+{
+        int             ret       = 0;
+        gf_boolean_t    need_flip = _gf_false;
+
+        switch (priv->client_pid) {
+        case GF_CLIENT_PID_GSYNCD:
+                /* valid xattr(s): *xtime, volume-mark* */
+                gf_log("glusterfs-fuse", GF_LOG_DEBUG, "PID: %d, checking xattr(s): "
+                       "volume-mark*, *xtime", priv->client_pid);
+                if ( (strcmp (okey, UNPRIV_XA_NS".glusterfs.volume-mark") == 0)
+                     || (fnmatch (UNPRIV_XA_NS".glusterfs.volume-mark.*", okey, FNM_PERIOD) == 0)
+                     || (fnmatch (UNPRIV_XA_NS".glusterfs.*.xtime", okey, FNM_PERIOD) == 0) )
+                        need_flip = _gf_true;
+                break;
+
+        case GF_CLIENT_PID_HADOOP:
+                /* valid xattr(s): pathinfo */
+                gf_log("glusterfs-fuse", GF_LOG_DEBUG, "PID: %d, checking xattr(s): "
+                       "pathinfo", priv->client_pid);
+                if (strcmp (okey, UNPRIV_XA_NS".glusterfs.pathinfo") == 0)
+                        need_flip = _gf_true;
+                break;
+        }
+
+        if (need_flip) {
+                gf_log ("glusterfs-fuse", GF_LOG_DEBUG, "flipping %s to "PRIV_XA_NS" equivalent",
+                        okey);
+                ret = fuse_do_flip_xattr_ns (okey, PRIV_XA_NS, nkey);
+        } else {
+                /* if we cannot match, continue with what we got */
+                ret = fuse_xattr_alloc_default (okey, nkey);
+        }
+
+        return ret;
+}
