@@ -47,6 +47,9 @@
 
 typedef ssize_t (*mnt3_serializer) (struct iovec outmsg, void *args);
 
+extern void *
+mount3udp_thread (void *argv);
+
 
 /* Generic reply function for MOUNTv3 specific replies. */
 int
@@ -56,11 +59,12 @@ mnt3svc_submit_reply (rpcsvc_request_t *req, void *arg, mnt3_serializer sfunc)
         struct iobuf            *iob = NULL;
         struct mount3_state     *ms = NULL;
         int                     ret = -1;
+        struct iobref           *iobref = NULL;
 
         if (!req)
                 return -1;
 
-        ms = (struct mount3_state *)nfs_rpcsvc_request_program_private (req);
+        ms = (struct mount3_state *)rpcsvc_request_program_private (req);
         if (!ms) {
                 gf_log (GF_MNT, GF_LOG_ERROR, "mount state not found");
                 goto ret;
@@ -69,6 +73,7 @@ mnt3svc_submit_reply (rpcsvc_request_t *req, void *arg, mnt3_serializer sfunc)
         /* First, get the io buffer into which the reply in arg will
          * be serialized.
          */
+        /* TODO: use 'xdrproc_t' instead of 'sfunc' to get the xdr-size */
         iob = iobuf_get (ms->iobpool);
         if (!iob) {
                 gf_log (GF_MNT, GF_LOG_ERROR, "Failed to get iobuf");
@@ -81,9 +86,18 @@ mnt3svc_submit_reply (rpcsvc_request_t *req, void *arg, mnt3_serializer sfunc)
          */
         outmsg.iov_len = sfunc (outmsg, arg);
 
+        iobref = iobref_new ();
+        if (iobref == NULL) {
+                gf_log (GF_MNT, GF_LOG_ERROR, "Failed to get iobref");
+                goto ret;
+        }
+
+        iobref_add (iobref, iob);
+
         /* Then, submit the message for transmission. */
-        ret = nfs_rpcsvc_submit_message (req, outmsg, iob);
+        ret = rpcsvc_submit_message (req, &outmsg, 1, NULL, 0, iobref);
         iobuf_unref (iob);
+        iobref_unref (iobref);
         if (ret == -1) {
                 gf_log (GF_MNT, GF_LOG_ERROR, "Reply submission failed");
                 goto ret;
@@ -181,6 +195,7 @@ mnt3svc_update_mountlist (struct mount3_state *ms, rpcsvc_request_t *req,
 {
         struct mountentry       *me = NULL;
         int                     ret = -1;
+        char                    *colon = NULL;
 
         if ((!ms) || (!req) || (!expname))
                 return -1;
@@ -195,10 +210,14 @@ mnt3svc_update_mountlist (struct mount3_state *ms, rpcsvc_request_t *req,
         /* Must get the IP or hostname of the client so we
          * can map it into the mount entry.
          */
-        ret = nfs_rpcsvc_conn_peername (req->conn, me->hostname, MNTPATHLEN);
+        ret = rpcsvc_transport_peername (req->trans, me->hostname, MNTPATHLEN);
         if (ret == -1)
                 goto free_err;
 
+        colon = strrchr (me->hostname, ':');
+        if (colon) {
+                *colon = '\0';
+        }
         LOCK (&ms->mountlock);
         {
                 list_add_tail (&me->mlist, &ms->mountlist);
@@ -260,16 +279,17 @@ mnt3svc_lookup_mount_cbk (call_frame_t *frame, void  *cookie,
                 return -1;
 
         mntxl = (xlator_t *)cookie;
-        ms = (struct mount3_state *)nfs_rpcsvc_request_program_private (req);
+        ms = (struct mount3_state *)rpcsvc_request_program_private (req);
         if (!ms) {
                 gf_log (GF_MNT, GF_LOG_ERROR, "mount state not found");
                 op_ret = -1;
                 op_errno = EINVAL;
         }
 
-        if (op_ret == -1)
+        if (op_ret == -1) {
+                gf_log (GF_NFS, GF_LOG_ERROR, "error=%s", strerror (op_errno));
                 status = mnt3svc_errno_to_mnterr (op_errno);
-
+        }
         if (status != MNT3_OK)
                 goto xmit_res;
 
@@ -287,9 +307,9 @@ xmit_res:
         gf_log (GF_MNT, GF_LOG_DEBUG, "MNT reply: fh %s, status: %d", fhstr,
                 status);
         if (op_ret == 0) {
-                svc = nfs_rpcsvc_request_service (req);
-                autharrlen = nfs_rpcsvc_auth_array (svc, mntxl->name, autharr,
-                                                    10);
+                svc = rpcsvc_request_service (req);
+                autharrlen = rpcsvc_auth_array (svc, mntxl->name, autharr,
+                                                10);
         }
 
         res = mnt3svc_set_mountres3 (status, &fh, autharr, autharrlen);
@@ -339,11 +359,11 @@ mnt3svc_mount_inode (rpcsvc_request_t *req, struct mount3_state *ms,
         if ((!req) || (!xl) || (!ms) || (!exportinode))
                 return ret;
 
-        ret = nfs_inode_loc_fill (exportinode, &exportloc);
+        ret = nfs_inode_loc_fill (exportinode, &exportloc, NFS_RESOLVE_EXIST);
         if (ret < 0) {
                 gf_log (GF_MNT, GF_LOG_ERROR, "Loc fill failed for export inode"
-                        ": ino %"PRIu64", volume: %s",
-                        exportinode->ino, xl->name);
+                        ": gfid %s, volume: %s",
+                        uuid_utoa (exportinode->gfid), xl->name);
                 goto err;
         }
 
@@ -380,7 +400,7 @@ mnt3svc_volume_mount (rpcsvc_request_t *req, struct mount3_state *ms,
         rootgfid[15] = 1;
         exportinode = inode_find (exp->vol->itable, rootgfid);
         if (!exportinode) {
-                gf_log (GF_MNT, GF_LOG_ERROR, "Faild to get root inode");
+                gf_log (GF_MNT, GF_LOG_ERROR, "Failed to get root inode");
                 ret = -ENOENT;
                 goto err;
         }
@@ -505,7 +525,8 @@ __mnt3_resolve_export_subdir_comp (mnt3_resolve_t *mres)
                                   &mres->resolveloc, NFS_RESOLVE_CREATE);
         if ((ret < 0) && (ret != -2)) {
                 gf_log (GF_MNT, GF_LOG_ERROR, "Failed to resolve and create "
-                        "inode: parent gfid %s, entry %s", uuid_utoa (mres->resolveloc.inode->gfid), nextcomp);
+                        "inode: parent gfid %s, entry %s",
+                        uuid_utoa (mres->resolveloc.inode->gfid), nextcomp);
                 ret = -EFAULT;
                 goto err;
         }
@@ -537,6 +558,8 @@ mnt3_resolve_subdir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         mres = frame->local;
         mntxl = (xlator_t *)cookie;
         if (op_ret == -1) {
+                gf_log (GF_NFS, GF_LOG_ERROR, "path=%s (%s)",
+                        mres->resolveloc.path, strerror (op_errno));
                 mntstat = mnt3svc_errno_to_mnterr (op_errno);
                 goto err;
         }
@@ -561,9 +584,9 @@ err:
         if (op_ret == -1) {
                 gf_log (GF_MNT, GF_LOG_DEBUG, "Mount reply status: %d",
                         mntstat);
-                svc = nfs_rpcsvc_request_service (mres->req);
-                autharrlen = nfs_rpcsvc_auth_array (svc, mntxl->name, autharr,
-                                                    10);
+                svc = rpcsvc_request_service (mres->req);
+                autharrlen = rpcsvc_auth_array (svc, mntxl->name, autharr,
+                                                10);
 
                 res = mnt3svc_set_mountres3 (mntstat, &fh, autharr, autharrlen);
                 mnt3svc_submit_reply (mres->req, (void *)&res,
@@ -738,24 +761,38 @@ int
 mnt3_check_client_net (struct mount3_state *ms, rpcsvc_request_t *req,
                        xlator_t *targetxl)
 {
-        rpcsvc_t        *svc = NULL;
-        int             ret = -1;
+
+        rpcsvc_t                *svc = NULL;
+        rpc_transport_t         *trans = NULL;
+        struct sockaddr_storage sastorage = {0,};
+        char                    peer[RPCSVC_PEER_STRLEN] = {0,};
+        int                     ret = -1;
 
         if ((!ms) || (!req) || (!targetxl))
                 return -1;
 
-        svc = nfs_rpcsvc_request_service (req);
-        ret = nfs_rpcsvc_conn_peer_check (svc->options, targetxl->name,
-                                          nfs_rpcsvc_request_conn (req));
+        svc = rpcsvc_request_service (req);
+
+        trans = rpcsvc_request_transport (req);
+        ret = rpcsvc_transport_peeraddr (trans, peer, RPCSVC_PEER_STRLEN,
+                                         &sastorage, sizeof (sastorage));
+        if (ret != 0) {
+                gf_log (GF_MNT, GF_LOG_WARNING, "Failed to get peer addr: %s",
+                        gai_strerror (ret));
+        }
+
+        ret = rpcsvc_transport_peer_check (svc->options, targetxl->name,
+                                           trans);
         if (ret == RPCSVC_AUTH_REJECT) {
-                gf_log (GF_MNT, GF_LOG_TRACE, "Peer not allowed");
+                gf_log (GF_MNT, GF_LOG_INFO, "Peer %s  not allowed", peer);
                 goto err;
         }
 
-        ret = nfs_rpcsvc_conn_privport_check (svc, targetxl->name,
-                                              nfs_rpcsvc_request_conn (req));
+        ret = rpcsvc_transport_privport_check (svc, targetxl->name,
+                                               rpcsvc_request_transport (req));
         if (ret == RPCSVC_AUTH_REJECT) {
-                gf_log (GF_MNT, GF_LOG_TRACE, "Unprivileged port not allowed");
+                gf_log (GF_MNT, GF_LOG_INFO, "Peer %s rejected. Unprivileged "
+                        "port not allowed", peer);
                 goto err;
         }
 
@@ -804,19 +841,17 @@ mnt3_find_export (rpcsvc_request_t *req, char *path, struct mnt3_export **e)
         int                     ret = -EFAULT;
         struct mount3_state     *ms = NULL;
         struct mnt3_export      *exp = NULL;
-        struct nfs_state        *nfs = NULL;
 
         if ((!req) || (!path) || (!e))
                 return -1;
 
-        ms = (struct mount3_state *)nfs_rpcsvc_request_program_private (req);
+        ms = (struct mount3_state *) rpcsvc_request_program_private (req);
         if (!ms) {
                 gf_log (GF_MNT, GF_LOG_ERROR, "Mount state not present");
-                nfs_rpcsvc_request_seterr (req, SYSTEM_ERR);
+                rpcsvc_request_seterr (req, SYSTEM_ERR);
                 goto err;
         }
 
-        nfs = (struct nfs_state *)ms->nfsx->private;
         gf_log (GF_MNT, GF_LOG_DEBUG, "dirpath: %s", path);
         exp = mnt3_mntpath_to_export (ms, path);
         if (exp) {
@@ -857,17 +892,17 @@ mnt3svc_mnt (rpcsvc_request_t *req)
 
         pvec.iov_base = path;
         pvec.iov_len = MNTPATHLEN;
-        ret = xdr_to_mountpath (pvec, req->msg);
+        ret = xdr_to_mountpath (pvec, req->msg[0]);
         if (ret == -1) {
                 gf_log (GF_MNT, GF_LOG_ERROR, "Failed to decode args");
-                nfs_rpcsvc_request_seterr (req, GARBAGE_ARGS);
+                rpcsvc_request_seterr (req, GARBAGE_ARGS);
                 goto rpcerr;
         }
 
-        ms = (struct mount3_state *)nfs_rpcsvc_request_program_private (req);
+        ms = (struct mount3_state *)rpcsvc_request_program_private (req);
         if (!ms) {
                 gf_log (GF_MNT, GF_LOG_ERROR, "Mount state not present");
-                nfs_rpcsvc_request_seterr (req, SYSTEM_ERR);
+                rpcsvc_request_seterr (req, SYSTEM_ERR);
                 ret = -1;
                 goto rpcerr;
         }
@@ -924,8 +959,7 @@ mnt3svc_null (rpcsvc_request_t *req)
                 gf_log (GF_MNT, GF_LOG_ERROR, "Got NULL request!");
                 return 0;
         }
-
-        nfs_rpcsvc_submit_generic (req, dummyvec, NULL);
+        rpcsvc_submit_generic (req, &dummyvec, 1,  NULL, 0, NULL);
         return 0;
 }
 
@@ -1031,19 +1065,19 @@ mnt3svc_dump (rpcsvc_request_t *req)
         if (!req)
                 return -1;
 
-        ms = (struct mount3_state *)nfs_rpcsvc_request_program_private (req);
+        ms = (struct mount3_state *)rpcsvc_request_program_private (req);
         if (!ms) {
-                nfs_rpcsvc_request_seterr (req, SYSTEM_ERR);
+                rpcsvc_request_seterr (req, SYSTEM_ERR);
                 goto rpcerr;
         }
 
         sfunc = (mnt3_serializer)xdr_serialize_mountlist;
         mlist = mnt3svc_build_mountlist (ms, &ret);
-        arg = mlist;
-        
+        arg = &mlist;
+
         if (!mlist) {
                 if (ret != 0) {
-                        nfs_rpcsvc_request_seterr (req, SYSTEM_ERR);
+                        rpcsvc_request_seterr (req, SYSTEM_ERR);
                         ret = -1;
                         goto rpcerr;
                 } else {
@@ -1137,6 +1171,7 @@ mnt3svc_umnt (rpcsvc_request_t *req)
         int                     ret = -1;
         struct mount3_state     *ms = NULL;
         mountstat3              mstat = MNT3_OK;
+        char                    *colon = NULL;
 
         if (!req)
                 return -1;
@@ -1144,56 +1179,43 @@ mnt3svc_umnt (rpcsvc_request_t *req)
         /* Remove the mount point from the exports list. */
         pvec.iov_base = dirpath;
         pvec.iov_len = MNTPATHLEN;
-        ret = xdr_to_mountpath (pvec, req->msg);;
+        ret = xdr_to_mountpath (pvec, req->msg[0]);
         if (ret == -1) {
                 gf_log (GF_MNT, GF_LOG_ERROR, "Failed decode args");
-                nfs_rpcsvc_request_seterr (req, GARBAGE_ARGS);
+                rpcsvc_request_seterr (req, GARBAGE_ARGS);
                 goto rpcerr;
         }
 
-        ms = (struct mount3_state *)nfs_rpcsvc_request_program_private (req);
+        ms = (struct mount3_state *)rpcsvc_request_program_private (req);
         if (!ms) {
                 gf_log (GF_MNT, GF_LOG_ERROR, "Mount state not present");
-                nfs_rpcsvc_request_seterr (req, SYSTEM_ERR);
+                rpcsvc_request_seterr (req, SYSTEM_ERR);
                 ret = -1;
                 goto rpcerr;
         }
 
-        ret = nfs_rpcsvc_conn_peername (req->conn, hostname, MNTPATHLEN);
+        ret = rpcsvc_transport_peername (req->trans, hostname, MNTPATHLEN);
         if (ret != 0) {
                 gf_log (GF_MNT, GF_LOG_ERROR, "Failed to get remote name: %s",
                         gai_strerror (ret));
-                goto try_umount_with_addr;
-        }
-
-        gf_log (GF_MNT, GF_LOG_DEBUG, "dirpath: %s, hostname: %s", dirpath,
-                hostname);
-        ret = mnt3svc_umount (ms, dirpath, hostname);
-
-        /* Unmount succeeded with the given hostname. */
-        if (ret == 0)
-                goto snd_reply;
-
-try_umount_with_addr:
-        if (ret != 0)
-                ret = nfs_rpcsvc_conn_peeraddr (req->conn, hostname, MNTPATHLEN,
-                                                NULL, 0);
-
-        if (ret != 0) {
-                gf_log (GF_MNT, GF_LOG_ERROR, "Failed to get remote addr: %s",
-                        gai_strerror (ret));
-                nfs_rpcsvc_request_seterr (req, SYSTEM_ERR);
                 goto rpcerr;
         }
 
+        colon = strrchr (hostname, ':');
+        if (colon) {
+                *colon= '\0';
+        }
         gf_log (GF_MNT, GF_LOG_DEBUG, "dirpath: %s, hostname: %s", dirpath,
                 hostname);
         ret = mnt3svc_umount (ms, dirpath, hostname);
-        if (ret == -1)
-                mstat = MNT3ERR_INVAL;
 
-        ret = 0;
-snd_reply:
+        if (ret == -1) {
+                ret = 0;
+                mstat = MNT3ERR_NOENT;
+        }
+        /* FIXME: also take care of the corner case where the
+         * client was resolvable at mount but not at the umount - vice-versa.
+         */
         mnt3svc_submit_reply (req, &mstat,
                               (mnt3_serializer)xdr_serialize_mountstat3);
 
@@ -1250,10 +1272,10 @@ mnt3svc_umntall (rpcsvc_request_t *req)
         if (!req)
                 return ret;
 
-        ms = (struct mount3_state *)nfs_rpcsvc_request_program_private (req);
+        ms = (struct mount3_state *)rpcsvc_request_program_private (req);
         if (!ms) {
                 gf_log (GF_MNT, GF_LOG_ERROR, "Mount state not present");
-                nfs_rpcsvc_request_seterr (req, SYSTEM_ERR);
+                rpcsvc_request_seterr (req, SYSTEM_ERR);
                 goto rpcerr;
         }
 
@@ -1309,8 +1331,8 @@ mnt3_xlchildren_to_exports (rpcsvc_t *svc, struct mount3_state *ms)
 
                 strcpy (elist->ex_dir, ent->expname);
 
-                addrstr = nfs_rpcsvc_volume_allowed (svc->options,
-                                                     ent->vol->name);
+                addrstr = rpcsvc_volume_allowed (svc->options,
+                                                 ent->vol->name);
                 if (addrstr)
                         addrstr = gf_strdup (addrstr);
                 else
@@ -1357,15 +1379,15 @@ mnt3svc_export (rpcsvc_request_t *req)
         if (!req)
                 return -1;
 
-        ms = (struct mount3_state *)nfs_rpcsvc_request_program_private (req);
+        ms = (struct mount3_state *)rpcsvc_request_program_private (req);
         if (!ms) {
                 gf_log (GF_MNT, GF_LOG_ERROR, "mount state not found");
-                nfs_rpcsvc_request_seterr (req, SYSTEM_ERR);
+                rpcsvc_request_seterr (req, SYSTEM_ERR);
                 goto err;
         }
 
         /* Using the children translator names, build the export list */
-        elist = mnt3_xlchildren_to_exports (nfs_rpcsvc_request_service (req),
+        elist = mnt3_xlchildren_to_exports (rpcsvc_request_service (req),
                                             ms);
         /* Do not return error when exports list is empty. An exports list can
          * be empty when no subvolumes have come up. No point returning error
@@ -1387,6 +1409,87 @@ err:
         return ret;
 }
 
+/* just declaring, definition is way down below */
+rpcsvc_program_t        mnt3prog;
+
+/* nfs3_rootfh used by mount3udp thread needs to access mount3prog.private
+ * directly as we don't have nfs xlator pointer to dereference it. But thats OK
+ */
+
+struct nfs3_fh *
+nfs3_rootfh (char* path)
+{
+        struct mount3_state     *ms = NULL;
+        struct nfs3_fh          *fh = NULL;
+        struct mnt3_export      *exp = NULL;
+        inode_t                 *inode = NULL;
+        char                    *tmp = NULL;
+
+        ms = mnt3prog.private;
+        exp = mnt3_mntpath_to_export (ms, path);
+        if (exp == NULL)
+                goto err;
+
+        tmp = (char *)path;
+        tmp = strchr (tmp, '/');
+        if (tmp == NULL)
+                tmp = "/";
+
+        inode = inode_from_path (exp->vol->itable, tmp);
+        if (inode == NULL)
+                goto err;
+
+        fh = GF_CALLOC (1, sizeof(*fh), gf_nfs_mt_nfs3_fh);
+        if (fh == NULL)
+                goto err;
+        nfs3_build_fh (inode, exp->volumeid, fh);
+
+err:
+        if (inode)
+                inode_unref (inode);
+        return fh;
+}
+
+int
+mount3udp_add_mountlist (char *host, dirpath *expname)
+{
+        struct mountentry       *me = NULL;
+        struct mount3_state     *ms = NULL;
+        char                    *export = NULL;
+
+        ms = mnt3prog.private;
+        me = GF_CALLOC (1, sizeof (*me), gf_nfs_mt_mountentry);
+        if (!me)
+                return -1;
+        export = (char *)expname;
+        while (*export == '/')
+                export++;
+
+        strcpy (me->exname, export);
+        strcpy (me->hostname, host);
+        INIT_LIST_HEAD (&me->mlist);
+        LOCK (&ms->mountlock);
+        {
+                list_add_tail (&me->mlist, &ms->mountlist);
+        }
+        UNLOCK (&ms->mountlock);
+        return 0;
+}
+
+int
+mount3udp_delete_mountlist (char *hostname, dirpath *expname)
+{
+        struct mount3_state     *ms = NULL;
+        char                    *export = NULL;
+
+        ms = mnt3prog.private;
+        export = (char *)expname;
+        while (*export == '/')
+                export++;
+        __mnt3svc_umount (ms, export, hostname);
+        return 0;
+}
+
 
 struct mnt3_export *
 mnt3_init_export_ent (struct mount3_state *ms, xlator_t *xl, char *exportpath,
@@ -1431,7 +1534,10 @@ mnt3_init_export_ent (struct mount3_state *ms, xlator_t *xl, char *exportpath,
                 exp->exptype = MNT3_EXPTYPE_VOLUME;
                 ret = snprintf (exp->expname, alloclen, "/%s", xl->name);
         }
-
+        if (ret < 0) {
+                gf_log (xl->name, GF_LOG_WARNING,
+                        "failed to get the export name");
+        }
         /* Just copy without discrimination, we'll determine whether to
          * actually use it when a mount request comes in and a file handle
          * needs to be built.
@@ -1716,13 +1822,35 @@ mnt3_init_state (xlator_t *nfsx)
         return ms;
 }
 
+int
+mount_init_state (xlator_t *nfsx)
+{
+        int              ret = -1;
+        struct nfs_state *nfs = NULL;
+
+        if (!nfsx)
+                goto out;
+
+        nfs = (struct nfs_state *)nfs_state (nfsx);
+        /*Maintaining global state for MOUNT1 and MOUNT3*/
+        nfs->mstate =  mnt3_init_state (nfsx);
+        if (!nfs->mstate) {
+                gf_log (GF_NFS, GF_LOG_ERROR, "Failed to allocate"
+                        "mount state");
+                goto out;
+        }
+        ret = 0;
+out:
+        return ret;
+}
+
 rpcsvc_actor_t  mnt3svc_actors[MOUNT3_PROC_COUNT] = {
-        {"NULL", MOUNT3_NULL, mnt3svc_null, NULL, NULL},
-        {"MNT", MOUNT3_MNT, mnt3svc_mnt, NULL, NULL},
-        {"DUMP", MOUNT3_DUMP, mnt3svc_dump, NULL, NULL},
-        {"UMNT", MOUNT3_UMNT, mnt3svc_umnt, NULL, NULL},
-        {"UMNTALL", MOUNT3_UMNTALL, mnt3svc_umntall, NULL, NULL},
-        {"EXPORT", MOUNT3_EXPORT, mnt3svc_export, NULL, NULL}
+        {"NULL", MOUNT3_NULL, mnt3svc_null, NULL, NULL, 0},
+        {"MNT", MOUNT3_MNT, mnt3svc_mnt, NULL, NULL, 0},
+        {"DUMP", MOUNT3_DUMP, mnt3svc_dump, NULL, NULL, 0},
+        {"UMNT", MOUNT3_UMNT, mnt3svc_umnt, NULL, NULL, 0},
+        {"UMNTALL", MOUNT3_UMNTALL, mnt3svc_umntall, NULL, NULL, 0},
+        {"EXPORT", MOUNT3_EXPORT, mnt3svc_export, NULL, NULL, 0}
 };
 
 
@@ -1735,30 +1863,74 @@ rpcsvc_program_t        mnt3prog = {
                         .prognum        = MOUNT_PROGRAM,
                         .progver        = MOUNT_V3,
                         .progport       = GF_MOUNTV3_PORT,
-                        .progaddrfamily = AF_INET,
-                        .proghost       = NULL,
                         .actors         = mnt3svc_actors,
                         .numactors      = MOUNT3_PROC_COUNT,
+                        .min_auth       = AUTH_NULL,
 };
 
 
+
 rpcsvc_program_t *
 mnt3svc_init (xlator_t *nfsx)
 {
         struct mount3_state     *mstate = NULL;
+        struct nfs_state        *nfs = NULL;
+        dict_t                  *options = NULL;
+        char                    *portstr = NULL;
+        int                      ret = -1;
+        pthread_t                udp_thread;
 
-        if (!nfsx)
+        if (!nfsx || !nfsx->private)
                 return NULL;
 
+        nfs = (struct nfs_state *)nfsx->private;
+
         gf_log (GF_MNT, GF_LOG_DEBUG, "Initing Mount v3 state");
-        mstate = mnt3_init_state (nfsx);
+        mstate = (struct mount3_state *)nfs->mstate;
         if (!mstate) {
                 gf_log (GF_MNT, GF_LOG_ERROR, "Mount v3 state init failed");
                 goto err;
         }
 
         mnt3prog.private = mstate;
+        options = dict_new ();
 
+        ret = gf_asprintf (&portstr, "%d", GF_MOUNTV3_PORT);
+        if (ret == -1)
+                goto err;
+
+        ret = dict_set_dynstr (options, "transport.socket.listen-port", portstr);
+        if (ret == -1)
+                goto err;
+        ret = dict_set_str (options, "transport-type", "socket");
+        if (ret == -1) {
+                gf_log (GF_NFS, GF_LOG_ERROR, "dict_set_str error");
+                goto err;
+        }
+
+        if (nfs->allow_insecure) {
+                ret = dict_set_str (options, "rpc-auth-allow-insecure", "on");
+                if (ret == -1) {
+                        gf_log (GF_NFS, GF_LOG_ERROR, "dict_set_str error");
+                        goto err;
+                }
+                ret = dict_set_str (options, "rpc-auth.ports.insecure", "on");
+                if (ret == -1) {
+                        gf_log (GF_NFS, GF_LOG_ERROR, "dict_set_str error");
+                        goto err;
+                }
+        }
+
+        rpcsvc_create_listeners (nfs->rpcsvc, options, nfsx->name);
+        if (ret == -1) {
+                gf_log (GF_NFS, GF_LOG_ERROR, "Unable to create listeners");
+                dict_unref (options);
+                goto err;
+        }
+
+        if (nfs->mount_udp) {
+                pthread_create (&udp_thread, NULL, mount3udp_thread, NULL);
+        }
         return &mnt3prog;
 err:
         return NULL;
@@ -1766,12 +1938,12 @@ err:
 
 
 rpcsvc_actor_t  mnt1svc_actors[MOUNT1_PROC_COUNT] = {
-        {"NULL", MOUNT1_NULL, mnt3svc_null, NULL, NULL},
-        {{0}, },
-        {"DUMP", MOUNT1_DUMP, mnt3svc_dump, NULL, NULL},
-        {"UMNT", MOUNT1_UMNT, mnt3svc_umnt, NULL, NULL},
-        {{0}, },
-        {"EXPORT", MOUNT1_EXPORT, mnt3svc_export, NULL, NULL}
+        {"NULL", MOUNT1_NULL, mnt3svc_null, NULL, NULL, 0},
+        {{0, 0}, },
+        {"DUMP", MOUNT1_DUMP, mnt3svc_dump, NULL, NULL, 0},
+        {"UMNT", MOUNT1_UMNT, mnt3svc_umnt, NULL, NULL, 0},
+        {{0, 0}, },
+        {"EXPORT", MOUNT1_EXPORT, mnt3svc_export, NULL, NULL, 0}
 };
 
 rpcsvc_program_t        mnt1prog = {
@@ -1779,10 +1951,9 @@ rpcsvc_program_t        mnt1prog = {
                         .prognum        = MOUNT_PROGRAM,
                         .progver        = MOUNT_V1,
                         .progport       = GF_MOUNTV1_PORT,
-                        .progaddrfamily = AF_INET,
-                        .proghost       = NULL,
                         .actors         = mnt1svc_actors,
                         .numactors      = MOUNT1_PROC_COUNT,
+                        .min_auth       = AUTH_NULL,
 };
 
 
@@ -1790,12 +1961,18 @@ rpcsvc_program_t *
 mnt1svc_init (xlator_t *nfsx)
 {
         struct mount3_state     *mstate = NULL;
+        struct nfs_state        *nfs = NULL;
+        dict_t                  *options = NULL;
+        char                    *portstr = NULL;
+        int                      ret = -1;
 
-        if (!nfsx)
+        if (!nfsx || !nfsx->private)
                 return NULL;
 
+        nfs = (struct nfs_state *)nfsx->private;
+
         gf_log (GF_MNT, GF_LOG_DEBUG, "Initing Mount v1 state");
-        mstate = mnt3_init_state (nfsx);
+        mstate = (struct mount3_state *)nfs->mstate;
         if (!mstate) {
                 gf_log (GF_MNT, GF_LOG_ERROR, "Mount v3 state init failed");
                 goto err;
@@ -1803,9 +1980,42 @@ mnt1svc_init (xlator_t *nfsx)
 
         mnt1prog.private = mstate;
 
+        options = dict_new ();
+
+        ret = gf_asprintf (&portstr, "%d", GF_MOUNTV1_PORT);
+        if (ret == -1)
+                goto err;
+
+        ret = dict_set_dynstr (options, "transport.socket.listen-port", portstr);
+        if (ret == -1)
+                goto err;
+        ret = dict_set_str (options, "transport-type", "socket");
+        if (ret == -1) {
+                gf_log (GF_NFS, GF_LOG_ERROR, "dict_set_str error");
+                goto err;
+        }
+
+        if (nfs->allow_insecure) {
+                ret = dict_set_str (options, "rpc-auth-allow-insecure", "on");
+                if (ret == -1) {
+                        gf_log (GF_NFS, GF_LOG_ERROR, "dict_set_str error");
+                        goto err;
+                }
+                ret = dict_set_str (options, "rpc-auth.ports.insecure", "on");
+                if (ret == -1) {
+                        gf_log (GF_NFS, GF_LOG_ERROR, "dict_set_str error");
+                        goto err;
+                }
+        }
+
+        rpcsvc_create_listeners (nfs->rpcsvc, options, nfsx->name);
+        if (ret == -1) {
+                gf_log (GF_NFS, GF_LOG_ERROR, "Unable to create listeners");
+                dict_unref (options);
+                goto err;
+        }
+
         return &mnt1prog;
 err:
         return NULL;
 }
-
-
