@@ -43,7 +43,7 @@ struct nfs_direntcache {
  * different NFS versions can simply call a standard interface and have fop
  * interface dependent functions be handled internally.
  * This structure is part of such an  abstraction. The fops layer stores any
- * state is requires in the fd. For eg, the dirent cache for a directory fd_t.
+ * state is requires in the fd. E.g. the dirent cache for a directory fd_t.
  */
 typedef struct nfs_fop_fdcontext {
         pthread_mutex_t         lock;
@@ -92,7 +92,7 @@ nfs_fsync (xlator_t *nfsx, xlator_t *xl, nfs_user_t *nfu, fd_t *fd,
 
 extern int
 nfs_write (xlator_t *nfsx, xlator_t *xl, nfs_user_t *nfu, fd_t *fd,
-           struct iobuf *srciob, struct iovec *vector, int32_t count,
+           struct iobref *srciobref, struct iovec *vector, int32_t count,
            off_t offset, fop_writev_cbk_t cbk, void *local);
 
 extern int
@@ -161,4 +161,7 @@ nfs_opendir (xlator_t *nfsx, xlator_t *fopxl, nfs_user_t *nfu, loc_t *pathloc,
 extern int
 nfs_access (xlator_t *nfsx, xlator_t *xl, nfs_user_t *nfu, loc_t *pathloc,
             int32_t accesstest, fop_access_cbk_t cbk, void *local);
+extern int
+nfs_lk (xlator_t *nfsx, xlator_t *xl, nfs_user_t *nfu, fd_t *fd,
+        int cmd, struct gf_flock *flock, fop_lk_cbk_t cbk, void *local);
 #endif
