@@ -39,7 +39,8 @@
 #include "mount3.h"
 #include "nfs3.h"
 #include "nfs-mem-types.h"
-#include "statedump.h"
+#include "nfs3-helpers.h"
+#include "nlm4.h"
 
 /* Every NFS version must call this function with the init function
  * for its particular version.
@@ -82,8 +83,8 @@ nfs_deinit_versions (struct list_head *versions, xlator_t *this)
                    version->deinit (this);
                 */
                 if (version->program)
-                        nfs_rpcsvc_program_unregister (nfs->rpcsvc,
-                                                  *(version->program));
+                        rpcsvc_program_unregister (nfs->rpcsvc,
+                                                   (version->program));
 
                 list_del (&version->list);
                 GF_FREE (version);
@@ -92,7 +93,6 @@ nfs_deinit_versions (struct list_head *versions, xlator_t *this)
         return 0;
 }
 
-
 int
 nfs_init_versions (struct nfs_state *nfs, xlator_t *this)
 {
@@ -118,17 +118,26 @@ nfs_init_versions (struct nfs_state *nfs, xlator_t *this)
                         ret = -1;
                         goto err;
                 }
-                prog->actorxl = this;
+//                prog->actorxl = this;
                 version->program = prog;
                 if (nfs->override_portnum)
                         prog->progport = nfs->override_portnum;
                 gf_log (GF_NFS, GF_LOG_DEBUG, "Starting program: %s",
                         prog->progname);
-                ret = nfs_rpcsvc_program_register (nfs->rpcsvc, *prog);
+
+                ret = rpcsvc_program_register (nfs->rpcsvc, prog);
                 if (ret == -1) {
                         gf_log (GF_NFS, GF_LOG_ERROR, "Program init failed");
                         goto err;
                 }
+                if (rpcsvc_register_portmap_enabled(nfs->rpcsvc)) {
+                        ret = rpcsvc_program_register_portmap (prog,
+                                                               prog->progport);
+                        if (ret == -1) {
+                                gf_log (GF_NFS, GF_LOG_ERROR, "Program registration failed");
+                                goto err;
+                        }
+                }
         }
 
         ret = 0;
@@ -164,6 +173,15 @@ nfs_add_all_initiators (struct nfs_state *nfs)
                 goto ret;
         }
 
+        if (nfs->enable_nlm == _gf_true) {
+                ret = nfs_add_initer (&nfs->versions, nlm4svc_init);
+                if (ret == -1) {
+                        gf_log (GF_NFS, GF_LOG_ERROR, "Failed to add protocol"
+                                " initializer");
+                        goto ret;
+                }
+        }
+
         ret = 0;
 ret:
         return ret;
@@ -266,7 +284,7 @@ nfs_startup_subvolume (xlator_t *nfsx, xlator_t *xl)
                 goto err;
         }
 
-        ret = nfs_inode_loc_fill (xl->itable->root, &rootloc);
+        ret = nfs_root_loc_fill (xl->itable, &rootloc);
         if (ret == -1) {
                 gf_log (GF_NFS, GF_LOG_CRITICAL, "Failed to init root loc");
                 goto err;
@@ -367,7 +385,7 @@ nfs_init_subvolumes (struct nfs_state *nfs, xlator_list_t *cl)
         }
 
         LOCK_INIT (&nfs->svinitlock);
-        nfs->initedxl = GF_CALLOC (svcount, sizeof (xlator_t *), 
+        nfs->initedxl = GF_CALLOC (svcount, sizeof (xlator_t *),
                                    gf_nfs_mt_xlator_t );
         if (!nfs->initedxl) {
                 gf_log (GF_NFS, GF_LOG_ERROR, "Failed to allocated inited xls");
@@ -440,13 +458,30 @@ nfs_request_user_init (nfs_user_t *nfu, rpcsvc_request_t *req)
         if ((!req) || (!nfu))
                 return;
 
-        gidarr = nfs_rpcsvc_auth_unix_auxgids (req, &gids);
-        nfs_user_create (nfu, nfs_rpcsvc_request_uid (req),
-                         nfs_rpcsvc_request_gid (req), gidarr, gids);
+        gidarr = rpcsvc_auth_unix_auxgids (req, &gids);
+        nfs_user_create (nfu, rpcsvc_request_uid (req),
+                         rpcsvc_request_gid (req), gidarr, gids);
 
         return;
 }
 
+void
+nfs_request_primary_user_init (nfs_user_t *nfu, rpcsvc_request_t *req,
+                               uid_t uid, gid_t gid)
+{
+        gid_t           *gidarr = NULL;
+        int             gids = 0;
+
+        if ((!req) || (!nfu))
+                return;
+
+        gidarr = rpcsvc_auth_unix_auxgids (req, &gids);
+        nfs_user_create (nfu, uid, gid, gidarr, gids);
+
+        return;
+}
+
+
 int32_t
 mem_acct_init (xlator_t *this)
 {
@@ -456,7 +491,7 @@ mem_acct_init (xlator_t *this)
                 return ret;
 
         ret = xlator_mem_acct_init (this, gf_nfs_mt_end + 1);
-        
+
         if (ret != 0) {
                 gf_log(this->name, GF_LOG_ERROR, "Memory accounting init"
                                 "failed");
@@ -475,7 +510,7 @@ nfs_init_state (xlator_t *this)
         unsigned int            fopspoolsize = 0;
         char                    *optstr = NULL;
         gf_boolean_t            boolt = _gf_false;
-        char                    *def_val = NULL;
+        struct stat             stbuf = {0,};
 
         if (!this)
                 return NULL;
@@ -492,42 +527,20 @@ nfs_init_state (xlator_t *this)
                 return NULL;
         }
 
-        /* RPC service needs to be started before NFS versions can be
-         * inited. */
-        nfs->rpcsvc =  nfs_rpcsvc_init (this->ctx, this->options);
-        if (!nfs->rpcsvc) {
-                gf_log (GF_NFS, GF_LOG_ERROR, "RPC service init failed");
-                goto free_nfs;
-        }
-
-        if (xlator_get_volopt_info (&this->volume_options, "nfs.mem-factor",
-                                    &def_val, NULL)) {
-                gf_log (this->name, GF_LOG_ERROR, "Default value of "
-                         "nfs.mem-factor not found");
-                ret = -1;
-                goto free_rpcsvc;
-        } else {
-                if (gf_string2uint (def_val, &nfs->memfactor)) {
-                        gf_log (this->name, GF_LOG_ERROR, "Default value of "
-                                 "nfs.mem-factor corrupt");
-                        ret = -1;
-                        goto free_rpcsvc;
-                }
-        }
-
+        nfs->memfactor = GF_NFS_DEFAULT_MEMFACTOR;
         if (dict_get (this->options, "nfs.mem-factor")) {
                 ret = dict_get_str (this->options, "nfs.mem-factor",
                                     &optstr);
                 if (ret < 0) {
                         gf_log (GF_NFS, GF_LOG_ERROR, "Failed to parse dict");
-                        goto free_foppool;
+                        goto free_rpcsvc;
                 }
 
                 ret = gf_string2uint (optstr, &nfs->memfactor);
                 if (ret < 0) {
                         gf_log (GF_NFS, GF_LOG_ERROR, "Failed to parse uint "
                                 "string");
-                        goto free_foppool;
+                        goto free_rpcsvc;
                 }
         }
 
@@ -560,21 +573,23 @@ nfs_init_state (xlator_t *this)
                         nfs->dynamicvolumes = GF_NFS_DVM_ON;
         }
 
-        if (xlator_get_volopt_info (&this->volume_options, "nfs.enable-ino32",
-                                    &def_val, NULL)) {
-                gf_log (this->name, GF_LOG_ERROR, "Default value of "
-                         "nfs.enable-ino32 not found");
-                ret = -1;
-                goto free_foppool;
-        } else {
-                if (gf_string2boolean (def_val,(gf_boolean_t *) &nfs->enable_ino32 )) {
-                        gf_log (this->name, GF_LOG_ERROR, "Default value of "
-                                 "nfs.enable-ino32 corrupt");
-                        ret = -1;
+        nfs->enable_nlm = _gf_true;
+        if (!dict_get_str (this->options, "nfs.nlm", &optstr)) {
+
+                ret = gf_string2boolean (optstr, &boolt);
+                if (ret < 0) {
+                        gf_log (GF_NFS, GF_LOG_ERROR, "Failed to parse"
+                                " bool string");
                         goto free_foppool;
                 }
+
+                if (boolt == _gf_false) {
+                        gf_log (GF_NFS, GF_LOG_INFO, "NLM is manually disabled");
+                        nfs->enable_nlm = _gf_false;
+                }
         }
 
+        nfs->enable_ino32 = 0;
         if (dict_get (this->options, "nfs.enable-ino32")) {
                 ret = dict_get_str (this->options, "nfs.enable-ino32",
                                     &optstr);
@@ -594,7 +609,6 @@ nfs_init_state (xlator_t *this)
                         nfs->enable_ino32 = 1;
         }
 
-        nfs->override_portnum = 0;
         if (dict_get (this->options, "nfs.port")) {
                 ret = dict_get_str (this->options, "nfs.port",
                                     &optstr);
@@ -611,6 +625,124 @@ nfs_init_state (xlator_t *this)
                 }
         }
 
+        if (dict_get(this->options, "transport.socket.listen-port") == NULL) {
+                if (nfs->override_portnum)
+                        ret = gf_asprintf (&optstr, "%d",
+                                           nfs->override_portnum);
+                else
+                        ret = gf_asprintf (&optstr, "%d", GF_NFS3_PORT);
+                if (ret == -1) {
+                        gf_log (GF_NFS, GF_LOG_ERROR, "failed mem-allocation");
+                        goto free_foppool;
+                }
+                ret = dict_set_dynstr (this->options,
+                                       "transport.socket.listen-port", optstr);
+                if (ret == -1) {
+                        gf_log (GF_NFS, GF_LOG_ERROR, "dict_set_dynstr error");
+                        goto free_foppool;
+                }
+        }
+
+        if (dict_get(this->options, "transport-type") == NULL) {
+                ret = dict_set_str (this->options, "transport-type", "socket");
+                if (ret == -1) {
+                        gf_log (GF_NFS, GF_LOG_ERROR, "dict_set_str error");
+                        goto free_foppool;
+                }
+        }
+
+        nfs->mount_udp = 0;
+        if (dict_get(this->options, "nfs.mount-udp")) {
+                ret = dict_get_str (this->options, "nfs.mount-udp", &optstr);
+                if (ret == -1) {
+                        gf_log (GF_NFS, GF_LOG_ERROR, "Failed to parse dict");
+                        goto free_foppool;
+                }
+
+                ret = gf_string2boolean (optstr, &boolt);
+                if (ret < 0) {
+                        gf_log (GF_NFS, GF_LOG_ERROR, "Failed to parse bool "
+                                "string");
+                        goto free_foppool;
+                }
+
+                if (boolt == _gf_true)
+                        nfs->mount_udp = 1;
+        }
+
+        /* support both options rpc-auth.ports.insecure and
+         * rpc-auth-allow-insecure for backward compatibility
+         */
+        nfs->allow_insecure = 1;
+        if (dict_get(this->options, "rpc-auth.ports.insecure")) {
+                ret = dict_get_str (this->options, "rpc-auth.ports.insecure",
+                                    &optstr);
+                if (ret < 0) {
+                        gf_log (GF_NFS, GF_LOG_ERROR, "Failed to parse dict");
+                        goto free_foppool;
+                }
+
+                ret = gf_string2boolean (optstr, &boolt);
+                if (ret < 0) {
+                        gf_log (GF_NFS, GF_LOG_ERROR, "Failed to parse bool "
+                                "string");
+                        goto free_foppool;
+                }
+
+                if (boolt == _gf_false)
+                        nfs->allow_insecure = 0;
+        }
+
+        if (dict_get(this->options, "rpc-auth-allow-insecure")) {
+                ret = dict_get_str (this->options, "rpc-auth-allow-insecure",
+                                    &optstr);
+                if (ret < 0) {
+                        gf_log (GF_NFS, GF_LOG_ERROR, "Failed to parse dict");
+                        goto free_foppool;
+                }
+
+                ret = gf_string2boolean (optstr, &boolt);
+                if (ret < 0) {
+                        gf_log (GF_NFS, GF_LOG_ERROR, "Failed to parse bool "
+                                "string");
+                        goto free_foppool;
+                }
+
+                if (boolt == _gf_false)
+                        nfs->allow_insecure = 0;
+        }
+
+        if (nfs->allow_insecure) {
+                /* blindly set both the options */
+                dict_del (this->options, "rpc-auth-allow-insecure");
+                ret = dict_set_str (this->options,
+                                    "rpc-auth-allow-insecure", "on");
+                if (ret == -1) {
+                        gf_log (GF_NFS, GF_LOG_ERROR, "dict_set_str error");
+                        goto free_foppool;
+                }
+                dict_del (this->options, "rpc-auth.ports.insecure");
+                ret = dict_set_str (this->options,
+                                    "rpc-auth.ports.insecure", "on");
+                if (ret == -1) {
+                        gf_log (GF_NFS, GF_LOG_ERROR, "dict_set_str error");
+                        goto free_foppool;
+                }
+        }
+
+        if (stat("/sbin/rpc.statd", &stbuf) == -1) {
+                gf_log (GF_NFS, GF_LOG_WARNING, "/sbin/rpc.statd not found. "
+                        "Disabling NLM");
+                nfs->enable_nlm = _gf_false;
+        }
+
+        nfs->rpcsvc =  rpcsvc_init (this, this->ctx, this->options, 0);
+        if (!nfs->rpcsvc) {
+                ret = -1;
+                gf_log (GF_NFS, GF_LOG_ERROR, "RPC service init failed");
+                goto free_foppool;
+        }
+
         this->private = (void *)nfs;
         INIT_LIST_HEAD (&nfs->versions);
 
@@ -623,7 +755,6 @@ free_foppool:
 free_rpcsvc:
         /*
          * rpcsvc_deinit */
-free_nfs:
         if (ret < 0) {
                 GF_FREE (nfs);
                 nfs = NULL;
@@ -632,35 +763,6 @@ free_nfs:
         return nfs;
 }
 
-int
-validate_options (xlator_t *this, char **op_errstr)
-{
-        int                 ret = 0;
-        volume_opt_list_t  *vol_opt = NULL;
-        volume_opt_list_t  *tmp;
-
-        if (!this) {
-                gf_log (this->name, GF_LOG_DEBUG, "'this' not a valid ptr");
-                ret =-1;
-                goto out;
-        }
-
-        if (list_empty (&this->volume_options))
-                goto out;
-
-        vol_opt = list_entry (this->volume_options.next,
-                                      volume_opt_list_t, list);
-        list_for_each_entry_safe (vol_opt, tmp, &this->volume_options, list) {
-                ret = validate_xlator_volume_options_attacherr (this,
-                                                                vol_opt->given_opt,
-                                                                op_errstr);
-        }
-
-out:
-        return ret;
-
-}
-
 
 int
 init (xlator_t *this) {
@@ -690,6 +792,20 @@ init (xlator_t *this) {
                 goto err;
         }
 
+        ret = mount_init_state (this);
+        if (ret == -1) {
+                gf_log (GF_NFS, GF_LOG_CRITICAL, "Failed to init Mount"
+                        "state");
+                goto err;
+        }
+
+        ret = nlm4_init_state (this);
+        if (ret == -1) {
+                gf_log (GF_NFS, GF_LOG_CRITICAL, "Failed to init NLM"
+                        "state");
+                goto err;
+        }
+
         ret = nfs_init_versions (nfs, this);
         if (ret == -1) {
                 gf_log (GF_NFS, GF_LOG_ERROR, "Failed to initialize "
@@ -702,8 +818,6 @@ init (xlator_t *this) {
                 goto err;
         }
 
-        LOCK_INIT (&nfs->lock);
-
         gf_log (GF_NFS, GF_LOG_INFO, "NFS service started");
 err:
 
@@ -714,10 +828,8 @@ err:
 int
 notify (xlator_t *this, int32_t event, void *data, ...)
 {
-        struct nfs_state        *nfs = NULL;
         xlator_t                *subvol = NULL;
 
-        nfs = (struct nfs_state *)this->private;
         subvol = (xlator_t *)data;
 
         gf_log (GF_NFS, GF_LOG_TRACE, "Notification received: %d",
@@ -753,59 +865,127 @@ fini (xlator_t *this)
         return 0;
 }
 
-int
-nfs_priv_dump (xlator_t *this)
+int32_t
+nfs_forget (xlator_t *this, inode_t *inode)
 {
+        uint64_t           ctx        = 0;
+        struct list_head  *head       = NULL;
 
-        char             key_prefix[GF_DUMP_MAX_BUF_LEN] = {0, };
-        struct nfs_state *nfs = NULL;
-        int64_t          ino_cnt = 0;
-        int64_t          entry_cnt = 0;
-        int64_t          intvl_ino_cnt = 0;
-        int64_t          intvl_entry_cnt = 0;
+        if (inode_ctx_del (inode, this, &ctx))
+                return -1;
 
-        if (!this || !this->private)
-                goto out;
+        head = (struct list_head *)ctx;
+        GF_FREE (head);
 
-        nfs = this->private;
-
-        gf_proc_dump_build_key (key_prefix, "xlator.nfs",
-                                "priv");
-        gf_proc_dump_add_section (key_prefix);
-
-        LOCK (&nfs->lock);
-        {
-                intvl_ino_cnt   = nfs->res_stat.intvl_ino_cnt;
-                intvl_entry_cnt = nfs->res_stat.intvl_entry_cnt;
-                ino_cnt         = nfs->res_stat.ino_cnt;
-                entry_cnt       = nfs->res_stat.entry_cnt;
-
-                nfs->res_stat.intvl_ino_cnt     = 0;
-                nfs->res_stat.intvl_entry_cnt   = 0;
-        }
-        UNLOCK (&nfs->lock);
-
-        gf_proc_dump_write ("Interval hard resolution (inode)", "%ld",
-                             intvl_ino_cnt);
-        gf_proc_dump_write ("Interval hard resolution (entry)", "%ld",
-                             intvl_entry_cnt);
-        gf_proc_dump_write ("Aggregate hard resolution (inode)", "%ld",
-                             ino_cnt);
-        gf_proc_dump_write ("Aggregate hard resoluton (entry)", "%ld",
-                             entry_cnt);
-
-out:
         return 0;
 }
 
-struct xlator_cbks cbks = { };
+gf_boolean_t
+_nfs_export_is_for_vol (char *exname, char *volname)
+{
+        gf_boolean_t    ret = _gf_false;
+        char            *tmp = NULL;
+
+        tmp = exname;
+        if (tmp[0] == '/')
+                tmp++;
+
+        if (!strcmp (tmp, volname))
+                ret = _gf_true;
+
+        return ret;
+}
+
+int
+nfs_priv_to_dict (xlator_t *this, dict_t *dict)
+{
+        int                     ret = -1;
+        struct nfs_state        *priv = NULL;
+        struct mountentry       *mentry = NULL;
+        char                    *volname = NULL;
+        char                    key[1024] = {0,};
+        int                     count = 0;
+
+        GF_VALIDATE_OR_GOTO (THIS->name, this, out);
+        GF_VALIDATE_OR_GOTO (THIS->name, dict, out);
+
+        priv = this->private;
+        GF_ASSERT (priv);
+
+        ret = dict_get_str (dict, "volname", &volname);
+        if (ret) {
+                gf_log (this->name, GF_LOG_ERROR, "Could not get volname");
+                goto out;
+        }
+
+        list_for_each_entry (mentry, &priv->mstate->mountlist, mlist) {
+                if (!_nfs_export_is_for_vol (mentry->exname, volname))
+                        continue;
+
+                memset (key, 0, sizeof (key));
+                snprintf (key, sizeof (key), "client%d.hostname", count);
+                ret = dict_set_str (dict, key, mentry->hostname);
+                if (ret) {
+                        gf_log (this->name, GF_LOG_ERROR,
+                                "Error writing hostname to dict");
+                        goto out;
+                }
+
+                /* No connection data available yet in nfs server.
+                 * Hence, setting to 0 to prevent cli failing
+                 */
+                memset (key, 0, sizeof (key));
+                snprintf (key, sizeof (key), "client%d.bytesread", count);
+                ret = dict_set_uint64 (dict, key, 0);
+                if (ret) {
+                        gf_log (this->name, GF_LOG_ERROR,
+                                "Error writing bytes read to dict");
+                        goto out;
+                }
+
+                memset (key, 0, sizeof (key));
+                snprintf (key, sizeof (key), "client%d.byteswrite", count);
+                ret = dict_set_uint64 (dict, key, 0);
+                if (ret) {
+                        gf_log (this->name, GF_LOG_ERROR,
+                                "Error writing bytes write to dict");
+                        goto out;
+                }
+
+                count++;
+        }
+
+        ret = dict_set_int32 (dict, "clientcount", count);
+        if (ret)
+                gf_log (this->name, GF_LOG_ERROR,
+                        "Error writing client count to dict");
+
+out:
+        gf_log (THIS->name, GF_LOG_DEBUG, "Returning %d", ret);
+        return ret;
+}
+
+extern int32_t
+nlm_priv (xlator_t *this);
+
+int32_t
+nfs_priv (xlator_t *this)
+{
+        return nlm_priv (this);
+}
+
+
+struct xlator_cbks cbks = {
+        .forget      = nfs_forget,
+};
+
 struct xlator_fops fops = { };
 
 struct xlator_dumpops dumpops = {
-        .priv        = nfs_priv_dump,
+        .priv           = nfs_priv,
+        .priv_to_dict   = nfs_priv_to_dict,
 };
 
-
 /* TODO: If needed, per-volume options below can be extended to be export
 + * specific also because after export-dir is introduced, a volume is not
 + * neccessarily an export whereas different subdirectories within that volume
@@ -816,13 +996,13 @@ struct volume_options options[] = {
           .type = GF_OPTION_TYPE_SIZET,
           .description = "Size in which the client should issue read requests"
                          " to the Gluster NFSv3 server. Must be a multiple of"
-                         " 4KiB."
+                         " 4KB."
         },
         { .key  = {"nfs3.write-size"},
           .type = GF_OPTION_TYPE_SIZET,
           .description = "Size in which the client should issue write requests"
                          " to the Gluster NFSv3 server. Must be a multiple of"
-                         " 4KiB."
+                         " 4KB."
         },
         { .key  = {"nfs3.readdir-size"},
           .type = GF_OPTION_TYPE_SIZET,
@@ -832,13 +1012,11 @@ struct volume_options options[] = {
         { .key  = {"nfs3.*.volume-access"},
           .type = GF_OPTION_TYPE_STR,
           .value = {"read-only", "read-write"},
-          .default_value = "read-write",
           .description = "Type of access desired for this subvolume: "
                          " read-only, read-write(default)"
         },
         { .key  = {"nfs3.*.trusted-write"},
           .type = GF_OPTION_TYPE_BOOL,
-          .default_value = "off",
           .description = "On an UNSTABLE write from client, return STABLE flag"
                          " to force client to not send a COMMIT request. In "
                          "some environments, combined with a replicated "
@@ -853,17 +1031,15 @@ struct volume_options options[] = {
         },
         { .key  = {"nfs3.*.trusted-sync"},
           .type = GF_OPTION_TYPE_BOOL,
-          .default_value = "off",
           .description = "All writes and COMMIT requests are treated as async."
                          " This implies that no write requests are guaranteed"
                          " to be on server disks when the write reply is "
-                         "received at the NFS client. Trusted sync includes"
+                         "received at the NFS client. Trusted sync includes "
                          " trusted-write behaviour. Off by default."
 
         },
         { .key  = {"nfs3.*.export-dir"},
-          .type = GF_OPTION_TYPE_STR,
-           .default_value = "",
+          .type = GF_OPTION_TYPE_PATH,
           .description = "By default, all subvolumes of nfs are exported as "
                          "individual exports. There are cases where a "
                          "subdirectory or subdirectories in the volume need to "
@@ -874,7 +1050,6 @@ struct volume_options options[] = {
         },
         { .key  = {"nfs3.export-dirs"},
           .type = GF_OPTION_TYPE_BOOL,
-          .default_value = "on",
           .description = "By default, all subvolumes of nfs are exported as "
                          "individual exports. There are cases where a "
                          "subdirectory or subdirectories in the volume need to "
@@ -884,7 +1059,6 @@ struct volume_options options[] = {
         },
         { .key  = {"nfs3.export-volumes"},
           .type = GF_OPTION_TYPE_BOOL,
-          .default_value = "on",
           .description = "Enable or disable exporting whole volumes, instead "
                          "if used in conjunction with nfs3.export-dir, can "
                          "allow setting up only subdirectories as exports. On "
@@ -892,7 +1066,6 @@ struct volume_options options[] = {
         },
         { .key  = {"rpc-auth.auth-unix"},
           .type = GF_OPTION_TYPE_BOOL,
-          .default_value = "",
           .description = "Disable or enable the AUTH_UNIX authentication type."
                          "Must always be enabled for better interoperability."
                          "However, can be disabled if needed. Enabled by"
@@ -900,16 +1073,23 @@ struct volume_options options[] = {
         },
         { .key  = {"rpc-auth.auth-null"},
           .type = GF_OPTION_TYPE_BOOL,
-          .default_value = "",
           .description = "Disable or enable the AUTH_NULL authentication type."
                          "Must always be enabled. This option is here only to"
                          " avoid unrecognized option warnings"
         },
         { .key  = {"rpc-auth.auth-unix.*"},
           .type = GF_OPTION_TYPE_BOOL,
-          .default_value = "on",
           .description = "Disable or enable the AUTH_UNIX authentication type "
-                         "for a particular exported volume over-riding defaults"
+                         "for a particular exported volume overriding defaults"
+                         " and general setting for AUTH_UNIX scheme. Must "
+                         "always be enabled for better interoperability."
+                         "However, can be disabled if needed. Enabled by"
+                         "default."
+        },
+        { .key  = {"rpc-auth.auth-unix.*.allow"},
+          .type = GF_OPTION_TYPE_STR,
+          .description = "Disable or enable the AUTH_UNIX authentication type "
+                         "for a particular exported volume overriding defaults"
                          " and general setting for AUTH_UNIX scheme. Must "
                          "always be enabled for better interoperability."
                          "However, can be disabled if needed. Enabled by"
@@ -917,48 +1097,42 @@ struct volume_options options[] = {
         },
         { .key  = {"rpc-auth.auth-null.*"},
           .type = GF_OPTION_TYPE_BOOL,
-          .default_value = "",
           .description = "Disable or enable the AUTH_NULL authentication type "
-                         "for a particular exported volume over-riding defaults"
+                         "for a particular exported volume overriding defaults"
                          " and general setting for AUTH_NULL. Must always be "
                          "enabled. This option is here only to avoid "
                          "unrecognized option warnings."
         },
         { .key  = {"rpc-auth.addr.allow"},
-          .type = GF_OPTION_TYPE_STR,
-          .default_value = "",
+          .type = GF_OPTION_TYPE_INTERNET_ADDRESS_LIST,
           .description = "Allow a comma separated list of addresses and/or"
                          " hostnames to connect to the server. By default, all"
-                         " connections are disallowed. This allows users to "
+                         " connections are allowed. This allows users to "
                          "define a general rule for all exported volumes."
         },
         { .key  = {"rpc-auth.addr.reject"},
-          .type = GF_OPTION_TYPE_STR,
-          .default_value = "",
+          .type = GF_OPTION_TYPE_INTERNET_ADDRESS_LIST,
           .description = "Reject a comma separated list of addresses and/or"
                          " hostnames from connecting to the server. By default,"
-                         " all connections are disallowed. This allows users to"
-                         " define a general rule for all exported volumes."
+                         " all connections are allowed. This allows users to"
+                         "define a general rule for all exported volumes."
         },
         { .key  = {"rpc-auth.addr.*.allow"},
-          .type = GF_OPTION_TYPE_STR,
-          .default_value = "",
+          .type = GF_OPTION_TYPE_INTERNET_ADDRESS_LIST,
           .description = "Allow a comma separated list of addresses and/or"
                          " hostnames to connect to the server. By default, all"
-                         " connections are disallowed. This allows users to "
+                         " connections are allowed. This allows users to "
                          "define a rule for a specific exported volume."
         },
         { .key  = {"rpc-auth.addr.*.reject"},
-          .type = GF_OPTION_TYPE_STR,
-          .default_value = "",
+          .type = GF_OPTION_TYPE_INTERNET_ADDRESS_LIST,
           .description = "Reject a comma separated list of addresses and/or"
                          " hostnames from connecting to the server. By default,"
-                         " all connections are disallowed. This allows users to"
-                         " define a rule for a specific exported volume."
+                         " all connections are allowed. This allows users to"
+                         "define a rule for a specific exported volume."
         },
         { .key  = {"rpc-auth.ports.insecure"},
           .type = GF_OPTION_TYPE_BOOL,
-          .default_value = "off",
           .description = "Allow client connections from unprivileged ports. By "
                          "default only privileged ports are allowed. This is a"
                          "global setting in case insecure ports are to be "
@@ -966,24 +1140,22 @@ struct volume_options options[] = {
         },
         { .key  = {"rpc-auth.ports.*.insecure"},
           .type = GF_OPTION_TYPE_BOOL,
-          .default_value = "",
           .description = "Allow client connections from unprivileged ports. By "
                          "default only privileged ports are allowed. Use this"
-                         " option to set enable or disable insecure ports for "
-                         "a specific subvolume and to over-ride global setting "
+                         " option to enable or disable insecure ports for "
+                         "a specific subvolume and to override the global setting "
                          " set by the previous option."
         },
         { .key  = {"rpc-auth.addr.namelookup"},
           .type = GF_OPTION_TYPE_BOOL,
-          .default_value = "on",
-          .description = "Users have the option of turning off name lookup for"
-                  " incoming client connections using this option. In some "
+          .description = "Users have the option of turning on name lookup for"
+                  " incoming client connections using this option. Use this "
+                  "option to turn on name lookups during address-based "
+                  "authentication. Turning this on will enable you to"
+                  " use hostnames in rpc-auth.addr.* filters. In some "
                   "setups, the name server can take too long to reply to DNS "
-                  "queries resulting in timeouts of mount requests. Use this "
-                  "option to turn off name lookups during address "
-                  "authentication. Note, turning this off will prevent you from"
-                  " using hostnames in rpc-auth.addr.* filters. By default, "
-                  " name lookup is on."
+                  "queries resulting in timeouts of mount requests. By default, "
+                  " name lookup is off"
         },
         { .key  = {"nfs.dynamic-volumes"},
           .type = GF_OPTION_TYPE_BOOL,
@@ -1001,29 +1173,29 @@ struct volume_options options[] = {
         },
         { .key  = {"nfs.enable-ino32"},
           .type = GF_OPTION_TYPE_BOOL,
-          .default_value = "off",
           .description = "For nfs clients or apps that do not support 64-bit "
                          "inode numbers, use this option to make NFS return "
-                         "32-bit inode numbers instead. Disabled by default so "
-                         "NFS returns 64-bit inode numbers by default."
+                         "32-bit inode numbers instead. Disabled by default, so "
+                         "NFS returns 64-bit inode numbers."
         },
         { .key  = {"rpc.register-with-portmap"},
           .type = GF_OPTION_TYPE_BOOL,
-          .default_value = "on",
           .description = "For systems that need to run multiple nfs servers, "
                          "only one registration is possible with "
                          "portmap service. Use this option to turn off portmap "
-                         "registration for Gluster NFS. On by default."
+                         "registration for Gluster NFS. On by default"
         },
         { .key  = {"nfs.port"},
           .type = GF_OPTION_TYPE_INT,
-          .default_value = "",
+          .min  = 1,
+          .max  = 0xffff,
           .description = "Use this option on systems that need Gluster NFS to "
                          "be associated with a non-default port number."
         },
         { .key  = {"nfs.mem-factor"},
           .type = GF_OPTION_TYPE_INT,
-          .default_value = "15",
+          .min  = 1,
+          .max  = 1024,
           .description = "Use this option to make NFS be faster on systems by "
                          "using more memory. This option specifies a multiple "
                          "that determines the total amount of memory used. "
@@ -1032,21 +1204,24 @@ struct volume_options options[] = {
                          "Please consult gluster-users list before using this "
                          "option."
         },
-        /* XXX These are synthetic options used to validate the options in   *
-         *     during volume set command and also to get visibiliity in      *
-         *     volume set help command                                       */
-        { .key  = {"nfs-disable"},
+        { .key  = {"nfs.*.disable"},
           .type = GF_OPTION_TYPE_BOOL,
-          .default_value = "off",
-          .description = "This option is used to enable/disable NFS server "
+          .description = "This option is used to start or stop NFS server"
                          "for individual volume."
         },
-        {
-          .key = {"nfs.transport-type"},
-          .type = GF_OPTION_TYPE_STR,
-          .default_value = "",
-          .description = "This options is used to change the transport type of"
-                        " NFS Server.",
+
+        { .key  = {"nfs.nlm"},
+          .type = GF_OPTION_TYPE_BOOL,
+          .description = "This option, if set to 'off', disables NLM server "
+                         "by not registering the service with the portmapper."
+                         " Set it to 'on' to re-enable it. Default value: 'on'"
+        },
+
+        { .key = {"nfs.mount-udp"},
+          .type = GF_OPTION_TYPE_BOOL,
+          .description = "set the option to 'on' to enable mountd on UDP. "
+                         "Needed by Solaris NFS clients if NLM support is"
+                         "needed"
         },
 
         { .key  = {NULL} },
