@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2007-2011 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
@@ -28,6 +28,7 @@
 #include <netdb.h>
 #include "authenticate.h"
 #include "dict.h"
+#include "rpc-transport.h"
 
 #define ADDR_DELIMITER " ,"
 #define PRIVILEGED_PORT_CEILING 1024
@@ -36,13 +37,6 @@
 #define AF_INET_SDP 27
 #endif
 
-/* TODO: duplicate declaration */
-typedef struct peer_info {
-        struct sockaddr_storage sockaddr;
-        socklen_t sockaddr_len;
-        char identifier[UNIX_PATH_MAX];
-}peer_info_t;
-
 auth_result_t
 gf_auth (dict_t *input_params, dict_t *config_params)
 {
@@ -232,14 +226,14 @@ out:
 
 struct volume_options options[] = {
         { .key   = {"auth.addr.*.allow"},
-          .type  = GF_OPTION_TYPE_ANY
+          .type  = GF_OPTION_TYPE_INTERNET_ADDRESS_LIST
         },
         { .key   = {"auth.addr.*.reject"},
-          .type  = GF_OPTION_TYPE_ANY
+          .type  = GF_OPTION_TYPE_INTERNET_ADDRESS_LIST
         },
         /* Backword compatibility */
         { .key   = {"auth.ip.*.allow"},
-          .type  = GF_OPTION_TYPE_ANY
+          .type  = GF_OPTION_TYPE_INTERNET_ADDRESS_LIST
         },
         { .key = {NULL} }
 };
