@@ -190,7 +190,7 @@ int
 _validate_volfile_checksum (xlator_t *this, char *key,
                             uint32_t checksum)
 {
-        char                 filename[ZR_PATH_MAX] = {0,};
+        char                 filename[PATH_MAX] = {0,};
         server_conf_t       *conf         = NULL;
         struct _volfile_ctx *temp_volfile = NULL;
         int                  ret          = 0;
@@ -253,7 +253,7 @@ server_getspec (rpcsvc_request_t *req)
         int32_t              op_errno               = ENOENT;
         int32_t              spec_fd                = -1;
         size_t               file_len               = 0;
-        char                 filename[ZR_PATH_MAX]  = {0,};
+        char                 filename[PATH_MAX]  = {0,};
         struct stat          stbuf                  = {0,};
         uint32_t             checksum               = 0;
         char                *key                    = NULL;
@@ -261,12 +261,10 @@ server_getspec (rpcsvc_request_t *req)
         xlator_t            *this                   = NULL;
         gf_getspec_req       args                   = {0,};
         gf_getspec_rsp       rsp                    = {0,};
-        server_connection_t *conn                   = NULL;
 
-        conn = req->trans->private;
         this = req->svc->mydata;
         conf = this->private;
-        if (xdr_to_glusterfs_req (req, &args, xdr_to_getspec_req)) {
+        if (!xdr_to_generic (req->msg[0], &args, (xdrproc_t)xdr_gf_getspec_req)) {
                 //failed to decode msg;
                 req->rpc_err = GARBAGE_ARGS;
                 op_errno = EINVAL;
@@ -326,7 +324,7 @@ fail:
         rsp.op_ret   = ret;
 
         server_submit_reply (NULL, req, &rsp, NULL, 0, NULL,
-                             (gfs_serialize_t)xdr_serialize_getspec_rsp);
+                             (xdrproc_t)xdr_gf_getspec_rsp);
 
         return 0;
 }
@@ -356,11 +354,13 @@ server_setvolume (rpcsvc_request_t *req)
         int32_t              op_errno      = EINVAL;
         int32_t              fop_version   = 0;
         int32_t              mgmt_version  = 0;
+        uint32_t             lk_version    = 0;
         char                *buf           = NULL;
+        gf_boolean_t        cancelled      = _gf_false;
 
         params = dict_new ();
         reply  = dict_new ();
-        if (xdr_to_glusterfs_req (req, &args, xdr_to_setvolume_req)) {
+        if (!xdr_to_generic (req->msg[0], &args, (xdrproc_t)xdr_gf_setvolume_req)) {
                 //failed to decode msg;
                 req->rpc_err = GARBAGE_ARGS;
                 goto fail;
@@ -410,8 +410,37 @@ server_setvolume (rpcsvc_request_t *req)
                 goto fail;
         }
 
+        /*lk_verion :: [1..2^31-1]*/
+        ret = dict_get_uint32 (params, "clnt-lk-version", &lk_version);
+        if (ret < 0) {
+                ret = dict_set_str (reply, "ERROR",
+                                    "lock state version not supplied");
+                if (ret < 0)
+                        gf_log (this->name, GF_LOG_DEBUG,
+                                "failed to set error msg");
+
+                op_ret = -1;
+                op_errno = EINVAL;
+                goto fail;
+        }
 
         conn = server_connection_get (this, process_uuid);
+        if (!conn) {
+                op_ret = -1;
+                op_errno = ENOMEM;
+                goto fail;
+        }
+
+        gf_log (this->name, GF_LOG_DEBUG, "Connected to %s", conn->id);
+        cancelled = server_cancel_conn_timer (this, conn);
+        if (cancelled)//Do connection_put on behalf of grace-timer-handler.
+                server_connection_put (this, conn, NULL);
+        if (conn->lk_version != 0 &&
+            conn->lk_version != lk_version) {
+                (void) server_connection_cleanup (this, conn,
+                                                  INTERNAL_LOCKS | POSIX_LOCKS);
+        }
+
         if (req->trans->xl_private != conn)
                 req->trans->xl_private = conn;
 
@@ -538,7 +567,7 @@ server_setvolume (rpcsvc_request_t *req)
 
                 gf_log (this->name, GF_LOG_INFO,
                         "accepted client from %s (version: %s)",
-                        (peerinfo) ? peerinfo->identifier : "<>",
+                        conn->id,
                         (clnt_version) ? clnt_version : "old");
                 op_ret = 0;
                 conn->bound_xl = xl;
@@ -549,7 +578,7 @@ server_setvolume (rpcsvc_request_t *req)
         } else {
                 gf_log (this->name, GF_LOG_ERROR,
                         "Cannot authenticate client from %s %s",
-                        (peerinfo) ? peerinfo->identifier : "<>",
+                        conn->id,
                         (clnt_version) ? clnt_version : "old");
 
                 op_ret = -1;
@@ -597,6 +626,12 @@ server_setvolume (rpcsvc_request_t *req)
                 gf_log (this->name, GF_LOG_DEBUG,
                         "failed to set 'process-uuid'");
 
+        ret = dict_set_uint32 (reply, "clnt-lk-version",
+                               conn->lk_version);
+        if (ret)
+                gf_log (this->name, GF_LOG_WARNING,
+                        "failed to set 'clnt-lk-version'");
+
         ret = dict_set_uint64 (reply, "transport-ptr",
                                ((uint64_t) (long) req->trans));
         if (ret)
@@ -630,7 +665,7 @@ fail:
         rsp.op_errno = gf_errno_to_error (op_errno);
 
         server_submit_reply (NULL, req, &rsp, NULL, 0, NULL,
-                             (gfs_serialize_t)xdr_serialize_setvolume_rsp);
+                             (xdrproc_t)xdr_gf_setvolume_rsp);
 
 
         if (args.dict.dict_val)
@@ -660,17 +695,60 @@ server_ping (rpcsvc_request_t *req)
         rsp.op_ret = 0;
 
         server_submit_reply (NULL, req, &rsp, NULL, 0, NULL,
-                             xdr_serialize_common_rsp);
+                             (xdrproc_t)xdr_gf_common_rsp);
 
         return 0;
 }
 
+int
+server_set_lk_version (rpcsvc_request_t *req)
+{
+        int                     op_ret          = -1;
+        int                     op_errno        = EINVAL;
+        gf_set_lk_ver_req       args            = {0, };
+        gf_set_lk_ver_rsp       rsp             = {0,};
+        server_connection_t     *conn           = NULL;
+        xlator_t                *this           = NULL;
+
+        this = req->svc->mydata;
+        //TODO: Decide on an appropriate errno for the error-path
+        //below
+        if (!this)
+                goto fail;
+
+        if (!xdr_to_generic (req->msg[0], &args,
+                             (xdrproc_t)xdr_gf_set_lk_ver_req)) {
+                //failed to decode msg;
+                req->rpc_err = GARBAGE_ARGS;
+                goto fail;
+        }
+
+        conn = server_connection_get (this, args.uid);
+        conn->lk_version = args.lk_ver;
+        server_connection_put (this, conn, NULL);
+
+        rsp.lk_ver   = args.lk_ver;
+
+        op_ret = 0;
+fail:
+        rsp.op_ret   = op_ret;
+        rsp.op_errno = op_errno;
+        server_submit_reply (NULL, req, &rsp, NULL, 0, NULL,
+                             (xdrproc_t)xdr_gf_set_lk_ver_rsp);
+
+        if (args.uid != NULL) {
+                free (args.uid);
+        }
+
+        return 0;
+}
 
 rpcsvc_actor_t gluster_handshake_actors[] = {
-        [GF_HNDSK_NULL]      = {"NULL",      GF_HNDSK_NULL,      server_null, NULL, NULL },
-        [GF_HNDSK_SETVOLUME] = {"SETVOLUME", GF_HNDSK_SETVOLUME, server_setvolume, NULL, NULL },
-        [GF_HNDSK_GETSPEC]   = {"GETSPEC",   GF_HNDSK_GETSPEC,   server_getspec, NULL, NULL },
-        [GF_HNDSK_PING]      = {"PING",      GF_HNDSK_PING,      server_ping, NULL, NULL },
+        [GF_HNDSK_NULL]       = {"NULL",      GF_HNDSK_NULL,      server_null, NULL, NULL, 0},
+        [GF_HNDSK_SETVOLUME]  = {"SETVOLUME", GF_HNDSK_SETVOLUME, server_setvolume, NULL, NULL, 0},
+        [GF_HNDSK_GETSPEC]    = {"GETSPEC",   GF_HNDSK_GETSPEC,   server_getspec, NULL, NULL, 0},
+        [GF_HNDSK_PING]       = {"PING",      GF_HNDSK_PING,      server_ping, NULL, NULL, 0},
+        [GF_HNDSK_SET_LK_VER] = {"SET_LK_VER", GF_HNDSK_SET_LK_VER, server_set_lk_version, NULL, NULL },
 };
 
 
