@@ -36,41 +36,76 @@
 #include "authenticate.h"
 #include "rpcsvc.h"
 
-struct iobuf *
-gfs_serialize_reply (rpcsvc_request_t *req, void *arg, gfs_serialize_t sfunc,
-                     struct iovec *outmsg)
+void
+grace_time_handler (void *data)
 {
-        struct iobuf            *iob = NULL;
-        ssize_t                  retlen = -1;
+        server_connection_t     *conn = NULL;
+        xlator_t                *this = NULL;
+        gf_boolean_t            cancelled = _gf_false;
+        gf_boolean_t            detached = _gf_false;
+
+        conn = data;
+        this = conn->this;
+
+        GF_VALIDATE_OR_GOTO (THIS->name, conn, out);
+        GF_VALIDATE_OR_GOTO (THIS->name, this, out);
+
+        gf_log (this->name, GF_LOG_INFO, "grace timer expired for %s", conn->id);
+
+        cancelled = server_cancel_conn_timer (this, conn);
+        if (cancelled) {
+                //conn should not be destroyed in conn_put, so take a ref.
+                server_conn_ref (conn);
+                server_connection_put (this, conn, &detached);
+                if (detached)//reconnection did not happen :-(
+                        server_connection_cleanup (this, conn,
+                                                  INTERNAL_LOCKS | POSIX_LOCKS);
+                server_conn_unref (conn);
+        }
+out:
+        return;
+}
+
+struct iobuf *
+gfs_serialize_reply (rpcsvc_request_t *req, void *arg, struct iovec *outmsg,
+                     xdrproc_t xdrproc)
+{
+        struct iobuf *iob      = NULL;
+        ssize_t       retlen   = 0;
+        ssize_t       xdr_size = 0;
 
         GF_VALIDATE_OR_GOTO ("server", req, ret);
 
         /* First, get the io buffer into which the reply in arg will
          * be serialized.
          */
-        iob = iobuf_get (req->svc->ctx->iobuf_pool);
-        if (!iob) {
-                gf_log_callingfn ("", GF_LOG_ERROR, "Failed to get iobuf");
-                goto ret;
-        }
+        if (arg && xdrproc) {
+                xdr_size = xdr_sizeof (xdrproc, arg);
+                iob = iobuf_get2 (req->svc->ctx->iobuf_pool, xdr_size);
+                if (!iob) {
+                        gf_log_callingfn (THIS->name, GF_LOG_ERROR,
+                                          "Failed to get iobuf");
+                        goto ret;
+                };
 
-        iobuf_to_iovec (iob, outmsg);
-        /* Use the given serializer to translate the give C structure in arg
-         * to XDR format which will be written into the buffer in outmsg.
-         */
-        /* retlen is used to received the error since size_t is unsigned and we
-         * need -1 for error notification during encoding.
-         */
-        retlen = sfunc (*outmsg, arg);
-        if (retlen == -1) {
-                /* Failed to Encode 'GlusterFS' msg in RPC is not exactly
-                   failure of RPC return values.. client should get
-                   notified about this, so there are no missing frames */
-                gf_log_callingfn ("", GF_LOG_ERROR, "Failed to encode message");
-                req->rpc_err = GARBAGE_ARGS;
-                retlen = 0;
-        }
+                iobuf_to_iovec (iob, outmsg);
+                /* Use the given serializer to translate the give C structure in arg
+                 * to XDR format which will be written into the buffer in outmsg.
+                 */
+                /* retlen is used to received the error since size_t is unsigned and we
+                 * need -1 for error notification during encoding.
+                 */
 
+                retlen = xdr_serialize_generic (*outmsg, arg, xdrproc);
+                if (retlen == -1) {
+                        /* Failed to Encode 'GlusterFS' msg in RPC is not exactly
+                           failure of RPC return values.. client should get
+                           notified about this, so there are no missing frames */
+                        gf_log_callingfn ("", GF_LOG_ERROR, "Failed to encode message");
+                        req->rpc_err = GARBAGE_ARGS;
+                        retlen = 0;
+                }
+        }
         outmsg->iov_len = retlen;
 ret:
         if (retlen == -1) {
@@ -83,25 +118,30 @@ ret:
 
 
 
-/* Generic reply function for NFSv3 specific replies. */
 int
 server_submit_reply (call_frame_t *frame, rpcsvc_request_t *req, void *arg,
                      struct iovec *payload, int payloadcount,
-                     struct iobref *iobref, gfs_serialize_t sfunc)
+                     struct iobref *iobref, xdrproc_t xdrproc)
 {
         struct iobuf           *iob        = NULL;
         int                     ret        = -1;
         struct iovec            rsp        = {0,};
         server_state_t         *state      = NULL;
         char                    new_iobref = 0;
+        server_connection_t    *conn       = NULL;
+        gf_boolean_t            lk_heal    = _gf_false;
 
         GF_VALIDATE_OR_GOTO ("server", req, ret);
 
         if (frame) {
                 state = CALL_STATE (frame);
                 frame->local = NULL;
+                conn  = SERVER_CONNECTION(frame);
         }
 
+        if (conn)
+                lk_heal = ((server_conf_t *) conn->this->private)->lk_heal;
+
         if (!iobref) {
                 iobref = iobref_new ();
                 if (!iobref) {
@@ -111,7 +151,7 @@ server_submit_reply (call_frame_t *frame, rpcsvc_request_t *req, void *arg,
                 new_iobref = 1;
         }
 
-        iob = gfs_serialize_reply (req, arg, sfunc, &rsp);
+        iob = gfs_serialize_reply (req, arg, &rsp, xdrproc);
         if (!iob) {
                 gf_log ("", GF_LOG_ERROR, "Failed to serialize reply");
                 goto ret;
@@ -134,6 +174,14 @@ server_submit_reply (call_frame_t *frame, rpcsvc_request_t *req, void *arg,
         iobuf_unref (iob);
         if (ret == -1) {
                 gf_log_callingfn ("", GF_LOG_ERROR, "Reply submission failed");
+                if (frame && conn && !lk_heal) {
+                        server_connection_cleanup (frame->this, conn,
+                                                  INTERNAL_LOCKS | POSIX_LOCKS);
+                } else {
+                        /* TODO: Failure of open(dir), create, inodelk, entrylk
+                           or lk fops send failure must be handled specially. */
+                        ;
+                }
                 goto ret;
         }
 
@@ -144,6 +192,8 @@ ret:
         }
 
         if (frame) {
+                if (frame->root->trans)
+                        server_conn_unref (frame->root->trans);
                 STACK_DESTROY (frame->root);
         }
 
@@ -156,16 +206,33 @@ ret:
 
 /* */
 int
-xdr_to_glusterfs_req (rpcsvc_request_t *req, void *arg, gfs_serialize_t sfunc)
+server_fd_to_dict (xlator_t *this, dict_t *dict)
 {
+        server_conf_t           *conf = NULL;
+        server_connection_t     *trav = NULL;
+        char                    key[GF_DUMP_MAX_BUF_LEN] = {0,};
+        int                     count = 0;
         int                     ret = -1;
 
-        GF_VALIDATE_OR_GOTO ("server", req, out);
+        GF_VALIDATE_OR_GOTO (THIS->name, this, out);
+        GF_VALIDATE_OR_GOTO (this->name, dict, out);
 
-        ret = sfunc (req->msg[0], arg);
+        conf = this->private;
+        if (!conf)
+                return -1;
 
-        if (ret > 0)
-                ret = 0;
+        ret = pthread_mutex_trylock (&conf->mutex);
+        if (ret)
+                return -1;
+
+        list_for_each_entry (trav, &conf->conns, list) {
+                memset (key, 0, sizeof (key));
+                snprintf (key, sizeof (key), "conn%d", count++);
+                fdtable_dump_to_dict (trav->fdtable, key, dict);
+        }
+        pthread_mutex_unlock (&conf->mutex);
+
+        ret = dict_set_int32 (dict, "conncount", count);
 out:
         return ret;
 }
@@ -178,6 +245,7 @@ server_fd (xlator_t *this)
         char                 key[GF_DUMP_MAX_BUF_LEN];
         int                  i = 1;
         int                  ret = -1;
+        gf_boolean_t         section_added = _gf_false;
 
         GF_VALIDATE_OR_GOTO ("server", this, out);
 
@@ -189,41 +257,101 @@ server_fd (xlator_t *this)
         }
 
         gf_proc_dump_add_section("xlator.protocol.server.conn");
+        section_added = _gf_true;
 
         ret = pthread_mutex_trylock (&conf->mutex);
-        if (ret) {
-                gf_log("", GF_LOG_WARNING, "Unable to dump fdtable"
-                       " errno: %d", errno);
-                return -1;
-        }
+        if (ret)
+                goto out;
 
         list_for_each_entry (trav, &conf->conns, list) {
                 if (trav->id) {
                         gf_proc_dump_build_key(key,
-                                               "xlator.protocol.server.conn",
-                                               "%d.id", i);
+                                               "conn","%d.id", i);
                         gf_proc_dump_write(key, "%s", trav->id);
                 }
 
-                gf_proc_dump_build_key(key,"xlator.protocol.server.conn",
-                                       "%d.ref",i)
+                gf_proc_dump_build_key(key,"conn","%d.ref",i)
                         gf_proc_dump_write(key, "%d", trav->ref);
                 if (trav->bound_xl) {
                         gf_proc_dump_build_key(key,
-                                               "xlator.protocol.server.conn",
-                                               "%d.bound_xl", i);
+                                               "conn","%d.bound_xl", i);
                         gf_proc_dump_write(key, "%s", trav->bound_xl->name);
                 }
 
                 gf_proc_dump_build_key(key,
-                                       "xlator.protocol.server.conn",
-                                       "%d.id", i);
+                                       "conn","%d.id", i);
                 fdtable_dump(trav->fdtable,key);
                 i++;
         }
         pthread_mutex_unlock (&conf->mutex);
 
         ret = 0;
+out:
+        if (ret) {
+                if (section_added == _gf_false)
+                        gf_proc_dump_add_section("xlator.protocol.server.conn");
+                gf_proc_dump_write ("Unable to dump the list of connections",
+                                    "(Lock acquisition failed) %s",
+                                    this?this->name:"server");
+        }
+        return ret;
+}
+
+int
+server_priv_to_dict (xlator_t *this, dict_t *dict)
+{
+        server_conf_t   *conf = NULL;
+        rpc_transport_t *xprt = NULL;
+        peer_info_t     *peerinfo = NULL;
+        char            key[32] = {0,};
+        int             count = 0;
+        int             ret = -1;
+
+        GF_VALIDATE_OR_GOTO (THIS->name, this, out);
+        GF_VALIDATE_OR_GOTO (THIS->name, dict, out);
+
+        conf = this->private;
+        if (!conf)
+                return 0;
+        //TODO: Dump only specific info to dict
+
+        pthread_mutex_lock (&conf->mutex);
+        {
+                list_for_each_entry (xprt, &conf->xprt_list, list) {
+                        peerinfo = &xprt->peerinfo;
+                        memset (key, 0, sizeof (key));
+                        snprintf (key, sizeof (key), "client%d.hostname",
+                                  count);
+                        ret = dict_set_str (dict, key, peerinfo->identifier);
+                        if (ret)
+                                goto unlock;
+
+                        memset (key, 0, sizeof (key));
+                        snprintf (key, sizeof (key), "client%d.bytesread",
+                                  count);
+                        ret = dict_set_uint64 (dict, key,
+                                               xprt->total_bytes_read);
+                        if (ret)
+                                goto unlock;
+
+                        memset (key, 0, sizeof (key));
+                        snprintf (key, sizeof (key), "client%d.byteswrite",
+                                  count);
+                        ret = dict_set_uint64 (dict, key,
+                                               xprt->total_bytes_write);
+                        if (ret)
+                                goto unlock;
+
+                        count++;
+                }
+        }
+unlock:
+        pthread_mutex_unlock (&conf->mutex);
+        if (ret)
+                goto out;
+
+        ret = dict_set_int32 (dict, "clientcount", count);
+
 out:
         return ret;
 }
@@ -244,10 +372,19 @@ server_priv (xlator_t *this)
         if (!conf)
                 return 0;
 
-        list_for_each_entry (xprt, &conf->xprt_list, list) {
-                total_read  += xprt->total_bytes_read;
-                total_write += xprt->total_bytes_write;
+        gf_proc_dump_build_key (key, "xlator.protocol.server", "priv");
+        gf_proc_dump_add_section (key);
+
+        ret = pthread_mutex_trylock (&conf->mutex);
+        if (ret != 0)
+                goto out;
+        {
+                list_for_each_entry (xprt, &conf->xprt_list, list) {
+                        total_read  += xprt->total_bytes_read;
+                        total_write += xprt->total_bytes_write;
+                }
         }
+        pthread_mutex_unlock (&conf->mutex);
 
         gf_proc_dump_build_key(key, "server", "total-bytes-read");
         gf_proc_dump_write(key, "%"PRIu64, total_read);
@@ -257,6 +394,62 @@ server_priv (xlator_t *this)
 
         ret = 0;
 out:
+        if (ret)
+                gf_proc_dump_write ("Unable to print priv",
+                                    "(Lock acquisition failed) %s",
+                                    this?this->name:"server");
+
+        return ret;
+}
+
+int
+server_inode_to_dict (xlator_t *this, dict_t *dict)
+{
+        server_conf_t           *conf = NULL;
+        server_connection_t     *trav = NULL;
+        char                    key[32] = {0,};
+        int                     count = 0;
+        int                     ret = -1;
+        xlator_t                *prev_bound_xl = NULL;
+
+        GF_VALIDATE_OR_GOTO (THIS->name, this, out);
+        GF_VALIDATE_OR_GOTO (this->name, dict, out);
+
+        conf = this->private;
+        if (!conf)
+                return -1;
+
+        ret = pthread_mutex_trylock (&conf->mutex);
+        if (ret)
+                return -1;
+
+        list_for_each_entry (trav, &conf->conns, list) {
+                if (trav->bound_xl && trav->bound_xl->itable) {
+                        /* Presently every brick contains only one
+                         * bound_xl for all connections. This will lead
+                         * to duplicating of the inode lists, if listing
+                         * is done for every connection. This simple check
+                         * prevents duplication in the present case. If
+                         * need arises the check can be improved.
+                         */
+                        if (trav->bound_xl == prev_bound_xl)
+                                continue;
+                        prev_bound_xl = trav->bound_xl;
+
+                        memset (key, 0, sizeof (key));
+                        snprintf (key, sizeof (key), "conn%d", count);
+                        inode_table_dump_to_dict (trav->bound_xl->itable,
+                                                  key, dict);
+                        count++;
+                }
+        }
+        pthread_mutex_unlock (&conf->mutex);
+
+        ret = dict_set_int32 (dict, "conncount", count);
+
+out:
+        if (prev_bound_xl)
+                prev_bound_xl = NULL;
         return ret;
 }
 
@@ -268,6 +461,7 @@ server_inode (xlator_t *this)
         char                 key[GF_DUMP_MAX_BUF_LEN];
         int                  i = 1;
         int                  ret = -1;
+        xlator_t             *prev_bound_xl = NULL;
 
         GF_VALIDATE_OR_GOTO ("server", this, out);
 
@@ -279,17 +473,24 @@ server_inode (xlator_t *this)
         }
 
         ret = pthread_mutex_trylock (&conf->mutex);
-        if (ret) {
-                gf_log("", GF_LOG_WARNING, "Unable to dump itable"
-                       " errno: %d", errno);
-                return -1;
-        }
+        if (ret)
+                goto out;
 
         list_for_each_entry (trav, &conf->conns, list) {
                 if (trav->bound_xl && trav->bound_xl->itable) {
+                        /* Presently every brick contains only one
+                         * bound_xl for all connections. This will lead
+                         * to duplicating of the inode lists, if listing
+                         * is done for every connection. This simple check
+                         * prevents duplication in the present case. If
+                         * need arises the check can be improved.
+                         */
+                        if (trav->bound_xl == prev_bound_xl)
+                                continue;
+                        prev_bound_xl = trav->bound_xl;
+
                         gf_proc_dump_build_key(key,
-                                               "xlator.protocol.server.conn",
-                                               "%d.bound_xl.%s",
+                                               "conn","%d.bound_xl.%s",
                                                i, trav->bound_xl->name);
                         inode_table_dump(trav->bound_xl->itable,key);
                         i++;
@@ -299,6 +500,11 @@ server_inode (xlator_t *this)
 
         ret = 0;
 out:
+        if (ret)
+                gf_proc_dump_write ("Unable to dump the inode table",
+                                    "(Lock acquisition failed) %s",
+                                    this?this->name:"server");
+
         return ret;
 }
 
@@ -351,6 +557,9 @@ validate_auth_options (xlator_t *this, dict_t *dict)
         xlator_list_t *trav = NULL;
         data_pair_t   *pair = NULL;
         char          *tail = NULL;
+        char          *tmp_addr_list = NULL;
+        char          *addr = NULL;
+        char          *tmp_str = NULL;
 
         GF_VALIDATE_OR_GOTO ("server", this, out);
         GF_VALIDATE_OR_GOTO ("server", dict, out);
@@ -374,9 +583,49 @@ validate_auth_options (xlator_t *this, dict_t *dict)
 
                         if (*tail == '.') {
                                 error = 0;
-                                break;
+
+                                /* when we are here, the key is checked for
+                                 * valid auth.allow.<xlator>
+                                 * Now we verify the ip address
+                                 */
+                                if (!strcmp (pair->value->data, "*")) {
+                                     error = 0;
+                                     goto out;
+                                }
+
+                                tmp_addr_list = gf_strdup (pair->value->data);
+                                addr = strtok_r (tmp_addr_list, ",",
+                                                &tmp_str);
+                                if (!addr)
+                                        addr = pair->value->data;
+
+                                while (addr) {
+
+                                        if (valid_internet_address
+                                                        (addr, _gf_true)) {
+                                                error = 0;
+                                        } else {
+                                                error = -1;
+                                                gf_log (this->name, GF_LOG_ERROR,
+                                                        "internet address '%s'"
+                                                        " does not conform to"
+                                                        " standards.", addr);
+                                                goto out;
+
+                                        }
+                                        if (tmp_str)
+                                                addr = strtok_r (NULL, ",",
+                                                                 &tmp_str);
+                                        else
+                                                addr = NULL;
+                                }
+
+                                GF_FREE (tmp_addr_list);
+                                tmp_addr_list = NULL;
                         }
+
                 }
+
                 if (-1 == error) {
                         gf_log (this->name, GF_LOG_ERROR,
                                 "volume '%s' defined as subvolume, but no "
@@ -386,7 +635,10 @@ validate_auth_options (xlator_t *this, dict_t *dict)
                 }
                 trav = trav->next;
         }
+
 out:
+        if (tmp_addr_list)
+                GF_FREE (tmp_addr_list);
         return error;
 }
 
@@ -395,11 +647,11 @@ int
 server_rpc_notify (rpcsvc_t *rpc, void *xl, rpcsvc_event_t event,
                    void *data)
 {
-        xlator_t            *this = NULL;
-        rpc_transport_t     *xprt = NULL;
-        server_connection_t *conn = NULL;
-        server_conf_t       *conf = NULL;
-
+        gf_boolean_t         detached   = _gf_false;
+        xlator_t            *this       = NULL;
+        rpc_transport_t     *xprt       = NULL;
+        server_connection_t *conn       = NULL;
+        server_conf_t       *conf       = NULL;
 
         if (!xl || !data) {
                 gf_log_callingfn ("server", GF_LOG_WARNING,
@@ -424,26 +676,69 @@ server_rpc_notify (rpcsvc_t *rpc, void *xl, rpcsvc_event_t event,
                 */
                 INIT_LIST_HEAD (&xprt->list);
 
-                list_add_tail (&xprt->list, &conf->xprt_list);
+                pthread_mutex_lock (&conf->mutex);
+                {
+                        list_add_tail (&xprt->list, &conf->xprt_list);
+                }
+                pthread_mutex_unlock (&conf->mutex);
 
                 break;
         }
         case RPCSVC_EVENT_DISCONNECT:
+                /* transport has to be removed from the list upon disconnect
+                 * irrespective of whether lock self heal is off or on, since
+                 * new transport will be created upon reconnect.
+                 */
+                pthread_mutex_lock (&conf->mutex);
+                {
+                        list_del_init (&xprt->list);
+                }
+                pthread_mutex_unlock (&conf->mutex);
+
                 conn = get_server_conn_state (this, xprt);
-                if (conn)
-                        server_connection_cleanup (this, conn);
+                if (!conn)
+                        break;
 
-                gf_log (this->name, GF_LOG_INFO,
-                        "disconnected connection from %s",
-                        xprt->peerinfo.identifier);
+                gf_log (this->name, GF_LOG_INFO, "disconnecting connection"
+                        "from %s", conn->id);
 
-                list_del (&xprt->list);
+                /* If lock self heal is off, then destroy the
+                   conn object, else register a grace timer event */
+                if (!conf->lk_heal) {
+                        server_conn_ref (conn);
+                        server_connection_put (this, conn, &detached);
+                        if (detached)
+                                server_connection_cleanup (this, conn,
+                                                           INTERNAL_LOCKS |
+                                                           POSIX_LOCKS);
+                        server_conn_unref (conn);
+                } else {
+                        put_server_conn_state (this, xprt);
+                        server_connection_cleanup (this, conn, INTERNAL_LOCKS);
 
+                        pthread_mutex_lock (&conn->lock);
+                        {
+                                if (conn->timer)
+                                        goto unlock;
+
+                                gf_log (this->name, GF_LOG_INFO, "starting a grace "
+                                        "timer for %s", conn->id);
+
+                                conn->timer = gf_timer_call_after (this->ctx,
+                                                                   conf->grace_tv,
+                                                                   grace_time_handler,
+                                                                   conn);
+                        }
+                unlock:
+                        pthread_mutex_unlock (&conn->lock);
+                }
                 break;
         case RPCSVC_EVENT_TRANSPORT_DESTROY:
-                conn = get_server_conn_state (this, xprt);
-                if (conn)
-                        server_connection_put (this, conn);
+                /*- conn obj has been disassociated from xprt on first
+                 *  disconnect.
+                 *  conn cleanup and destruction is handed over to
+                 *  grace_time_handler or the subsequent handler that 'owns'
+                 *  the conn. Nothing left to be done here. */
                 break;
         default:
                 break;
@@ -471,34 +766,6 @@ out:
         return ret;
 }
 
-int
-validate_options ( xlator_t *this, char **op_errstr)
-{
-        int                 ret = 0;
-        volume_opt_list_t  *vol_opt = NULL;
-        volume_opt_list_t  *tmp;
-
-        if (!this) {
-                gf_log (this->name, GF_LOG_DEBUG, "'this' not a valid ptr");
-                ret =-1;
-                goto out;
-        }
-
-        if (list_empty (&this->volume_options))
-                goto out;
-
-        vol_opt = list_entry (this->volume_options.next,
-                                      volume_opt_list_t, list);
-         list_for_each_entry_safe (vol_opt, tmp, &this->volume_options, list) {
-                ret = validate_xlator_volume_options_attacherr (this,
-                                                                vol_opt->given_opt,
-                                                                op_errstr);
-        }
-
-out:
-
-        return ret;
-}
 
 static void
 _delete_auth_opt (dict_t *this,
@@ -536,6 +803,43 @@ _copy_auth_opt (dict_t *unused,
 }
 
 
+int
+server_init_grace_timer (xlator_t *this, dict_t *options,
+                         server_conf_t *conf)
+{
+        int32_t   ret            = -1;
+        int32_t   grace_timeout  = -1;
+        char     *lk_heal        = NULL;
+
+        GF_VALIDATE_OR_GOTO ("server", this, out);
+        GF_VALIDATE_OR_GOTO (this->name, options, out);
+        GF_VALIDATE_OR_GOTO (this->name, conf, out);
+
+        conf->lk_heal = _gf_false;
+
+        ret = dict_get_str (options, "lk-heal", &lk_heal);
+        if (!ret)
+                gf_string2boolean (lk_heal, &conf->lk_heal);
+
+        gf_log (this->name, GF_LOG_DEBUG, "lk-heal = %s",
+                (conf->lk_heal) ? "on" : "off");
+
+        ret = dict_get_int32 (options, "grace-timeout", &grace_timeout);
+        if (!ret)
+                conf->grace_tv.tv_sec = grace_timeout;
+        else
+                conf->grace_tv.tv_sec = 10;
+
+        gf_log (this->name, GF_LOG_DEBUG, "Server grace timeout "
+                "value = %"PRIu64, conf->grace_tv.tv_sec);
+
+        conf->grace_tv.tv_usec  = 0;
+
+        ret = 0;
+out:
+        return ret;
+}
+
 int
 reconfigure (xlator_t *this, dict_t *options)
 {
@@ -547,7 +851,7 @@ reconfigure (xlator_t *this, dict_t *options)
         gf_boolean_t              trace;
         data_t                   *data;
         int                       ret = 0;
-
+        char                     *statedump_path = NULL;
         conf = this->private;
 
         if (!conf) {
@@ -575,6 +879,27 @@ reconfigure (xlator_t *this, dict_t *options)
 			" to %d", conf->trace);
 
 	}
+
+        /*ret = dict_get_str (options, "statedump-path", &statedump_path);
+        if (!ret) {
+                gf_path_strip_trailing_slashes (statedump_path);
+                if (this->ctx->statedump_path)
+                        GF_FREE (this->ctx->statedump_path);
+                this->ctx->statedump_path = gf_strdup (statedump_path);
+        }*/
+        GF_OPTION_RECONF ("statedump-path", statedump_path,
+                          options, path, out);
+        if (!statedump_path) {
+                gf_log (this->name, GF_LOG_ERROR,
+                        "Error while reconfiguring statedump path");
+                ret = -1;
+                goto out;
+        }
+        gf_path_strip_trailing_slashes (statedump_path);
+        if (this->ctx->statedump_path)
+                GF_FREE (this->ctx->statedump_path);
+        this->ctx->statedump_path = gf_strdup (statedump_path);
+
         if (!conf->auth_modules)
                 conf->auth_modules = dict_new ();
 
@@ -609,6 +934,7 @@ reconfigure (xlator_t *this, dict_t *options)
                                         "Reconfigure not found for transport" );
                 }
         }
+        ret = server_init_grace_timer (this, options, conf);
 
 out:
         gf_log ("", GF_LOG_DEBUG, "returning %d", ret);
@@ -621,7 +947,7 @@ init (xlator_t *this)
         int32_t            ret      = -1;
         server_conf_t     *conf     = NULL;
         rpcsvc_listener_t *listener = NULL;
-
+        char              *statedump_path = NULL;
         GF_VALIDATE_OR_GOTO ("init", this, out);
 
         if (this->children == NULL) {
@@ -645,6 +971,10 @@ init (xlator_t *this)
         INIT_LIST_HEAD (&conf->xprt_list);
         pthread_mutex_init (&conf->mutex, NULL);
 
+        ret = server_init_grace_timer (this, this->options, conf);
+        if (ret)
+                goto out;
+
         ret = server_build_config (this, conf);
         if (ret)
                 goto out;
@@ -653,6 +983,22 @@ init (xlator_t *this)
         if (ret)
                 conf->conf_dir = CONFDIR;
 
+        /*ret = dict_get_str (this->options, "statedump-path", &statedump_path);
+        if (!ret) {
+                gf_path_strip_trailing_slashes (statedump_path);
+                this->ctx->statedump_path = statedump_path;
+        }*/
+        GF_OPTION_INIT ("statedump-path", statedump_path, path, out);
+        if (statedump_path) {
+                gf_path_strip_trailing_slashes (statedump_path);
+                this->ctx->statedump_path = gf_strdup (statedump_path);
+        } else {
+                gf_log (this->name, GF_LOG_ERROR,
+                        "Error setting statedump path");
+                ret = -1;
+                goto out;
+        }
+
         /* Authentication modules */
         conf->auth_modules = dict_new ();
         GF_VALIDATE_OR_GOTO(this->name, conf->auth_modules, out);
@@ -671,8 +1017,7 @@ init (xlator_t *this)
         }
 
         /* RPC related */
-        //conf->rpc = rpc_svc_init (&conf->rpc_conf);
-        conf->rpc = rpcsvc_init (this->ctx, this->options);
+        conf->rpc = rpcsvc_init (this, this->ctx, this->options, 0);
         if (conf->rpc == NULL) {
                 gf_log (this->name, GF_LOG_WARNING,
                         "creation of rpcsvc failed");
@@ -814,9 +1159,12 @@ struct xlator_cbks cbks = {
 };
 
 struct xlator_dumpops dumpops = {
-        .priv  = server_priv,
-        .fd    = server_fd,
-        .inode = server_inode,
+        .priv           = server_priv,
+        .fd             = server_fd,
+        .inode          = server_inode,
+        .priv_to_dict   = server_priv_to_dict,
+        .fd_to_dict     = server_fd_to_dict,
+        .inode_to_dict  = server_inode_to_dict,
 };
 
 
@@ -842,7 +1190,8 @@ struct volume_options options[] = {
         { .key   = {"inode-lru-limit"},
           .type  = GF_OPTION_TYPE_INT,
           .min   = 0,
-          .max   = (1 * GF_UNIT_MB)
+          .max   = (1 * GF_UNIT_MB),
+          .default_value = "16384",
         },
         { .key   = {"verify-volfile-checksum"},
           .type  = GF_OPTION_TYPE_BOOL
@@ -857,22 +1206,41 @@ struct volume_options options[] = {
         { .key   = {"rpc-auth-allow-insecure"},
           .type  = GF_OPTION_TYPE_BOOL,
         },
+        { .key           = {"statedump-path"},
+          .type          = GF_OPTION_TYPE_PATH,
+          .default_value = "/tmp",
+          .description = "Specifies directory in which gluster should save its"
+                         " statedumps. By default it is the /tmp directory"
+        },
+        { .key   = {"lk-heal"},
+          .type  = GF_OPTION_TYPE_BOOL,
+          .default_value = "off",
+        },
+        {.key  = {"grace-timeout"},
+         .type = GF_OPTION_TYPE_INT,
+         .min  = 10,
+         .max  = 1800,
+        },
+        {.key  = {"tcp-window-size"},
+         .type = GF_OPTION_TYPE_SIZET,
+         .min  = GF_MIN_SOCKET_WINDOW_SIZE,
+         .max  = GF_MAX_SOCKET_WINDOW_SIZE
+        },
 
-        /* XXX These are synthetic options which are actually recognized and *
-         *     validated in addr.c, added here to get visibiliity in         *
-         *     volume set help/help-xml                                      */
+        /*  The following two options are defined in addr.c, redifined here *
+         * for the sake of validation during volume set from cli            */
 
         { .key   = {"auth.addr.*.allow"},
-          .type  = GF_OPTION_TYPE_ANY,
-          .default_value = "*",
-          .description = "'IP addresses/Host name' of the clients which should"
-                         " be allowed to access the the volume."
+          .type  = GF_OPTION_TYPE_INTERNET_ADDRESS_LIST,
+          .description = "Allow a comma separated list of addresses and/or "
+                         "hostnames to connect to the server. By default, all"
+                         " connections are allowed."
         },
         { .key   = {"auth.addr.*.reject"},
-          .type  = GF_OPTION_TYPE_ANY,
-          .default_value = "",
-          .description = "'IP addresses/Host name' of the clients which should"
-                         " be denied to access the volume."
+          .type  = GF_OPTION_TYPE_INTERNET_ADDRESS_LIST,
+          .description = "Reject a comma separated list of addresses and/or "
+                         "hostnames to connect to the server. By default, all"
+                         " connections are allowed."
         },
 
         { .key   = {NULL} },
